@@ -8,22 +8,19 @@
    bits instead of 64 is not important; speed is considerably more
    important.  ANSI guarantees that "unsigned long" will be big enough,
    and always using it seems to have few disadvantages.  */
-typedef unsigned long uint32;
+typedef unsigned long cvs_uint32;
 
-struct MD5Context {
-	uint32 buf[4];
-	uint32 bits[2];
+struct cvs_MD5Context {
+	cvs_uint32 buf[4];
+	cvs_uint32 bits[2];
 	unsigned char in[64];
 };
 
-void MD5Init PROTO((struct MD5Context *context));
-void MD5Update PROTO((struct MD5Context *context, unsigned char const *buf, unsigned len));
-void MD5Final PROTO((unsigned char digest[16], struct MD5Context *context));
-void MD5Transform PROTO((uint32 buf[4], const unsigned char in[64]));
-
-/*
- * This is needed to make RSAREF happy on some MS-DOS compilers.
- */
-typedef struct MD5Context MD5_CTX;
+void cvs_MD5Init (struct cvs_MD5Context *context);
+void cvs_MD5Update (struct cvs_MD5Context *context,
+			   unsigned char const *buf, unsigned len);
+void cvs_MD5Final (unsigned char digest[16],
+			  struct cvs_MD5Context *context);
+void cvs_MD5Transform (cvs_uint32 buf[4], const unsigned char in[64]);
 
 #endif /* !MD5_H */
