@@ -1,11 +1,6 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
- *
- * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
- *                                  and others.
- *
- * Portions Copyright (c) 1992, Brian Berliner and Jeff Polk
- * Portions Copyright (c) 1989-1992, Brian Berliner
+ * Copyright (c) 1992, Brian Berliner and Jeff Polk
+ * Copyright (c) 1989-1992, Brian Berliner
  * 
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with the CVS source distribution.
@@ -18,12 +13,13 @@
 #ifdef CVS_ADMIN_GROUP
 #include <grp.h>
 #endif
-#include <assert.h>
 
-static Dtype admin_dirproc PROTO ((void *callerdat, const char *dir,
-                                   const char *repos, const char *update_dir,
-                                   List *entries));
-static int admin_fileproc PROTO ((void *callerdat, struct file_info *finfo));
+static Dtype admin_dirproc (void *callerdat, const char *dir,
+                            const char *repos, const char *update_dir,
+                            List *entries);
+static int admin_fileproc (void *callerdat, struct file_info *finfo);
+
+char *UserAdminOptions = "k";
 
 static const char *const admin_usage[] =
 {
@@ -117,10 +113,7 @@ struct admin_data
    happen depends on whether the option was specified as optional to
    getopt).  */
 static void
-arg_add (dat, opt, arg)
-    struct admin_data *dat;
-    int opt;
-    char *arg;
+arg_add (struct admin_data *dat, int opt, char *arg)
 {
     char *newelt = xmalloc ((arg == NULL ? 0 : strlen (arg)) + 3);
     strcpy (newelt, "-");
@@ -145,19 +138,17 @@ arg_add (dat, opt, arg)
 }
 
 int
-admin (argc, argv)
-    int argc;
-    char **argv;
+admin (int argc, char **argv)
 {
     int err;
 #ifdef CVS_ADMIN_GROUP
     struct group *grp;
-    struct group *getgrnam();
+    struct group *getgrnam(const char *);
 #endif
     struct admin_data admin_data;
     int c;
     int i;
-    int only_k_option;
+    int only_allowed_options;
 
     if (argc <= 1)
 	usage (admin_usage);
@@ -170,12 +161,12 @@ admin (argc, argv)
        example, admin_data->branch should be not `-bfoo' but simply `foo'. */
 
     optind = 0;
-    only_k_option = 1;
+    only_allowed_options = 1;
     while ((c = getopt (argc, argv,
 			"+ib::c:a:A:e::l::u::LUn:N:m:o:s:t::IqxV:k:")) != -1)
     {
-	if (c != 'k' && c != 'q')
-	    only_k_option = 0;
+	if (c != 'q' && !strchr(UserAdminOptions, c))
+	    only_allowed_options = 0;
 
 	switch (c)
 	{
@@ -232,12 +223,12 @@ admin (argc, argv)
 		break;
 
 	    case 'l':
-		/* Note that multiple -l options are legal.  */
+		/* Note that multiple -l options are valid.  */
 		arg_add (&admin_data, 'l', optarg);
 		break;
 
 	    case 'u':
-		/* Note that multiple -u options are legal.  */
+		/* Note that multiple -u options are valid.  */
 		arg_add (&admin_data, 'u', optarg);
 		break;
 
@@ -269,7 +260,7 @@ admin (argc, argv)
 		/* Mostly similar to cvs tag.  Could also be parsing
 		   the syntax of optarg, although for now we just pass
 		   it to rcs as-is.  Note that multiple -n options are
-		   legal.  */
+		   valid.  */
 		arg_add (&admin_data, 'n', optarg);
 		break;
 
@@ -277,14 +268,14 @@ admin (argc, argv)
 		/* Mostly similar to cvs tag.  Could also be parsing
 		   the syntax of optarg, although for now we just pass
 		   it to rcs as-is.  Note that multiple -N options are
-		   legal.  */
+		   valid.  */
 		arg_add (&admin_data, 'N', optarg);
 		break;
 
 	    case 'm':
 		/* Change log message.  Could also be parsing the syntax
 		   of optarg, although for now we just pass it to rcs
-		   as-is.  Note that multiple -m options are legal.  */
+		   as-is.  Note that multiple -m options are valid.  */
 		arg_add (&admin_data, 'm', optarg);
 		break;
 
@@ -295,7 +286,7 @@ admin (argc, argv)
 		   Other than that I'm not sure whether it matters much
 		   whether we parse it here or in admin_fileproc.
 
-		   Note that multiple -o options are illegal, in RCS
+		   Note that multiple -o options are invalid, in RCS
 		   as well as here.  */
 
 		if (admin_data.delete_revs != NULL)
@@ -309,7 +300,7 @@ admin (argc, argv)
 		break;
 
 	    case 's':
-		/* Note that multiple -s options are legal.  */
+		/* Note that multiple -s options are valid.  */
 		arg_add (&admin_data, 's', optarg);
 		break;
 
@@ -393,8 +384,8 @@ admin (argc, argv)
 	 */
         !current_parsed_root->isremote &&
 # endif	/* CLIENT_SUPPORT */
-        !only_k_option
-	&& (grp = getgrnam(CVS_ADMIN_GROUP)) != NULL)
+        !only_allowed_options &&
+	(grp = getgrnam(CVS_ADMIN_GROUP)) != NULL)
     {
 #ifdef HAVE_GETGROUPS
 	gid_t *grps;
@@ -520,13 +511,14 @@ admin (argc, argv)
     }
 #endif /* CLIENT_SUPPORT */
 
-    lock_tree_for_write (argc, argv, 0, W_LOCAL, 0);
+    lock_tree_promotably (argc, argv, 0, W_LOCAL, 0);
+
+    err = start_recursion
+	    (admin_fileproc, NULL, admin_dirproc,
+	     NULL, &admin_data,
+	     argc, argv, 0,
+	     W_LOCAL, 0, CVS_LOCK_WRITE, NULL, 1, NULL);
 
-    err = start_recursion (admin_fileproc, (FILESDONEPROC) NULL, admin_dirproc,
-			   (DIRLEAVEPROC) NULL, (void *)&admin_data,
-			   argc, argv, 0,
-			   W_LOCAL, 0, CVS_LOCK_NONE, (char *) NULL, 1,
-			   (char *) NULL);
     Lock_Cleanup ();
 
  return_it:
@@ -553,9 +545,7 @@ admin (argc, argv)
  */
 /* ARGSUSED */
 static int
-admin_fileproc (callerdat, finfo)
-    void *callerdat;
-    struct file_info *finfo;
+admin_fileproc (void *callerdat, struct file_info *finfo)
 {
     struct admin_data *admin_data = (struct admin_data *) callerdat;
     Vers_TS *vers;
@@ -821,13 +811,6 @@ admin_fileproc (callerdat, finfo)
 		{
 		    tag = xstrdup (arg + 2);
 		    rev = RCS_head (rcs);
-		    if (!rev)
-		    {
-			error (0, 0, "No head revision in archive file `%s'.",
-			       rcs->path);
-			status = 1;
-			continue;
-		    }
 		}
 		else
 		{
@@ -901,7 +884,7 @@ admin_fileproc (callerdat, finfo)
 		    memset ((void *) delta->text, 0, sizeof (Deltatext));
 		}
 		delta->text->version = xstrdup (delta->version);
-		delta->text->log = make_message_rcslegal (msg);
+		delta->text->log = make_message_rcsvalid (msg);
 		break;
 
 	    case 'l':
@@ -936,17 +919,15 @@ admin_fileproc (callerdat, finfo)
     return status;
 }
 
+
+
 /*
  * Print a warm fuzzy message
  */
 /* ARGSUSED */
 static Dtype
-admin_dirproc (callerdat, dir, repos, update_dir, entries)
-    void *callerdat;
-    const char *dir;
-    const char *repos;
-    const char *update_dir;
-    List *entries;
+admin_dirproc (void *callerdat, const char *dir, const char *repos,
+               const char *update_dir, List *entries)
 {
     if (!quiet)
 	error (0, 0, "Administrating %s", update_dir);
