@@ -1,56 +1,40 @@
-/*
- * Copyright (C) 1996-2005 The Free Software Foundation, Inc.
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- */
-
 /* Code for the buffer data structure.  */
 
-#include <assert.h>
 #include "cvs.h"
 #include "buffer.h"
 
 #if defined (SERVER_SUPPORT) || defined (CLIENT_SUPPORT)
 
-#ifdef HAVE_WINSOCK_H
-# include <winsock.h>
-#else
-# include <sys/socket.h>
-#endif
+# ifdef HAVE_WINSOCK_H
+#   include <winsock.h>
+# else
+#  include <sys/socket.h>
+# endif
 
 /* OS/2 doesn't have EIO.  FIXME: this whole notion of turning
    a different error into EIO strikes me as pretty dubious.  */
-#if !defined (EIO)
-#define EIO EBADPOS
-#endif
+# if !defined( EIO )
+#   define EIO EBADPOS
+# endif
 
 /* Linked list of available buffer_data structures.  */
 static struct buffer_data *free_buffer_data;
 
 /* Local functions.  */
-static void buf_default_memory_error PROTO ((struct buffer *));
-static void allocate_buffer_datas PROTO((void));
-static struct buffer_data *get_buffer_data PROTO((void));
+static void buf_default_memory_error (struct buffer *);
+static void allocate_buffer_datas (void);
+static struct buffer_data *get_buffer_data (void);
 
 /* Initialize a buffer structure.  */
 
 struct buffer *
-buf_initialize (input, output, flush, block, shutdown, memory, closure)
-     int (*input) PROTO((void *, char *, int, int, int *));
-     int (*output) PROTO((void *, const char *, int, int *));
-     int (*flush) PROTO((void *));
-     int (*block) PROTO((void *, int));
-     int (*shutdown) PROTO((struct buffer *));
-     void (*memory) PROTO((struct buffer *));
-     void *closure;
+buf_initialize (int (*input) (void *, char *, int, int, int *),
+	        int (*output) (void *, const char *, int, int *),
+                int (*flush) (void *),
+                int (*block) (void *, int),
+                int (*shutdown) (struct buffer *),
+                void (*memory) (struct buffer *),
+                void *closure)
 {
     struct buffer *buf;
 
@@ -71,8 +55,7 @@ buf_initialize (input, output, flush, block, shutdown, memory, closure)
 /* Free a buffer structure.  */
 
 void
-buf_free (buf)
-     struct buffer *buf;
+buf_free (struct buffer *buf)
 {
     if (buf->closure != NULL)
     {
@@ -90,15 +73,14 @@ buf_free (buf)
 /* Initialize a buffer structure which is not to be used for I/O.  */
 
 struct buffer *
-buf_nonio_initialize (memory)
-     void (*memory) PROTO((struct buffer *));
+buf_nonio_initialize( void (*memory) (struct buffer *) )
 {
     return (buf_initialize
-	    ((int (*) PROTO((void *, char *, int, int, int *))) NULL,
-	     (int (*) PROTO((void *, const char *, int, int *))) NULL,
-	     (int (*) PROTO((void *))) NULL,
-	     (int (*) PROTO((void *, int))) NULL,
-	     (int (*) PROTO((struct buffer *))) NULL,
+	    ((int (*) (void *, char *, int, int, int *)) NULL,
+	     (int (*) (void *, const char *, int, int *)) NULL,
+	     (int (*) (void *)) NULL,
+	     (int (*) (void *, int)) NULL,
+	     (int (*) (struct buffer *)) NULL,
 	     memory,
 	     (void *) NULL));
 }
@@ -106,8 +88,7 @@ buf_nonio_initialize (memory)
 /* Default memory error handler.  */
 
 static void
-buf_default_memory_error (buf)
-     struct buffer *buf;
+buf_default_memory_error (struct buffer *buf)
 {
     error (1, 0, "out of memory");
 }
@@ -115,22 +96,20 @@ buf_default_memory_error (buf)
 /* Allocate more buffer_data structures.  */
 
 static void
-allocate_buffer_datas ()
+allocate_buffer_datas (void)
 {
     struct buffer_data *alc;
     char *space;
     int i;
 
     /* Allocate buffer_data structures in blocks of 16.  */
-#define ALLOC_COUNT (16)
+# define ALLOC_COUNT (16)
 
-    alc = xmalloc (ALLOC_COUNT * sizeof (struct buffer_data));
+    alc = ((struct buffer_data *)
+	   xmalloc (ALLOC_COUNT * sizeof (struct buffer_data)));
     space = (char *) valloc (ALLOC_COUNT * BUFFER_DATA_SIZE);
-    if (!space)
-    {
-	free (alc);
+    if (alc == NULL || space == NULL)
 	return;
-    }
     for (i = 0; i < ALLOC_COUNT; i++, alc++, space += BUFFER_DATA_SIZE)
     {
 	alc->next = free_buffer_data;
@@ -142,7 +121,7 @@ allocate_buffer_datas ()
 /* Get a new buffer_data structure.  */
 
 static struct buffer_data *
-get_buffer_data ()
+get_buffer_data (void)
 {
     struct buffer_data *ret;
 
@@ -176,8 +155,7 @@ buf_empty (buf)
 
 /* See whether a buffer is empty.  */
 int
-buf_empty_p (buf)
-    struct buffer *buf;
+buf_empty_p (struct buffer *buf)
 {
     struct buffer_data *data;
 
@@ -189,15 +167,14 @@ buf_empty_p (buf)
 
 
 
-#ifdef SERVER_FLOWCONTROL
+# ifdef SERVER_FLOWCONTROL
 /*
  * Count how much data is stored in the buffer..
  * Note that each buffer is a xmalloc'ed chunk BUFFER_DATA_SIZE.
  */
 
 int
-buf_count_mem (buf)
-    struct buffer *buf;
+buf_count_mem (struct buffer *buf)
 {
     struct buffer_data *data;
     int mem = 0;
@@ -207,15 +184,12 @@ buf_count_mem (buf)
 
     return mem;
 }
-#endif /* SERVER_FLOWCONTROL */
+# endif /* SERVER_FLOWCONTROL */
 
 /* Add data DATA of length LEN to BUF.  */
 
 void
-buf_output (buf, data, len)
-    struct buffer *buf;
-    const char *data;
-    int len;
+buf_output (struct buffer *buf, const char *data, int len)
 {
     if (buf->data != NULL
 	&& (((buf->last->text + BUFFER_DATA_SIZE)
@@ -267,9 +241,7 @@ buf_output (buf, data, len)
 /* Add a '\0' terminated string to BUF.  */
 
 void
-buf_output0 (buf, string)
-    struct buffer *buf;
-    const char *string;
+buf_output0 (struct buffer *buf, const char *string)
 {
     buf_output (buf, string, strlen (string));
 }
@@ -277,9 +249,7 @@ buf_output0 (buf, string)
 /* Add a single character to BUF.  */
 
 void
-buf_append_char (buf, ch)
-    struct buffer *buf;
-    int ch;
+buf_append_char (struct buffer *buf, int ch)
 {
     if (buf->data != NULL
 	&& (buf->last->text + BUFFER_DATA_SIZE
@@ -304,8 +274,7 @@ buf_append_char (buf, ch)
  */
 
 int
-buf_send_output (buf)
-     struct buffer *buf;
+buf_send_output (struct buffer *buf)
 {
     if (buf->output == NULL)
 	abort ();
@@ -367,9 +336,7 @@ buf_send_output (buf)
  */
 
 int
-buf_flush (buf, block)
-     struct buffer *buf;
-     int block;
+buf_flush (struct buffer *buf, int block)
 {
     int nonblocking;
     int status;
@@ -407,8 +374,7 @@ buf_flush (buf, block)
  */
 
 int
-set_nonblock (buf)
-     struct buffer *buf;
+set_nonblock (struct buffer *buf)
 {
     int status;
 
@@ -429,8 +395,7 @@ set_nonblock (buf)
  */
 
 int
-set_block (buf)
-     struct buffer *buf;
+set_block (struct buffer *buf)
 {
     int status;
 
@@ -454,8 +419,7 @@ set_block (buf)
  */
 
 int
-buf_send_counted (buf)
-     struct buffer *buf;
+buf_send_counted (struct buffer *buf)
 {
     int size;
     struct buffer_data *data;
@@ -494,9 +458,7 @@ buf_send_counted (buf)
  */
 
 int
-buf_send_special_count (buf, count)
-     struct buffer *buf;
-     int count;
+buf_send_special_count (struct buffer *buf, int count)
 {
     struct buffer_data *data;
 
@@ -523,10 +485,8 @@ buf_send_special_count (buf, count)
 /* Append a list of buffer_data structures to an buffer.  */
 
 void
-buf_append_data (buf, data, last)
-     struct buffer *buf;
-     struct buffer_data *data;
-     struct buffer_data *last;
+buf_append_data (struct buffer *buf, struct buffer_data *data,
+                 struct buffer_data *last)
 {
     if (data != NULL)
     {
@@ -542,9 +502,7 @@ buf_append_data (buf, data, last)
    from the source buffer.  */
 
 void
-buf_append_buffer (to, from)
-     struct buffer *to;
-     struct buffer *from;
+buf_append_buffer (struct buffer *to, struct buffer *from)
 {
     buf_append_data (to, from->data, from->last);
     from->data = NULL;
@@ -562,11 +520,8 @@ buf_append_buffer (to, from)
  */
 
 int
-buf_read_file (f, size, retp, lastp)
-    FILE *f;
-    long size;
-    struct buffer_data **retp;
-    struct buffer_data **lastp;
+buf_read_file (FILE *f, long int size, struct buffer_data **retp,
+               struct buffer_data **lastp)
 {
     int status;
 
@@ -631,10 +586,8 @@ buf_read_file (f, size, retp, lastp)
  */
 
 int
-buf_read_file_to_eof (f, retp, lastp)
-     FILE *f;
-     struct buffer_data **retp;
-     struct buffer_data **lastp;
+buf_read_file_to_eof (FILE *f, struct buffer_data **retp,
+                      struct buffer_data **lastp)
 {
     int status;
 
@@ -690,8 +643,7 @@ buf_read_file_to_eof (f, retp, lastp)
 /* Return the number of bytes in a chain of buffer_data structures.  */
 
 int
-buf_chain_length (buf)
-     struct buffer_data *buf;
+buf_chain_length (struct buffer_data *buf)
 {
     int size = 0;
     while (buf)
@@ -705,8 +657,7 @@ buf_chain_length (buf)
 /* Return the number of bytes in a buffer.  */
 
 int
-buf_length (buf)
-    struct buffer *buf;
+buf_length (struct buffer *buf)
 {
     return buf_chain_length (buf->data);
 }
@@ -720,9 +671,7 @@ buf_length (buf)
  */
 
 int
-buf_input_data (buf, countp)
-     struct buffer *buf;
-     int *countp;
+buf_input_data (struct buffer *buf, int *countp)
 {
     if (buf->input == NULL)
 	abort ();
@@ -795,10 +744,7 @@ buf_input_data (buf, countp)
  */
 
 int
-buf_read_line (buf, line, lenp)
-     struct buffer *buf;
-     char **line;
-     int *lenp;
+buf_read_line (struct buffer *buf, char **line, int *lenp)
 {
     if (buf->input == NULL)
         abort ();
@@ -932,11 +878,7 @@ buf_read_line (buf, line, lenp)
  */
 
 int
-buf_read_data (buf, want, retdata, got)
-     struct buffer *buf;
-     int want;
-     char **retdata;
-     int *got;
+buf_read_data (struct buffer *buf, int want, char **retdata, int *got)
 {
     if (buf->input == NULL)
 	abort ();
@@ -1007,10 +949,7 @@ buf_read_data (buf, want, retdata, got)
  */
 
 void
-buf_copy_lines (outbuf, inbuf, command)
-     struct buffer *outbuf;
-     struct buffer *inbuf;
-     int command;
+buf_copy_lines (struct buffer *outbuf, struct buffer *inbuf, int command)
 {
     while (1)
     {
@@ -1091,10 +1030,7 @@ buf_copy_lines (outbuf, inbuf, command)
  */
 
 int
-buf_copy_counted (outbuf, inbuf, special)
-     struct buffer *outbuf;
-     struct buffer *inbuf;
-     int *special;
+buf_copy_counted (struct buffer *outbuf, struct buffer *inbuf, int *special)
 {
     *special = 0;
 
@@ -1238,8 +1174,7 @@ buf_copy_counted (outbuf, inbuf, special)
 /* Shut down a buffer.  This returns 0 on success, or an errno code.  */
 
 int
-buf_shutdown (buf)
-     struct buffer *buf;
+buf_shutdown (struct buffer *buf)
 {
     if (buf->shutdown)
 	return (*buf->shutdown) (buf);
@@ -1253,10 +1188,10 @@ buf_shutdown (buf)
    implement setting this type of buffer into nonblocking mode.  The
    closure field is just a FILE *.  */
 
-static int stdio_buffer_input PROTO((void *, char *, int, int, int *));
-static int stdio_buffer_output PROTO((void *, const char *, int, int *));
-static int stdio_buffer_flush PROTO((void *));
-static int stdio_buffer_shutdown PROTO((struct buffer *buf));
+static int stdio_buffer_input (void *, char *, int, int, int *);
+static int stdio_buffer_output (void *, const char *, int, int *);
+static int stdio_buffer_flush (void *);
+static int stdio_buffer_shutdown (struct buffer *buf);
 
 
 
@@ -1270,11 +1205,8 @@ struct stdio_buffer_closure
 
 
 struct buffer *
-stdio_buffer_initialize (fp, child_pid, input, memory)
-     FILE *fp;
-     int child_pid;
-     int input;
-     void (*memory) PROTO((struct buffer *));
+stdio_buffer_initialize (FILE *fp, int child_pid, int input,
+                         void (*memory) (struct buffer *))
 {
     struct stdio_buffer_closure *bc = xmalloc (sizeof (*bc));
 
@@ -1284,37 +1216,32 @@ stdio_buffer_initialize (fp, child_pid, input, memory)
     return buf_initialize (input ? stdio_buffer_input : NULL,
 			   input ? NULL : stdio_buffer_output,
 			   input ? NULL : stdio_buffer_flush,
-			   (int (*) PROTO((void *, int))) NULL,
-			   stdio_buffer_shutdown,
-			   memory,
-			   (void *) bc);
+			   NULL, stdio_buffer_shutdown, memory,
+			   bc);
 }
 
+
+
 /* Return the file associated with a stdio buffer. */
 FILE *
-stdio_buffer_get_file (buf)
-    struct buffer *buf;
+stdio_buffer_get_file (struct buffer *buf)
 {
     struct stdio_buffer_closure *bc;
 
-    assert(buf->shutdown == stdio_buffer_shutdown);
+    assert (buf->shutdown == stdio_buffer_shutdown);
 
-    bc = (struct stdio_buffer_closure *) buf->closure;
+    bc = buf->closure;
 
-    return(bc->fp);
+    return bc->fp;
 }
 
-/* The buffer input function for a buffer built on a stdio FILE.  */
 
+
+/* The buffer input function for a buffer built on a stdio FILE.  */
 static int
-stdio_buffer_input (closure, data, need, size, got)
-     void *closure;
-     char *data;
-     int need;
-     int size;
-     int *got;
+stdio_buffer_input (void *closure, char *data, int need, int size, int *got)
 {
-    struct stdio_buffer_closure *bc = (struct stdio_buffer_closure *) closure;
+    struct stdio_buffer_closure *bc = closure;
     int nbytes;
 
     /* Since stdio does its own buffering, we don't worry about
@@ -1359,16 +1286,13 @@ stdio_buffer_input (closure, data, need, size, got)
     return 0;
 }
 
-/* The buffer output function for a buffer built on a stdio FILE.  */
 
+
+/* The buffer output function for a buffer built on a stdio FILE.  */
 static int
-stdio_buffer_output (closure, data, have, wrote)
-     void *closure;
-     const char *data;
-     int have;
-     int *wrote;
+stdio_buffer_output (void *closure, const char *data, int have, int *wrote)
 {
-    struct stdio_buffer_closure *bc = (struct stdio_buffer_closure *) closure;
+    struct stdio_buffer_closure *bc = closure;
 
     *wrote = 0;
 
@@ -1398,8 +1322,7 @@ stdio_buffer_output (closure, data, have, wrote)
 
 /* The buffer flush function for a buffer built on a stdio FILE.  */
 static int
-stdio_buffer_flush (closure)
-     void *closure;
+stdio_buffer_flush (void *closure)
 {
     struct stdio_buffer_closure *bc = (struct stdio_buffer_closure *) closure;
 
@@ -1417,8 +1340,7 @@ stdio_buffer_flush (closure)
 
 
 static int
-stdio_buffer_shutdown (buf)
-    struct buffer *buf;
+stdio_buffer_shutdown (struct buffer *buf)
 {
     struct stdio_buffer_closure *bc = buf->closure;
     struct stat s;
@@ -1491,7 +1413,11 @@ stdio_buffer_shutdown (buf)
 
     if (closefp && fclose (bc->fp) == EOF)
     {
-	if (server_active)
+	if (0
+# ifdef SERVER_SUPPORT
+	    || server_active
+# endif /* SERVER_SUPPORT */
+           )
 	{
             /* Syslog this? */
 	}
@@ -1534,7 +1460,7 @@ stdio_buffer_shutdown (buf)
    bytes, and the input translation routine should shrink the data
    correspondingly.  */
 
-#define PACKET_SLOP (100)
+# define PACKET_SLOP (100)
 
 /* This structure is the closure field of a packetizing buffer.  */
 
@@ -1547,16 +1473,16 @@ struct packetizing_buffer
        untranslate the data in INPUT, storing the result in OUTPUT.
        SIZE is the amount of data in INPUT, and is also the size of
        OUTPUT.  This should return 0 on success, or an errno code.  */
-    int (*inpfn) PROTO((void *fnclosure, const char *input, char *output,
-			int size));
+    int (*inpfn) (void *fnclosure, const char *input, char *output,
+			int size);
     /* The output translation function.  This should translate the
        data in INPUT, storing the result in OUTPUT.  The first two
        bytes in INPUT will be the size of the data, and so will SIZE.
        This should set *TRANSLATED to the amount of translated data in
        OUTPUT.  OUTPUT is large enough to hold SIZE + PACKET_SLOP
        bytes.  This should return 0 on success, or an errno code.  */
-    int (*outfn) PROTO((void *fnclosure, const char *input, char *output,
-			int size, int *translated));
+    int (*outfn) (void *fnclosure, const char *input, char *output,
+			int size, int *translated);
     /* A closure for the translation function.  */
     void *fnclosure;
     /* For an input buffer, we may have to buffer up data here.  */
@@ -1576,21 +1502,25 @@ struct packetizing_buffer
     char *holddata;
 };
 
-static int packetizing_buffer_input PROTO((void *, char *, int, int, int *));
-static int packetizing_buffer_output PROTO((void *, const char *, int, int *));
-static int packetizing_buffer_flush PROTO((void *));
-static int packetizing_buffer_block PROTO((void *, int));
-static int packetizing_buffer_shutdown PROTO((struct buffer *));
+
+
+static int packetizing_buffer_input (void *, char *, int, int, int *);
+static int packetizing_buffer_output (void *, const char *, int, int *);
+static int packetizing_buffer_flush (void *);
+static int packetizing_buffer_block (void *, int);
+static int packetizing_buffer_shutdown (struct buffer *);
+
+
 
 /* Create a packetizing buffer.  */
-
 struct buffer *
-packetizing_buffer_initialize (buf, inpfn, outfn, fnclosure, memory)
-     struct buffer *buf;
-     int (*inpfn) PROTO ((void *, const char *, char *, int));
-     int (*outfn) PROTO ((void *, const char *, char *, int, int *));
-     void *fnclosure;
-     void (*memory) PROTO((struct buffer *));
+packetizing_buffer_initialize (struct buffer *buf,
+                               int (*inpfn) (void *, const char *, char *,
+                                             int),
+                               int (*outfn) (void *, const char *, char *,
+                                             int, int *),
+                               void *fnclosure,
+                               void (*memory) (struct buffer *))
 {
     struct packetizing_buffer *pb;
 
@@ -1620,17 +1550,14 @@ packetizing_buffer_initialize (buf, inpfn, outfn, fnclosure, memory)
 			   pb);
 }
 
-/* Input data from a packetizing buffer.  */
 
+
+/* Input data from a packetizing buffer.  */
 static int
-packetizing_buffer_input (closure, data, need, size, got)
-     void *closure;
-     char *data;
-     int need;
-     int size;
-     int *got;
+packetizing_buffer_input (void *closure, char *data, int need, int size,
+                          int *got)
 {
-    struct packetizing_buffer *pb = (struct packetizing_buffer *) closure;
+    struct packetizing_buffer *pb = closure;
 
     *got = 0;
 
@@ -1833,16 +1760,14 @@ packetizing_buffer_input (closure, data, need, size, got)
     return 0;
 }
 
-/* Output data to a packetizing buffer.  */
 
+
+/* Output data to a packetizing buffer.  */
 static int
-packetizing_buffer_output (closure, data, have, wrote)
-     void *closure;
-     const char *data;
-     int have;
-     int *wrote;
+packetizing_buffer_output (void *closure, const char *data, int have,
+                           int *wrote)
 {
-    struct packetizing_buffer *pb = (struct packetizing_buffer *) closure;
+    struct packetizing_buffer *pb = closure;
     char inbuf[BUFFER_DATA_SIZE + 2];
     char stack_outbuf[BUFFER_DATA_SIZE + PACKET_SLOP + 4];
     struct buffer_data *outdata;
@@ -1916,8 +1841,7 @@ packetizing_buffer_output (closure, data, have, wrote)
 
 /* Flush data to a packetizing buffer.  */
 static int
-packetizing_buffer_flush (closure)
-     void *closure;
+packetizing_buffer_flush (void *closure)
 {
     struct packetizing_buffer *pb = (struct packetizing_buffer *) closure;
 
@@ -1932,9 +1856,7 @@ packetizing_buffer_flush (closure)
 
 /* The block routine for a packetizing buffer.  */
 static int
-packetizing_buffer_block (closure, block)
-     void *closure;
-     int block;
+packetizing_buffer_block (void *closure, int block)
 {
     struct packetizing_buffer *pb = (struct packetizing_buffer *) closure;
 
@@ -1944,13 +1866,13 @@ packetizing_buffer_block (closure, block)
 	return set_nonblock (pb->buf);
 }
 
-/* Shut down a packetizing buffer.  */
 
+
+/* Shut down a packetizing buffer.  */
 static int
-packetizing_buffer_shutdown (buf)
-    struct buffer *buf;
+packetizing_buffer_shutdown (struct buffer *buf)
 {
-    struct packetizing_buffer *pb = (struct packetizing_buffer *) buf->closure;
+    struct packetizing_buffer *pb = buf->closure;
 
     return buf_shutdown (pb->buf);
 }
