@@ -1,11 +1,6 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
- *
- * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
- *                                  and others.
- *
- * Portions Copyright (C) 1992, Brian Berliner and Jeff Polk
- * Portions Copyright (C) 1989-1992, Brian Berliner
+ * Copyright (c) 1992, Brian Berliner and Jeff Polk
+ * Copyright (c) 1989-1992, Brian Berliner
  * 
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with the CVS source distribution.
@@ -38,14 +33,13 @@
  * edited by the user, if necessary (when the repository is moved, e.g.)
  */
 
-#include <assert.h>
 #include "cvs.h"
 
-static char *findslash PROTO((char *start, char *p));
-static int checkout_proc PROTO((int argc, char **argv, char *where,
+static char *findslash (char *start, char *p);
+static int checkout_proc (int argc, char **argv, char *where,
 		          char *mwhere, char *mfile, int shorten,
 		          int local_specified, char *omodule,
-		          char *msg));
+		          char *msg);
 
 static const char *const checkout_usage[] =
 {
@@ -72,13 +66,13 @@ static const char *const checkout_usage[] =
 
 static const char *const export_usage[] =
 {
-    "Usage: %s %s [-NRfln] [-r tag] [-D date] [-d dir] [-k kopt] module...\n",
+    "Usage: %s %s [-NRfln] [-r rev] [-D date] [-d dir] [-k kopt] module...\n",
     "\t-N\tDon't shorten module paths if -d specified.\n",
     "\t-f\tForce a head revision match if tag/date not found.\n",
     "\t-l\tLocal directory only, not recursive\n",
     "\t-R\tProcess directories recursively (default).\n",
     "\t-n\tDo not run module program (if any).\n",
-    "\t-r tag\tExport tagged revisions.\n",
+    "\t-r rev\tExport revision or tag.\n",
     "\t-D date\tExport revisions as of date.\n",
     "\t-d dir\tExport into dir instead of module name.\n",
     "\t-k kopt\tUse RCS kopt -k option on checkout.\n",
@@ -102,9 +96,7 @@ static char *history_name;
 static enum mtype m_type;
 
 int
-checkout (argc, argv)
-    int argc;
-    char **argv;
+checkout (int argc, char **argv)
 {
     int i;
     int c;
@@ -175,9 +167,11 @@ checkout (argc, argv)
 		break;
 	    case 'Q':
 	    case 'q':
+#ifdef SERVER_SUPPORT
 		/* The CVS 1.5 client sends these options (in addition to
 		   Global_option requests), so we must ignore them.  */
 		if (!server_active)
+#endif
 		    error (1, 0,
 			   "-q or -Q must be specified before \"%s\"",
 			   cvs_cmd_name);
@@ -417,22 +411,20 @@ checkout (argc, argv)
  *  being able to resolve a path or failing ot chdir to a path.
  */
 int
-safe_location (where)
-    char *where;
+safe_location (char *where)
 {
     char *current;
-    char *where_location;
     char *hardpath;
     size_t hardpath_len;
     int retval;
 
-    if (trace)
-	(void) fprintf (stderr, "%s-> safe_location( where=%s )\n",
-			CLIENT_SERVER_STR,
-			where ? where : "(null)");
+    TRACE( TRACE_FUNCTION, "safe_location( where=%s )",
+           where ? where : "(null)" );
 
+#ifdef CLIENT_SUPPORT
     /* Don't compare remote CVSROOTs to our destination directory. */
-    if (current_parsed_root->isremote) return 1;
+    if ( current_parsed_root->isremote ) return 1;
+#endif /* CLIENT_SUPPORT */
 
     /* set current - even if where is set we'll need to cd back... */
     current = xgetwd ();
@@ -441,65 +433,64 @@ safe_location (where)
 
     hardpath = xresolvepath ( current_parsed_root->directory );
 
-    /* if where is set, set current to where, where - last_component( where ),
-     * or fail, depending on whether the directories exist or not.
+    /* if where is set, set current to as much of where as exists,
+     * or fail.
      */
-    if( where != NULL )
+    if (where != NULL)
     {
-	if( chdir( where ) != -1 )
+	char *where_this_pass = xstrdup (where);
+	while (1)
 	{
-	    /* where */
-	    where_location = xgetwd();
-	    if( where_location == NULL )
-		error( 1, errno, "could not get working directory" );
-
-	    if( chdir( current ) == -1 )
-		error( 1, errno, "could not change directory to `%s'", current );
-
-	    free( current );
-	    current = where_location;
-        }
-	else if( errno == ENOENT )
-	{
-	    if ( last_component( where ) != where )
+	    if (CVS_CHDIR (where_this_pass) != -1)
 	    {
-		/* where - last_component( where ) */
+		/* where */
+		free (where_this_pass);
+		where_this_pass = xgetwd();
+		if (where_this_pass == NULL)
+		    error (1, errno, "could not get working directory");
+
+		if (CVS_CHDIR (current) == -1)
+		    error (1, errno,
+			   "could not restore directory to `%s'", current);
+
+		free (current);
+		current = where_this_pass;
+		break;
+	    }
+	    else if (errno == ENOENT)
+	    {
+		/* where_this_pass - last_component (where_this_pass) */
 		char *parent;
 
-		/* strip the last_component */
-		where_location = xstrdup (where);
-                /* It's okay to cast out the const below since we know we just
-                 * allocated where_location and can do what we like with it.
-                 */
-		parent = (char *)last_component (where_location);
-		parent[-1] = '\0';
-
-		if( chdir( where_location ) != -1 )
+		/* It's okay to cast out the const below since we know we
+		 * allocated where_this_pass and have control of it.
+		 */
+		if ((parent = (char *)last_component (where_this_pass))
+		        != where_this_pass)
 		{
-		    free( where_location );
-		    where_location = xgetwd();
-		    if( where_location == NULL )
-			error( 1, errno, "could not get working directory (nominally `%s')", where_location );
-
-		    if( chdir( current ) == -1 )
-			error( 1, errno, "could not change directory to `%s'", current );
-
-		    free( current );
-		    current = where_location;
+		    /* strip the last_component */
+		    parent[-1] = '\0';
+		    /* continue */
 		}
 		else
-		    /* fail */
-		    error( 1, errno, "could not change directory to requested checkout directory `%s'", where_location );
+		{
+		    /* ERRNO == ENOENT
+		     *   && last_component (where_this_pass) == where_this_pass
+		     * means we've tried all the parent diretories and not one
+		     * exists, so there is no need to test any portion of where
+		     * - it is all being created.
+		     */
+		    free (where_this_pass);
+		    break;
+		}
 	    }
-	    /* else: ERRNO == ENOENT & last_component(where) == where
-	     * for example, 'cvs co -d newdir module', where newdir hasn't
-	     * been created yet, so leave current set to '.' and check that
-	     */
-	}
-	else
-	    /* fail */
-	    error( 1, errno, "could not change directory to requested checkout directory `%s'", where );
-    }
+	    else
+		/* we don't know how to handle other errors, so fail */
+		error (1, errno, "\
+could not change directory to requested checkout directory `%s'",
+		       where_this_pass);
+	} /* while (1) */
+    } /* where != NULL */
 
     hardpath_len = strlen (hardpath);
     if (strlen (current) >= hardpath_len
@@ -519,7 +510,7 @@ safe_location (where)
     else
 	retval = 1;
     free (current);
-    free (hardpath);
+    free( hardpath );
     return retval;
 }
 
@@ -530,23 +521,16 @@ struct dir_to_build
     /* The path to the directory.  */
     char *dirpath;
 
-    /* If set, don't build the directory, just change to it.
-       The caller will also want to set REPOSITORY to NULL.  */
-    int just_chdir;
-
     struct dir_to_build *next;
 };
 
-static int build_dirs_and_chdir PROTO ((struct dir_to_build *list,
-					int sticky));
+static int build_dirs_and_chdir (struct dir_to_build *list,
+					int sticky);
 
-static void build_one_dir PROTO ((char *, char *, int));
+static void build_one_dir (char *, char *, int);
 
 static void
-build_one_dir (repository, dirpath, sticky)
-    char *repository;
-    char *dirpath;
-    int sticky;
+build_one_dir (char *repository, char *dirpath, int sticky)
 {
     FILE *fp;
 
@@ -592,17 +576,7 @@ build_one_dir (repository, dirpath, sticky)
  */
 /* ARGSUSED */
 static int
-checkout_proc (argc, argv, where_orig, mwhere, mfile, shorten,
-	       local_specified, omodule, msg)
-    int argc;
-    char **argv;
-    char *where_orig;
-    char *mwhere;
-    char *mfile;
-    int shorten;
-    int local_specified;
-    char *omodule;
-    char *msg;
+checkout_proc (int argc, char **argv, char *where_orig, char *mwhere, char *mfile, int shorten, int local_specified, char *omodule, char *msg)
 {
     char *myargv[2];
     int err = 0;
@@ -612,6 +586,18 @@ checkout_proc (argc, argv, where_orig, mwhere, mfile, shorten,
     char *oldupdate = NULL;
     char *where;
 
+    if (trace)
+	(void) fprintf (stderr,
+	                "%s-> checkout_proc (%s, %s, %s, %d, %d, %s, %s)\n",
+			CLIENT_SERVER_STR,
+			where_orig ? where_orig : "(null)",
+			mwhere ? mwhere : "(null)",
+			mfile ? mfile : "(null)",
+	                shorten, local_specified,
+			omodule ? omodule : "(null)",
+			msg ? msg : "(null)"
+	               );
+
     /*
      * OK, so we're doing the checkout! Our args are as follows: 
      *  argc,argv contain either dir or dir followed by a list of files 
@@ -629,7 +615,8 @@ checkout_proc (argc, argv, where_orig, mwhere, mfile, shorten,
 			  + strlen (argv[0])
 			  + (mfile == NULL ? 0 : strlen (mfile))
 			  + 10);
-    (void) sprintf (repository, "%s/%s", current_parsed_root->directory, argv[0]);
+    (void) sprintf (repository, "%s/%s",
+                    current_parsed_root->directory, argv[0]);
     Sanitize_Repository_Name (repository);
 
 
@@ -819,8 +806,6 @@ internal error: %s doesn't start with %s in checkout_proc",
 	head->repository = NULL;
 	head->dirpath = xstrdup (where);
 	head->next = NULL;
-	head->just_chdir = 0;
-
 
 	/* Make a copy of the repository name to play with. */
 	reposcopy = xstrdup (repository);
@@ -860,23 +845,6 @@ internal error: %s doesn't start with %s in checkout_proc",
 	    new->next = head;
 	    head = new;
 
-	    /* If where consists of multiple pathname components,
-	       then we want to just cd into it, without creating
-	       directories or modifying CVS directories as we go.
-	       In CVS 1.9 and earlier, the code actually does a
-	       CVS_CHDIR up-front; I'm not going to try to go back
-	       to that exact code but this is somewhat similar
-	       in spirit.  */
-	    if (where_orig != NULL
-		&& cp - where < strlen (where_orig))
-	    {
-		new->repository = NULL;
-		new->just_chdir = 1;
-		continue;
-	    }
-
-	    new->just_chdir = 0;
-
 	    /* Now figure out what repository directory to generate.
                The most complete case would be something like this:
 
@@ -939,7 +907,7 @@ internal error: %s doesn't start with %s in checkout_proc",
 	   may not have a thing to do with where the sources are
 	   being checked out.  If it does, build_dirs_and_chdir
 	   will take care of creating adm files here. */
-	/* FIXME: checking is_absolute (where) is a horrid kludge;
+	/* FIXME: checking where_is_absolute is a horrid kludge;
 	   I suspect we probably can just skip the call to
 	   build_one_dir whenever the -d command option was specified
 	   to checkout.  */
@@ -1054,8 +1022,7 @@ internal error: %s doesn't start with %s in checkout_proc",
 	which = W_REPOS;
 	if (tag != NULL && !tag_validated)
 	{
-	    tag_check_valid (tag, argc - 1, argv + 1, 0, aflag,
-			     repository);
+	    tag_check_valid (tag, argc - 1, argv + 1, 0, aflag, NULL);
 	    tag_validated = 1;
 	}
     }
@@ -1094,12 +1061,12 @@ internal error: %s doesn't start with %s in checkout_proc",
 	if (!pipeout)
 	    history_write (m_type == CHECKOUT ? 'O' : 'E', preload_update_dir,
 			   history_name, where, repository);
-	err += do_update (0, (char **) NULL, options, tag, date,
-			  force_tag_match, 0 /* !local */ ,
-			  1 /* update -d */ , aflag, checkout_prune_dirs,
-			  pipeout, which, join_rev1, join_rev2,
-			  preload_update_dir, m_type == CHECKOUT,
-			  repository);
+	err += do_update ( 0, (char **) NULL, options, tag, date,
+			   force_tag_match, 0 /* !local */ ,
+			   1 /* update -d */ , aflag, checkout_prune_dirs,
+			   pipeout, which, join_rev1, join_rev2,
+			   preload_update_dir, m_type == CHECKOUT,
+			   repository );
 	goto out;
     }
 
@@ -1152,11 +1119,11 @@ internal error: %s doesn't start with %s in checkout_proc",
 		       repository);
 
     /* go ahead and call update now that everything is set */
-    err += do_update (argc - 1, argv + 1, options, tag, date,
-		      force_tag_match, local_specified, 1 /* update -d */,
-		      aflag, checkout_prune_dirs, pipeout, which, join_rev1,
-		      join_rev2, preload_update_dir, m_type == CHECKOUT,
-		      repository);
+    err += do_update ( argc - 1, argv + 1, options, tag, date,
+		       force_tag_match, local_specified, 1 /* update -d */,
+		       aflag, checkout_prune_dirs, pipeout, which, join_rev1,
+		       join_rev2, preload_update_dir, m_type == CHECKOUT,
+		       repository );
 out:
     free (preload_update_dir);
     preload_update_dir = oldupdate;
@@ -1166,9 +1133,7 @@ out:
 }
 
 static char *
-findslash (start, p)
-    char *start;
-    char *p;
+findslash (char *start, char *p)
 {
     for (;;)
     {
@@ -1184,7 +1149,7 @@ findslash (start, p)
    directory, give a fatal error.  Otherwise, the directory is guaranteed
    to exist when we return.  */
 char *
-emptydir_name ()
+emptydir_name (void)
 {
     char *repository;
 
@@ -1229,9 +1194,7 @@ emptydir_name ()
  *  The only nonfatal error this function may return is if the CHDIR fails.
  */
 static int
-build_dirs_and_chdir (dirs, sticky)
-    struct dir_to_build *dirs;
-    int sticky;
+build_dirs_and_chdir (struct dir_to_build *dirs, int sticky)
 {
     int retval = 0;
     struct dir_to_build *nextdir;
@@ -1239,12 +1202,10 @@ build_dirs_and_chdir (dirs, sticky)
     while (dirs != NULL)
     {
 	const char *dir = last_component (dirs->dirpath);
+	int made_dir = 0;
 
-	if (!dirs->just_chdir)
-	{
-	    mkdir_if_needed (dir);
-	    Subdir_Register (NULL, NULL, dir);
-	}
+	made_dir = !mkdir_if_needed (dir);
+	if (made_dir) Subdir_Register (NULL, NULL, dir);
 
 	if (CVS_CHDIR (dir) < 0)
 	{
@@ -1254,7 +1215,8 @@ build_dirs_and_chdir (dirs, sticky)
 	}
 	if (dirs->repository != NULL)
 	{
-	    build_one_dir (dirs->repository, dirs->dirpath, sticky);
+	    if (made_dir)
+		build_one_dir (dirs->repository, dirs->dirpath, sticky);
 	    free (dirs->repository);
 	}
 	nextdir = dirs->next;
