@@ -1,11 +1,6 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
- *
- * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
- *                                  and others.
- *
- * Portions Copyright (C) 1992, Brian Berliner and Jeff Polk
- * Portions Copyright (C) 1989-1992, Brian Berliner
+ * Copyright (c) 1992, Brian Berliner and Jeff Polk
+ * Copyright (c) 1989-1992, Brian Berliner
  *
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with the CVS source distribution.
@@ -19,51 +14,45 @@
  *
  */
 
-#include <assert.h>
 #include "cvs.h"
 #include "getline.h"
 #include "edit.h"
 #include "fileattr.h"
 #include "hardlink.h"
 
-static Dtype check_direntproc PROTO ((void *callerdat, const char *dir,
-                                      const char *repos,
-                                      const char *update_dir,
-                                      List *entries));
-static int check_fileproc PROTO ((void *callerdat, struct file_info *finfo));
-static int check_filesdoneproc PROTO ((void *callerdat, int err,
-                                       const char *repos,
-                                       const char *update_dir,
-                                       List *entries));
-static int checkaddfile PROTO((const char *file, const char *repository,
-                               const char *tag, const char *options,
-                               RCSNode **rcsnode));
-static Dtype commit_direntproc PROTO ((void *callerdat, const char *dir,
-                                       const char *repos,
-                                       const char *update_dir,
-                                       List *entries));
-static int commit_dirleaveproc PROTO ((void *callerdat, const char *dir,
-                                       int err, const char *update_dir,
-                                       List *entries));
-static int commit_fileproc PROTO ((void *callerdat, struct file_info *finfo));
-static int commit_filesdoneproc PROTO ((void *callerdat, int err,
-                                        const char *repository,
-                                        const char *update_dir,
-                                        List *entries));
-static int finaladd PROTO((struct file_info *finfo, char *revision, char *tag,
-			   char *options));
-static int findmaxrev PROTO((Node * p, void *closure));
-static int lock_RCS PROTO((const char *user, RCSNode *rcs, const char *rev,
-			   const char *repository));
-static int precommit_list_proc PROTO((Node * p, void *closure));
-static int precommit_proc PROTO((const char *repository, const char *filter));
-static int remove_file PROTO ((struct file_info *finfo, char *tag,
-			       char *message));
-static void fixaddfile PROTO((const char *rcs));
-static void fixbranch PROTO((RCSNode *, char *branch));
-static void unlockrcs PROTO((RCSNode *rcs));
-static void ci_delproc PROTO((Node *p));
-static void masterlist_delproc PROTO((Node *p));
+static Dtype check_direntproc (void *callerdat, const char *dir,
+                               const char *repos, const char *update_dir,
+                               List *entries);
+static int check_fileproc (void *callerdat, struct file_info *finfo);
+static int check_filesdoneproc (void *callerdat, int err, const char *repos,
+				const char *update_dir, List *entries);
+static int checkaddfile (const char *file, const char *repository,
+                         const char *tag, const char *options,
+                         RCSNode **rcsnode);
+static Dtype commit_direntproc (void *callerdat, const char *dir,
+                                const char *repos, const char *update_dir,
+                                List *entries);
+static int commit_dirleaveproc (void *callerdat, const char *dir, int err,
+				const char *update_dir, List *entries);
+static int commit_fileproc (void *callerdat, struct file_info *finfo);
+static int commit_filesdoneproc (void *callerdat, int err,
+                                 const char *repository,
+				 const char *update_dir, List *entries);
+static int finaladd (struct file_info *finfo, char *revision, char *tag,
+		     char *options);
+static int findmaxrev (Node * p, void *closure);
+static int lock_RCS (const char *user, RCSNode *rcs, const char *rev,
+                     const char *repository);
+static int precommit_list_to_args_proc (Node * p, void *closure);
+static int precommit_proc (const char *repository, const char *filter,
+                           void *closure);
+static int remove_file (struct file_info *finfo, char *tag,
+			char *message);
+static void fixaddfile (const char *rcs);
+static void fixbranch (RCSNode *, char *branch);
+static void unlockrcs (RCSNode *rcs);
+static void ci_delproc (Node *p);
+static void masterlist_delproc (Node *p);
 
 struct commit_info
 {
@@ -86,7 +75,6 @@ static char *write_dirtag;
 static int write_dirnonbranch;
 static char *logfile;
 static List *mulist;
-static List *saved_ulist;
 static char *saved_message;
 static time_t last_register_time;
 
@@ -140,18 +128,9 @@ struct find_data {
 
 
 
-static Dtype find_dirent_proc PROTO ((void *callerdat, const char *dir,
-                                      const char *repository,
-                                      const char *update_dir,
-                                      List *entries));
-
 static Dtype
-find_dirent_proc (callerdat, dir, repository, update_dir, entries)
-    void *callerdat;
-    const char *dir;
-    const char *repository;
-    const char *update_dir;
-    List *entries;
+find_dirent_proc (void *callerdat, const char *dir, const char *repository,
+                  const char *update_dir, List *entries)
 {
     struct find_data *find_data = (struct find_data *)callerdat;
 
@@ -182,12 +161,8 @@ static struct find_data *find_data_static;
 
 
 
-static void find_ignproc PROTO ((const char *, const char *));
-
 static void
-find_ignproc (file, dir)
-    const char *file;
-    const char *dir;
+find_ignproc (const char *file, const char *dir)
 {
     struct question *p;
 
@@ -201,18 +176,9 @@ find_ignproc (file, dir)
 
 
 
-static int find_filesdoneproc PROTO ((void *callerdat, int err,
-                                      const char *repository,
-                                      const char *update_dir,
-                                      List *entries));
-
 static int
-find_filesdoneproc (callerdat, err, repository, update_dir, entries)
-    void *callerdat;
-    int err;
-    const char *repository;
-    const char *update_dir;
-    List *entries;
+find_filesdoneproc (void *callerdat, int err, const char *repository,
+                    const char *update_dir, List *entries)
 {
     struct find_data *find_data = (struct find_data *)callerdat;
     find_data->repository = repository;
@@ -232,16 +198,12 @@ find_filesdoneproc (callerdat, err, repository, update_dir, entries)
 
 
 
-static int find_fileproc PROTO ((void *callerdat, struct file_info *finfo));
-
 /* Machinery to find out what is modified, added, and removed.  It is
    possible this should be broken out into a new client_classify function;
    merging it with classify_file is almost sure to be a mess, though,
    because classify_file has all kinds of repository processing.  */
 static int
-find_fileproc (callerdat, finfo)
-    void *callerdat;
-    struct file_info *finfo;
+find_fileproc (void *callerdat, struct file_info *finfo)
 {
     Vers_TS *vers;
     enum classify_type status;
@@ -272,7 +234,7 @@ find_fileproc (callerdat, finfo)
 	if (vers->ts_user == NULL)
 	    error (0, 0, "nothing known about `%s'", finfo->fullname);
 	else
-	    error (0, 0, "use `%s add' to create an entry for %s",
+	    error (0, 0, "use `%s add' to create an entry for `%s'",
 		   program_name, finfo->fullname);
 	freevers_ts (&vers);
 	return 1;
@@ -350,12 +312,8 @@ find_fileproc (callerdat, finfo)
 
 
 
-static int copy_ulist PROTO ((Node *, void *));
-
 static int
-copy_ulist (node, data)
-    Node *node;
-    void *data;
+copy_ulist (Node *node, void *data)
 {
     struct find_data *args = (struct find_data *)data;
     args->argv[args->argc++] = node->key;
@@ -363,15 +321,15 @@ copy_ulist (node, data)
 }
 #endif /* CLIENT_SUPPORT */
 
+
+
 #ifdef SERVER_SUPPORT
 # define COMMIT_OPTIONS "+nlRm:fF:r:"
 #else /* !SERVER_SUPPORT */
 # define COMMIT_OPTIONS "+lRm:fF:r:"
 #endif /* SERVER_SUPPORT */
 int
-commit (argc, argv)
-    int argc;
-    char **argv;
+commit (int argc, char **argv)
 {
     int c;
     int err = 0;
@@ -388,12 +346,16 @@ commit (argc, argv)
     /* FIXME: Shouldn't this check be much more closely related to the
        readonly user stuff (CVSROOT/readers, &c).  That is, why should
        root be able to "cvs init", "cvs import", &c, but not "cvs ci"?  */
-    /* Who we are on the client side doesn't affect logging.  */
-    if (geteuid () == (uid_t) 0 && !current_parsed_root->isremote)
+    if (geteuid () == (uid_t) 0
+#  ifdef CLIENT_SUPPORT
+	/* Who we are on the client side doesn't affect logging.  */
+	&& !current_parsed_root->isremote
+#  endif
+	)
     {
 	struct passwd *pw;
 
-	if ((pw = (struct passwd *) getpwnam (getcaller ())) == NULL)
+	if ((pw = getpwnam (getcaller ())) == NULL)
 	    error (1, 0,
                    "your apparent username (%s) is unknown to this system",
                    getcaller ());
@@ -412,7 +374,6 @@ commit (argc, argv)
 		/* Silently ignore -n for compatibility with old
 		 * clients.
 		 */
-		if (!server_active) error(0, 0, "the `-n' option is obsolete");
 		break;
 #endif /* SERVER_SUPPORT */
 	    case 'm':
@@ -503,11 +464,10 @@ commit (argc, argv)
 	   a little unneeded network traffic.  */
 	find_args.force = force_ci || saved_tag != NULL;
 
-	err = start_recursion (find_fileproc, find_filesdoneproc,
-			       find_dirent_proc, (DIRLEAVEPROC) NULL,
-			       (void *)&find_args,
-			       argc, argv, local, W_LOCAL, 0, CVS_LOCK_NONE,
-			       (char *) NULL, 0, (char *) NULL);
+	err = start_recursion
+	    (find_fileproc, find_filesdoneproc, find_dirent_proc, NULL,
+	     &find_args, argc, argv, local, W_LOCAL, 0, CVS_LOCK_NONE,
+	     NULL, 0, NULL );
 	if (err)
 	    error (1, 0, "correct above errors first!");
 
@@ -551,7 +511,7 @@ commit (argc, argv)
 	 * The protocol is designed this way.  This is a feature.
 	 */
 	if (use_editor)
-	    do_editor (".", &saved_message, (char *)NULL, find_args.ulist);
+	    do_editor (".", &saved_message, NULL, find_args.ulist);
 
 	/* We always send some sort of message, even if empty.  */
 	option_with_arg ("-m", saved_message ? saved_message : "");
@@ -643,8 +603,7 @@ commit (argc, argv)
 
 	    fp = cvs_temp_file (&fname);
 	    if (fp == NULL)
-		error (1, 0, "cannot create temporary file %s",
-		       fname ? fname : "(null)");
+		error (1, 0, "cannot create temporary file %s", fname);
 	    if (fwrite (saved_message, 1, strlen (saved_message), fp)
 		!= strlen (saved_message))
 		error (1, errno, "cannot write temporary file %s", fname);
@@ -666,7 +625,7 @@ commit (argc, argv)
 
     wrap_setup ();
 
-    lock_tree_for_write (argc, argv, local, W_LOCAL, aflag);
+    lock_tree_promotably (argc, argv, local, W_LOCAL, aflag);
 
     /*
      * Set up the master update list and hard link list
@@ -690,14 +649,10 @@ commit (argc, argv)
      * Run the recursion processor to verify the files are all up-to-date
      */
     err = start_recursion (check_fileproc, check_filesdoneproc,
-			   check_direntproc, (DIRLEAVEPROC) NULL, NULL, argc,
-			   argv, local, W_LOCAL, aflag, CVS_LOCK_NONE,
-			   (char *) NULL, 1, (char *) NULL);
+                           check_direntproc, NULL, NULL, argc, argv, local,
+                           W_LOCAL, aflag, CVS_LOCK_NONE, NULL, 1, NULL);
     if (err)
-    {
-	Lock_Cleanup ();
 	error (1, 0, "correct above errors first!");
-    }
 
     /*
      * Run the recursion processor to commit the files
@@ -705,9 +660,9 @@ commit (argc, argv)
     write_dirnonbranch = 0;
     if (noexec == 0)
 	err = start_recursion (commit_fileproc, commit_filesdoneproc,
-			       commit_direntproc, commit_dirleaveproc, NULL,
-			       argc, argv, local, W_LOCAL, aflag, CVS_LOCK_NONE,
-			       (char *) NULL, 1, (char *) NULL);
+                               commit_direntproc, commit_dirleaveproc, NULL,
+                               argc, argv, local, W_LOCAL, aflag,
+                               CVS_LOCK_WRITE, NULL, 1, NULL);
 
     /*
      * Unlock all the dirs and clean up
@@ -715,14 +670,15 @@ commit (argc, argv)
     Lock_Cleanup ();
     dellist (&mulist);
 
-    if (server_active)
-	return err;
-
     /* see if we need to sleep before returning to avoid time-stamp races */
-    if (last_register_time)
-    {
-	sleep_past (last_register_time);
-    }
+    if (
+#ifdef SERVER_SUPPORT
+	/* But only sleep on the client.  */
+        !server_active &&
+#endif
+	last_register_time
+       )
+	    sleep_past (last_register_time);
 
     return err;
 }
@@ -734,9 +690,7 @@ commit (argc, argv)
 
 static
 Ctype
-classify_file_internal (finfo, vers)
-    struct file_info *finfo;
-    Vers_TS **vers;
+classify_file_internal (struct file_info *finfo, Vers_TS **vers)
 {
     int save_noexec, save_quiet, save_really_quiet;
     Ctype status;
@@ -824,9 +778,7 @@ classify_file_internal (finfo, vers)
  */
 /* ARGSUSED */
 static int
-check_fileproc (callerdat, finfo)
-    void *callerdat;
-    struct file_info *finfo;
+check_fileproc (void *callerdat, struct file_info *finfo)
 {
     Ctype status;
     const char *xdir;
@@ -835,6 +787,7 @@ check_fileproc (callerdat, finfo)
     Vers_TS *vers;
     struct commit_info *ci;
     struct logfile_info *li;
+    int retval = 1;
 
     size_t cvsroot_len = strlen (current_parsed_root->directory);
 
@@ -846,11 +799,11 @@ check_fileproc (callerdat, finfo)
 
     if (strncmp (finfo->repository, current_parsed_root->directory,
                  cvsroot_len) == 0
-	&& ISDIRSEP (finfo->repository[cvsroot_len])
+	&& ISSLASH (finfo->repository[cvsroot_len])
 	&& strncmp (finfo->repository + cvsroot_len + 1,
 		    CVSROOTADM,
 		    sizeof (CVSROOTADM) - 1) == 0
-	&& ISDIRSEP (finfo->repository[cvsroot_len + sizeof (CVSROOTADM)])
+	&& ISSLASH (finfo->repository[cvsroot_len + sizeof (CVSROOTADM)])
 	&& strcmp (finfo->repository + cvsroot_len + sizeof (CVSROOTADM) + 1,
 		   CVSNULLREPOS) == 0
 	)
@@ -871,11 +824,10 @@ check_fileproc (callerdat, finfo)
 	case T_CHECKOUT:
 	case T_PATCH:
 	case T_NEEDS_MERGE:
+	case T_CONFLICT:
 	case T_REMOVE_ENTRY:
 	    error (0, 0, "Up-to-date check failed for `%s'", finfo->fullname);
-	    freevers_ts (&vers);
-	    return 1;
-	case T_CONFLICT:
+	    goto out;
 	case T_MODIFIED:
 	case T_ADDED:
 	case T_REMOVED:
@@ -900,8 +852,7 @@ check_fileproc (callerdat, finfo)
 		    error (0, 0,
 			   "cannot commit with sticky date for file `%s'",
 			   finfo->fullname);
-		    freevers_ts (&vers);
-		    return 1;
+		    goto out;
 		}
 		if (status == T_MODIFIED && vers->tag &&
 		    !RCS_isbranch (finfo->rcs, vers->tag))
@@ -909,34 +860,42 @@ check_fileproc (callerdat, finfo)
 		    error (0, 0,
 			   "sticky tag `%s' for file `%s' is not a branch",
 			   vers->tag, finfo->fullname);
-		    freevers_ts (&vers);
-		    return 1;
+		    goto out;
 		}
 	    }
-	    if (status == T_CONFLICT && !force_ci)
+	    if (status == T_MODIFIED && !force_ci && vers->ts_conflict)
 	    {
-		error (0, 0,
-		      "file `%s' had a conflict and has not been modified",
-		       finfo->fullname);
-		freevers_ts (&vers);
-		return 1;
-	    }
-	    if (status == T_MODIFIED && !force_ci && file_has_markers (finfo))
-	    {
-		/* Make this a warning, not an error, because we have
-		   no way of knowing whether the "conflict indicators"
-		   are really from a conflict or whether they are part
-		   of the document itself (cvs.texinfo and sanity.sh in
-		   CVS itself, for example, tend to want to have strings
-		   like ">>>>>>>" at the start of a line).  Making people
-		   kludge this the way they need to kludge keyword
-		   expansion seems undesirable.  And it is worse than
-		   keyword expansion, because there is no -ko
-		   analogue.  */
-		error (0, 0,
-		       "\
+		/*
+		 * We found a "conflict" marker.
+		 *
+		 * If the timestamp on the file is the same as the
+		 * timestamp stored in the Entries file, we block the commit.
+		 */
+		if ( file_has_conflict ( finfo, vers->ts_conflict ) )
+		{
+		    error (0, 0,
+			  "file `%s' had a conflict and has not been modified",
+			   finfo->fullname);
+		    goto out;
+		}
+
+		if (file_has_markers (finfo))
+		{
+		    /* Make this a warning, not an error, because we have
+		       no way of knowing whether the "conflict indicators"
+		       are really from a conflict or whether they are part
+		       of the document itself (cvs.texinfo and sanity.sh in
+		       CVS itself, for example, tend to want to have strings
+		       like ">>>>>>>" at the start of a line).  Making people
+		       kludge this the way they need to kludge keyword
+		       expansion seems undesirable.  And it is worse than
+		       keyword expansion, because there is no -ko
+		       analogue.  */
+		    error (0, 0,
+			   "\
 warning: file `%s' seems to still contain conflict indicators",
-		       finfo->fullname);
+			   finfo->fullname);
+		}
 	    }
 
 	    if (status == T_REMOVED)
@@ -946,8 +905,7 @@ warning: file `%s' seems to still contain conflict indicators",
 		    error (0, 0,
 			   "`%s' should be removed and is still there (or is"
 			   " back again)", finfo->fullname);
-		    freevers_ts (&vers);
-		    return 1;
+		    goto out;
 		}
 
 		if (vers->tag && isdigit ((unsigned char) *vers->tag))
@@ -961,7 +919,7 @@ warning: file `%s' seems to still contain conflict indicators",
 			   "cannot remove file `%s' which has a numeric sticky"
 			   " tag of `%s'", finfo->fullname, vers->tag);
 		    freevers_ts (&vers);
-		    return 1;
+		    goto out;
 		}
 	    }
 	    if (status == T_ADDED)
@@ -974,8 +932,7 @@ warning: file `%s' seems to still contain conflict indicators",
 			error (0, 0,
 		    "cannot add file `%s' when RCS file `%s' already exists",
 			       finfo->fullname, finfo->rcs->path);
-			freevers_ts (&vers);
-			return 1;
+			goto out;
 		    }
 		}
 		else if (isdigit ((unsigned char) *vers->tag) &&
@@ -984,8 +941,7 @@ warning: file `%s' seems to still contain conflict indicators",
 		    error (0, 0,
 		"cannot add file `%s' with revision `%s'; must be on trunk",
 			       finfo->fullname, vers->tag);
-		    freevers_ts (&vers);
-		    return 1;
+		    goto out;
 		}
 	    }
 
@@ -1003,15 +959,14 @@ warning: file `%s' seems to still contain conflict indicators",
 	    {
 		struct master_lists *ml;
 
-		ulist = getlist ();
-		cilist = getlist ();
+		ml = (struct master_lists *)
+		    xmalloc( sizeof( struct master_lists ) );
+		ulist = ml->ulist = getlist();
+		cilist = ml->cilist = getlist();
+
 		p = getnode ();
 		p->key = xstrdup (xdir);
 		p->type = UPDATE;
-		ml = (struct master_lists *)
-		    xmalloc (sizeof (struct master_lists));
-		ml->ulist = ulist;
-		ml->cilist = cilist;
 		p->data = ml;
 		p->delproc = masterlist_delproc;
 		(void) addnode (mulist, p);
@@ -1087,8 +1042,7 @@ warning: file `%s' seems to still contain conflict indicators",
 	    break;
 	case T_UNKNOWN:
 	    error (0, 0, "nothing known about `%s'", finfo->fullname);
-	    freevers_ts (&vers);
-	    return 1;
+	    goto out;
 	case T_UPTODATE:
 	    break;
 	default:
@@ -1096,8 +1050,12 @@ warning: file `%s' seems to still contain conflict indicators",
 	    break;
     }
 
+    retval = 0;
+
+ out:
+
     freevers_ts (&vers);
-    return 0;
+    return retval;
 }
 
 
@@ -1108,12 +1066,8 @@ warning: file `%s' seems to still contain conflict indicators",
  */
 /* ARGSUSED */
 static Dtype
-check_direntproc (callerdat, dir, repos, update_dir, entries)
-    void *callerdat;
-    const char *dir;
-    const char *repos;
-    const char *update_dir;
-    List *entries;
+check_direntproc (void *callerdat, const char *dir, const char *repos,
+                  const char *update_dir, List *entries)
 {
     if (!isdir (dir))
 	return R_SKIP_ALL;
@@ -1127,20 +1081,71 @@ check_direntproc (callerdat, dir, repos, update_dir, entries)
 
 
 /*
- * Walklist proc to run pre-commit checks
+ * Walklist proc to generate an arg list from the line in commitinfo
  */
 static int
-precommit_list_proc (p, closure)
+precommit_list_to_args_proc (p, closure)
     Node *p;
     void *closure;
 {
-    struct logfile_info *li = p->data;
-    if (li->type == T_ADDED
-	|| li->type == T_MODIFIED
-	|| li->type == T_REMOVED)
+    struct format_cmdline_walklist_closure *c = closure;
+    struct logfile_info *li;
+    char *arg = NULL;
+    const char *f;
+    char *d;
+    size_t doff;
+
+    if (p->data == NULL) return 1;
+
+    f = c->format;
+    d = *c->d;
+    /* foreach requested attribute */
+    while (*f)
     {
-	run_arg (p->key);
+   	switch (*f++)
+	{
+	    case 's':
+		li = p->data;
+		if (li->type == T_ADDED
+			|| li->type == T_MODIFIED
+			|| li->type == T_REMOVED)
+		{
+		    arg = p->key;
+		}
+		break;
+	    default:
+		error (1, 0,
+		       "Unknown format character or not a list attribute: %c",
+		       f[-1]);
+		/* NOTREACHED */
+		break;
+	}
+	/* copy the attribute into an argument */
+	if (c->quotes)
+	{
+	    arg = cmdlineescape (c->quotes, arg);
+	}
+	else
+	{
+	    arg = cmdlinequote ('"', arg);
+	}
+	doff = d - *c->buf;
+	expand_string (c->buf, c->length, doff + strlen (arg));
+	d = *c->buf + doff;
+	strncpy (d, arg, strlen (arg));
+	d += strlen (arg);
+	free (arg);
+
+	/* and always put the extra space on.  we'll have to back up a char
+	 * when we're done, but that seems most efficient
+	 */
+	doff = d - *c->buf;
+	expand_string (c->buf, c->length, doff + 1);
+	d = *c->buf + doff;
+	*d++ = ' ';
     }
+    /* correct our original pointer into the buff */
+    *c->d = d;
     return 0;
 }
 
@@ -1150,34 +1155,51 @@ precommit_list_proc (p, closure)
  * Callback proc for pre-commit checking
  */
 static int
-precommit_proc (repository, filter)
-    const char *repository;
-    const char *filter;
+precommit_proc (const char *repository, const char *filter, void *closure)
 {
-    /* see if the filter is there, only if it's a full path */
-    if (isabsolute (filter))
-    {
-    	char *s, *cp;
+    char *newfilter = NULL;
+    char *cmdline;
+    const char *srepos = Short_Repository (repository);
+    List *ulist = (List *)closure;
 
-	s = xstrdup (filter);
-	for (cp = s; *cp; cp++)
-	    if (isspace ((unsigned char) *cp))
-	    {
-		*cp = '\0';
-		break;
-	    }
-	if (!isfile (s))
-	{
-	    error (0, errno, "cannot find pre-commit filter `%s'", s);
-	    free (s);
-	    return 1;			/* so it fails! */
-	}
-	free (s);
+#ifdef SUPPORT_OLD_INFO_FMT_STRINGS
+    if (!strchr (filter, '%'))
+    {
+	error (0, 0,
+               "warning: commitinfo line contains no format strings:\n"
+               "    \"%s\"\n"
+               "Appending defaults (\" %%r/%%p %%s\"), but please be aware that this usage is\n"
+               "deprecated.", filter);
+	newfilter = xmalloc (strlen (filter) + 10);
+	strcpy (newfilter, filter);
+	strcat (newfilter, " %r/%p %s");
+	filter = newfilter;
+    }
+#endif /* SUPPORT_OLD_INFO_FMT_STRINGS */
+
+    cmdline = format_cmdline (
+#ifdef SUPPORT_OLD_INFO_FMT_STRINGS
+        0, srepos,
+#endif /* SUPPORT_OLD_INFO_FMT_STRINGS */
+        filter,
+        "p", "s", srepos,
+        "r", "s", current_parsed_root->directory,
+        "s", ",", ulist, precommit_list_to_args_proc, (void *) NULL,
+        (char *)NULL
+	);
+
+    if (newfilter) free (newfilter);
+
+    if (!cmdline || !strlen (cmdline))
+    {
+	if (cmdline) free (cmdline);
+	error (0, 0, "precommit proc resolved to the empty string!");
+	return 1;
     }
 
-    run_setup (filter);
-    run_arg (repository);
-    (void) walklist (saved_ulist, precommit_list_proc, NULL);
+    run_setup (cmdline);
+    free (cmdline);
+
     return run_exec (RUN_TTY, RUN_TTY, RUN_TTY, RUN_NORMAL|RUN_REALLY);
 }
 
@@ -1188,29 +1210,28 @@ precommit_proc (repository, filter)
  */
 /* ARGSUSED */
 static int
-check_filesdoneproc (callerdat, err, repos, update_dir, entries)
-    void *callerdat;
-    int err;
-    const char *repos;
-    const char *update_dir;
-    List *entries;
+check_filesdoneproc (void *callerdat, int err, const char *repos,
+                     const char *update_dir, List *entries)
 {
     int n;
     Node *p;
+    List *saved_ulist;
 
     /* find the update list for this dir */
     p = findnode (mulist, update_dir);
     if (p != NULL)
 	saved_ulist = ((struct master_lists *) p->data)->ulist;
     else
-	saved_ulist = (List *) NULL;
+	saved_ulist = NULL;
 
     /* skip the checks if there's nothing to do */
     if (saved_ulist == NULL || saved_ulist->list->next == saved_ulist->list)
 	return err;
 
     /* run any pre-commit checks */
-    if ((n = Parse_Info (CVSROOTADM_COMMITINFO, repos, precommit_proc, 1)) > 0)
+    n = Parse_Info (CVSROOTADM_COMMITINFO, repos, precommit_proc, PIOPT_ALL,
+                    saved_ulist);
+    if (n > 0)
     {
 	error (0, 0, "Pre-commit check failed");
 	err += n;
@@ -1229,9 +1250,7 @@ static char *sbranch;
 
 /* ARGSUSED */
 static int
-commit_fileproc (callerdat, finfo)
-    void *callerdat;
-    struct file_info *finfo;
+commit_fileproc (void *callerdat, struct file_info *finfo)
 {
     Node *p;
     int err = 0;
@@ -1275,7 +1294,11 @@ commit_fileproc (callerdat, finfo)
     if (!got_message)
     {
 	got_message = 1;
-	if (!server_active && use_editor)
+	if (
+#ifdef SERVER_SUPPORT
+	    !server_active &&
+#endif
+	    use_editor)
 	    do_editor (finfo->update_dir, &saved_message,
 		       finfo->repository, ulist);
 	do_verify (&saved_message, finfo->repository);
@@ -1451,18 +1474,12 @@ out:
  */
 /* ARGSUSED */
 static int
-commit_filesdoneproc (callerdat, err, repository, update_dir, entries)
-    void *callerdat;
-    int err;
-    const char *repository;
-    const char *update_dir;
-    List *entries;
+commit_filesdoneproc (void *callerdat, int err, const char *repository,
+                      const char *update_dir, List *entries)
 {
     Node *p;
     List *ulist;
 
-    assert (repository);
-
     p = findnode (mulist, update_dir);
     if (p == NULL)
 	return err;
@@ -1506,12 +1523,16 @@ commit_filesdoneproc (callerdat, err, repository, update_dir, entries)
 		    || admin_dir[p - repository + cvsrootlen] == '/');
 	    admin_dir[p - repository + cvsrootlen] = '\0';
 
-	    cvs_output (program_name, 0);
-	    cvs_output (" ", 1);
-	    cvs_output (cvs_cmd_name, 0);
-	    cvs_output (": Rebuilding administrative file database\n", 0);
+	    if (!really_quiet)
+	    {
+		cvs_output (program_name, 0);
+		cvs_output (" ", 1);
+		cvs_output (cvs_cmd_name, 0);
+		cvs_output (": Rebuilding administrative file database\n", 0);
+	    }
 	    mkmodules (admin_dir);
 	    free (admin_dir);
+	    WriteTemplate (".", 1, repository);
 	}
     }
 
@@ -1525,12 +1546,8 @@ commit_filesdoneproc (callerdat, err, repository, update_dir, entries)
  */
 /* ARGSUSED */
 static Dtype
-commit_direntproc (callerdat, dir, repos, update_dir, entries)
-    void *callerdat;
-    const char *dir;
-    const char *repos;
-    const char *update_dir;
-    List *entries;
+commit_direntproc (void *callerdat, const char *dir, const char *repos,
+                   const char *update_dir, List *entries)
 {
     Node *p;
     List *ulist;
@@ -1551,9 +1568,13 @@ commit_direntproc (callerdat, dir, repos, update_dir, entries)
 	return R_SKIP_FILES;
 
     /* get commit message */
-    real_repos = Name_Repository (dir, update_dir);
     got_message = 1;
-    if (!server_active && use_editor)
+    real_repos = Name_Repository (dir, update_dir);
+    if (
+#ifdef SERVER_SUPPORT
+        !server_active &&
+#endif
+        use_editor)
 	do_editor (update_dir, &saved_message, real_repos, ulist);
     do_verify (&saved_message, real_repos);
     free (real_repos);
@@ -1567,12 +1588,8 @@ commit_direntproc (callerdat, dir, repos, update_dir, entries)
  */
 /* ARGSUSED */
 static int
-commit_dirleaveproc (callerdat, dir, err, update_dir, entries)
-    void *callerdat;
-    const char *dir;
-    int err;
-    const char *update_dir;
-    List *entries;
+commit_dirleaveproc (void *callerdat, const char *dir, int err,
+                     const char *update_dir, List *entries)
 {
     /* update the per-directory tag info */
     /* FIXME?  Why?  The "commit examples" node of cvs.texinfo briefly
@@ -1595,9 +1612,7 @@ commit_dirleaveproc (callerdat, dir, err, update_dir, entries)
  * find the maximum major rev number in an entries file
  */
 static int
-findmaxrev (p, closure)
-    Node *p;
-    void *closure;
+findmaxrev (Node *p, void *closure)
 {
     int thisrev;
     Entnode *entdata = p->data;
@@ -1619,10 +1634,7 @@ findmaxrev (p, closure)
    Return value is 0 on success, or >0 on error (in which case we have
    printed an error message).  */
 static int
-remove_file (finfo, tag, message)
-    struct file_info *finfo;
-    char *tag;
-    char *message;
+remove_file (struct file_info *finfo, char *tag, char *message)
 {
     int retcode;
 
@@ -1662,11 +1674,6 @@ remove_file (finfo, tag, message)
     /* we are removing the file from either the head or a branch */
     /* commit a new, dead revision. */
 
-    /* Print message indicating that file is going to be removed. */
-    cvs_output ("Removing ", 0);
-    cvs_output (finfo->fullname, 0);
-    cvs_output (";\n", 0);
-
     rev = NULL;
     lockflag = 1;
     if (branch)
@@ -1737,22 +1744,18 @@ remove_file (finfo, tag, message)
     if (corev != NULL)
 	free (corev);
 
-    retcode = RCS_checkin (finfo->rcs, finfo->file, message, rev, 0,
+    retcode = RCS_checkin (finfo->rcs, finfo->file, message, rev,
 			   RCS_FLAGS_DEAD | RCS_FLAGS_QUIET);
     if (retcode	!= 0)
     {
 	if (!quiet)
 	    error (0, retcode == -1 ? errno : 0,
 		   "failed to commit dead revision for `%s'", finfo->fullname);
-	if (prev_rev != NULL)
-	    free (prev_rev);
 	return 1;
     }
     /* At this point, the file has been committed as removed.  We should
        probably tell the history file about it  */
-    corev = rev ? RCS_getbranch (finfo->rcs, rev, 1) : RCS_head (finfo->rcs);
-    history_write ('R', NULL, corev, finfo->file, finfo->repository);
-    free (corev);
+    history_write ('R', NULL, finfo->rcs->head, finfo->file, finfo->repository);
 
     if (rev != NULL)
 	free (rev);
@@ -1762,13 +1765,17 @@ remove_file (finfo, tag, message)
 	RCS_setattic (finfo->rcs, 1);
 
     /* Print message that file was removed. */
-    cvs_output (old_path, 0);
-    cvs_output ("  <--  ", 0);
-    cvs_output (finfo->file, 0);
-    cvs_output ("\nnew revision: delete; previous revision: ", 0);
-    cvs_output (prev_rev, 0);
-    cvs_output ("\ndone\n", 0);
-    free(prev_rev);
+    if (!really_quiet)
+    {
+	cvs_output (old_path, 0);
+	cvs_output ("  <--  ", 0);
+	cvs_output (finfo->file, 0);
+	cvs_output ("\nnew revision: delete; previous revision: ", 0);
+	cvs_output (prev_rev, 0);
+	cvs_output ("\n", 0);
+    }
+
+    free (prev_rev);
 
     free (old_path);
 
@@ -1782,11 +1789,7 @@ remove_file (finfo, tag, message)
  * Do the actual checkin for added files
  */
 static int
-finaladd (finfo, rev, tag, options)
-    struct file_info *finfo;
-    char *rev;
-    char *tag;
-    char *options;
+finaladd (struct file_info *finfo, char *rev, char *tag, char *options)
 {
     int ret;
 
@@ -1815,8 +1818,7 @@ finaladd (finfo, rev, tag, options)
  * Unlock an rcs file
  */
 static void
-unlockrcs (rcs)
-    RCSNode *rcs;
+unlockrcs (RCSNode *rcs)
 {
     int retcode;
 
@@ -1837,8 +1839,7 @@ unlockrcs (rcs)
  * to be removed without reparsing the file as we do below.
  */
 static void
-fixaddfile (rcs)
-    const char *rcs;
+fixaddfile (const char *rcs)
 {
     RCSNode *rcsfile;
     int save_really_quiet;
@@ -1861,9 +1862,7 @@ fixaddfile (rcs)
  * put the branch back on an rcs file
  */
 static void
-fixbranch (rcs, branch)
-    RCSNode *rcs;
-    char *branch;
+fixbranch (RCSNode *rcs, char *branch)
 {
     int retcode;
 
@@ -1907,12 +1906,8 @@ fixbranch (rcs, branch)
  */
 
 static int
-checkaddfile (file, repository, tag, options, rcsnode)
-    const char *file;
-    const char *repository;
-    const char *tag;
-    const char *options;
-    RCSNode **rcsnode;
+checkaddfile (const char *file, const char *repository, const char *tag,
+              const char *options, RCSNode **rcsnode)
 {
     RCSNode *rcs;
     char *fname;
@@ -2003,15 +1998,6 @@ checkaddfile (file, repository, tag, options, rcsnode)
 	else
 	    opt = NULL;
 
-	/* This message is an artifact of the time when this
-	   was implemented via "rcs -i".  It should be revised at
-	   some point (does the "initial revision" in the message from
-	   RCS_checkin indicate that this is a new file?  Or does the
-	   "RCS file" message serve some function?).  */
-	cvs_output ("RCS file: ", 0);
-	cvs_output (rcsname, 0);
-	cvs_output ("\ndone\n", 0);
-
 	if (add_rcs_file (NULL, rcsname, file, NULL, opt,
 			  NULL, NULL, 0, NULL,
 			  desc, desclen, NULL) != 0)
@@ -2074,8 +2060,7 @@ checkaddfile (file, repository, tag, options, rcsnode)
 	/* and lock it */
 	if (lock_RCS (file, rcs, rev, repository))
 	{
-	    error (0, 0, "cannot lock revision %s in `%s'.",
-		   rev ? rev : tag ? tag : "HEAD", rcs->path);
+	    error (0, 0, "cannot lock `%s'.", rcs->path);
 	    if (rev != NULL)
 		free (rev);
 	    goto out;
@@ -2113,14 +2098,13 @@ checkaddfile (file, repository, tag, options, rcsnode)
 	    /* commit a dead revision. */
 	    (void) sprintf (tmp, "file %s was initially added on branch %s.",
 			    file, tag);
-	    retcode = RCS_checkin (rcs, NULL, tmp, NULL, 0,
+	    retcode = RCS_checkin (rcs, NULL, tmp, NULL,
 				   RCS_FLAGS_DEAD | RCS_FLAGS_QUIET);
 	    free (tmp);
 	    if (retcode != 0)
 	    {
 		error (retcode == -1 ? 1 : 0, retcode == -1 ? errno : 0,
 		       "could not create initial dead revision %s", rcs->path);
-		free (fname);
 		goto out;
 	    }
 
@@ -2133,7 +2117,7 @@ checkaddfile (file, repository, tag, options, rcsnode)
 	    rcs = RCS_parse (file, repository);
 	    if (rcs == NULL)
 	    {
-		error (0, 0, "could not read %s in %s", file, repository);
+		error (0, 0, "could not read %s", rcs->path);
 		goto out;
 	    }
 	    *rcsnode = rcs;
@@ -2141,8 +2125,7 @@ checkaddfile (file, repository, tag, options, rcsnode)
 	    /* and lock it once again. */
 	    if (lock_RCS (file, rcs, NULL, repository))
 	    {
-		error (0, 0, "cannot lock initial revision in `%s'.",
-		       rcs->path);
+		error (0, 0, "cannot lock `%s'.", rcs->path);
 		goto out;
 	    }
 	}
@@ -2155,25 +2138,12 @@ checkaddfile (file, repository, tag, options, rcsnode)
 	    char *head;
 	    char *magicrev;
 	    int retcode;
-	    time_t headtime = -1;
-	    char *revnum, *tmp;
-	    FILE *fp;
-	    time_t t = -1;
-	    struct tm *ct;
 
 	    fixbranch (rcs, sbranch);
 
 	    head = RCS_getversion (rcs, NULL, NULL, 0, (int *) NULL);
-	    if (!head)
-		error (1, 0, "No head revision in archive file `%s'.",
-		       rcs->path);
 	    magicrev = RCS_magicrev (rcs, head);
 
-	    /* If this is not a new branch, then we will want a dead
-	       version created before this one. */
-	    if (!newfile)
-		headtime = RCS_getrevtime (rcs, head, 0, 0);
-
 	    retcode = RCS_settag (rcs, tag, magicrev);
 	    RCS_rewrite (rcs, NULL, NULL);
 
@@ -2186,76 +2156,13 @@ checkaddfile (file, repository, tag, options, rcsnode)
 		       "could not stub branch %s for %s", tag, rcs->path);
 		goto out;
 	    }
-	    /* We need to add a dead version here to avoid -rtag -Dtime
-	       checkout problems between when the head version was
-	       created and now. */
-	    if (!newfile && headtime != -1)
-	    {
-		/* move the new file out of the way. */
-		fname = xmalloc (strlen (file) + sizeof (CVSADM)
-			     + sizeof (CVSPREFIX) + 10);
-		(void) sprintf (fname, "%s/%s%s", CVSADM, CVSPREFIX, file);
-		rename_file (file, fname);
-
-		/* Create empty FILE.  Can't use copy_file with a DEVNULL
-		   argument -- copy_file now ignores device files. */
-		fp = fopen (file, "w");
-		if (fp == NULL)
-		    error (1, errno, "cannot open %s for writing", file);
-		if (fclose (fp) < 0)
-		    error (0, errno, "cannot close %s", file);
-
-		/* As we will be hacking the delta date, put the time
-		   this was added into the log message. */
-		t = time(NULL);
-		ct = gmtime(&t);
-		tmp = xmalloc (strlen (file) + strlen (tag) + 80);
-
-		(void) sprintf (tmp,
-			       "file %s was added on branch %s on %d-%02d-%02d %02d:%02d:%02d +0000",
-				 file, tag,
-				 ct->tm_year + (ct->tm_year < 100 ? 0 : 1900),
-				 ct->tm_mon + 1, ct->tm_mday,
-				 ct->tm_hour, ct->tm_min, ct->tm_sec);
-			 
-		/* commit a dead revision. */
-		revnum = RCS_whatbranch (rcs, tag);
-		retcode = RCS_checkin (rcs, NULL, tmp, revnum, headtime,
-				       RCS_FLAGS_DEAD |
-				       RCS_FLAGS_QUIET |
-				       RCS_FLAGS_USETIME);
-		free (revnum);
-		free (tmp);
-
-		if (retcode != 0)
-		{
-		    error (retcode == -1 ? 1 : 0, retcode == -1 ? errno : 0,
-			   "could not created dead stub %s for %s", tag,
-			   rcs->path);
-		    goto out;
-		}
-
-		/* put the new file back where it was */
-		rename_file (fname, file);
-		free (fname);
-
-		/* double-check that the file was written correctly */
-		freercsnode (&rcs);
-		rcs = RCS_parse (file, repository);
-		if (rcs == NULL)
-		{
-		    error (0, 0, "could not read %s", rcs->path);
-		    goto out;
-		}
-		*rcsnode = rcs;
-	    }
 	}
 	else
 	{
 	    /* lock the branch. (stubbed branches need not be locked.)  */
 	    if (lock_RCS (file, rcs, NULL, repository))
 	    {
-		error (0, 0, "cannot lock head revision in `%s'.", rcs->path);
+		error (0, 0, "cannot lock `%s'.", rcs->path);
 		goto out;
 	    }
 	}
@@ -2303,11 +2210,8 @@ checkaddfile (file, repository, tag, options, rcsnode)
  * put the branch back as the head if there are any errors.
  */
 static int
-lock_RCS (user, rcs, rev, repository)
-    const char *user;
-    RCSNode *rcs;
-    const char *rev;
-    const char *repository;
+lock_RCS (const char *user, RCSNode *rcs, const char *rev,
+          const char *repository)
 {
     char *branch = NULL;
     int err = 0;
@@ -2385,8 +2289,7 @@ lock_RCS (user, rcs, rev, repository)
  * free an UPDATE node's data
  */
 void
-update_delproc (p)
-    Node *p;
+update_delproc (Node *p)
 {
     struct logfile_info *li = p->data;
 
@@ -2403,8 +2306,7 @@ update_delproc (p)
  * Free the commit_info structure in p.
  */
 static void
-ci_delproc (p)
-    Node *p;
+ci_delproc (Node *p)
 {
     struct commit_info *ci = p->data;
 
@@ -2421,8 +2323,7 @@ ci_delproc (p)
  * Free the commit_info structure in p.
  */
 static void
-masterlist_delproc (p)
-    Node *p;
+masterlist_delproc (Node *p)
 {
     struct master_lists *ml = p->data;
 
