@@ -1,11 +1,6 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
- *
- * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
- *                                  and others.
- *
- * Portions Copyright (C) 1992, Brian Berliner and Jeff Polk
- * Portions Copyright (C) 1989-1992, Brian Berliner
+ * Copyright (c) 1992, Brian Berliner and Jeff Polk
+ * Copyright (c) 1989-1992, Brian Berliner
  * 
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with the CVS source distribution.
@@ -18,6 +13,8 @@
 
 #include "cvs.h"
 
+
+
 /* update_dir includes dir as its last component.
 
    Return value is 0 for success, or 1 if we printed a warning message.
@@ -27,29 +24,18 @@
    don't print warnings; all errors are fatal then.  */
 
 int
-Create_Admin (dir, update_dir, repository, tag, date, nonbranch, warn,
-	      dotemplate)
-    const char *dir;
-    const char *update_dir;
-    const char *repository;
-    const char *tag;
-    const char *date;
-    int nonbranch;
-    int warn;
-    int dotemplate;
+Create_Admin (const char *dir, const char *update_dir, const char *repository,
+              const char *tag, const char *date, int nonbranch, int warn,
+              int dotemplate)
 {
     FILE *fout;
     char *cp;
     char *reposcopy;
     char *tmp;
 
-    if (trace)
-    {
-	fprintf (stderr, "%s-> Create_Admin (%s, %s, %s, %s, %s, %d, %d, %d)\n",
-		 CLIENT_SERVER_STR,
-		 dir, update_dir, repository, tag ? tag : "",
-		 date ? date : "", nonbranch, warn, dotemplate);
-    }
+    TRACE ( 1, "Create_Admin (%s, %s, %s, %s, %s, %d, %d, %d)",
+	    dir, update_dir, repository, tag ? tag : "",
+	    date ? date : "", nonbranch, warn, dotemplate );
 
     if (noexec)
 	return 0;
@@ -168,18 +154,7 @@ Create_Admin (dir, update_dir, repository, tag, date, nonbranch, warn,
     /* Create a new CVS/Tag file */
     WriteTag (dir, tag, date, nonbranch, update_dir, repository);
 
-#ifdef SERVER_SUPPORT
-    if (server_active && dotemplate)
-    {
-	server_template (update_dir, repository);
-    }
-
-    if (trace)
-    {
-	fprintf (stderr, "%c<- Create_Admin\n",
-		 (server_active) ? 'S' : ' ');
-    }
-#endif
+    TRACE (1, "Create_Admin");
 
     free (reposcopy);
     free (tmp);
