@@ -16,75 +16,51 @@
 # include <config.h>		/* this is stuff found via autoconf */
 #endif /* CONFIG_H */
 
-/* Changed from if __STDC__ to ifdef __STDC__ because of Sun's acc compiler */
+/* Add GNU attribute suppport.  */
+#ifndef __attribute__
+/* This feature is available in gcc versions 2.5 and later.  */
+# if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 5) || __STRICT_ANSI__
+#  define __attribute__(Spec) /* empty */
+# else
+#   if __GNUC__ == 2 && __GNUC_MINOR__ < 96
+#    define __pure__	/* empty */
+#   endif
+#   if __GNUC__ < 3
+#    define __malloc__	/* empty */
+#   endif
+# endif
+/* The __-protected variants of `format' and `printf' attributes
+   are accepted by gcc versions 2.6.4 (effectively 2.7) and later.  */
+# if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 7)
+#  define __const__	const
+#  define __format__	format
+#  define __noreturn__	noreturn
+#  define __printf__	printf
+# endif
+#endif /* __attribute__ */
 
-#ifdef __STDC__
-#define	PTR	void *
-#else
-#define	PTR	char *
-#endif
+/* begin GNULIB headers */
+#include "exit.h"
+#include "vasnprintf.h"
+#include "xalloc.h"
+#include "xsize.h"
+/* end GNULIB headers */
 
-/* Add prototype support.  */
-#ifndef PROTO
-#if defined (USE_PROTOTYPES) ? USE_PROTOTYPES : defined (__STDC__)
-#define PROTO(ARGS) ARGS
-#else
-#define PROTO(ARGS) ()
-#endif
-#endif
-
-#include <stdio.h>
+#if ! STDC_HEADERS
+char *getenv();
+#endif /* ! STDC_HEADERS */
 
 /* Under OS/2, <stdio.h> doesn't define popen()/pclose(). */
 #ifdef USE_OWN_POPEN
 #include "popen.h"
 #endif
 
-/* Begin GNULIB headers.  */
-#include "xsize.h"
-/* End GNULIB headers.  */
-
-#ifdef STDC_HEADERS
-#include <stdlib.h>
-#else
-extern void exit ();
-extern char *getenv();
-#endif
-
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-
-#ifdef HAVE_STRING_H
-#include <string.h>
-#else
-#include <strings.h>
-#endif
-
 #ifdef SERVER_SUPPORT
 /* If the system doesn't provide strerror, it won't be declared in
    string.h.  */
-char *strerror ();
+char *strerror (int);
 #endif
 
-#ifdef HAVE_FNMATCH
-# include <fnmatch.h> /* This is supposed to be available on Posix systems */
-#else /* HAVE_FNMATCH */
-# include "fnmatch.h" /* Our substitute */
-#endif /* HAVE_FNMATCH */
-
-#include <ctype.h>
-#include <pwd.h>
-#include <signal.h>
-
-#ifdef HAVE_ERRNO_H
-#include <errno.h>
-#else
-#ifndef errno
-extern int errno;
-#endif /* !errno */
-#endif /* HAVE_ERRNO_H */
-
 #include "system.h"
 
 #include "hash.h"
@@ -109,17 +85,14 @@ extern int errno;
 #include "rcs.h"
 
 
-/* This actually gets set in system.h.  Note that the _ONLY_ reason for
-   this is if various system calls (getwd, getcwd, readlink) require/want
-   us to use it.  All other parts of CVS allocate pathname buffers
-   dynamically, and we want to keep it that way.  */
-#ifndef PATH_MAX
-#ifdef MAXPATHLEN
-#define	PATH_MAX MAXPATHLEN+2
-#else
-#define	PATH_MAX 1024+2
-#endif
-#endif /* PATH_MAX */
+
+/* Note that the _ONLY_ reason for PATH_MAX is if various system calls (getwd,
+ * getcwd, readlink) require/want us to use it.  All other parts of CVS
+ * allocate pathname buffers dynamically, and we want to keep it that way.
+ */
+#include "pathmax.h"
+
+
 
 /* Definitions for the CVS Administrative directory and the files it contains.
    Here as #define's to make changing the names a simple task.  */
@@ -181,7 +154,6 @@ extern int errno;
 #define	CVSROOTADM_RCSINFO	"rcsinfo"
 #define CVSROOTADM_COMMITINFO	"commitinfo"
 #define CVSROOTADM_TAGINFO      "taginfo"
-#define	CVSROOTADM_EDITINFO	"editinfo"
 #define CVSROOTADM_VERIFYMSG    "verifymsg"
 #define	CVSROOTADM_HISTORY	"history"
 #define CVSROOTADM_VALTAGS	"val-tags"
@@ -208,7 +180,9 @@ extern int errno;
 
 #define	CVSLCK		"#cvs.lock"
 #define	CVSRFL		"#cvs.rfl"
+#define	CVSPFL		"#cvs.pfl"
 #define	CVSWFL		"#cvs.wfl"
+#define CVSPFLPAT	"#cvs.pfl.*"	/* wildcard expr to match plocks */
 #define CVSRFLPAT	"#cvs.rfl.*"	/* wildcard expr to match read locks */
 #define	CVSEXT_LOG	",t"
 #define	CVSPREFIX	",,"
@@ -263,17 +237,17 @@ extern int errno;
 #define	CVSREAD_ENV	"CVSREAD"	/* make files read-only */
 #define	CVSREAD_DFLT	0		/* writable files by default */
 
+#define	CVSREADONLYFS_ENV "CVSREADONLYFS" /* repository is read-only */
+
 #define	TMPDIR_ENV	"TMPDIR"	/* Temporary directory */
+#define	CVS_PID_ENV	"CVS_PID"	/* pid of running cvs */
 
 #define	EDITOR1_ENV	"CVSEDITOR"	/* which editor to use */
 #define	EDITOR2_ENV	"VISUAL"	/* which editor to use */
 #define	EDITOR3_ENV	"EDITOR"	/* which editor to use */
 
 #define	CVSROOT_ENV	"CVSROOT"	/* source directory root */
-/* Define CVSROOT_DFLT to a fallback value for CVSROOT.
- *
-#undef	CVSROOT_DFL
- */
+#define	CVSROOT_DFLT	NULL		/* No dflt; must set for checkout */
 
 #define	IGNORE_ENV	"CVSIGNORE"	/* More files to ignore */
 #define WRAPPER_ENV     "CVSWRAPPERS"   /* name of the wrapper file */
@@ -367,6 +341,9 @@ typedef enum direnter_type Dtype;
 #define CVS_LOCK_READ	1
 #define CVS_LOCK_WRITE	2
 
+/* Option flags for Parse_Info() */
+#define PIOPT_ALL 1	/* accept "all" keyword */
+
 extern const char *program_name, *program_path, *cvs_cmd_name;
 extern char *Tmpdir, *Editor;
 extern int cvsadmin_root;
@@ -388,14 +365,18 @@ extern char *CVSroot_cmdline;
 extern List *root_directories;
 extern cvsroot_t *current_parsed_root;
 
-extern char *emptydir_name PROTO ((void));
-extern int safe_location PROTO ((char *));
+char *emptydir_name (void);
+int safe_location (char *);
 
 extern int trace;		/* Show all commands */
 extern int noexec;		/* Don't modify disk anywhere */
+extern int readonlyfs;		/* fail on all write locks; succeed all read locks */
 extern int logoff;		/* Don't write history entry */
 
 extern int top_level_admin;
+#ifdef SUPPORT_OLD_INFO_FMT_STRINGS
+extern int UseNewInfoFmtStrings;
+#endif /* SUPPORT_OLD_INFO_FMT_STRINGS */
 
 
 #define LOGMSG_REREAD_NEVER 0	/* do_verify - never  reread message */
@@ -413,8 +394,8 @@ extern char hostname[];
 
 /* Externs that are included directly in the CVS sources */
 
-int RCS_merge PROTO((RCSNode *, const char *, const char *, const char *,
-                     const char *, const char *));
+int RCS_merge (RCSNode *, const char *, const char *, const char *,
+               const char *, const char *);
 /* Flags used by RCS_* functions.  See the description of the individual
    functions for which flags mean what for each function.  */
 #define RCS_FLAGS_FORCE 1
@@ -422,85 +403,103 @@ int RCS_merge PROTO((RCSNode *, const char *, const char *, const char *,
 #define RCS_FLAGS_QUIET 4
 #define RCS_FLAGS_MODTIME 8
 #define RCS_FLAGS_KEEPFILE 16
-#define RCS_FLAGS_USETIME 32
 
-extern int RCS_exec_rcsdiff PROTO ((RCSNode *rcsfile,
-				    const char *opts, const char *options,
-				    const char *rev1, const char *rev1_cache,
-                                    const char *rev2, const char *label1,
-                                    const char *label2, const char *workfile));
-extern int diff_exec PROTO ((const char *file1, const char *file2,
-			     const char *label1, const char *label2,
-			     const char *options, const char *out));
+int RCS_exec_rcsdiff (RCSNode *rcsfile,
+                      const char *opts, const char *options,
+                      const char *rev1, const char *rev1_cache,
+                      const char *rev2,
+                      const char *label1, const char *label2,
+                      const char *workfile);
+int diff_exec (const char *file1, const char *file2,
+               const char *label1, const char *label2,
+               const char *options, const char *out);
 
 
 #include "error.h"
 
-DBM *open_module PROTO((void));
-FILE *open_file PROTO((const char *, const char *));
-List *Find_Directories PROTO((char *repository, int which, List *entries));
-void Entries_Close PROTO((List *entries));
-List *Entries_Open PROTO ((int aflag, char *update_dir));
-void Subdirs_Known PROTO((List *entries));
-void Subdir_Register PROTO((List *, const char *, const char *));
-void Subdir_Deregister PROTO((List *, const char *, const char *));
+DBM *open_module (void);
+FILE *open_file (const char *, const char *);
+List *Find_Directories (char *repository, int which, List *entries);
+void Entries_Close (List *entries);
+List *Entries_Open (int aflag, char *update_dir);
+void Subdirs_Known (List *entries);
+void Subdir_Register (List *, const char *, const char *);
+void Subdir_Deregister (List *, const char *, const char *);
 
-char *Make_Date PROTO((char *rawdate));
-char *date_from_time_t PROTO ((time_t));
-void date_to_internet PROTO ((char *, const char *));
-void date_to_tm PROTO ((struct tm *, const char *));
-void tm_to_internet PROTO ((char *, const struct tm *));
+char *Make_Date (char *rawdate);
+char *date_from_time_t (time_t);
+void date_to_internet (char *, const char *);
+void date_to_tm (struct tm *, const char *);
+void tm_to_internet (char *, const struct tm *);
+char *gmformat_time_t (time_t unixtime);
+char *format_date_alloc (char *text);
 
-char *Name_Repository PROTO((const char *dir, const char *update_dir));
-const char *Short_Repository PROTO((const char *repository));
-void Sanitize_Repository_Name PROTO((char *repository));
+char *Name_Repository (const char *dir, const char *update_dir);
+const char *Short_Repository (const char *repository);
+void Sanitize_Repository_Name (char *repository);
 
-char *previous_rev PROTO ((RCSNode *rcs, const char *rev));
-char *gca PROTO ((const char *rev1, const char *rev2));
-extern void check_numeric PROTO ((const char *, int, char **));
-char *getcaller PROTO ((void));
-char *time_stamp PROTO ((const char *file));
+char *Name_Root (char *dir, char *update_dir);
+void free_cvsroot_t (cvsroot_t *root_in);
+cvsroot_t *parse_cvsroot (const char *root)
+	__attribute__ ((__malloc__));
+cvsroot_t *local_cvsroot (const char *dir)
+	__attribute__ ((__malloc__));
+void Create_Root (const char *dir, const char *rootdir);
+void root_allow_add (char *);
+void root_allow_free (void);
+int root_allow_ok (char *);
 
-void *xmalloc PROTO((size_t bytes));
-void *xrealloc PROTO((void *ptr, size_t bytes));
-void expand_string PROTO ((char **, size_t *, size_t));
-void xrealloc_and_strcat PROTO ((char **, size_t *, const char *));
-char *xstrdup PROTO((const char *str));
-int strip_trailing_newlines PROTO((char *str));
-int pathname_levels PROTO ((const char *path));
+char *previous_rev (RCSNode *rcs, const char *rev);
+char *gca (const char *rev1, const char *rev2);
+void check_numeric (const char *, int, char **);
+char *getcaller (void);
+char *entries_time (time_t unixtime);
+time_t unix_time_stamp (const char *file);
+char *time_stamp (const char *file);
 
-typedef	int (*CALLPROC)	PROTO((const char *repository, const char *value));
-int Parse_Info PROTO((const char *infofile, const char *repository,
-                      CALLPROC callproc, int all));
-extern int parse_config PROTO ((char *));
+void *xmalloc (size_t bytes)
+	__attribute__((__malloc__));
+void *xrealloc (void *ptr, size_t bytes)
+	__attribute__ ((__malloc__));
+void expand_string (char **, size_t *, size_t);
+void xrealloc_and_strcat (char **, size_t *, const char *);
+char *xstrdup (const char *str)
+	__attribute__ ((__malloc__));
+int strip_trailing_newlines (char *str);
+int pathname_levels (const char *path);
 
-typedef	RETSIGTYPE (*SIGCLEANUPPROC)	PROTO(());
-int SIG_register PROTO((int sig, SIGCLEANUPPROC sigcleanup));
-int isdir PROTO((const char *file));
-int isfile PROTO((const char *file));
-int islink PROTO((const char *file));
-int isdevice PROTO ((const char *));
-int isreadable PROTO((const char *file));
-int iswritable PROTO((const char *file));
-int isaccessible PROTO((const char *file, const int mode));
-int isabsolute PROTO((const char *filename));
+typedef	int (*CALLPROC)	(const char *repository, const char *value,
+                         void *closure);
+int Parse_Info (const char *infofile, const char *repository,
+                CALLPROC callproc, int opt, void *closure);
+int parse_config (char *);
+
+typedef	RETSIGTYPE (*SIGCLEANUPPROC)	();
+int SIG_register (int sig, SIGCLEANUPPROC sigcleanup);
+int isdir (const char *file);
+int isfile (const char *file);
+int islink (const char *file);
+int isdevice (const char *file);
+int isreadable (const char *file);
+int iswritable (const char *file);
+int isaccessible (const char *file, const int mode);
+int isabsolute (const char *filename);
 #ifdef HAVE_READLINK
-char *xreadlink PROTO((const char *link));
-#endif
-char *xresolvepath PROTO((const char *path));
-const char *last_component PROTO((const char *path));
-char *get_homedir PROTO ((void));
-char *strcat_filename_onto_homedir PROTO ((const char *, const char *));
-char *cvs_temp_name PROTO ((void));
-FILE *cvs_temp_file PROTO ((char **filename));
-
-int numdots PROTO((const char *s));
-char *increment_revnum PROTO ((const char *));
-int compare_revnums PROTO ((const char *, const char *));
-int unlink_file PROTO((const char *f));
-int unlink_file_dir PROTO((const char *f));
-
+char *xreadlink (const char *link);
+#endif /* HAVE_READLINK */
+char *xresolvepath (const char *path);
+const char *last_component (const char *path);
+char *get_homedir (void);
+char *strcat_filename_onto_homedir (const char *, const char *);
+char *cvs_temp_name (void);
+FILE *cvs_temp_file (char **filename);
 
+int numdots (const char *s);
+char *increment_revnum (const char *);
+int compare_revnums (const char *, const char *);
+int ls (int argc, char *argv[]);
+int unlink_file (const char *f);
+int unlink_file_dir (const char *f);
 
 /* This is the structure that the recursion processor passes to the
    fileproc to tell it about a particular file.  */
@@ -531,70 +530,78 @@ struct file_info
     RCSNode *rcs;
 };
 
-
-
-int update PROTO((int argc, char *argv[]));
+int update (int argc, char *argv[]);
 /* The only place this is currently used outside of update.c is add.c.
  * Restricting its use to update.c seems to be in the best interest of
  * modularity, but I can't think of a good way to get an update of a
  * resurrected file done and print the fact otherwise.
  */
-void write_letter PROTO ((struct file_info *finfo, int letter));
-int xcmp PROTO((const char *file1, const char *file2));
-int yesno PROTO((void));
-void *valloc PROTO((size_t bytes));
-time_t get_date PROTO((char *date, struct timeb *now));
-extern int Create_Admin PROTO ((const char *dir, const char *update_dir,
-                                const char *repository, const char *tag,
-                                const char *date,
-                                int nonbranch, int warn, int dotemplate));
-extern int expand_at_signs PROTO ((const char *, off_t, FILE *));
+void write_letter (struct file_info *finfo, int letter);
+int xcmp (const char *file1, const char *file2);
+int yesno (void);
+void *valloc (size_t bytes);
+
+/* Need this until we back out the get_date () proto again and use a current
+ * version of getdate.y from GNULIB.
+ */
+#include "xtime.h"
+time_t get_date (char *date, struct timeb *now);
+
+int Create_Admin (const char *dir, const char *update_dir,
+                  const char *repository, const char *tag, const char *date,
+                  int nonbranch, int warn, int dotemplate);
+int expand_at_signs (const char *, off_t, FILE *);
 
 /* Locking subsystem (implemented in lock.c).  */
 
-int Reader_Lock PROTO((char *xrepository));
-void Lock_Cleanup PROTO((void));
+int Reader_Lock (char *xrepository);
+void Simple_Lock_Cleanup (void);
+void Lock_Cleanup (void);
 
 /* Writelock an entire subtree, well the part specified by ARGC, ARGV, LOCAL,
    and AFLAG, anyway.  */
-void lock_tree_for_write PROTO ((int argc, char **argv, int local, int which,
-				 int aflag));
+void lock_tree_promotably (int argc, char **argv, int local, int which,
+			   int aflag);
 
 /* See lock.c for description.  */
-extern void lock_dir_for_write PROTO ((char *));
+void lock_dir_for_write (const char *);
 
 /* LockDir setting from CVSROOT/config.  */
 extern char *lock_dir;
 
-void Scratch_Entry PROTO((List * list, const char *fname));
-void ParseTag PROTO((char **tagp, char **datep, int *nonbranchp));
-void WriteTag PROTO ((const char *dir, const char *tag, const char *date,
-                      int nonbranch, const char *update_dir,
-                      const char *repository));
-void cat_module PROTO((int status));
-void check_entries PROTO((char *dir));
-void close_module PROTO((DBM * db));
-void copy_file PROTO((const char *from, const char *to));
-void fperrmsg PROTO((FILE * fp, int status, int errnum, char *message,...));
-void free_names PROTO((int *pargc, char *argv[]));
+/* AllowedAdminOptions setting from CVSROOT/config.  */
+extern char *UserAdminOptions;
 
-extern int ign_name PROTO ((char *name));
-void ign_add PROTO((char *ign, int hold));
-void ign_add_file PROTO((char *file, int hold));
-void ign_setup PROTO((void));
-void ign_dir_add PROTO((char *name));
-int ignore_directory PROTO((const char *name));
-typedef void (*Ignore_proc) PROTO ((const char *, const char *));
-extern void ignore_files PROTO ((List *, List *, const char *, Ignore_proc));
+void Scratch_Entry (List * list, const char *fname);
+void ParseTag (char **tagp, char **datep, int *nonbranchp);
+void WriteTag (const char *dir, const char *tag, const char *date,
+               int nonbranch, const char *update_dir, const char *repository);
+void WriteTemplate (const char *update_dir, int dotemplate,
+                    const char *repository);
+void cat_module (int status);
+void check_entries (char *dir);
+void close_module (DBM * db);
+void copy_file (const char *from, const char *to);
+void fperrmsg (FILE * fp, int status, int errnum, char *message,...);
+void free_names (int *pargc, char *argv[]);
+
+int ign_name (char *name);
+void ign_add (char *ign, int hold);
+void ign_add_file (char *file, int hold);
+void ign_setup (void);
+void ign_dir_add (char *name);
+int ignore_directory (const char *name);
+typedef void (*Ignore_proc) (const char *, const char *);
+void ignore_files (List *, List *, const char *, Ignore_proc);
 extern int ign_inhibit_server;
 
 #include "update.h"
 
-void line2argv PROTO ((int *pargc, char ***argv, char *line, char *sepchars));
-void make_directories PROTO((const char *name));
-void make_directory PROTO((const char *name));
-extern int mkdir_if_needed PROTO ((const char *name));
-void rename_file PROTO((const char *from, const char *to));
+void line2argv (int *pargc, char ***argv, char *line, char *sepchars);
+void make_directories (const char *name);
+void make_directory (const char *name);
+int mkdir_if_needed (const char *name);
+void rename_file (const char *from, const char *to);
 /* Expand wildcards in each element of (ARGC,ARGV).  This is according to the
    files which exist in the current directory, and accordingly to OS-specific
    conventions regarding wildcard syntax.  It might be desirable to change the
@@ -602,75 +609,79 @@ void rename_file PROTO((const char *from, const char *to));
    in the working directory).  The result is placed in *PARGC and *PARGV;
    the *PARGV array itself and all the strings it contains are newly
    malloc'd.  It is OK to call it with PARGC == &ARGC or PARGV == &ARGV.  */
-extern void expand_wild PROTO ((int argc, char **argv, 
-                                int *pargc, char ***pargv));
+void expand_wild (int argc, char **argv, 
+                  int *pargc, char ***pargv);
 
 #ifdef SERVER_SUPPORT
-extern int cvs_casecmp PROTO ((const char *, const char *));
+int cvs_casecmp (const char *, const char *);
 #endif
 
-void strip_trailing_slashes PROTO((char *path));
-void update_delproc PROTO((Node * p));
-void usage PROTO((const char *const *cpp));
-void xchmod PROTO((const char *fname, int writable));
-char *xgetwd PROTO((void));
-List *Find_Names PROTO((char *repository, int which, int aflag,
-                        List **optentries));
-void Register PROTO((List * list, const char *fname, const char *vn,
-                     const char *ts, const char *options, const char *tag,
-                     const char *date, const char *ts_conflict));
-void Update_Logfile PROTO((const char *repository, const char *xmessage,
-                           FILE * xlogfp, List * xchanges));
-void do_editor PROTO((const char *dir, char **messagep,
-		      const char *repository, List * changes));
+/* exithandle.c */
+void signals_register (RETSIGTYPE (*handler)(int));
+void cleanup_register (void (*handler) (void));
 
-void do_verify PROTO((char **messagep, const char *repository));
+void strip_trailing_slashes (char *path);
+void update_delproc (Node * p);
+void usage (const char *const *cpp);
+void xchmod (const char *fname, int writable);
+char *xgetwd (void);
+List *Find_Names (char *repository, int which, int aflag,
+		  List ** optentries);
+void Register (List * list, const char *fname, const char *vn, const char *ts,
+               const char *options, const char *tag, const char *date,
+               const char *ts_conflict);
+void Update_Logfile (const char *repository, const char *xmessage,
+                     FILE * xlogfp, List * xchanges);
+void do_editor (const char *dir, char **messagep,
+                const char *repository, List * changes);
 
-typedef	int (*CALLBACKPROC)	PROTO((int argc, char *argv[], char *where,
+void do_verify (char **messagep, const char *repository);
+
+typedef	int (*CALLBACKPROC)	(int argc, char *argv[], char *where,
 	char *mwhere, char *mfile, int shorten, int local_specified,
-	char *omodule, char *msg));
+	char *omodule, char *msg);
 
-typedef	int (*FILEPROC) PROTO ((void *callerdat, struct file_info *finfo));
-typedef	int (*FILESDONEPROC) PROTO ((void *callerdat, int err,
-                                     const char *repository,
-                                     const char *update_dir,
-                                     List *entries));
-typedef	Dtype (*DIRENTPROC) PROTO ((void *callerdat, const char *dir,
-				    const char *repos, const char *update_dir,
-				    List *entries));
-typedef	int (*DIRLEAVEPROC) PROTO ((void *callerdat, const char *dir, int err,
-				    const char *update_dir, List *entries));
 
-extern int mkmodules PROTO ((char *dir));
-extern int init PROTO ((int argc, char **argv));
+typedef	int (*FILEPROC) (void *callerdat, struct file_info *finfo);
+typedef	int (*FILESDONEPROC) (void *callerdat, int err,
+                              const char *repository, const char *update_dir,
+                              List *entries);
+typedef	Dtype (*DIRENTPROC) (void *callerdat, const char *dir,
+                             const char *repos, const char *update_dir,
+                             List *entries);
+typedef	int (*DIRLEAVEPROC) (void *callerdat, const char *dir, int err,
+                             const char *update_dir, List *entries);
 
-int do_module PROTO((DBM * db, char *mname, enum mtype m_type, char *msg,
+int mkmodules (char *dir);
+int init (int argc, char **argv);
+
+int do_module (DBM * db, char *mname, enum mtype m_type, char *msg,
 		CALLBACKPROC callback_proc, char *where, int shorten,
 		int local_specified, int run_module_prog, int build_dirs,
-		char *extra_arg));
-void history_write PROTO((int type, const char *update_dir, const char *revs,
-                          const char *name, const char *repository));
-int start_recursion PROTO((FILEPROC fileproc, FILESDONEPROC filesdoneproc,
+		char *extra_arg);
+void history_write (int type, const char *update_dir, const char *revs,
+                    const char *name, const char *repository);
+int start_recursion (FILEPROC fileproc, FILESDONEPROC filesdoneproc,
 		     DIRENTPROC direntproc, DIRLEAVEPROC dirleaveproc,
 		     void *callerdat,
 		     int argc, char *argv[], int local, int which,
 		     int aflag, int locktype, char *update_preload,
-		     int dosrcs, char *repository));
-void SIG_beginCrSect PROTO((void));
-void SIG_endCrSect PROTO((void));
-int SIG_inCrSect PROTO((void));
-void read_cvsrc PROTO((int *argc, char ***argv, const char *cmdname));
+		     int dosrcs, char *repository);
+void SIG_beginCrSect (void);
+void SIG_endCrSect (void);
+int SIG_inCrSect (void);
+void read_cvsrc (int *argc, char ***argv, const char *cmdname);
 
-char *make_message_rcslegal PROTO((const char *message));
-extern int file_has_conflict PROTO ((const struct file_info *,
-				     const char *ts_conflict));
-extern int file_has_markers PROTO ((const struct file_info *));
-extern void get_file PROTO ((const char *, const char *, const char *,
-			     char **, size_t *, size_t *));
-extern char *shell_escape PROTO((char *buf, const char *str));
-char *backup_file PROTO((const char *file, const char *suffix));
-extern void resolve_symlink PROTO ((char **filename));
-void sleep_past PROTO ((time_t desttime));
+char *make_message_rcsvalid (const char *message);
+int file_has_conflict (const struct file_info *,
+                       const char *ts_conflict);
+int file_has_markers (const struct file_info *);
+void get_file (const char *, const char *, const char *,
+               char **, size_t *, size_t *);
+char *shell_escape (char *buf, const char *str);
+char *backup_file (const char *file, const char *suffix);
+void resolve_symlink (char **filename);
+void sleep_past (time_t desttime);
 
 /* flags for run_exec(), the fast system() for CVS */
 #define	RUN_NORMAL            0x0000    /* no special behaviour */
@@ -681,19 +692,48 @@ void sleep_past PROTO ((time_t desttime));
 #define	RUN_SIGIGNORE         0x0010    /* ignore interrupts for command */
 #define	RUN_TTY               (char *)0 /* for the benefit of lint */
 
-void run_arg PROTO((const char *s));
-void run_print PROTO((FILE * fp));
-void run_setup PROTO ((const char *prog));
-int run_exec PROTO((const char *stin, const char *stout, const char *sterr,
-		    int flags));
+void run_arg (const char *s);
+void run_print (FILE * fp);
+void run_setup (const char *prog);
+int run_exec (const char *stin, const char *stout, const char *sterr,
+              int flags);
+/* for format_cmdline function - when a list variable is bound to a user string,
+ * we need to pass some data through walklist into the callback function.
+ * We use this struct.
+ */
+struct format_cmdline_walklist_closure
+{
+    const char *format;	/* the format string the user passed us */
+    char **buf;		/* *dest = our NUL terminated and possibly too short
+			 * destination string
+			 */
+    size_t *length;	/* *dlen = how many bytes have already been allocated to
+			 * *dest.
+			 */
+    char **d;		/* our pointer into buf where the next char should go */
+    char quotes;	/* quotes we are currently between, if any */
+#ifdef SUPPORT_OLD_INFO_FMT_STRINGS
+    int onearg;
+    int firstpass;
+    const char *srepos;
+#endif /* SUPPORT_OLD_INFO_FMT_STRINGS */
+    void *closure;	/* our user defined closure */
+};
+char *cmdlinequote (char quotes, char *s);
+char *cmdlineescape (char quotes, char *s);
+#ifdef SUPPORT_OLD_INFO_FMT_STRINGS
+char *format_cmdline (int oldway, const char *srepos, const char *format, ...);
+#else /* SUPPORT_OLD_INFO_FMT_STRINGS */
+char *format_cmdline (const char *format, ...);
+#endif /* SUPPORT_OLD_INFO_FMT_STRINGS */
 
 /* other similar-minded stuff from run.c.  */
-FILE *run_popen PROTO((const char *, const char *));
-int piped_child PROTO((const char **, int *, int *));
-void close_on_exec PROTO((int));
+FILE *run_popen (const char *, const char *);
+int piped_child (char *const *, int *, int *);
+void close_on_exec (int);
+
+pid_t waitpid (pid_t, int *, int);
 
-pid_t waitpid PROTO((pid_t, int *, int));
-
 /*
  * a struct vers_ts contains all the information about a file including the
  * user and rcs file names, and the version checked out and the head.
@@ -708,9 +748,9 @@ struct vers_ts
 
        NULL = file is not mentioned in Entries (this is also used for a
 	      directory).
-       "" = ILLEGAL!  The comment used to say that it meant "no user file"
+       "" = INVALID!  The comment used to say that it meant "no user file"
 	    but as far as I know CVS didn't actually use it that way.
-	    Note that according to cvs.texinfo, "" is not legal in the
+	    Note that according to cvs.texinfo, "" is not valid in the
 	    Entries file.
        0 = user file is new
        -vers = user file to be removed.  */
@@ -764,25 +804,25 @@ struct vers_ts
 };
 typedef struct vers_ts Vers_TS;
 
-Vers_TS *Version_TS PROTO ((struct file_info *finfo, char *options, char *tag,
+Vers_TS *Version_TS (struct file_info *finfo, char *options, char *tag,
 			    char *date, int force_tag_match,
-			    int set_time));
-void freevers_ts PROTO ((Vers_TS ** versp));
+			    int set_time);
+void freevers_ts (Vers_TS ** versp);
 
 /* Miscellaneous CVS infrastructure which layers on top of the recursion
    processor (for example, needs struct file_info).  */
 
-int Checkin PROTO ((int type, struct file_info *finfo, char *rev,
-		    char *tag, char *options, char *message));
-int No_Difference PROTO ((struct file_info *finfo, Vers_TS *vers));
+int Checkin (int type, struct file_info *finfo, char *rev,
+	     char *tag, char *options, char *message);
+int No_Difference (struct file_info *finfo, Vers_TS *vers);
 /* TODO: can the finfo argument to special_file_mismatch be changed? -twp */
-int special_file_mismatch PROTO ((struct file_info *finfo,
-				  char *rev1, char *rev2));
+int special_file_mismatch (struct file_info *finfo,
+				  char *rev1, char *rev2);
 
 /* CVSADM_BASEREV stuff, from entries.c.  */
-extern char *base_get PROTO ((struct file_info *));
-extern void base_register PROTO ((struct file_info *, char *));
-extern void base_deregister PROTO ((struct file_info *));
+char *base_get (struct file_info *);
+void base_register (struct file_info *, char *);
+void base_deregister (struct file_info *);
 
 /*
  * defines for Classify_File() to determine the current state of a file.
@@ -805,9 +845,8 @@ enum classify_type
 };
 typedef enum classify_type Ctype;
 
-Ctype Classify_File PROTO
-    ((struct file_info *finfo, char *tag, char *date, char *options,
-      int force_tag_match, int aflag, Vers_TS **versp, int pipeout));
+Ctype Classify_File (struct file_info *finfo, char *tag, char *date, char *options,
+      int force_tag_match, int aflag, Vers_TS **versp, int pipeout);
 
 /*
  * structure used for list nodes passed to Update_Logfile() and
@@ -834,41 +873,42 @@ typedef enum {
     WRAP_RCSOPTION
 } WrapMergeHas;
 
-void  wrap_setup PROTO((void));
-int   wrap_name_has PROTO((const char *name,WrapMergeHas has));
-char *wrap_rcsoption PROTO ((const char *fileName, int asFlag));
-char *wrap_tocvs_process_file PROTO((const char *fileName));
-int   wrap_merge_is_copy PROTO((const char *fileName));
-void wrap_fromcvs_process_file PROTO ((const char *fileName));
-void wrap_add_file PROTO((const char *file,int temp));
-void wrap_add PROTO((char *line,int temp));
-void wrap_send PROTO ((void));
+void  wrap_setup (void);
+int   wrap_name_has (const char *name,WrapMergeHas has);
+char *wrap_rcsoption (const char *fileName, int asFlag);
+char *wrap_tocvs_process_file (const char *fileName);
+int   wrap_merge_is_copy (const char *fileName);
+void wrap_fromcvs_process_file (const char *fileName);
+void wrap_add_file (const char *file,int temp);
+void wrap_add (char *line,int temp);
+void wrap_send (void);
 #if defined(SERVER_SUPPORT) || defined(CLIENT_SUPPORT)
-void wrap_unparse_rcs_options PROTO ((char **, int));
+void wrap_unparse_rcs_options (char **, int);
 #endif /* SERVER_SUPPORT || CLIENT_SUPPORT */
 
 /* Pathname expansion */
-char *expand_path PROTO((const char *name, const char *file, int line));
+char *expand_path (const char *name, const char *file, int line,
+                   int formatsafe);
 
 /* User variables.  */
 extern List *variable_list;
 
-extern void variable_set PROTO ((char *nameval));
+void variable_set (char *nameval);
 
-int watch PROTO ((int argc, char **argv));
-int edit PROTO ((int argc, char **argv));
-int unedit PROTO ((int argc, char **argv));
-int editors PROTO ((int argc, char **argv));
-int watchers PROTO ((int argc, char **argv));
-extern int annotate PROTO ((int argc, char **argv));
-extern int add PROTO ((int argc, char **argv));
-extern int admin PROTO ((int argc, char **argv));
-extern int checkout PROTO ((int argc, char **argv));
-extern int commit PROTO ((int argc, char **argv));
-extern int diff PROTO ((int argc, char **argv));
-extern int history PROTO ((int argc, char **argv));
-extern int import PROTO ((int argc, char **argv));
-extern int cvslog PROTO ((int argc, char **argv));
+int watch (int argc, char **argv);
+int edit (int argc, char **argv);
+int unedit (int argc, char **argv);
+int editors (int argc, char **argv);
+int watchers (int argc, char **argv);
+int annotate (int argc, char **argv);
+int add (int argc, char **argv);
+int admin (int argc, char **argv);
+int checkout (int argc, char **argv);
+int commit (int argc, char **argv);
+int diff (int argc, char **argv);
+int history (int argc, char **argv);
+int import (int argc, char **argv);
+int cvslog (int argc, char **argv);
 #ifdef AUTH_CLIENT_SUPPORT
 /* Some systems (namely Mac OS X) have conflicting definitions for these
  * functions.  Avoid them.
@@ -879,40 +919,62 @@ extern int cvslog PROTO ((int argc, char **argv));
 #ifdef HAVE_LOGOUT
 # define logout		cvs_logout
 #endif /* HAVE_LOGOUT */
-extern int login PROTO((int argc, char **argv));
-extern int logout PROTO((int argc, char **argv));
+int login (int argc, char **argv);
+int logout (int argc, char **argv);
 #endif /* AUTH_CLIENT_SUPPORT */
-extern int patch PROTO((int argc, char **argv));
-extern int release PROTO((int argc, char **argv));
-extern int cvsremove PROTO((int argc, char **argv));
-extern int rtag PROTO((int argc, char **argv));
-extern int cvsstatus PROTO((int argc, char **argv));
-extern int cvstag PROTO((int argc, char **argv));
-extern int version PROTO((int argc, char **argv));
-
-extern unsigned long int lookup_command_attribute PROTO((char *));
+int patch (int argc, char **argv);
+int release (int argc, char **argv);
+int cvsremove (int argc, char **argv);
+int rtag (int argc, char **argv);
+int cvsstatus (int argc, char **argv);
+int cvstag (int argc, char **argv);
+int version (int argc, char **argv);
+
+unsigned long int lookup_command_attribute (char *);
 
 #if defined(AUTH_CLIENT_SUPPORT) || defined(AUTH_SERVER_SUPPORT)
-char *scramble PROTO ((char *str));
-char *descramble PROTO ((char *str));
+char *scramble (char *str);
+char *descramble (char *str);
 #endif /* AUTH_CLIENT_SUPPORT || AUTH_SERVER_SUPPORT */
 
 #ifdef AUTH_CLIENT_SUPPORT
-char *get_cvs_password PROTO((void));
-int get_cvs_port_number PROTO((const cvsroot_t *root));
-char *normalize_cvsroot PROTO((const cvsroot_t *root));
+char *get_cvs_password (void);
+/* get_cvs_port_number() is not pure since the /etc/services file could change
+ * between calls.  */
+int get_cvs_port_number (const cvsroot_t *root);
+/* normalize_cvsroot() is not pure since it calls get_cvs_port_number.  */
+char *normalize_cvsroot (const cvsroot_t *root)
+	__attribute__ ((__malloc__));
 #endif /* AUTH_CLIENT_SUPPORT */
 
-extern void tag_check_valid PROTO ((char *, int, char **, int, int, char *));
-extern void tag_check_valid_join PROTO ((char *, int, char **, int, int,
-					 char *));
-
+void tag_check_valid (char *, int, char **, int, int, char *);
+void tag_check_valid_join (char *, int, char **, int, int,
+                           char *);
+
 #include "server.h"
 
 /* From server.c and documented there.  */
-extern void cvs_output PROTO ((const char *, size_t));
-extern void cvs_output_binary PROTO ((char *, size_t));
-extern void cvs_outerr PROTO ((const char *, size_t));
-extern void cvs_flusherr PROTO ((void));
-extern void cvs_flushout PROTO ((void));
-extern void cvs_output_tagged PROTO ((const char *, const char *));
+void cvs_output (const char *, size_t);
+void cvs_output_binary (char *, size_t);
+void cvs_outerr (const char *, size_t);
+void cvs_flusherr (void);
+void cvs_flushout (void);
+void cvs_output_tagged (const char *, const char *);
+
+/* The trace function from subr.c */
+void cvs_trace (int level, const char *fmt, ...)
+  __attribute__ ((__format__ (__printf__, 2, 3)));
+#define TRACE cvs_trace
+/* Trace levels:
+ *
+ * TRACE_FUNCTION	Trace function calls, often including function
+ * 			arguments.  This is the trace level that, historically,
+ * 			applied to all trace calls.
+ * TRACE_FLOW		Include the flow control functions, such as
+ * 			start_recursion, do_recursion, and walklist in the
+ * 			function traces.
+ * TRACE_DATA		Trace important internal function data.
+ */ 
+#define TRACE_FUNCTION		1
+#define TRACE_FLOW		2
+#define TRACE_DATA		3
