@@ -1,11 +1,6 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
- *
- * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
- *                                  and others.
- *
- * Portions Copyright (C) 1992, Brian Berliner and Jeff Polk
- * Portions Copyright (C) 1989-1992, Brian Berliner
+ * Copyright (c) 1992, Brian Berliner and Jeff Polk
+ * Copyright (c) 1989-1992, Brian Berliner
  * 
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with the CVS source distribution.
@@ -19,7 +14,6 @@
  * files.
  */
 
-#include <assert.h>
 #include "cvs.h"
 
 enum diff_file
@@ -31,23 +25,19 @@ enum diff_file
     DIFF_SAME
 };
 
-static Dtype diff_dirproc PROTO ((void *callerdat, const char *dir,
-                                  const char *pos_repos,
-                                  const char *update_dir,
-                                  List *entries));
-static int diff_filesdoneproc PROTO ((void *callerdat, int err,
-                                      const char *repos,
-                                      const char *update_dir,
-                                      List *entries));
-static int diff_dirleaveproc PROTO ((void *callerdat, const char *dir,
-                                     int err, const char *update_dir,
-                                     List *entries));
-static enum diff_file diff_file_nodiff PROTO(( struct file_info *finfo,
-					       Vers_TS *vers,
-					       enum diff_file, 
-					       char **rev1_cache ));
-static int diff_fileproc PROTO ((void *callerdat, struct file_info *finfo));
-static void diff_mark_errors PROTO((int err));
+static Dtype diff_dirproc (void *callerdat, const char *dir,
+                           const char *pos_repos, const char *update_dir,
+                           List *entries);
+static int diff_filesdoneproc (void *callerdat, int err,
+                               const char *repos, const char *update_dir,
+                               List *entries);
+static int diff_dirleaveproc (void *callerdat, const char *dir,
+                              int err, const char *update_dir,
+                              List *entries);
+static enum diff_file diff_file_nodiff (struct file_info *finfo, Vers_TS *vers,
+                                        enum diff_file, char **rev1_cache );
+static int diff_fileproc (void *callerdat, struct file_info *finfo);
+static void diff_mark_errors (int err);
 
 
 /* Global variables.  Would be cleaner if we just put this stuff in a
@@ -243,9 +233,7 @@ static struct option const longopts[] =
    a larger issue than the changes included here.  */
 
 int
-diff (argc, argv)
-    int argc;
-    char **argv;
+diff (int argc, char **argv)
 {
     char tmp[50];
     int c, err = 0;
@@ -436,10 +424,10 @@ diff (argc, argv)
     wrap_setup ();
 
     /* start the recursion processor */
-    err = start_recursion (diff_fileproc, diff_filesdoneproc, diff_dirproc,
-			   diff_dirleaveproc, NULL, argc, argv, local,
-			   which, 0, CVS_LOCK_READ, (char *) NULL, 1,
-			   (char *) NULL);
+    err = start_recursion
+	    ( diff_fileproc, diff_filesdoneproc, diff_dirproc,
+	      diff_dirleaveproc, NULL, argc, argv, local,
+	      which, 0, CVS_LOCK_READ, (char *) NULL, 1, (char *) NULL );
 
     /* clean up */
     free (options);
@@ -458,9 +446,7 @@ diff (argc, argv)
  */
 /* ARGSUSED */
 static int
-diff_fileproc (callerdat, finfo)
-    void *callerdat;
-    struct file_info *finfo;
+diff_fileproc (void *callerdat, struct file_info *finfo)
 {
     int status, err = 2;		/* 2 == trouble, like rcsdiff */
     Vers_TS *vers;
@@ -663,26 +649,24 @@ diff_fileproc (callerdat, finfo)
      */
     label1 = NULL;
     label2 = NULL;
-    if (!have_rev1_label)
-    {
-	if (empty_file == DIFF_ADDED)
-	    label1 =
-		make_file_label (DEVNULL, NULL, NULL);
-	else
-	    label1 =
-                make_file_label (finfo->fullname, use_rev1,
-                                 vers ? vers->srcfile : NULL);
-    }
-
+    /* The user cannot set the rev2 label without first setting the rev1
+     * label.
+     */
     if (!have_rev2_label)
     {
 	if (empty_file == DIFF_REMOVED)
-	    label2 =
-		make_file_label (DEVNULL, NULL, NULL);
+	    label2 = make_file_label (DEVNULL, NULL, NULL);
 	else
-	    label2 =
-                make_file_label (finfo->fullname, use_rev2,
-                                 vers ? vers->srcfile : NULL);
+	    label2 = make_file_label (finfo->fullname, use_rev2,
+	                              vers ? vers->srcfile : NULL);
+	if (!have_rev1_label)
+	{
+	    if (empty_file == DIFF_ADDED)
+		label1 = make_file_label (DEVNULL, NULL, NULL);
+	    else
+		label1 = make_file_label (finfo->fullname, use_rev1,
+		                          vers ? vers->srcfile : NULL);
+	}
     }
 
     if (empty_file == DIFF_ADDED || empty_file == DIFF_REMOVED)
@@ -805,13 +789,14 @@ out:
  * Remember the exit status for each file.
  */
 static void
-diff_mark_errors (err)
-    int err;
+diff_mark_errors (int err)
 {
     if (err > diff_errors)
 	diff_errors = err;
 }
 
+
+
 /*
  * Print a warm fuzzy message when we enter a dir
  *
@@ -819,12 +804,8 @@ diff_mark_errors (err)
  */
 /* ARGSUSED */
 static Dtype
-diff_dirproc (callerdat, dir, pos_repos, update_dir, entries)
-    void *callerdat;
-    const char *dir;
-    const char *pos_repos;
-    const char *update_dir;
-    List *entries;
+diff_dirproc (void *callerdat, const char *dir, const char *pos_repos,
+              const char *update_dir, List *entries)
 {
     /* XXX - check for dirs we don't want to process??? */
 
@@ -837,51 +818,52 @@ diff_dirproc (callerdat, dir, pos_repos, update_dir, entries)
     return (R_PROCESS);
 }
 
+
+
 /*
  * Concoct the proper exit status - done with files
  */
 /* ARGSUSED */
 static int
-diff_filesdoneproc (callerdat, err, repos, update_dir, entries)
-    void *callerdat;
-    int err;
-    const char *repos;
-    const char *update_dir;
-    List *entries;
+diff_filesdoneproc (void *callerdat, int err, const char *repos,
+                    const char *update_dir, List *entries)
 {
     return (diff_errors);
 }
 
+
+
 /*
  * Concoct the proper exit status - leaving directories
  */
 /* ARGSUSED */
 static int
-diff_dirleaveproc (callerdat, dir, err, update_dir, entries)
-    void *callerdat;
-    const char *dir;
-    int err;
-    const char *update_dir;
-    List *entries;
+diff_dirleaveproc (void *callerdat, const char *dir, int err,
+                   const char *update_dir, List *entries)
 {
     return (diff_errors);
 }
 
+
+
 /*
  * verify that a file is different
  */
 static enum diff_file
-diff_file_nodiff( finfo, vers, empty_file, rev1_cache )
-    struct file_info *finfo;
-    Vers_TS *vers;
-    enum diff_file empty_file;
-    char **rev1_cache;		/* Cache the content of rev1 if we have to look
+diff_file_nodiff(struct file_info *finfo, Vers_TS *vers, enum diff_file empty_file, char **rev1_cache)
+                            
+                  
+                              
+                      		/* Cache the content of rev1 if we have to look
 				 * it up.
 				 */
 {
     Vers_TS *xvers;
     int retcode;
 
+    TRACE (TRACE_FUNCTION, "diff_file_nodiff (%s, %d)",
+           finfo->fullname ? finfo->fullname : "(null)", empty_file);
+
     /* free up any old use_rev* variables and reset 'em */
     if (use_rev1)
 	free (use_rev1);
