@@ -16,7 +16,7 @@
 #include "edit.h"
 #include "fileattr.h"
 
-static int watch_onoff PROTO ((int, char **));
+static int watch_onoff (int, char **);
 
 static int setting_default;
 static int turning_on;
@@ -25,36 +25,30 @@ static int setting_tedit;
 static int setting_tunedit;
 static int setting_tcommit;
 
-static int onoff_fileproc PROTO ((void *callerdat, struct file_info *finfo));
+
 
 static int
-onoff_fileproc (callerdat, finfo)
-    void *callerdat;
-    struct file_info *finfo;
+onoff_fileproc (void *callerdat, struct file_info *finfo)
 {
     fileattr_set (finfo->file, "_watched", turning_on ? "" : NULL);
     return 0;
 }
 
-static int onoff_filesdoneproc PROTO ((void *, int, char *, char *, List *));
+
 
 static int
-onoff_filesdoneproc (callerdat, err, repository, update_dir, entries)
-    void *callerdat;
-    int err;
-    char *repository;
-    char *update_dir;
-    List *entries;
+onoff_filesdoneproc (void *callerdat, int err, const char *repository,
+                     const char *update_dir, List *entries)
 {
     if (setting_default)
 	fileattr_set (NULL, "_watched", turning_on ? "" : NULL);
     return err;
 }
 
+
+
 static int
-watch_onoff (argc, argv)
-    int argc;
-    char **argv;
+watch_onoff (int argc, char **argv)
 {
     int c;
     int local = 0;
@@ -89,6 +83,7 @@ watch_onoff (argc, argv)
 
 	if (local)
 	    send_arg ("-l");
+	send_arg ("--");
 	send_files (argc, argv, local, 0, SEND_NO_CONTENTS);
 	send_file_names (argc, argv, SEND_EXPAND_WILD);
 	send_to_server (turning_on ? "watch-on\012" : "watch-off\012", 0);
@@ -98,41 +93,36 @@ watch_onoff (argc, argv)
 
     setting_default = (argc <= 0);
 
-    lock_tree_for_write (argc, argv, local, W_LOCAL, 0);
+    lock_tree_promotably (argc, argv, local, W_LOCAL, 0);
 
-    err = start_recursion (onoff_fileproc, onoff_filesdoneproc,
-			   (DIRENTPROC) NULL, (DIRLEAVEPROC) NULL, NULL,
-			   argc, argv, local, W_LOCAL, 0, 0, (char *)NULL,
-			   0);
+    err = start_recursion
+	(onoff_fileproc, onoff_filesdoneproc,
+	 NULL, NULL, NULL,
+	 argc, argv, local, W_LOCAL, 0, CVS_LOCK_WRITE,
+	 NULL, 0, NULL);
 
     Lock_Cleanup ();
     return err;
 }
 
 int
-watch_on (argc, argv)
-    int argc;
-    char **argv;
+watch_on (int argc, char **argv)
 {
     turning_on = 1;
     return watch_onoff (argc, argv);
 }
 
 int
-watch_off (argc, argv)
-    int argc;
-    char **argv;
+watch_off (int argc, char **argv)
 {
     turning_on = 0;
     return watch_onoff (argc, argv);
 }
 
-static int dummy_fileproc PROTO ((void *callerdat, struct file_info *finfo));
+static int dummy_fileproc (void *callerdat, struct file_info *finfo);
 
 static int
-dummy_fileproc (callerdat, finfo)
-    void *callerdat;
-    struct file_info *finfo;
+dummy_fileproc (void *callerdat, struct file_info *finfo)
 {
     /* This is a pretty hideous hack, but the gist of it is that recurse.c
        won't call notify_check unless there is a fileproc, so we can't just
@@ -140,7 +130,7 @@ dummy_fileproc (callerdat, finfo)
     return 0;
 }
 
-static int ncheck_fileproc PROTO ((void *callerdat, struct file_info *finfo));
+static int ncheck_fileproc (void *callerdat, struct file_info *finfo);
 
 /* Check for and process notifications.  Local only.  I think that doing
    this as a fileproc is the only way to catch all the
@@ -149,9 +139,7 @@ static int ncheck_fileproc PROTO ((void *callerdat, struct file_info *finfo));
    processed the directory.  */
 
 static int
-ncheck_fileproc (callerdat, finfo)
-    void *callerdat;
-    struct file_info *finfo;
+ncheck_fileproc (void *callerdat, struct file_info *finfo)
 {
     int notif_type;
     char *filename;
@@ -219,15 +207,12 @@ ncheck_fileproc (callerdat, finfo)
     return 0;
 }
 
-static int send_notifications PROTO ((int, char **, int));
+static int send_notifications (int, char **, int);
 
 /* Look through the CVSADM_NOTIFY file and process each item there
    accordingly.  */
 static int
-send_notifications (argc, argv, local)
-    int argc;
-    char **argv;
-    int local;
+send_notifications (int argc, char **argv, int local)
 {
     int err = 0;
 
@@ -237,19 +222,20 @@ send_notifications (argc, argv, local)
        notifications stay in CVSADM_NOTIFY to be sent next time.  */
     if (current_parsed_root->isremote)
     {
-	if (strcmp (command_name, "release") != 0)
+	if (strcmp (cvs_cmd_name, "release") != 0)
 	{
 	    start_server ();
 	    ign_setup ();
 	}
 
-	err += start_recursion (dummy_fileproc, (FILESDONEPROC) NULL,
-				(DIRENTPROC) NULL, (DIRLEAVEPROC) NULL, NULL,
-				argc, argv, local, W_LOCAL, 0, 0, (char *)NULL,
-				0);
+	err += start_recursion
+	    ( dummy_fileproc, (FILESDONEPROC) NULL,
+	      (DIRENTPROC) NULL, (DIRLEAVEPROC) NULL, NULL,
+	      argc, argv, local, W_LOCAL, 0, 0, (char *) NULL,
+	      0, (char *) NULL );
 
 	send_to_server ("noop\012", 0);
-	if (strcmp (command_name, "release") == 0)
+	if (strcmp (cvs_cmd_name, "release") == 0)
 	    err += get_server_responses ();
 	else
 	    err += get_responses_and_close ();
@@ -259,22 +245,22 @@ send_notifications (argc, argv, local)
     {
 	/* Local.  */
 
-	lock_tree_for_write (argc, argv, local, W_LOCAL, 0);
-	err += start_recursion (ncheck_fileproc, (FILESDONEPROC) NULL,
-				(DIRENTPROC) NULL, (DIRLEAVEPROC) NULL, NULL,
-				argc, argv, local, W_LOCAL, 0, 0, (char *)NULL,
-				0);
+	lock_tree_promotably (argc, argv, local, W_LOCAL, 0);
+	err += start_recursion
+	    (ncheck_fileproc, NULL, NULL, NULL, NULL,
+	     argc, argv, local, W_LOCAL, 0, CVS_LOCK_WRITE,
+	     NULL, 0, NULL);
 	Lock_Cleanup ();
     }
     return err;
 }
-
-static int edit_fileproc PROTO ((void *callerdat, struct file_info *finfo));
+
+
+
+static int edit_fileproc (void *callerdat, struct file_info *finfo);
 
 static int
-edit_fileproc (callerdat, finfo)
-    void *callerdat;
-    struct file_info *finfo;
+edit_fileproc (void *callerdat, struct file_info *finfo)
 {
     FILE *fp;
     time_t now;
@@ -363,9 +349,7 @@ static const char *const edit_usage[] =
 };
 
 int
-edit (argc, argv)
-    int argc;
-    char **argv;
+edit (int argc, char **argv)
 {
     int local = 0;
     int c;
@@ -440,22 +424,21 @@ edit (argc, argv)
 
     /* No need to readlock since we aren't doing anything to the
        repository.  */
-    err = start_recursion (edit_fileproc, (FILESDONEPROC) NULL,
-			   (DIRENTPROC) NULL, (DIRLEAVEPROC) NULL, NULL,
-			   argc, argv, local, W_LOCAL, 0, 0, (char *)NULL,
-			   0);
+    err = start_recursion
+	( edit_fileproc, (FILESDONEPROC) NULL,
+	  (DIRENTPROC) NULL, (DIRLEAVEPROC) NULL, NULL,
+	  argc, argv, local, W_LOCAL, 0, 0, (char *) NULL,
+	  0, (char *) NULL );
 
     err += send_notifications (argc, argv, local);
 
     return err;
 }
 
-static int unedit_fileproc PROTO ((void *callerdat, struct file_info *finfo));
+static int unedit_fileproc (void *callerdat, struct file_info *finfo);
 
 static int
-unedit_fileproc (callerdat, finfo)
-    void *callerdat;
-    struct file_info *finfo;
+unedit_fileproc (void *callerdat, struct file_info *finfo)
 {
     FILE *fp;
     time_t now;
@@ -533,7 +516,7 @@ unedit_fileproc (callerdat, finfo)
 	   now.  */
 	if (node != NULL)
 	{
-	    entdata = (Entnode *) node->data;
+	    entdata = node->data;
 	    if (baserev == NULL)
 	    {
 		/* This can only happen if the CVS/Baserev file got
@@ -576,9 +559,7 @@ static const char *const unedit_usage[] =
 };
 
 int
-unedit (argc, argv)
-    int argc;
-    char **argv;
+unedit (int argc, char **argv)
 {
     int local = 0;
     int c;
@@ -609,19 +590,21 @@ unedit (argc, argv)
 
     /* No need to readlock since we aren't doing anything to the
        repository.  */
-    err = start_recursion (unedit_fileproc, (FILESDONEPROC) NULL,
-			   (DIRENTPROC) NULL, (DIRLEAVEPROC) NULL, NULL,
-			   argc, argv, local, W_LOCAL, 0, 0, (char *)NULL,
-			   0);
+    err = start_recursion
+	( unedit_fileproc, (FILESDONEPROC) NULL,
+	  (DIRENTPROC) NULL, (DIRLEAVEPROC) NULL, NULL,
+	  argc, argv, local, W_LOCAL, 0, 0, (char *) NULL,
+	  0, (char *) NULL );
 
     err += send_notifications (argc, argv, local);
 
     return err;
 }
 
+
+
 void
-mark_up_to_date (file)
-    char *file;
+mark_up_to_date (const char *file)
 {
     char *base;
 
@@ -636,12 +619,10 @@ mark_up_to_date (file)
     free (base);
 }
 
-
+
+
 void
-editor_set (filename, editor, val)
-    char *filename;
-    char *editor;
-    char *val;
+editor_set (const char *filename, const char *editor, const char *val)
 {
     char *edlist;
     char *newlist;
@@ -662,73 +643,47 @@ editor_set (filename, editor, val)
 
 struct notify_proc_args {
     /* What kind of notification, "edit", "tedit", etc.  */
-    char *type;
+    const char *type;
     /* User who is running the command which causes notification.  */
-    char *who;
+    const char *who;
     /* User to be notified.  */
-    char *notifyee;
+    const char *notifyee;
     /* File.  */
-    char *file;
+    const char *file;
 };
 
-/* Pass as a static until we get around to fixing Parse_Info to pass along
-   a void * where we can stash it.  */
-static struct notify_proc_args *notify_args;
 
-static int notify_proc PROTO ((char *repository, char *filter));
 
 static int
-notify_proc (repository, filter)
-    char *repository;
-    char *filter;
+notify_proc (const char *repository, const char *filter, void *closure)
 {
+    char *cmdline;
     FILE *pipefp;
-    char *prog;
-    char *expanded_prog;
-    char *p;
-    char *q;
-    char *srepos;
-    struct notify_proc_args *args = notify_args;
+    const char *srepos = Short_Repository (repository);
+    struct notify_proc_args *args = (struct notify_proc_args *)closure;
 
-    srepos = Short_Repository (repository);
-    prog = xmalloc (strlen (filter) + strlen (args->notifyee) + 1);
-    /* Copy FILTER to PROG, replacing the first occurrence of %s with
-       the notifyee.  We only allocated enough memory for one %s, and I doubt
-       there is a need for more.  */
-    for (p = filter, q = prog; *p != '\0'; ++p)
+    cmdline = format_cmdline (
+#ifdef SUPPORT_OLD_INFO_FMT_STRINGS
+		    0, srepos,
+#endif /* SUPPORT_OLD_INFO_FMT_STRINGS */
+	filter,
+    	"p", "s", srepos,
+	"r", "s", current_parsed_root->directory,
+	"s", "s", args->notifyee,
+	NULL
+	);
+    if (!cmdline || !strlen(cmdline))
     {
-	if (p[0] == '%')
-	{
-	    if (p[1] == 's')
-	    {
-		strcpy (q, args->notifyee);
-		q += strlen (q);
-		strcpy (q, p + 2);
-		q += strlen (q);
-		break;
-	    }
-	    else
-		continue;
-	}
-	*q++ = *p;
-    }
-    *q = '\0';
-
-    /* FIXME: why are we calling expand_proc?  Didn't we already
-       expand it in Parse_Info, before passing it to notify_proc?  */
-    expanded_prog = expand_path (prog, "notify", 0);
-    if (!expanded_prog)
-    {
-	free (prog);
-	return 1;
+	if (cmdline) free (cmdline);
+	error(0, 0, "pretag proc resolved to the empty string!");
+	return (1);
     }
 
-    pipefp = run_popen (expanded_prog, "w");
+    pipefp = run_popen (cmdline, "w");
     if (pipefp == NULL)
     {
-	error (0, errno, "cannot write entry to notify filter: %s", prog);
-	free (prog);
-	free (expanded_prog);
+	error (0, errno, "cannot write entry to notify filter: %s", cmdline);
+	free(cmdline);
 	return 1;
     }
 
@@ -739,22 +694,18 @@ notify_proc (repository, filter)
     /* Lots more potentially useful information we could add here; see
        logfile_write for inspiration.  */
 
-    free (prog);
-    free (expanded_prog);
+    free(cmdline);
     return (pclose (pipefp));
 }
 
+
+
 /* FIXME: this function should have a way to report whether there was
    an error so that server.c can know whether to report Notified back
    to the client.  */
 void
-notify_do (type, filename, who, val, watches, repository)
-    int type;
-    char *filename;
-    char *who;
-    char *val;
-    char *watches;
-    char *repository;
+notify_do (int type, const char *filename, const char *who, const char *val,
+           const char *watches, const char *repository)
 {
     static struct addremove_args blank;
     struct addremove_args args;
@@ -918,18 +869,24 @@ notify_do (type, filename, who, val, watches, repository)
 
 	    if (args.notifyee == NULL)
 	    {
-		args.notifyee = xmalloc (endp - p + 1);
-		strncpy (args.notifyee, p, endp - p);
-		args.notifyee[endp - p] = '\0';
+		char *tmp;
+		tmp = xmalloc (endp - p + 1);
+		strncpy (tmp, p, endp - p);
+		tmp[endp - p] = '\0';
+		args.notifyee = tmp;
 	    }
 
-	    notify_args = &args;
 	    args.type = notif;
 	    args.who = who;
 	    args.file = filename;
 
-	    (void) Parse_Info (CVSROOTADM_NOTIFY, repository, notify_proc, 1);
-	    free (args.notifyee);
+	    (void) Parse_Info (CVSROOTADM_NOTIFY, repository, notify_proc,
+			PIOPT_ALL, &args);
+            /* It's okay to cast out the const for the free() below since we
+             * just allocated this a few lines above.  The const was for
+             * everybody else.
+             */
+	    free ((char *)args.notifyee);
 	}
 
 	p = nextp;
@@ -964,12 +921,12 @@ notify_do (type, filename, who, val, watches, repository)
     }
 }
 
+
+
 #ifdef CLIENT_SUPPORT
 /* Check and send notifications.  This is only for the client.  */
 void
-notify_check (repository, update_dir)
-    char *repository;
-    char *update_dir;
+notify_check (const char *repository, const char *update_dir)
 {
     FILE *fp;
     char *line = NULL;
@@ -1032,12 +989,10 @@ static const char *const editors_usage[] =
     NULL
 };
 
-static int editors_fileproc PROTO ((void *callerdat, struct file_info *finfo));
+static int editors_fileproc (void *callerdat, struct file_info *finfo);
 
 static int
-editors_fileproc (callerdat, finfo)
-    void *callerdat;
-    struct file_info *finfo;
+editors_fileproc (void *callerdat, struct file_info *finfo)
 {
     char *them;
     char *p;
@@ -1087,9 +1042,7 @@ editors_fileproc (callerdat, finfo)
 }
 
 int
-editors (argc, argv)
-    int argc;
-    char **argv;
+editors (int argc, char **argv)
 {
     int local = 0;
     int c;
@@ -1125,6 +1078,7 @@ editors (argc, argv)
 
 	if (local)
 	    send_arg ("-l");
+	send_arg ("--");
 	send_files (argc, argv, local, 0, SEND_NO_CONTENTS);
 	send_file_names (argc, argv, SEND_EXPAND_WILD);
 	send_to_server ("editors\012", 0);
@@ -1132,8 +1086,9 @@ editors (argc, argv)
     }
 #endif /* CLIENT_SUPPORT */
 
-    return start_recursion (editors_fileproc, (FILESDONEPROC) NULL,
-			    (DIRENTPROC) NULL, (DIRLEAVEPROC) NULL, NULL,
-			    argc, argv, local, W_LOCAL, 0, 1, (char *)NULL,
-			    0);
+    return start_recursion
+	( editors_fileproc, (FILESDONEPROC) NULL,
+	  (DIRENTPROC) NULL, (DIRLEAVEPROC) NULL, NULL,
+	  argc, argv, local, W_LOCAL, 0, 1, (char *) NULL,
+	  0, (char *) NULL );
 }
