@@ -13,10 +13,9 @@
 #include "cvs.h"
 #include "getline.h"
 #include "fileattr.h"
-#include <assert.h>
 
-static void fileattr_read PROTO ((void));
-static int writeattr_proc PROTO ((Node *, void *));
+static void fileattr_read (void);
+static int writeattr_proc (Node *, void *);
 
 /* Where to look for CVSREP_FILEATTR.  */
 static char *fileattr_stored_repos;
@@ -39,11 +38,12 @@ struct unrecog {
 };
 static struct unrecog *unrecog_head;
 
+
+
 /* Note that if noone calls fileattr_get, this is very cheap.  No stat(),
    no open(), no nothing.  */
 void
-fileattr_startdir (repos)
-    const char *repos;
+fileattr_startdir (const char *repos)
 {
     assert (fileattr_stored_repos == NULL);
     fileattr_stored_repos = xstrdup (repos);
@@ -52,9 +52,10 @@ fileattr_startdir (repos)
     assert (unrecog_head == NULL);
 }
 
+
+
 static void
-fileattr_delproc (node)
-    Node *node;
+fileattr_delproc (Node *node)
 {
     assert (node->data != NULL);
     free (node->data);
@@ -63,7 +64,7 @@ fileattr_delproc (node)
 
 /* Read all the attributes for the current directory into memory.  */
 static void
-fileattr_read ()
+fileattr_read (void)
 {
     char *fname;
     FILE *fp;
@@ -139,7 +140,6 @@ fileattr_read ()
 		       "file attribute database corruption: tab missing in %s",
 		       fname);
 	    ++p;
-	    if (fileattr_default_attrs) free (fileattr_default_attrs);
 	    fileattr_default_attrs = xstrdup (p);
 	}
 	else
@@ -165,9 +165,7 @@ fileattr_read ()
 }
 
 char *
-fileattr_get (filename, attrname)
-    const char *filename;
-    const char *attrname;
+fileattr_get (const char *filename, const char *attrname)
 {
     Node *node;
     size_t attrname_len = strlen (attrname);
@@ -208,9 +206,7 @@ fileattr_get (filename, attrname)
 }
 
 char *
-fileattr_get0 (filename, attrname)
-    const char *filename;
-    const char *attrname;
+fileattr_get0 (const char *filename, const char *attrname)
 {
     char *cp;
     char *cpend;
@@ -229,12 +225,7 @@ fileattr_get0 (filename, attrname)
 }
 
 char *
-fileattr_modify (list, attrname, attrval, namevalsep, entsep)
-    char *list;
-    const char *attrname;
-    const char *attrval;
-    int namevalsep;
-    int entsep;
+fileattr_modify (char *list, const char *attrname, const char *attrval, int namevalsep, int entsep)
 {
     char *retval;
     char *rp;
@@ -326,10 +317,7 @@ fileattr_modify (list, attrname, attrval, namevalsep, entsep)
 }
 
 void
-fileattr_set (filename, attrname, attrval)
-    const char *filename;
-    const char *attrname;
-    const char *attrval;
+fileattr_set (const char *filename, const char *attrname, const char *attrval)
 {
     Node *node;
     char *p;
@@ -386,8 +374,7 @@ fileattr_set (filename, attrname, attrval)
 }
 
 char *
-fileattr_getall (filename)
-    const char *filename;
+fileattr_getall (const char *filename)
 {
     Node *node;
     char *p;
@@ -413,9 +400,7 @@ fileattr_getall (filename)
 }
 
 void
-fileattr_setall (filename, attrs)
-    const char *filename;
-    const char *attrs;
+fileattr_setall (const char *filename, const char *attrs)
 {
     Node *node;
 
@@ -466,8 +451,7 @@ fileattr_setall (filename, attrs)
 }
 
 void
-fileattr_newfile (filename)
-    const char *filename;
+fileattr_newfile (const char *filename)
 {
     Node *node;
 
@@ -495,9 +479,7 @@ fileattr_newfile (filename)
 }
 
 static int
-writeattr_proc (node, data)
-    Node *node;
-    void *data;
+writeattr_proc (Node *node, void *data)
 {
     FILE *fp = (FILE *)data;
     fputs ("F", fp);
@@ -509,7 +491,7 @@ writeattr_proc (node, data)
 }
 
 void
-fileattr_write ()
+fileattr_write (void)
 {
     FILE *fp;
     char *fname;
@@ -590,7 +572,6 @@ fileattr_write ()
 	    {
 		error (0, errno, "cannot make directory %s", repname);
 		(void) umask (omask);
-		free (fname);
 		free (repname);
 		return;
 	    }
@@ -602,7 +583,6 @@ fileattr_write ()
 	{
 	    error (0, errno, "cannot write %s", fname);
 	    (void) umask (omask);
-	    free (fname);
 	    return;
 	}
     }
@@ -633,7 +613,7 @@ fileattr_write ()
 }
 
 void
-fileattr_free ()
+fileattr_free (void)
 {
     /* Note that attrs_modified will ordinarily be zero, but there are
        a few cases in which fileattr_write will fail to zero it (if
