@@ -17,26 +17,22 @@
    definitions under operating systems (like, say, Windows NT) with different
    file system semantics.  */
 
-#include <assert.h>
 #include "cvs.h"
 
-static int deep_remove_dir PROTO((const char *path));
+static int deep_remove_dir (const char *path);
 
 /*
  * Copies "from" to "to".
  */
 void
-copy_file (from, to)
-    const char *from;
-    const char *to;
+copy_file (const char *from, const char *to)
 {
     struct stat sb;
     struct utimbuf t;
     int fdin, fdout;
 
-    if (trace)
-	(void) fprintf (stderr, "%s-> copy(%s,%s)\n",
-			CLIENT_SERVER_STR, from, to);
+    TRACE ( 1, "copy(%s,%s)", from, to );
+
     if (noexec)
 	return;
 
@@ -52,8 +48,8 @@ copy_file (from, to)
 
     if (isdevice (from))
     {
-#if defined(HAVE_MKNOD) && defined(HAVE_ST_RDEV)
-	if (stat (from, &sb) < 0)
+#if defined(HAVE_MKNOD) && defined(HAVE_STRUCT_STAT_ST_RDEV)
+	if( CVS_STAT( from, &sb ) < 0 )
 	    error (1, errno, "cannot stat %s", from);
 	mknod (to, sb.st_mode, sb.st_rdev);
 #else
@@ -120,12 +116,11 @@ copy_file (from, to)
  * link which points to a directory.
  */
 int
-isdir (file)
-    const char *file;
+isdir (const char *file)
 {
     struct stat sb;
 
-    if (stat (file, &sb) < 0)
+    if( CVS_STAT( file, &sb ) < 0 )
 	return (0);
     return (S_ISDIR (sb.st_mode));
 }
@@ -134,8 +129,7 @@ isdir (file)
  * Returns non-zero if the argument file is a symbolic link.
  */
 int
-islink (file)
-    const char *file;
+islink (const char *file)
 {
 #ifdef S_ISLNK
     struct stat sb;
@@ -153,8 +147,7 @@ islink (file)
  * character special device.
  */
 int
-isdevice (file)
-    const char *file;
+isdevice (const char *file)
 {
     struct stat sb;
 
@@ -175,8 +168,7 @@ isdevice (file)
  * Returns non-zero if the argument file exists.
  */
 int
-isfile (file)
-    const char *file;
+isfile (const char *file)
 {
     return isaccessible(file, F_OK);
 }
@@ -185,8 +177,7 @@ isfile (file)
  * Returns non-zero if the argument file is readable.
  */
 int
-isreadable (file)
-    const char *file;
+isreadable (const char *file)
 {
     return isaccessible(file, R_OK);
 }
@@ -195,8 +186,7 @@ isreadable (file)
  * Returns non-zero if the argument file is writable.
  */
 int
-iswritable (file)
-    const char *file;
+iswritable (const char *file)
 {
     return isaccessible(file, W_OK);
 }
@@ -207,18 +197,16 @@ iswritable (file)
  * bits set.
  */
 int
-isaccessible (file, mode)
-    const char *file;
-    const int mode;
+isaccessible (const char *file, const int mode)
 {
 #ifdef SETXID_SUPPORT
     struct stat sb;
     int umask = 0;
     int gmask = 0;
     int omask = 0;
-    int uid;
+    int uid, mask;
     
-    if (stat(file, &sb) == -1)
+    if( CVS_STAT( file, &sb ) == -1 )
 	return 0;
     if (mode == F_OK)
 	return 1;
@@ -226,10 +214,11 @@ isaccessible (file, mode)
     uid = geteuid();
     if (uid == 0)		/* superuser */
     {
-	if (mode & X_OK)
-	    return sb.st_mode & (S_IXUSR|S_IXGRP|S_IXOTH);
-	else
+	if (!(mode & X_OK) || (sb.st_mode & (S_IXUSR|S_IXGRP|S_IXOTH)))
 	    return 1;
+
+	errno = EACCES;
+	return 0;
     }
 	
     if (mode & R_OK)
@@ -251,12 +240,11 @@ isaccessible (file, mode)
 	omask |= S_IXOTH;
     }
 
-    if (sb.st_uid == uid)
-	return (sb.st_mode & umask) == umask;
-    else if (sb.st_gid == getegid())
-	return (sb.st_mode & gmask) == gmask;
-    else
-	return (sb.st_mode & omask) == omask;
+    mask = sb.st_uid == uid ? umask : sb.st_gid == getegid() ? gmask : omask;
+    if ((sb.st_mode & mask) == mask)
+	return 1;
+    errno = EACCES;
+    return 0;
 #else
     return access(file, mode) == 0;
 #endif
@@ -266,9 +254,7 @@ isaccessible (file, mode)
  * Open a file and die if it fails
  */
 FILE *
-open_file (name, mode)
-    const char *name;
-    const char *mode;
+open_file (const char *name, const char *mode)
 {
     FILE *fp;
 
@@ -281,12 +267,11 @@ open_file (name, mode)
  * Make a directory and die if it fails
  */
 void
-make_directory (name)
-    const char *name;
+make_directory (const char *name)
 {
     struct stat sb;
 
-    if (stat (name, &sb) == 0 && (!S_ISDIR (sb.st_mode)))
+    if( CVS_STAT( name, &sb ) == 0 && ( !S_ISDIR( sb.st_mode ) ) )
 	    error (0, 0, "%s already exists but is not a directory", name);
     if (!noexec && mkdir (name, 0777) < 0)
 	error (1, errno, "cannot make directory %s", name);
@@ -297,8 +282,7 @@ make_directory (name)
  * goes wrong.
  */
 void
-make_directories (name)
-    const char *name;
+make_directories (const char *name)
 {
     char *cp;
 
@@ -326,13 +310,13 @@ make_directories (name)
    other errors.  Returns 0 if directory was created; 1 if it already
    existed.  */
 int
-mkdir_if_needed (name)
-    char *name;
+mkdir_if_needed (const char *name)
 {
     if (mkdir (name, 0777) < 0)
     {
-	if (errno != EEXIST && !isdir (name))
-	    error (1, errno, "cannot make directory %s", name);
+	int save_errno = errno;
+	if (save_errno != EEXIST && !isdir (name))
+	    error (1, save_errno, "cannot make directory %s", name);
 	return 1;
     }
     return 0;
@@ -346,9 +330,7 @@ mkdir_if_needed (name)
  * have unexpected consequences for some uses of xchmod.
  */
 void
-xchmod (fname, writable)
-    char *fname;
-    int writable;
+xchmod (const char *fname, int writable)
 {
     struct stat sb;
     mode_t mode, oumask;
@@ -356,7 +338,7 @@ xchmod (fname, writable)
     if (preserve_perms)
 	return;
 
-    if (stat (fname, &sb) < 0)
+    if( CVS_STAT( fname, &sb ) < 0 )
     {
 	if (!noexec)
 	    error (0, errno, "cannot stat %s", fname);
@@ -376,10 +358,8 @@ xchmod (fname, writable)
 	mode = sb.st_mode & ~(S_IWRITE | S_IWGRP | S_IWOTH) & ~oumask;
     }
 
-    if (trace)
-	(void) fprintf (stderr, "%s-> chmod(%s,%o)\n",
-			CLIENT_SERVER_STR, fname,
-			(unsigned int) mode);
+    TRACE ( 1, "chmod(%s,%o)", fname, (unsigned int) mode );
+
     if (noexec)
 	return;
 
@@ -391,13 +371,10 @@ xchmod (fname, writable)
  * Rename a file and die if it fails
  */
 void
-rename_file (from, to)
-    const char *from;
-    const char *to;
+rename_file (const char *from, const char *to)
 {
-    if (trace)
-	(void) fprintf (stderr, "%s-> rename(%s,%s)\n",
-			CLIENT_SERVER_STR, from, to);
+    TRACE ( 1, "rename(%s,%s)", from, to );
+
     if (noexec)
 	return;
 
@@ -409,41 +386,38 @@ rename_file (from, to)
  * unlink a file, if possible.
  */
 int
-unlink_file (f)
-    const char *f;
+unlink_file (const char *f)
 {
-    if (trace)
-	(void) fprintf (stderr, "%s-> unlink_file(%s)\n",
-			CLIENT_SERVER_STR, f);
+    TRACE ( 1, "unlink_file(%s)", f );
+
     if (noexec)
 	return (0);
 
     return (CVS_UNLINK (f));
 }
 
+
+
 /*
  * Unlink a file or dir, if possible.  If it is a directory do a deep
  * removal of all of the files in the directory.  Return -1 on error
  * (in which case errno is set).
  */
 int
-unlink_file_dir (f)
-    const char *f;
+unlink_file_dir (const char *f)
 {
     struct stat sb;
 
-    if (trace
 #ifdef SERVER_SUPPORT
 	/* This is called by the server parent process in contexts where
 	   it is not OK to send output (e.g. after we sent "ok" to the
 	   client).  */
-	&& !server_active
+	if (!server_active)
 #endif
-	)
-	(void) fprintf (stderr, "-> unlink_file_dir(%s)\n", f);
+	TRACE (TRACE_FUNCTION, "unlink_file_dir(%s)", f);
 
     if (noexec)
-	return (0);
+	return 0;
 
     /* For at least some unices, if root tries to unlink() a directory,
        instead of doing something rational like returning EISDIR,
@@ -453,7 +427,7 @@ unlink_file_dir (f)
        call to stat() and the call to unlink(), we'll still corrupt
        the filesystem.  Where is the Unix Haters Handbook when you need
        it?  */
-    if (stat (f, &sb) < 0)
+    if( CVS_STAT( f, &sb ) < 0 )
     {
 	if (existence_error (errno))
 	{
@@ -467,13 +441,14 @@ unlink_file_dir (f)
     return CVS_UNLINK (f);
 }
 
+
+
 /* Remove a directory and everything it contains.  Returns 0 for
  * success, -1 for failure (in which case errno is set).
  */
 
 static int
-deep_remove_dir (path)
-    const char *path;
+deep_remove_dir (const char *path)
 {
     DIR		  *dirp;
     struct dirent *dp;
@@ -508,11 +483,11 @@ deep_remove_dir (path)
 		/* See comment in unlink_file_dir explanation of why we use
 		   isdir instead of just calling unlink and checking the
 		   status.  */
-		if (isdir(buf)) 
+		if (isdir (buf)) 
 		{
-		    if (deep_remove_dir(buf))
+		    if (deep_remove_dir (buf))
 		    {
-			CVS_CLOSEDIR(dirp);
+			CVS_CLOSEDIR (dirp);
 			free (buf);
 			return -1;
 		    }
@@ -521,7 +496,7 @@ deep_remove_dir (path)
 		{
 		    if (CVS_UNLINK (buf) != 0)
 		    {
-			CVS_CLOSEDIR(dirp);
+			CVS_CLOSEDIR (dirp);
 			free (buf);
 			return -1;
 		    }
@@ -548,14 +523,13 @@ deep_remove_dir (path)
     return 0;
 }
 
+
+
 /* Read NCHARS bytes from descriptor FD into BUF.
    Return the number of characters successfully read.
    The number returned is always NCHARS unless end-of-file or error.  */
 static size_t
-block_read (fd, buf, nchars)
-    int fd;
-    char *buf;
-    size_t nchars;
+block_read (int fd, char *buf, size_t nchars)
 {
     char *bp = buf;
     size_t nread;
@@ -589,9 +563,7 @@ block_read (fd, buf, nchars)
  * (i.e. major/minor device numbers, links, etc.
  */
 int
-xcmp (file1, file2)
-    const char *file1;
-    const char *file2;
+xcmp (const char *file1, const char *file2)
 {
     char *buf1, *buf2;
     struct stat sb1, sb2;
@@ -609,6 +581,7 @@ xcmp (file1, file2)
 
     /* If FILE1 and FILE2 are symlinks, they are equal if they point to
        the same thing. */
+#ifdef S_ISLNK
     if (S_ISLNK (sb1.st_mode) && S_ISLNK (sb2.st_mode))
     {
 	int result;
@@ -619,12 +592,13 @@ xcmp (file1, file2)
 	free (buf2);
 	return result;
     }
+#endif
 
     /* If FILE1 and FILE2 are devices, they are equal if their device
        numbers match. */
     if (S_ISBLK (sb1.st_mode) || S_ISCHR (sb1.st_mode))
     {
-#ifdef HAVE_ST_RDEV
+#ifdef HAVE_STRUCT_STAT_ST_RDEV
 	if (sb1.st_rdev == sb2.st_rdev)
 	    return 0;
 	else
@@ -694,7 +668,7 @@ xcmp (file1, file2)
  * now.
  */
 char *
-cvs_temp_name ()
+cvs_temp_name (void)
 {
     char *fn;
     FILE *fp;
@@ -724,25 +698,15 @@ cvs_temp_name ()
  *	unique file name or NULL on failure.
  *
  *  ERRORS
- *	on error, errno will be set to some value either by CVS_FOPEN or
- *	whatever system function is called to generate the temporary file name
+ *	On error, errno will be set to some value either by CVS_FOPEN or
+ *	whatever system function is called to generate the temporary file name.
+ *	The value of filename is undefined on error.
  */
-/* There are at least four functions for generating temporary
- * filenames.  We use mkstemp (BSD 4.3) if possible, else tempnam (SVID 3),
- * else mktemp (BSD 4.3), and as last resort tmpnam (POSIX).  Reason is that
- * mkstemp, tempnam, and mktemp both allow to specify the directory in which
- * the temporary file will be created.
- *
- * And the _correct_ way to use the deprecated functions probably involves
- * opening file descriptors using O_EXCL & O_CREAT and even doing the annoying
- * NFS locking thing, but until I hear of more problems, I'm not going to
- * bother.
- */
-FILE *cvs_temp_file (filename)
-    char **filename;
+FILE *cvs_temp_file (char **filename)
 {
     char *fn;
     FILE *fp;
+    int fd;
 
     /* FIXME - I'd like to be returning NULL here in noexec mode, but I think
      * some of the rcs & diff functions which rely on a temp file run in
@@ -751,11 +715,6 @@ FILE *cvs_temp_file (filename)
 
     assert (filename != NULL);
 
-#ifdef HAVE_MKSTEMP
-
-    {
-    int fd;
-
     fn = xmalloc (strlen (Tmpdir) + 11);
     sprintf (fn, "%s/%s", Tmpdir, "cvsXXXXXX" );
     fd = mkstemp (fn);
@@ -766,8 +725,8 @@ FILE *cvs_temp_file (filename)
     if (fd == -1) fp = NULL;
     else if ((fp = CVS_FDOPEN (fd, "w+")) == NULL)
     {
-	/* attempt to close and unlink the file since mkstemp returned sucessfully and
-	 * we believe it's been created and opened
+	/* Attempt to close and unlink the file since mkstemp returned
+	 * sucessfully and we believe it's been created and opened.
 	 */
  	int save_errno = errno;
 	if (close (fd))
@@ -778,98 +737,51 @@ FILE *cvs_temp_file (filename)
     }
 
     if (fp == NULL) free (fn);
-    /* mkstemp is defined to open mode 0600 using glibc 2.0.7+ */
-    /* FIXME - configure can probably tell us which version of glibc we are
-     * linking to and not chmod for 2.0.7+
+
+    /* mkstemp is defined to open mode 0600 using glibc 2.0.7+.  There used
+     * to be a complicated #ifdef checking the library versions here and then
+     * a chmod 0600 on the temp file for versions of glibc less than 2.1.  This
+     * is rather a special case, leaves a race condition open regardless, and
+     * one could hope that sysadmins have read the relevant security
+     * announcements and upgraded by now to a version with a fix committed in
+     * January of 1999.
+     *
+     * If it is decided at some point that old, buggy versions of glibc should
+     * still be catered to, a umask of 0600 should be set before file creation
+     * instead then reset after file creation since this would avoid the race
+     * condition that the chmod left open to exploitation.
      */
-    else chmod (fn, 0600);
-
-    }
-
-#elif HAVE_TEMPNAM
-
-    /* tempnam has been deprecated due to under-specification */
-
-    fn = tempnam (Tmpdir, "cvs");
-    if (fn == NULL) fp = NULL;
-    else if ((fp = CVS_FOPEN (fn, "w+")) == NULL) free (fn);
-    else chmod (fn, 0600);
-
-    /* tempnam returns a pointer to a newly malloc'd string, so there's
-     * no need for a xstrdup
-     */
-
-#elif HAVE_MKTEMP
-
-    /* mktemp has been deprecated due to the BSD 4.3 specification specifying
-     * that XXXXXX will be replaced by a PID and a letter, creating only 26
-     * possibilities, a security risk, and a race condition.
-     */
-
-    {
-    char *ifn;
-
-    ifn = xmalloc (strlen (Tmpdir) + 11);
-    sprintf (ifn, "%s/%s", Tmpdir, "cvsXXXXXX" );
-    fn = mktemp (ifn);
-
-    if (fn == NULL) fp = NULL;
-    else fp = CVS_FOPEN (fn, "w+");
-
-    if (fp == NULL) free (ifn);
-    else chmod (fn, 0600);
-
-    }
-
-#else	/* use tmpnam if all else fails */
-
-    /* tmpnam is deprecated */
-
-    {
-    char ifn[L_tmpnam + 1];
-
-    fn = tmpnam (ifn);
-
-    if (fn == NULL) fp = NULL;
-    else if ((fp = CVS_FOPEN (ifn, "w+")) != NULL)
-    {
-	fn = xstrdup (ifn);
-	chmod (fn, 0600);
-    }
-
-    }
-
-#endif
 
     *filename = fn;
     return fp;
 }
 
-/* Return non-zero iff FILENAME is absolute.
-   Trivial under Unix, but more complicated under other systems.  */
-int
-isabsolute (filename)
-    const char *filename;
-{
-    return filename[0] == '/';
-}
 
-/*
- * Return a string (dynamically allocated) with the name of the file to which
- * LINK is symlinked.
+
+#ifdef HAVE_READLINK
+/* char *
+ * xreadlink ( const char *link )
+ *
+ * Like the X/OPEN and 4.4BSD readlink() function, but allocates and returns
+ * its own buf.
+ *
+ * INPUTS
+ *  link	The original path.
+ *
+ * RETURNS
+ *  The resolution of the final symbolic link in the path.
+ *
+ * ERRORS
+ *  This function exits with a fatal error if it fails to read the link for
+ *  any reason.
  */
 char *
-xreadlink (link)
-    const char *link;
+xreadlink (const char *link)
 {
     char *file = NULL;
-    char *tfile;
     int buflen = 128;
     int link_name_len;
 
-    if (!islink (link))
-	return NULL;
-
     /* Get the name of the file to which `from' is linked.
        FIXME: what portability issues arise here?  Are readlink &
        ENAMETOOLONG defined on all systems? -twp */
@@ -886,19 +798,57 @@ xreadlink (link)
 
     file[link_name_len] = '\0';
 
-    tfile = xstrdup (file);
-    free (file);
+    return file;
+}
+#endif /* HAVE_READLINK */
 
-    return tfile;
+
+
+/* char *
+ * xresolvepath ( const char *path )
+ *
+ * Like xreadlink(), but resolve all links in a path.
+ *
+ * INPUTS
+ *  path	The original path.
+ *
+ * RETURNS
+ *  The path with any symbolic links expanded.
+ *
+ * ERRORS
+ *  This function exits with a fatal error if it fails to read the link for
+ *  any reason.
+ */
+char *
+xresolvepath (const char *path)
+{
+    char *hardpath;
+    char *owd;
+
+    assert ( isdir ( path ) );
+
+    /* FIXME - If HAVE_READLINK is defined, we should probably walk the path
+     * bit by bit calling xreadlink().
+     */
+
+    owd = xgetwd();
+    if ( CVS_CHDIR ( path ) < 0)
+	error ( 1, errno, "cannot chdir to %s", path );
+    if ( ( hardpath = xgetwd() ) == NULL )
+	error (1, errno, "cannot getwd in %s", path);
+    if ( CVS_CHDIR ( owd ) < 0)
+	error ( 1, errno, "cannot chdir to %s", owd );
+    free (owd);
+    return hardpath;
 }
 
 
+
 /* Return a pointer into PATH's last component.  */
-char *
-last_component (path)
-    char *path;
+const char *
+last_component (const char *path)
 {
-    char *last = strrchr (path, '/');
+    const char *last = strrchr (path, '/');
     
     if (last && (last != path))
         return last + 1;
@@ -906,6 +856,8 @@ last_component (path)
         return path;
 }
 
+
+
 /* Return the home directory.  Returns a pointer to storage
    managed by this function or its callees (currently getenv).
    This function will return the same thing every time it is
@@ -930,7 +882,7 @@ last_component (path)
    iffy, although I suppose some people probably are relying on it for
    .cvsrc and such, in the cases where it works.  */
 char *
-get_homedir ()
+get_homedir (void)
 {
     static char *home = NULL;
     char *env;
@@ -954,22 +906,44 @@ get_homedir ()
     return home;
 }
 
+/* Compose a path to a file in the home directory.  This is necessary because
+ * of different behavior on UNIX and VMS.  See the notes in vms/filesubr.c.
+ *
+ * A more clean solution would be something more along the lines of a
+ * "join a directory to a filename" kind of thing which was not specific to
+ * the homedir.  This should aid portability between UNIX, Mac, Windows, VMS,
+ * and possibly others.  This is already handled by Perl - it might be
+ * interesting to see how much of the code was written in C since Perl is under
+ * the GPL and the Artistic license - we might be able to use it.
+ */
+char *
+strcat_filename_onto_homedir (const char *dir, const char *file)
+{
+    char *path = xmalloc (strlen (dir) + 1 + strlen(file) + 1);
+    sprintf (path, "%s/%s", dir, file);
+    return path;
+}
+
 /* See cvs.h for description.  On unix this does nothing, because the
    shell expands the wildcards.  */
 void
-expand_wild (argc, argv, pargc, pargv)
-    int argc;
-    char **argv;
-    int *pargc;
-    char ***pargv;
+expand_wild (int argc, char **argv, int *pargc, char ***pargv)
 {
     int i;
+    if (size_overflow_p (xtimes (argc, sizeof (char *)))) {
+	*pargc = 0;
+	*pargv = NULL;
+	error (0, 0, "expand_wild: too many arguments");
+	return;
+    }
     *pargc = argc;
-    *pargv = (char **) xmalloc (argc * sizeof (char *));
+    *pargv = xmalloc (xtimes (argc, sizeof (char *)));
     for (i = 0; i < argc; ++i)
 	(*pargv)[i] = xstrdup (argv[i]);
 }
 
+
+
 #ifdef SERVER_SUPPORT
 /* Case-insensitive string compare.  I know that some systems
    have such a routine, but I'm not sure I see any reasons for
@@ -977,12 +951,10 @@ expand_wild (argc, argv, pargc, pargv)
    looked into whether this is a performance bottleneck; I would guess
    not).  */
 int
-cvs_casecmp (str1, str2)
-    char *str1;
-    char *str2;
+cvs_casecmp (const char *str1, const char *str2)
 {
-    char *p;
-    char *q;
+    const char *p;
+    const char *q;
     int pqdiff;
 
     p = str1;
@@ -996,105 +968,4 @@ cvs_casecmp (str1, str2)
     }
     return pqdiff;
 }
-
-/* Case-insensitive file open.  As you can see, this is an expensive
-   call.  We don't regard it as our main strategy for dealing with
-   case-insensitivity.  Returns errno code or 0 for success.  Puts the
-   new file in *FP.  NAME and MODE are as for fopen.  If PATHP is not
-   NULL, then put a malloc'd string containing the pathname as found
-   into *PATHP.  *PATHP is only set if the return value is 0.
-
-   Might be cleaner to separate the file finding (which just gives
-   *PATHP) from the file opening (which the caller can do).  For one
-   thing, might make it easier to know whether to put NAME or *PATHP
-   into error messages.  */
-int
-fopen_case (name, mode, fp, pathp)
-    char *name;
-    char *mode;
-    FILE **fp;
-    char **pathp;
-{
-    struct dirent *dp;
-    DIR *dirp;
-    char *dir;
-    char *fname;
-    char *found_name;
-    int retval;
-
-    /* Separate NAME into directory DIR and filename within the directory
-       FNAME.  */
-    dir = xstrdup (name);
-    fname = strrchr (dir, '/');
-    if (fname == NULL)
-	error (1, 0, "internal error: relative pathname in fopen_case");
-    *fname++ = '\0';
-
-    found_name = NULL;
-    dirp = CVS_OPENDIR (dir);
-    if (dirp == NULL)
-    {
-	if (existence_error (errno))
-	{
-	    /* This can happen if we are looking in the Attic and the Attic
-	       directory does not exist.  Return the error to the caller;
-	       they know what to do with it.  */
-	    retval = errno;
-	    goto out;
-	}
-	else
-	{
-	    /* Give a fatal error; that way the error message can be
-	       more specific than if we returned the error to the caller.  */
-	    error (1, errno, "cannot read directory %s", dir);
-	}
-    }
-    errno = 0;
-    while ((dp = CVS_READDIR (dirp)) != NULL)
-    {
-	if (cvs_casecmp (dp->d_name, fname) == 0)
-	{
-	    if (found_name != NULL)
-		error (1, 0, "%s is ambiguous; could mean %s or %s",
-		       fname, dp->d_name, found_name);
-	    found_name = xstrdup (dp->d_name);
-	}
-    }
-    if (errno != 0)
-	error (1, errno, "cannot read directory %s", dir);
-    CVS_CLOSEDIR (dirp);
-
-    if (found_name == NULL)
-    {
-	*fp = NULL;
-	retval = ENOENT;
-    }
-    else
-    {
-	char *p;
-
-	/* Copy the found name back into DIR.  We are assuming that
-	   found_name is the same length as fname, which is true as
-	   long as the above code is just ignoring case and not other
-	   aspects of filename syntax.  */
-	p = dir + strlen (dir);
-	*p++ = '/';
-	strcpy (p, found_name);
-	*fp = fopen (dir, mode);
-	if (*fp == NULL)
-	    retval = errno;
-	else
-	    retval = 0;
-    }
-
-    if (pathp == NULL)
-	free (dir);
-    else if (retval != 0)
-	free (dir);
-    else
-	*pathp = dir;
-    free (found_name);
- out:
-    return retval;
-}
 #endif /* SERVER_SUPPORT */
