@@ -1,18 +1,9 @@
 /*
- * Copyright (C) 1994-2005 The Free Software Foundation, Inc.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
+ *    You may distribute under the terms of the GNU General Public License
+ *    as specified in the README file that comes with the CVS 1.0 kit.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- */
-
-/* **************** History of Users and Module ****************
+ * **************** History of Users and Module ****************
  *
  * LOGGING:  Append record to "${CVSROOT}/CVSROOTADM/CVSROOTADM_HISTORY".
  *
@@ -207,17 +198,17 @@ static struct hrec
 static long hrec_idx;
 
 
-static void fill_hrec PROTO((char *line, struct hrec * hr));
-static int accept_hrec PROTO((struct hrec * hr, struct hrec * lr));
-static int select_hrec PROTO((struct hrec * hr));
-static int sort_order PROTO((const PTR l, const PTR r));
-static int within PROTO((char *find, char *string));
-static void expand_modules PROTO((void));
-static void read_hrecs PROTO((char *fname));
-static void report_hrecs PROTO((void));
-static void save_file PROTO((char *dir, char *name, char *module));
-static void save_module PROTO((char *module));
-static void save_user PROTO((char *name));
+static void fill_hrec (char *line, struct hrec * hr);
+static int accept_hrec (struct hrec * hr, struct hrec * lr);
+static int select_hrec (struct hrec * hr);
+static int sort_order (const void *l, const void *r);
+static int within (char *find, char *string);
+static void expand_modules (void);
+static void read_hrecs (char *fname);
+static void report_hrecs (void);
+static void save_file (char *dir, char *name, char *module);
+static void save_module (char *module);
+static void save_user (char *name);
 
 #define USER_INCREMENT	2
 #define FILE_INCREMENT	128
@@ -245,7 +236,7 @@ static short tz_local;
 static time_t tz_seconds_east_of_GMT;
 static char *tz_name = "+0000";
 
-char *logHistory;
+char *logHistory = ALL_HISTORY_REC_TYPES;
 
 /* -r, -t, or -b options, malloc'd.  These are "" if the option in
    question is not specified or is overridden by another option.  The
@@ -327,9 +318,7 @@ static const char *const history_usg[] =
    - Always sort timestamp last.
 */
 static int
-sort_order (l, r)
-    const PTR l;
-    const PTR r;
+sort_order (const void *l, const void *r)
 {
     int i;
     const struct hrec *left = (const struct hrec *) l;
@@ -378,9 +367,7 @@ sort_order (l, r)
 }
 
 int
-history (argc, argv)
-    int argc;
-    char **argv;
+history (int argc, char **argv)
 {
     int i, c;
     char *fname;
@@ -451,7 +438,7 @@ history (argc, argv)
 		backto = xstrdup (optarg);
 		break;
 	    case 'f':			/* For specified file */
-		save_file (NULL, optarg, NULL);
+		save_file ("", optarg, (char *) NULL);
 		break;
 	    case 'm':			/* Full module report */
 		if (!module_report++) report_count++;
@@ -460,7 +447,7 @@ history (argc, argv)
 		save_module (optarg);
 		break;
 	    case 'p':			/* For specified directory */
-		save_file (optarg, NULL, NULL);
+		save_file (optarg, "", (char *) NULL);
 		break;
 	    case 'r':			/* Since specified Tag/Rev */
 		if (since_date || *since_tag || *backto)
@@ -543,7 +530,7 @@ history (argc, argv)
     argc -= optind;
     argv += optind;
     for (i = 0; i < argc; i++)
-	save_file (NULL, argv[i], NULL);
+	save_file ("", argv[i], (char *) NULL);
 
 
     /* ================ Now analyze the arguments a bit */
@@ -695,7 +682,7 @@ history (argc, argv)
     read_hrecs (fname);
     if(hrec_count>0)
     {
-	qsort ((PTR) hrec_head, hrec_count, 
+	qsort ((void *) hrec_head, hrec_count, 
 		sizeof (struct hrec), sort_order);
     }
     report_hrecs ();
@@ -707,16 +694,14 @@ history (argc, argv)
     free (backto);
     free (rec_types);
 
-    return (0);
+    return 0;
 }
 
+
+
 void
-history_write (type, update_dir, revs, name, repository)
-    int type;
-    const char *update_dir;
-    const char *revs;
-    const char *name;
-    const char *repository;
+history_write (int type, const char *update_dir, const char *revs,
+               const char *name, const char *repository)
 {
     char *fname;
     char *workdir;
@@ -733,10 +718,9 @@ history_write (type, update_dir, revs, name, repository)
 				 * readonlyfs.
 				 */
 	return;
-    if (strchr (logHistory, type) == NULL)
+    if ( strchr(logHistory, type) == NULL )	
 	return;
-    fname = xmalloc (strlen (current_parsed_root->directory)
-		     + sizeof (CVSROOTADM)
+    fname = xmalloc (strlen (current_parsed_root->directory) + sizeof (CVSROOTADM)
 		     + sizeof (CVSROOTADM_HISTORY) + 3);
     (void) sprintf (fname, "%s/%s/%s", current_parsed_root->directory,
 		    CVSROOTADM, CVSROOTADM_HISTORY);
@@ -757,16 +741,10 @@ history_write (type, update_dir, revs, name, repository)
 	goto out;
     }
 
-    if (trace)
-	fprintf (stderr, "%s-> fopen(%s,a)\n",
-		 CLIENT_SERVER_STR, fname);
+    TRACE ( 1, "fopen(%s,a)", fname );
+
     if (noexec)
 	goto out;
-
-    if (!history_lock (current_parsed_root->directory))
-	/* history_lock() will already have printed an error on failure.  */
-	goto out;
-
     fd = CVS_OPEN (fname, O_WRONLY | O_APPEND | OPEN_BINARY, 0666);
     if (fd < 0)
     {
@@ -802,13 +780,13 @@ history_write (type, update_dir, revs, name, repository)
 		char *homedir;
 
 		if (save_cwd (&cwd))
-		    error_exit ();
+		    exit (EXIT_FAILURE);
 
-		if (CVS_CHDIR (pwdir) < 0 || (homedir = xgetwd ()) == NULL)
+		if ( CVS_CHDIR (pwdir) < 0 || (homedir = xgetwd ()) == NULL)
 		    homedir = pwdir;
 
 		if (restore_cwd (&cwd, NULL))
-		    error_exit ();
+		    exit (EXIT_FAILURE);
 		free_cwd (&cwd);
 
 		i = strlen (homedir);
@@ -911,7 +889,6 @@ history_write (type, update_dir, revs, name, repository)
 	error (1, errno, "cannot close history file: %s", fname);
     free (workdir);
  out:
-    clear_history_lock ();
     free (fname);
 }
 
@@ -920,8 +897,7 @@ history_write (type, update_dir, revs, name, repository)
  *		username ("") matches any user.
  */
 static void
-save_user (name)
-    char *name;
+save_user (char *name)
 {
     if (user_count == user_max)
     {
@@ -949,10 +925,7 @@ save_user (name)
  *	- else it is matched against the file name.
  */
 static void
-save_file (dir, name, module)
-    char *dir;
-    char *name;
-    char *module;
+save_file (char *dir, char *name, char *module)
 {
     char *cp;
     struct file_list_str *fl;
@@ -968,9 +941,7 @@ save_file (dir, name, module)
 	file_list = xrealloc (file_list, xtimes (file_max, sizeof (*fl)));
     }
     fl = &file_list[file_count++];
-    fl->l_file = cp = xmalloc (dir ? strlen (dir) : 0
-			       + name ? strlen (name) : 0
-			       + 2);
+    fl->l_file = cp = xmalloc (strlen (dir) + strlen (name) + 2);
     fl->l_module = module;
 
     if (dir && *dir)
@@ -1001,8 +972,7 @@ save_file (dir, name, module)
 }
 
 static void
-save_module (module)
-    char *module;
+save_module (char *module)
 {
     if (mod_count == mod_max)
     {
@@ -1018,7 +988,7 @@ save_module (module)
 }
 
 static void
-expand_modules ()
+expand_modules (void)
 {
 }
 
@@ -1041,9 +1011,7 @@ expand_modules ()
 	} while (0)
 
 static void
-fill_hrec (line, hr)
-    char *line;
-    struct hrec *hr;
+fill_hrec (char *line, struct hrec *hr)
 {
     char *cp;
     int c;
@@ -1100,8 +1068,7 @@ fill_hrec (line, hr)
  * than the whole block, we're done. 
  */
 static void
-read_hrecs (fname)
-    char *fname;
+read_hrecs (char *fname)
 {
     unsigned char *cpstart, *cpend, *cp, *nl;
     char *hrline;
@@ -1208,8 +1175,7 @@ read_hrecs (fname)
 
 /* Utility program for determining whether "find" is inside "string" */
 static int
-within (find, string)
-    char *find, *string;
+within (char *find, char *string)
 {
     int c, len;
 
@@ -1235,8 +1201,7 @@ within (find, string)
  * this record should be remembered for printing.
  */
 static int
-select_hrec (hr)
-    struct hrec *hr;
+select_hrec (struct hrec *hr)
 {
     char **cpp, *cp, *cp2;
     struct file_list_str *fl;
@@ -1411,8 +1376,6 @@ select_hrec (hr)
 		    if (within (cp, cp2))
 		    {
 			hr->mod = fl->l_module;
-			if (cmpfile != NULL)
-			    free (cmpfile);
 			break;
 		    }
 		    if (cmpfile != NULL)
@@ -1444,7 +1407,7 @@ select_hrec (hr)
  * selections are more easily done after the qsort by "accept_hrec".
  */
 static void
-report_hrecs ()
+report_hrecs (void)
 {
     struct hrec *hr, *lr;
     struct tm *tm;
@@ -1596,8 +1559,7 @@ report_hrecs ()
 }
 
 static int
-accept_hrec (lr, hr)
-    struct hrec *hr, *lr;
+accept_hrec (struct hrec *lr, struct hrec *hr)
 {
     int ty;
 
