@@ -18,21 +18,20 @@
 
 #include "cvs.h"
 #include "savecwd.h"
-#include <assert.h>
 
-static char *get_comment PROTO((char *user));
-static int add_rev PROTO((char *message, RCSNode *rcs, char *vfile,
-			  char *vers));
-static int add_tags PROTO((RCSNode *rcs, char *vfile, char *vtag, int targc,
-		     char *targv[]));
-static int import_descend PROTO((char *message, char *vtag, int targc, char *targv[]));
-static int import_descend_dir PROTO((char *message, char *dir, char *vtag,
-			       int targc, char *targv[]));
-static int process_import_file PROTO((char *message, char *vfile, char *vtag,
-				int targc, char *targv[]));
-static int update_rcs_file PROTO((char *message, char *vfile, char *vtag, int targc,
-			    char *targv[], int inattic));
-static void add_log PROTO((int ch, char *fname));
+static char *get_comment (const char *user);
+static int add_rev (char *message, RCSNode *rcs, char *vfile,
+			  char *vers);
+static int add_tags (RCSNode *rcs, char *vfile, char *vtag, int targc,
+		     char *targv[]);
+static int import_descend (char *message, char *vtag, int targc, char *targv[]);
+static int import_descend_dir (char *message, char *dir, char *vtag,
+			       int targc, char *targv[]);
+static int process_import_file (char *message, char *vfile, char *vtag,
+				int targc, char *targv[]);
+static int update_rcs_file (char *message, char *vfile, char *vtag, int targc,
+			    char *targv[], int inattic);
+static void add_log (int ch, char *fname);
 
 static int repos_len;
 static char *vhead;
@@ -58,9 +57,7 @@ static const char *const import_usage[] =
 };
 
 int
-import (argc, argv)
-    int argc;
-    char **argv;
+import (int argc, char **argv)
 {
     char *message = NULL;
     char *tmpfile;
@@ -91,7 +88,7 @@ import (argc, argv)
 #endif
 		    error (1, 0,
 			   "-q or -Q must be specified before \"%s\"",
-			   command_name);
+			   cvs_cmd_name);
 		break;
 	    case 'd':
 #ifdef SERVER_SUPPORT
@@ -157,6 +154,21 @@ import (argc, argv)
 	use_file_modtime = 1;
 #endif
 
+    /* Don't allow "CVS" as any directory in module path.
+     *
+     * Could abstract this to valid_module_path, but I don't think we'll need
+     * to call it from anywhere else.
+     */
+    if ((cp = strstr(argv[0], "CVS")) &&   /* path contains "CVS" AND ... */
+        ((cp == argv[0]) || ISSLASH(*(cp-1))) && /* /^CVS/ OR m#/CVS# AND ... */
+        ((*(cp+3) == '\0') || ISSLASH(*(cp+3))) /* /CVS$/ OR m#CVS/# */
+       )
+    {
+        error (0, 0,
+               "The word `CVS' is reserved by CVS and may not be used");
+        error (1, 0, "as a directory in a path or as a file name.");
+    }
+
     for (i = 1; i < argc; i++)		/* check the tags for validity */
     {
 	int j;
@@ -168,8 +180,7 @@ import (argc, argv)
     }
 
     /* XXX - this should be a module, not just a pathname */
-    if (! isabsolute (argv[0])
-	&& pathname_levels (argv[0]) == 0)
+    if (!isabsolute (argv[0]) && pathname_levels (argv[0]) == 0)
     {
 	if (current_parsed_root == NULL)
 	{
@@ -216,7 +227,11 @@ import (argc, argv)
     }
 #endif
 
-    if (use_editor)
+    if (
+#ifdef SERVER_SUPPORT
+        !server_active &&
+#endif
+        use_editor)
     {
 	do_editor ((char *) NULL, &message,
 #ifdef CLIENT_SUPPORT
@@ -247,8 +262,7 @@ import (argc, argv)
 
 	if (vbranch[0] != '\0')
 	    option_with_arg ("-b", vbranch);
-	if (message)
-	    option_with_arg ("-m", message);
+	option_with_arg ("-m", message ? message : "");
 	if (keyword_opt != NULL)
 	    option_with_arg ("-k", keyword_opt);
 	/* The only ignore processing which takes place on the server side
@@ -376,7 +390,7 @@ import (argc, argv)
     li->type = T_TITLE;
     li->tag = xstrdup (vbranch);
     li->rev_old = li->rev_new = NULL;
-    p->data = (char *) li;
+    p->data = li;
     (void) addnode (ulist, p);
     Update_Logfile (repository, message, logfp, ulist);
     dellist (&ulist);
@@ -402,11 +416,7 @@ import (argc, argv)
    Returns 0 for success, or >0 on error (in which case a message
    will have been printed).  */
 static int
-import_descend (message, vtag, targc, targv)
-    char *message;
-    char *vtag;
-    int targc;
-    char *targv[];
+import_descend (char *message, char *vtag, int targc, char **targv)
 {
     DIR *dirp;
     struct dirent *dp;
@@ -519,12 +529,7 @@ import_descend (message, vtag, targc, targv)
  * Process the argument import file.
  */
 static int
-process_import_file (message, vfile, vtag, targc, targv)
-    char *message;
-    char *vfile;
-    char *vtag;
-    int targc;
-    char *targv[];
+process_import_file (char *message, char *vfile, char *vtag, int targc, char **targv)
 {
     char *rcs;
     int inattic = 0;
@@ -570,7 +575,8 @@ process_import_file (message, vfile, vtag, targc, targv)
 		node = findnode_fn (entries, vfile);
 		if (node != NULL)
 		{
-		    Entnode *entdata = (Entnode *) node->data;
+		    Entnode *entdata = node->data;
+
 		    if (entdata->type == ENT_FILE)
 		    {
 			assert (entdata->options[0] == '-'
@@ -607,13 +613,7 @@ process_import_file (message, vfile, vtag, targc, targv)
  * (possibly already existing) vendor branch.
  */
 static int
-update_rcs_file (message, vfile, vtag, targc, targv, inattic)
-    char *message;
-    char *vfile;
-    char *vtag;
-    int targc;
-    char *targv[];
-    int inattic;
+update_rcs_file (char *message, char *vfile, char *vtag, int targc, char **targv, int inattic)
 {
     Vers_TS *vers;
     int letter;
@@ -650,7 +650,8 @@ update_rcs_file (message, vfile, vtag, targc, targv, inattic)
            not NULL?  */
 	expand = vers->srcfile->expand != NULL &&
 			vers->srcfile->expand[0] == 'b' ? "-kb" : "-ko";
-	different = RCS_cmp_file (vers->srcfile, vers->vn_rcs, expand, vfile);
+	different = RCS_cmp_file( vers->srcfile, vers->vn_rcs, (char **)NULL,
+	                          (char *)NULL, expand, vfile );
 	if (tocvsPath)
 	    if (unlink_file_dir (tocvsPath) < 0)
 		error (0, errno, "cannot remove %s", tocvsPath);
@@ -699,11 +700,7 @@ update_rcs_file (message, vfile, vtag, targc, targv, inattic)
  * Add the revision to the vendor branch
  */
 static int
-add_rev (message, rcs, vfile, vers)
-    char *message;
-    RCSNode *rcs;
-    char *vfile;
-    char *vers;
+add_rev (char *message, RCSNode *rcs, char *vfile, char *vers)
 {
     int locked, status, ierrno;
     char *tocvsPath;
@@ -763,12 +760,7 @@ add_rev (message, rcs, vfile, vers)
  * 1.1.1.2, ...).
  */
 static int
-add_tags (rcs, vfile, vtag, targc, targv)
-    RCSNode *rcs;
-    char *vfile;
-    char *vtag;
-    int targc;
-    char *targv[];
+add_tags (RCSNode *rcs, char *vfile, char *vtag, int targc, char **targv)
 {
     int i, ierrno;
     Vers_TS *vers;
@@ -927,9 +919,10 @@ static const struct compair comtable[] =
 /* must always be last		 */
 };
 
+
+
 static char *
-get_comment (user)
-    char *user;
+get_comment (const char *user)
 {
     char *cp, *suffix;
     char *suffix_path;
@@ -974,50 +967,44 @@ get_comment (user)
 }
 
 /* Create a new RCS file from scratch.
-
-   This probably should be moved to rcs.c now that it is called from
-   places outside import.c.
-
-   Return value is 0 for success, or nonzero for failure (in which
-   case an error message will have already been printed).  */
+ *
+ * This probably should be moved to rcs.c now that it is called from
+ * places outside import.c.
+ *
+ * INPUTS
+ *   message    Log message for the addition.  Not used if add_vhead == NULL.
+ *   rcs        Filename of the RCS file to create.
+ *   user       Filename of the file to serve as the contents of the initial
+ *              revision.  Even if add_vhead is NULL, we use this to determine
+ *              the modes to give the new RCS file.
+ *   add_vhead  Revision number of head that we are adding.  Normally 1.1 but
+ *              could be another revision as long as ADD_VBRANCH is a branch
+ *              from it.  If NULL, then just add an empty file without any
+ *              revisions (similar to the one created by "rcs -i").
+ *   key_opt    Keyword expansion mode, e.g., "b" for binary.  NULL means the
+ *              default behavior.
+ *   add_vbranch
+ *              Vendor branch to import to, or NULL if none.  If non-NULL, then
+ *              vtag should also be non-NULL.
+ *   vtag
+ *   targc      Number of elements in TARGV.
+ *   targv      The list of tags to attached to this imported revision.
+ *   desctext   If non-NULL, description for the file.  If NULL, the
+ *              description will be empty.
+ *   desclen    The number of bytes in desctext.
+ *   add_logfp  Write errors to here as well as via error (), or NULL if we
+ *              should use only error ().
+ *
+ * RETURNS
+ *   Return value is 0 for success, or nonzero for failure (in which
+ *   case an error message will have already been printed).
+ */
 int
-add_rcs_file (message, rcs, user, add_vhead, key_opt,
-	      add_vbranch, vtag, targc, targv,
-	      desctext, desclen, add_logfp)
-    /* Log message for the addition.  Not used if add_vhead == NULL.  */
-    char *message;
-    /* Filename of the RCS file to create.  */
-    char *rcs;
-    /* Filename of the file to serve as the contents of the initial
-       revision.  Even if add_vhead is NULL, we use this to determine
-       the modes to give the new RCS file.  */
-    char *user;
-
-    /* Revision number of head that we are adding.  Normally 1.1 but
-       could be another revision as long as ADD_VBRANCH is a branch
-       from it.  If NULL, then just add an empty file without any
-       revisions (similar to the one created by "rcs -i").  */
-    char *add_vhead;
-
-    /* Keyword expansion mode, e.g., "b" for binary.  NULL means the
-       default behavior.  */
-    char *key_opt;
-
-    /* Vendor branch to import to, or NULL if none.  If non-NULL, then
-       vtag should also be non-NULL.  */
-    char *add_vbranch;
-    char *vtag;
-    int targc;
-    char *targv[];
-
-    /* If non-NULL, description for the file.  If NULL, the description
-       will be empty.  */
-    char *desctext;
-    size_t desclen;
-
-    /* Write errors to here as well as via error (), or NULL if we should
-       use only error ().  */
-    FILE *add_logfp;
+add_rcs_file (const char *message, const char *rcs, const char *user,
+              const char *add_vhead, const char *key_opt,
+              const char *add_vbranch, const char *vtag, int targc,
+              char **targv, const char *desctext, size_t desclen,
+              FILE *add_logfp)
 {
     FILE *fprcs, *fpuser;
     struct stat sb;
@@ -1028,8 +1015,7 @@ add_rcs_file (message, rcs, user, add_vhead, key_opt,
     int i, ierrno, err = 0;
     mode_t mode;
     char *tocvsPath;
-    char *userfile;
-    char *local_opt = key_opt;
+    const char *userfile;
     char *free_opt = NULL;
     mode_t file_type;
 
@@ -1043,11 +1029,11 @@ add_rcs_file (message, rcs, user, add_vhead, key_opt,
        or the other.  Before making a change of this sort, should think
        about what is best, document it (in cvs.texinfo and NEWS), &c.  */
 
-    if (local_opt == NULL)
+    if (key_opt == NULL)
     {
 	if (wrap_name_has (user, WRAP_RCSOPTION))
 	{
-	    local_opt = free_opt = wrap_rcsoption (user, 0);
+	    key_opt = free_opt = wrap_rcsoption (user, 0);
 	}
     }
 
@@ -1084,7 +1070,7 @@ add_rcs_file (message, rcs, user, add_vhead, key_opt,
     if (!preserve_perms || file_type == S_IFREG)
     {
 	fpuser = CVS_FOPEN (userfile,
-			    ((local_opt != NULL && strcmp (local_opt, "b") == 0)
+			    ((key_opt != NULL && strcmp (key_opt, "b") == 0)
 			     ? "rb"
 			     : "r")
 	    );
@@ -1154,9 +1140,9 @@ add_rcs_file (message, rcs, user, add_vhead, key_opt,
 	goto write_error;
     }
 
-    if (local_opt != NULL && strcmp (local_opt, "kv") != 0)
+    if (key_opt != NULL && strcmp (key_opt, "kv") != 0)
     {
-	if (fprintf (fprcs, "expand   @%s@;\012", local_opt) < 0)
+	if (fprintf (fprcs, "expand   @%s@;\012", key_opt) < 0)
 	{
 	    goto write_error;
 	}
@@ -1225,7 +1211,7 @@ add_rcs_file (message, rcs, user, add_vhead, key_opt,
 		    case S_IFREG: break;
 		    case S_IFCHR:
 		    case S_IFBLK:
-#ifdef HAVE_ST_RDEV
+#ifdef HAVE_STRUCT_STAT_ST_RDEV
 			if (fprintf (fprcs, "special\t%s %lu;\012",
 				     (file_type == S_IFCHR
 				      ? "character"
@@ -1282,7 +1268,7 @@ userfile);
 			case S_IFREG: break;
 			case S_IFCHR:
 			case S_IFBLK:
-#ifdef HAVE_ST_RDEV
+#ifdef HAVE_STRUCT_STAT_ST_RDEV
 			    if (fprintf (fprcs, "special\t%s %lu;\012",
 					 (file_type == S_IFCHR
 					  ? "character"
@@ -1453,26 +1439,22 @@ read_error:
     return (err + 1);
 }
 
+
+
 /*
  * Write SIZE bytes at BUF to FP, expanding @ signs into double @
  * signs.  If an error occurs, return a negative value and set errno
  * to indicate the error.  If not, return a nonnegative value.
  */
 int
-expand_at_signs (buf, size, fp)
-    char *buf;
-    off_t size;
-    FILE *fp;
+expand_at_signs (const char *buf, off_t size, FILE *fp)
 {
-    register char *cp, *next;
+    register const char *cp, *next;
 
     cp = buf;
     while ((next = memchr (cp, '@', size)) != NULL)
     {
-	int len;
-
-	++next;
-	len = next - cp;
+	size_t len = ++next - cp;
 	if (fwrite (cp, 1, len, fp) != len)
 	    return EOF;
 	if (putc ('@', fp) == EOF)
@@ -1491,9 +1473,7 @@ expand_at_signs (buf, size, fp)
  * Write an update message to (potentially) the screen and the log file.
  */
 static void
-add_log (ch, fname)
-    int ch;
-    char *fname;
+add_log (int ch, char *fname)
 {
     if (!really_quiet)			/* write to terminal */
     {
@@ -1532,12 +1512,7 @@ add_log (ch, fname)
  * Note that we do not follow symbolic links here, which is a feature!
  */
 static int
-import_descend_dir (message, dir, vtag, targc, targv)
-    char *message;
-    char *dir;
-    char *vtag;
-    int targc;
-    char *targv[];
+import_descend_dir (char *message, char *dir, char *vtag, int targc, char **targv)
 {
     struct saved_cwd cwd;
     char *cp;
@@ -1622,7 +1597,7 @@ import_descend_dir (message, dir, vtag, targc, targv)
     else
 	repository[0] = '\0';
     if (restore_cwd (&cwd, NULL))
-	error_exit ();
+	exit (EXIT_FAILURE);
     free_cwd (&cwd);
     return (err);
 }
