@@ -112,34 +112,33 @@ struct log_data_and_rcs
     RCSNode *rcs;
 };
 
-static int rlog_proc PROTO((int argc, char **argv, char *xwhere,
-			    char *mwhere, char *mfile, int shorten,
-			    int local_specified, char *mname, char *msg));
-static Dtype log_dirproc PROTO ((void *callerdat, char *dir,
-				 char *repository, char *update_dir,
-				 List *entries));
-static int log_fileproc PROTO ((void *callerdat, struct file_info *finfo));
-static struct option_revlist *log_parse_revlist PROTO ((const char *));
-static void log_parse_date PROTO ((struct log_data *, const char *));
-static void log_parse_list PROTO ((List **, const char *));
-static struct revlist *log_expand_revlist PROTO ((RCSNode *,
-						  struct option_revlist *,
-						  int));
-static void log_free_revlist PROTO ((struct revlist *));
-static int log_version_requested PROTO ((struct log_data *, struct revlist *,
-					 RCSNode *, RCSVers *));
-static int log_symbol PROTO ((Node *, void *));
-static int log_count PROTO ((Node *, void *));
-static int log_fix_singledate PROTO ((Node *, void *));
-static int log_count_print PROTO ((Node *, void *));
-static void log_tree PROTO ((struct log_data *, struct revlist *,
-			     RCSNode *, const char *));
-static void log_abranch PROTO ((struct log_data *, struct revlist *,
-				RCSNode *, const char *));
-static void log_version PROTO ((struct log_data *, struct revlist *,
-				RCSNode *, RCSVers *, int));
-static int log_branch PROTO ((Node *, void *));
-static int version_compare PROTO ((const char *, const char *, int));
+static int rlog_proc (int argc, char **argv, char *xwhere,
+                      char *mwhere, char *mfile, int shorten,
+                      int local_specified, char *mname, char *msg);
+static Dtype log_dirproc (void *callerdat, const char *dir,
+                          const char *repository, const char *update_dir,
+                          List *entries);
+static int log_fileproc (void *callerdat, struct file_info *finfo);
+static struct option_revlist *log_parse_revlist (const char *);
+static void log_parse_date (struct log_data *, const char *);
+static void log_parse_list (List **, const char *);
+static struct revlist *log_expand_revlist (RCSNode *, struct option_revlist *,
+                                           int);
+static void log_free_revlist (struct revlist *);
+static int log_version_requested (struct log_data *, struct revlist *,
+					 RCSNode *, RCSVers *);
+static int log_symbol (Node *, void *);
+static int log_count (Node *, void *);
+static int log_fix_singledate (Node *, void *);
+static int log_count_print (Node *, void *);
+static void log_tree (struct log_data *, struct revlist *,
+			     RCSNode *, const char *);
+static void log_abranch (struct log_data *, struct revlist *,
+				RCSNode *, const char *);
+static void log_version (struct log_data *, struct revlist *,
+				RCSNode *, RCSVers *, int);
+static int log_branch (Node *, void *);
+static int version_compare (const char *, const char *, int);
 
 static struct log_data log_data;
 static int is_rlog;
@@ -175,15 +174,13 @@ static const char *const log_usage[] =
 
 #ifdef CLIENT_SUPPORT
 
-/* Helper function for send_arg_list.  */
-static int send_one PROTO ((Node *, void *));
 
+
+/* Helper function for send_arg_list.  */
 static int
-send_one (node, closure)
-    Node *node;
-    void *closure;
+send_one (Node *node, void *closure)
 {
-    char *option = (char *) closure;
+    char *option = closure;
 
     send_to_server ("Argument ", 0);
     send_to_server (option, 0);
@@ -198,33 +195,31 @@ send_one (node, closure)
     return 0;
 }
 
+
+
 /* For each element in ARG, send an argument consisting of OPTION
    concatenated with that element.  */
-static void send_arg_list PROTO ((char *, List *));
-
 static void
-send_arg_list (option, arg)
-    char *option;
-    List *arg;
+send_arg_list (char *option, List *arg)
 {
     if (arg == NULL)
 	return;
-    walklist (arg, send_one, (void *)option);
+    walklist (arg, send_one, option);
 }
 
 #endif
 
+
+
 int
-cvslog (argc, argv)
-    int argc;
-    char **argv;
+cvslog (int argc, char **argv)
 {
     int c;
     int err = 0;
     int local = 0;
     struct option_revlist **prl;
 
-    is_rlog = (strcmp (command_name, "rlog") == 0);
+    is_rlog = (strcmp (cvs_cmd_name, "rlog") == 0);
 
     if (argc == -1)
 	usage (log_usage);
@@ -383,6 +378,7 @@ cvslog (argc, argv)
 	dellist (&log_data.statelist);
 	send_arg_list ("-w", log_data.authorlist);
 	dellist (&log_data.authorlist);
+	send_arg ("--");
 
 	if (is_rlog)
 	{
@@ -414,16 +410,15 @@ cvslog (argc, argv)
 	db = open_module ();
 	for (i = 0; i < argc; i++)
 	{
-	    err += do_module (db, argv[i], MISC, "Logging", rlog_proc,
-			     (char *) NULL, 0, local, 0, 0, (char *) NULL);
+             err += do_module (db, argv[i], MISC, "Logging", rlog_proc,
+                               NULL, 0, local, 0, 0, NULL);
 	}
 	close_module (db);
     }
     else
     {
-	err = rlog_proc (argc + 1, argv - 1, (char *) NULL,
-			 (char *) NULL, (char *) NULL, 0, local, (char *) NULL,
-			 (char *) NULL);
+        err = rlog_proc (argc + 1, argv - 1, NULL, NULL, NULL, 0, local, NULL,
+                         NULL);
     }
 
     while (log_data.revlist)
@@ -459,65 +454,65 @@ cvslog (argc, argv)
     dellist (&log_data.statelist);
     dellist (&log_data.authorlist);
 
-    return (err);
+    return err;
 }
 
 
+
 static int
-rlog_proc (argc, argv, xwhere, mwhere, mfile, shorten, local, mname, msg)
-    int argc;
-    char **argv;
-    char *xwhere;
-    char *mwhere;
-    char *mfile;
-    int shorten;
-    int local;
-    char *mname;
-    char *msg;
+rlog_proc (int argc, char **argv, char *xwhere, char *mwhere, char *mfile,
+           int shorten, int local, char *mname, char *msg)
 {
     /* Begin section which is identical to patch_proc--should this
        be abstracted out somehow?  */
     char *myargv[2];
     int err = 0;
     int which;
-    char *repository;
+    char *repository = NULL;
     char *where;
 
     if (is_rlog)
     {
-	repository = xmalloc (strlen (current_parsed_root->directory) + strlen (argv[0])
+	repository = xmalloc (strlen (current_parsed_root->directory)
+                              + strlen (argv[0])
 			      + (mfile == NULL ? 0 : strlen (mfile) + 1) + 2);
-	(void) sprintf (repository, "%s/%s", current_parsed_root->directory, argv[0]);
-	where = xmalloc (strlen (argv[0]) + (mfile == NULL ? 0 : strlen (mfile) + 1)
+	(void)sprintf (repository, "%s/%s",
+                       current_parsed_root->directory, argv[0]);
+	where = xmalloc (strlen (argv[0])
+                         + (mfile == NULL ? 0 : strlen (mfile) + 1)
 			 + 1);
-	(void) strcpy (where, argv[0]);
+	(void)strcpy (where, argv[0]);
 
-	/* if mfile isn't null, we need to set up to do only part of the module */
+	/* If mfile isn't null, we need to set up to do only part of theu
+         * module.
+         */
 	if (mfile != NULL)
 	{
 	    char *cp;
 	    char *path;
 
-	    /* if the portion of the module is a path, put the dir part on repos */
+	    /* If the portion of the module is a path, put the dir part on
+             * repos.
+             */
 	    if ((cp = strrchr (mfile, '/')) != NULL)
 	    {
 		*cp = '\0';
-		(void) strcat (repository, "/");
-		(void) strcat (repository, mfile);
-		(void) strcat (where, "/");
-		(void) strcat (where, mfile);
+		(void)strcat (repository, "/");
+		(void)strcat (repository, mfile);
+		(void)strcat (where, "/");
+		(void)strcat (where, mfile);
 		mfile = cp + 1;
 	    }
 
 	    /* take care of the rest */
 	    path = xmalloc (strlen (repository) + strlen (mfile) + 5);
-	    (void) sprintf (path, "%s/%s", repository, mfile);
+	    (void)sprintf (path, "%s/%s", repository, mfile);
 	    if (isdir (path))
 	    {
 		/* directory means repository gets the dir tacked on */
-		(void) strcpy (repository, path);
-		(void) strcat (where, "/");
-		(void) strcat (where, mfile);
+		(void)strcpy (repository, path);
+		(void)strcat (where, "/");
+		(void)strcat (where, mfile);
 	    }
 	    else
 	    {
@@ -530,38 +525,42 @@ rlog_proc (argc, argv, xwhere, mwhere, mfile, shorten, local, mname, msg)
 	}
 
 	/* cd to the starting repository */
-	if ( CVS_CHDIR (repository) < 0)
+	if (CVS_CHDIR (repository) < 0)
 	{
 	    error (0, errno, "cannot chdir to %s", repository);
 	    free (repository);
-	    return (1);
+	    free (where);
+	    return 1;
 	}
-	free (repository);
 	/* End section which is identical to patch_proc.  */
 
 	which = W_REPOS | W_ATTIC;
     }
     else
     {
+        repository = NULL;
         where = NULL;
         which = W_LOCAL | W_REPOS | W_ATTIC;
     }
 
-    err = start_recursion (log_fileproc, (FILESDONEPROC) NULL, log_dirproc,
-			   (DIRLEAVEPROC) NULL, (void *) &log_data,
-			   argc - 1, argv + 1, local, which, 0, 1,
-			   where, 1);
+    err = start_recursion (log_fileproc, NULL, log_dirproc,
+			   NULL, &log_data,
+			   argc - 1, argv + 1, local, which, 0, CVS_LOCK_READ,
+			   where, 1, repository);
+
+    if (!(which & W_LOCAL)) free (repository);
+    if (where) free (where);
+
     return err;
 }
 
 
+
 /*
  * Parse a revision list specification.
  */
-
 static struct option_revlist *
-log_parse_revlist (argstring)
-    const char *argstring;
+log_parse_revlist (const char *argstring)
 {
     char *orig_copy, *copy;
     struct option_revlist *ret, **pr;
@@ -588,7 +587,7 @@ log_parse_revlist (argstring)
 	if (comma != NULL)
 	    *comma++ = '\0';
 
-	r = (struct option_revlist *) xmalloc (sizeof *r);
+	r = xmalloc (sizeof *r);
 	r->next = NULL;
 	r->first = copy;
 	r->branchhead = 0;
@@ -631,13 +630,13 @@ log_parse_revlist (argstring)
     return ret;
 }
 
+
+
 /*
  * Parse a date specification.
  */
 static void
-log_parse_date (log_data, argstring)
-    struct log_data *log_data;
-    const char *argstring;
+log_parse_date (struct log_data *log_data, const char *argstring)
 {
     char *orig_copy, *copy;
 
@@ -650,7 +649,7 @@ log_parse_date (log_data, argstring)
 	struct datelist *nd, **pd;
 	char *cpend, *cp, *ds, *de;
 
-	nd = (struct datelist *) xmalloc (sizeof *nd);
+	nd = xmalloc (sizeof *nd);
 
 	cpend = strchr (copy, ';');
 	if (cpend != NULL)
@@ -728,13 +727,13 @@ log_parse_date (log_data, argstring)
     free (orig_copy);
 }
 
+
+
 /*
  * Parse a comma separated list of items, and add each one to *PLIST.
  */
 static void
-log_parse_list (plist, argstring)
-    List **plist;
-    const char *argstring;
+log_parse_list (List **plist, const char *argstring)
 {
     while (1)
     {
@@ -753,7 +752,7 @@ log_parse_list (plist, argstring)
 	    len = cp - argstring;
 	    p->key = xmalloc (len + 1);
 	    strncpy (p->key, argstring, len);
-	    p->key[len + 1] = '\0';
+	    p->key[len] = '\0';
 	}
 
 	if (*plist == NULL)
@@ -768,12 +767,10 @@ log_parse_list (plist, argstring)
     }
 }
 
-static int printlock_proc PROTO ((Node *, void *));
+
 
 static int
-printlock_proc (lock, foo)
-    Node *lock;
-    void *foo;
+printlock_proc (Node *lock, void *foo)
 {
     cvs_output ("\n\t", 2);
     cvs_output (lock->data, 0);
@@ -782,20 +779,20 @@ printlock_proc (lock, foo)
     return 0;
 }
 
+
+
 /*
  * Do an rlog on a file
  */
 static int
-log_fileproc (callerdat, finfo)
-    void *callerdat;
-    struct file_info *finfo;
+log_fileproc (void *callerdat, struct file_info *finfo)
 {
-    struct log_data *log_data = (struct log_data *) callerdat;
+    struct log_data *log_data = callerdat;
     Node *p;
     int selrev = -1;
     RCSNode *rcsfile;
     char buf[50];
-    struct revlist *revlist;
+    struct revlist *revlist = NULL;
     struct log_data_and_rcs log_data_and_rcs;
 
     if ((rcsfile = finfo->rcs) == NULL)
@@ -804,22 +801,21 @@ log_fileproc (callerdat, finfo)
 	p = findnode (finfo->entries, finfo->file);
 	if (p != NULL)
 	{
-	    Entnode *e;
-	    
-	    e = (Entnode *) p->data;
+	    Entnode *e = p->data;
+
 	    if (e->version[0] == '0' && e->version[1] == '\0')
 	    {
 		if (!really_quiet)
 		    error (0, 0, "%s has been added, but not committed",
 			   finfo->file);
-		return(0);
+		return 0;
 	    }
 	}
 	
 	if (!really_quiet)
 	    error (0, 0, "nothing known about %s", finfo->file);
 	
-	return (1);
+	return 1;
     }
 
     if (log_data->sup_header || !log_data->nameonly)
@@ -832,7 +828,8 @@ log_fileproc (callerdat, finfo)
 	   revisions.  */
 	revlist = log_expand_revlist (rcsfile, log_data->revlist,
 				      log_data->default_branch);
-	if (log_data->sup_header || (!log_data->header && !log_data->long_header))
+	if (log_data->sup_header
+            || (!log_data->header && !log_data->long_header))
 	{
 	    log_data_and_rcs.log_data = log_data;
 	    log_data_and_rcs.revlist = revlist;
@@ -845,11 +842,15 @@ log_fileproc (callerdat, finfo)
 	       start date for each specific revision.  */
 	    if (log_data->singledatelist != NULL)
 		walklist (rcsfile->versions, log_fix_singledate,
-			  (void *) &log_data_and_rcs);
+			  &log_data_and_rcs);
 
 	    selrev = walklist (rcsfile->versions, log_count_print,
-			       (void *) &log_data_and_rcs);
-	    if (log_data->sup_header && selrev == 0) return 0;
+			       &log_data_and_rcs);
+	    if (log_data->sup_header && selrev == 0)
+	    {
+		log_free_revlist (revlist);
+		return 0;
+	    }
 	}
 
     }
@@ -858,6 +859,7 @@ log_fileproc (callerdat, finfo)
     {
 	cvs_output (rcsfile->path, 0);
 	cvs_output ("\n", 1);
+	log_free_revlist (revlist);
 	return 0;
     }
 
@@ -914,16 +916,16 @@ log_fileproc (callerdat, finfo)
 
 		cvs_output ("\n\t", 2);
 		cp2 = cp;
-		while (! isspace ((unsigned char) *cp2) && *cp2 != '\0')
+		while (!isspace ((unsigned char)*cp2) && *cp2 != '\0')
 		    ++cp2;
 		cvs_output (cp, cp2 - cp);
 		cp = cp2;
-		while (isspace ((unsigned char) *cp) && *cp != '\0')
+		while (isspace ((unsigned char)*cp) && *cp != '\0')
 		    ++cp;
 	}
     }
 
-    if (! log_data->notags)
+    if (!log_data->notags)
     {
 	List *syms;
 
@@ -951,14 +953,14 @@ log_fileproc (callerdat, finfo)
 
     cvs_output ("\n", 1);
 
-    if (! log_data->header || log_data->long_header)
+    if (!log_data->header || log_data->long_header)
     {
 	cvs_output ("description:\n", 0);
 	if (rcsfile->desc != NULL)
 	    cvs_output (rcsfile->desc, 0);
     }
 
-    if (! log_data->header && ! log_data->long_header && rcsfile->head != NULL)
+    if (!log_data->header && ! log_data->long_header && rcsfile->head != NULL)
     {
 	p = findnode (rcsfile->versions, rcsfile->head);
 	if (p == NULL)
@@ -966,9 +968,8 @@ log_fileproc (callerdat, finfo)
 		   finfo->fullname);
 	while (p != NULL)
 	{
-	    RCSVers *vers;
+	    RCSVers *vers = p->data;
 
-	    vers = (RCSVers *) p->data;
 	    log_version (log_data, revlist, rcsfile, vers, 1);
 	    if (vers->next == NULL)
 		p = NULL;
@@ -1008,15 +1009,15 @@ log_fileproc (callerdat, finfo)
     return 0;
 }
 
+
+
 /*
  * Fix up a revision list in order to compare it against versions.
  * Expand any symbolic revisions.
  */
 static struct revlist *
-log_expand_revlist (rcs, revlist, default_branch)
-    RCSNode *rcs;
-    struct option_revlist *revlist;
-    int default_branch;
+log_expand_revlist (RCSNode *rcs, struct option_revlist *revlist,
+                    int default_branch)
 {
     struct option_revlist *r;
     struct revlist *ret, **pr;
@@ -1027,7 +1028,7 @@ log_expand_revlist (rcs, revlist, default_branch)
     {
 	struct revlist *nr;
 
-	nr = (struct revlist *) xmalloc (sizeof *nr);
+	nr = xmalloc (sizeof *nr);
 	nr->inclusive = r->inclusive;
 
 	if (r->first == NULL && r->last == NULL)
@@ -1056,7 +1057,7 @@ log_expand_revlist (rcs, revlist, default_branch)
 		    free (branch);
 		}
 	    }
-	    if (nr->first == NULL)
+	    if (nr->first == NULL && !really_quiet)
 	    {
 		error (0, 0, "warning: no branch `%s' in `%s'",
 		       r->first, rcs->path);
@@ -1078,8 +1079,8 @@ log_expand_revlist (rcs, revlist, default_branch)
 		if (RCS_nodeisbranch (rcs, r->first))
 		    nr->first = RCS_whatbranch (rcs, r->first);
 		else
-		    nr->first = RCS_gettag (rcs, r->first, 1, (int *) NULL);
-		if (nr->first == NULL)
+		    nr->first = RCS_gettag (rcs, r->first, 1, NULL);
+		if (nr->first == NULL && !really_quiet)
 		{
 		    error (0, 0, "warning: no revision `%s' in `%s'",
 			   r->first, rcs->path);
@@ -1096,8 +1097,8 @@ log_expand_revlist (rcs, revlist, default_branch)
 		if (RCS_nodeisbranch (rcs, r->last))
 		    nr->last = RCS_whatbranch (rcs, r->last);
 		else
-		    nr->last = RCS_gettag (rcs, r->last, 1, (int *) NULL);
-		if (nr->last == NULL)
+		    nr->last = RCS_gettag (rcs, r->last, 1, NULL);
+		if (nr->last == NULL && !really_quiet)
 		{
 		    error (0, 0, "warning: no revision `%s' in `%s'",
 			   r->last, rcs->path);
@@ -1119,7 +1120,7 @@ log_expand_revlist (rcs, revlist, default_branch)
 
 		    nr->first = xstrdup (nr->last);
 		    cp = strrchr (nr->first, '.');
-		    strcpy (cp, ".0");
+		    strcpy (cp + 1, "0");
 		}
 	    }
 	    else if (r->last == NULL && nr->first != NULL)
@@ -1136,13 +1137,30 @@ log_expand_revlist (rcs, revlist, default_branch)
 		    *cp = '\0';
 		}
 	    }
-	    else if (nr->first != NULL && nr->last != NULL)
+	    else if (nr->first == NULL || nr->last == NULL)
+		nr->fields = 0;
+	    else if (strcmp (nr->first, nr->last) == 0)
+		nr->fields = numdots (nr->last) + 1;
+	    else
 	    {
-		nr->fields = numdots (nr->first) + 1;
-		if (nr->fields != numdots (nr->last) + 1
-		    || (nr->fields > 2
-			&& version_compare (nr->first, nr->last,
-					    nr->fields - 1) != 0))
+		int ord;
+		int dots1 = numdots (nr->first);
+		int dots2 = numdots (nr->last);
+		if (dots1 > dots2 || (dots1 == dots2 &&
+		    version_compare (nr->first, nr->last, dots1 + 1) > 0))
+		{
+		    char *tmp = nr->first;
+		    nr->first = nr->last;
+		    nr->last = tmp;
+		    nr->fields = dots2 + 1;
+		    dots2 = dots1;
+		    dots1 = nr->fields - 1;
+		}
+		else
+		    nr->fields = dots1 + 1;
+		dots1 += (nr->fields & 1);
+		ord = version_compare (nr->first, nr->last, dots1);
+		if (ord > 0 || (nr->fields > 2 && ord < 0))
 		{
 		    error (0, 0,
 			   "invalid branch or revision pair %s:%s in `%s'",
@@ -1153,17 +1171,46 @@ log_expand_revlist (rcs, revlist, default_branch)
 		    nr->last = NULL;
 		    nr->fields = 0;
 		}
-		else if (version_compare (nr->first, nr->last, nr->fields) > 0)
+		else
 		{
-		    char *tmp;
+		    if (nr->fields <= dots2 && (nr->fields & 1))
+		    {
+			char *p = xmalloc (strlen (nr->first) + 3);
+			strcpy (p, nr->first);
+			strcat (p, ".0");
+			free (nr->first);
+			nr->first = p;
+			++nr->fields;
+		    }
+		    while (nr->fields <= dots2)
+		    {
+			char *p;
+			int i;
 
-		    tmp = nr->first;
-		    nr->first = nr->last;
-		    nr->last = tmp;
+			nr->next = NULL;
+			*pr = nr;
+			nr = xmalloc (sizeof *nr);
+			nr->inclusive = 1;
+			nr->first = xstrdup ((*pr)->last);
+			nr->last = xstrdup ((*pr)->last);
+			nr->fields = (*pr)->fields;
+			p = (*pr)->last;
+			for (i = 0; i < nr->fields; i++)
+			    p = strchr (p, '.') + 1;
+			p[-1] = '\0';
+			p = strchr (nr->first + (p - (*pr)->last), '.');
+			if (p != NULL)
+			{
+			    *++p = '0';
+			    *++p = '\0';
+			    nr->fields += 2;
+			}
+			else
+			    ++nr->fields;
+			pr = &(*pr)->next;
+		    }
 		}
 	    }
-	    else
-		nr->fields = 0;
 	}
 
 	nr->next = NULL;
@@ -1178,7 +1225,7 @@ log_expand_revlist (rcs, revlist, default_branch)
     {
 	struct revlist *nr;
 
-	nr = (struct revlist *) xmalloc (sizeof *nr);
+	nr = xmalloc (sizeof *nr);
 	if (rcs->branch != NULL)
 	    nr->first = xstrdup (rcs->branch);
 	else
@@ -1200,12 +1247,13 @@ log_expand_revlist (rcs, revlist, default_branch)
     return ret;
 }
 
+
+
 /*
  * Free a revlist created by log_expand_revlist.
  */
 static void
-log_free_revlist (revlist)
-    struct revlist *revlist;
+log_free_revlist (struct revlist *revlist)
 {
     struct revlist *r;
 
@@ -1224,16 +1272,15 @@ log_free_revlist (revlist)
     }
 }
 
+
+
 /*
  * Return nonzero if a revision should be printed, based on the
  * options provided.
  */
 static int
-log_version_requested (log_data, revlist, rcs, vnode)
-    struct log_data *log_data;
-    struct revlist *revlist;
-    RCSNode *rcs;
-    RCSVers *vnode;
+log_version_requested (struct log_data *log_data, struct revlist *revlist,
+                       RCSNode *rcs, RCSVers *vnode)
 {
     /* Handle the list of states from the -s option.  */
     if (log_data->statelist != NULL
@@ -1306,10 +1353,10 @@ log_version_requested (log_data, revlist, rcs, vnode)
 	vfields = numdots (v) + 1;
 	for (r = revlist; r != NULL; r = r->next)
 	{
-	    if (vfields == r->fields + (r->fields & 1) &&
-		(r->inclusive ? version_compare (v, r->first, r->fields) >= 0 :
-				version_compare (v, r->first, r->fields) > 0)
-		    && version_compare (v, r->last, r->fields) <= 0)
+            if (vfields == r->fields + (r->fields & 1) &&
+                (r->inclusive ? version_compare (v, r->first, r->fields) >= 0 :
+                                version_compare (v, r->first, r->fields) > 0)
+                && version_compare (v, r->last, r->fields) <= 0)
 	    {
 		return 1;
 	    }
@@ -1325,14 +1372,14 @@ log_version_requested (log_data, revlist, rcs, vnode)
     return 1;
 }
 
+
+
 /*
  * Output a single symbol.  This is called via walklist.
  */
 /*ARGSUSED*/
 static int
-log_symbol (p, closure)
-    Node *p;
-    void *closure;
+log_symbol (Node *p, void *closure)
 {
     cvs_output ("\n\t", 2);
     cvs_output (p->key, 0);
@@ -1341,28 +1388,28 @@ log_symbol (p, closure)
     return 0;
 }
 
+
+
 /*
  * Count the number of entries on a list.  This is called via walklist.
  */
 /*ARGSUSED*/
 static int
-log_count (p, closure)
-    Node *p;
-    void *closure;
+log_count (Node *p, void *closure)
 {
     return 1;
 }
 
+
+
 /*
  * Sort out a single date specification by narrowing down the date
  * until we find the specific selected revision.
  */
 static int
-log_fix_singledate (p, closure)
-    Node *p;
-    void *closure;
+log_fix_singledate (Node *p, void *closure)
 {
-    struct log_data_and_rcs *data = (struct log_data_and_rcs *) closure;
+    struct log_data_and_rcs *data = closure;
     Node *pv;
     RCSVers *vnode;
     struct datelist *holdsingle, *holddate;
@@ -1372,7 +1419,7 @@ log_fix_singledate (p, closure)
     if (pv == NULL)
 	error (1, 0, "missing version `%s' in RCS file `%s'",
 	       p->key, data->rcs->path);
-    vnode = (RCSVers *) pv->data;
+    vnode = pv->data;
 
     /* We are only interested if this revision passes any other tests.
        Temporarily clear log_data->singledatelist to avoid confusing
@@ -1412,15 +1459,15 @@ log_fix_singledate (p, closure)
     return 0;
 }
 
+
+
 /*
  * Count the number of revisions we are going to print.
  */
 static int
-log_count_print (p, closure)
-    Node *p;
-    void *closure;
+log_count_print (Node *p, void *closure)
 {
-    struct log_data_and_rcs *data = (struct log_data_and_rcs *) closure;
+    struct log_data_and_rcs *data = closure;
     Node *pv;
 
     pv = findnode (data->rcs->versions, p->key);
@@ -1428,22 +1475,21 @@ log_count_print (p, closure)
 	error (1, 0, "missing version `%s' in RCS file `%s'",
 	       p->key, data->rcs->path);
     if (log_version_requested (data->log_data, data->revlist, data->rcs,
-			       (RCSVers *) pv->data))
+			       pv->data))
 	return 1;
     else
 	return 0;
 }
 
+
+
 /*
  * Print the list of changes, not including the trunk, in reverse
  * order for each branch.
  */
 static void
-log_tree (log_data, revlist, rcs, ver)
-    struct log_data *log_data;
-    struct revlist *revlist;
-    RCSNode *rcs;
-    const char *ver;
+log_tree (struct log_data *log_data, struct revlist *revlist, RCSNode *rcs,
+          const char *ver)
 {
     Node *p;
     RCSVers *vnode;
@@ -1452,7 +1498,7 @@ log_tree (log_data, revlist, rcs, ver)
     if (p == NULL)
 	error (1, 0, "missing version `%s' in RCS file `%s'",
 	       ver, rcs->path);
-    vnode = (RCSVers *) p->data;
+    vnode = p->data;
     if (vnode->next != NULL)
 	log_tree (log_data, revlist, rcs, vnode->next);
     if (vnode->branches != NULL)
@@ -1471,15 +1517,14 @@ log_tree (log_data, revlist, rcs, ver)
     }
 }
 
+
+
 /*
  * Log the changes for a branch, in reverse order.
  */
 static void
-log_abranch (log_data, revlist, rcs, ver)
-    struct log_data *log_data;
-    struct revlist *revlist;
-    RCSNode *rcs;
-    const char *ver;
+log_abranch (struct log_data *log_data, struct revlist *revlist, RCSNode *rcs,
+             const char *ver)
 {
     Node *p;
     RCSVers *vnode;
@@ -1488,22 +1533,20 @@ log_abranch (log_data, revlist, rcs, ver)
     if (p == NULL)
 	error (1, 0, "missing version `%s' in RCS file `%s'",
 	       ver, rcs->path);
-    vnode = (RCSVers *) p->data;
+    vnode = p->data;
     if (vnode->next != NULL)
 	log_abranch (log_data, revlist, rcs, vnode->next);
     log_version (log_data, revlist, rcs, vnode, 0);
 }
 
+
+
 /*
  * Print the log output for a single version.
  */
 static void
-log_version (log_data, revlist, rcs, ver, trunk)
-    struct log_data *log_data;
-    struct revlist *revlist;
-    RCSNode *rcs;
-    RCSVers *ver;
-    int trunk;
+log_version (struct log_data *log_data, struct revlist *revlist, RCSNode *rcs, 
+             RCSVers *ver, int trunk)
 {
     Node *p;
     int year, mon, mday, hour, min, sec;
@@ -1523,22 +1566,23 @@ log_version (log_data, revlist, rcs, ver, trunk)
 	cvs_output (p->data, 0);
 	cvs_output (";", 1);
     }
+    cvs_output ("\n", 1);
 
-    cvs_output ("\ndate: ", 0);
-    (void) sscanf (ver->date, SDATEFORM, &year, &mon, &mday, &hour, &min,
-		   &sec);
+    cvs_output_tagged ("text", "date: ");
+    (void)sscanf (ver->date, SDATEFORM, &year, &mon, &mday, &hour, &min,
+		  &sec);
     if (year < 1900)
 	year += 1900;
-    sprintf (buf, "%04d/%02d/%02d %02d:%02d:%02d", year, mon, mday,
+    sprintf (buf, "%04d-%02d-%02d %02d:%02d:%02d +0000", year, mon, mday,
 	     hour, min, sec);
-    cvs_output (buf, 0);
+    cvs_output_tagged ("date", buf);
 
-    cvs_output (";  author: ", 0);
-    cvs_output (ver->author, 0);
+    cvs_output_tagged ("text", ";  author: ");
+    cvs_output_tagged ("text", ver->author);
 
-    cvs_output (";  state: ", 0);
-    cvs_output (ver->state, 0);
-    cvs_output (";", 1);
+    cvs_output_tagged ("text", ";  state: ");
+    cvs_output_tagged ("text", ver->state);
+    cvs_output_tagged ("text", ";");
 
     if (! trunk)
     {
@@ -1559,55 +1603,55 @@ log_version (log_data, revlist, rcs, ver, trunk)
 	if (nextp == NULL)
 	    error (1, 0, "missing version `%s' in `%s'", ver->next,
 		   rcs->path);
-	nextver = (RCSVers *) nextp->data;
+	nextver = nextp->data;
 	pdel = findnode (nextver->other, ";add");
 	padd = findnode (nextver->other, ";delete");
     }
 
     if (padd != NULL)
     {
-	cvs_output ("  lines: +", 0);
-	cvs_output (padd->data, 0);
-	cvs_output (" -", 2);
-	cvs_output (pdel->data, 0);
+	cvs_output_tagged ("text", "  lines: +");
+	cvs_output_tagged ("text", padd->data);
+	cvs_output_tagged ("text", " -");
+	cvs_output_tagged ("text", pdel->data);
     }
+    cvs_output_tagged ("newline", NULL);
 
     if (ver->branches != NULL)
     {
-	cvs_output ("\nbranches:", 0);
-	walklist (ver->branches, log_branch, (void *) NULL);
+	cvs_output ("branches:", 0);
+	walklist (ver->branches, log_branch, NULL);
+	cvs_output ("\n", 1);
     }
 
-    cvs_output ("\n", 1);
-
     p = findnode (ver->other, "log");
     /* The p->date == NULL case is the normal one for an empty log
        message (rcs-14 in sanity.sh).  I don't think the case where
        p->data is "" can happen (getrcskey in rcs.c checks for an
        empty string and set the value to NULL in that case).  My guess
        would be the p == NULL case would mean an RCS file which was
-       missing the "log" keyword (which is illegal according to
+       missing the "log" keyword (which is invalid according to
        rcsfile.5).  */
-    if (p == NULL || p->data == NULL || p->data[0] == '\0')
+    if (p == NULL || p->data == NULL || *(char *)p->data == '\0')
 	cvs_output ("*** empty log message ***\n", 0);
     else
     {
 	/* FIXME: Technically, the log message could contain a null
            byte.  */
 	cvs_output (p->data, 0);
-	if (p->data[strlen (p->data) - 1] != '\n')
+	if (((char *)p->data)[strlen (p->data) - 1] != '\n')
 	    cvs_output ("\n", 1);
     }
 }
 
+
+
 /*
  * Output a branch version.  This is called via walklist.
  */
 /*ARGSUSED*/
 static int
-log_branch (p, closure)
-    Node *p;
-    void *closure;
+log_branch (Node *p, void *closure)
 {
     cvs_output ("  ", 2);
     if ((numdots (p->key) & 1) == 0)
@@ -1626,34 +1670,31 @@ log_branch (p, closure)
     return 0;
 }
 
+
+
 /*
  * Print a warm fuzzy message
  */
 /* ARGSUSED */
 static Dtype
-log_dirproc (callerdat, dir, repository, update_dir, entries)
-    void *callerdat;
-    char *dir;
-    char *repository;
-    char *update_dir;
-    List *entries;
+log_dirproc (void *callerdat, const char *dir, const char *repository,
+             const char *update_dir, List *entries)
 {
     if (!isdir (dir))
-	return (R_SKIP_ALL);
+	return R_SKIP_ALL;
 
     if (!quiet)
 	error (0, 0, "Logging %s", update_dir);
-    return (R_PROCESS);
+    return R_PROCESS;
 }
 
+
+
 /*
  * Compare versions.  This is taken from RCS compartial.
  */
 static int
-version_compare (v1, v2, len)
-    const char *v1;
-    const char *v2;
-    int len;
+version_compare (const char *v1, const char *v2, int len)
 {
     while (1)
     {
