@@ -1,10 +1,5 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
- *
- * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
- *                                  and others.
- *
- * Portions Copyright (c) 1995, Cyclic Software, Bloomington, IN, USA
+ * Copyright (c) 1995, Cyclic Software, Bloomington, IN, USA
  * 
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with CVS.
@@ -14,7 +9,6 @@
 
 #include "cvs.h"
 #include "getline.h"
-extern char * getpass (const char *prompt);
 
 #ifdef AUTH_CLIENT_SUPPORT   /* This covers the rest of the file. */
 
@@ -33,11 +27,11 @@ extern char * getpass (const char *prompt);
 /* If non-NULL, get_cvs_password() will just return this. */
 static char *cvs_password = NULL;
 
-static char *construct_cvspass_filename PROTO ((void));
+static char *construct_cvspass_filename (void);
 
 /* The return value will need to be freed. */
 static char *
-construct_cvspass_filename ()
+construct_cvspass_filename (void)
 {
     char *homedir;
     char *passfile;
@@ -102,11 +96,9 @@ construct_cvspass_filename ()
  *	called on the same linebuf
  */
 static char *
-password_entry_parseline (cvsroot_canonical, warn, linenumber, linebuf)
-    const char *cvsroot_canonical;
-    const unsigned char warn;
-    const int linenumber;
-    char *linebuf;
+password_entry_parseline (const char *cvsroot_canonical,
+			  const unsigned char warn, const int linenumber,
+			  char *linebuf)
 {
     char *password = NULL;
     char *p;
@@ -116,20 +108,20 @@ password_entry_parseline (cvsroot_canonical, warn, linenumber, linebuf)
     {
 	/* Yes: slurp '^/\d+\D' and parse the rest of the line according to version number */
 	char *q;
-	unsigned long int entry_version = 0;
+	unsigned long int entry_version;
 
 	if (isspace(*(linebuf + 1)))
-	{
 	    /* special case since strtoul ignores leading white space */
-	    q = linebuf + 1;
-	}
+	    entry_version = 0;
 	else
-	{
 	    entry_version = strtoul (linebuf + 1, &q, 10);
-	    if (q != linebuf + 1)
-		/* assume a delimiting seperator */
-		q++;
-	}
+
+	if (q == linebuf + 1)
+	    /* no valid digits found by strtoul */
+	    entry_version = 0;
+	else
+	    /* assume a delimiting seperator */
+	    q++;
 
 	switch (entry_version)
 	{
@@ -280,10 +272,7 @@ typedef enum password_entry_operation_e {
 } password_entry_operation_t;
 
 static char *
-password_entry_operation (operation, root, newpassword)
-    password_entry_operation_t operation;
-    cvsroot_t *root;
-    char *newpassword;
+password_entry_operation (password_entry_operation_t operation, cvsroot_t *root, char *newpassword)
 {
     char *passfile;
     FILE *fp;
@@ -311,7 +300,6 @@ internal error: can only call password_entry_operation with pserver method");
      */
 
     passfile = construct_cvspass_filename ();
-    errno = 0; /* Standard C doesn't require errno be set on error */
     fp = CVS_FOPEN (passfile, "r");
     if (fp == NULL)
     {
@@ -320,7 +308,7 @@ internal error: can only call password_entry_operation with pserver method");
     }
 
     /* Check each line to see if we have this entry already. */
-    line = 0;
+    line = 0L;
     while ((line_length = getline (&linebuf, &linebuf_len, fp)) >= 0)
     {
 	line++;
@@ -383,17 +371,15 @@ process:
 	FILE *tmp_fp;
 
 	/* open the original file again */
-	errno = 0; /* Standard C doesn't require errno be set on error */
 	fp = CVS_FOPEN (passfile, "r");
 	if (fp == NULL)
 	    error (1, errno, "failed to open %s for reading", passfile);
 
 	/* create and open a temp file */
 	if ((tmp_fp = cvs_temp_file (&tmp_name)) == NULL)
-	    error (1, errno, "unable to open temp file %s",
-		   tmp_name ? tmp_name : "(null)");
+	    error (1, errno, "unable to open temp file %s", tmp_name);
 
-	line = 0;
+	line = 0L;
 	while ((line_length = getline (&linebuf, &linebuf_len, fp)) >= 0)
 	{
 	    line++;
@@ -458,14 +444,13 @@ process:
     if (!noexec && operation == password_entry_add
 	    && (password == NULL || strcmp (password, newpassword)))
     {
-	errno = 0; /* Standard C doesn't require errno be set on error */
 	if ((fp = CVS_FOPEN (passfile, "a")) == NULL)
 	    error (1, errno, "could not open %s for writing", passfile);
 
 	if (fprintf (fp, "/1 %s %s\n", cvsroot_canonical, newpassword) == EOF)
 	    error (1, errno, "cannot write %s", passfile);
 	if (fclose (fp) < 0)
-	    error (1, errno, "cannot close %s", passfile);
+	    error (0, errno, "cannot close %s", passfile);
     }
 
     /* Utter, total, raving paranoia, I know. */
@@ -521,9 +506,7 @@ static const char *const login_usage[] =
 };
 
 int
-login (argc, argv)
-    int argc;
-    char **argv;
+login (int argc, char **argv)
 {
     char *typed_password;
     char *cvsroot_canonical;
@@ -570,42 +553,23 @@ login (argc, argv)
     password_entry_operation (password_entry_add, current_parsed_root,
                               typed_password);
 
-    free_cvs_password (typed_password);
+    memset (typed_password, 0, strlen (typed_password));
+    free (typed_password);
+
+    free (cvs_password);
     free (cvsroot_canonical);
+    cvs_password = NULL;
 
     return 0;
 }
 
 
 
-/* Free the password returned by get_cvs_password() and also free the
- * saved cvs_password if they are different pointers. Be paranoid
- * about the in-memory copy of the password and overwrite it with zero
- * bytes before doing the free().
- */
-void
-free_cvs_password (char *password)
-{
-    if (password && password != cvs_password)
-    {
-	memset (password, 0, strlen (password));
-	free (password);
-    }
-
-    if (cvs_password)
-    {
-	memset (cvs_password, 0, strlen (cvs_password));
-	free (cvs_password);
-	cvs_password = NULL;
-    }
-}
-
-/* Returns the _scrambled_ password in freshly allocated memory.  The server
- * must descramble before hashing and comparing.  If password file not found,
- * or password not found in the file, just return NULL.
- */
+/* Returns the _scrambled_ password.  The server must descramble
+   before hashing and comparing.  If password file not found, or
+   password not found in the file, just return NULL. */
 char *
-get_cvs_password ()
+get_cvs_password (void)
 {
     if (current_parsed_root->password)
 	return scramble (current_parsed_root->password);
@@ -614,7 +578,7 @@ get_cvs_password ()
        context, then assume they have supplied the correct, scrambled
        password. */
     if (cvs_password)
-	return xstrdup (cvs_password);
+	return cvs_password;
 
     if (getenv ("CVS_PASSWORD") != NULL)
     {
@@ -651,9 +615,7 @@ static const char *const logout_usage[] =
 
 /* Remove any entry for the CVSRoot repository found in .cvspass. */
 int
-logout (argc, argv)
-    int argc;
-    char **argv;
+logout (int argc, char **argv)
 {
     char *cvsroot_canonical;
 
