@@ -3,7 +3,8 @@
  *    Copyright (c) 1989-1992, Brian Berliner
  *
  *    You may distribute under the terms of the GNU General Public License
- *    as specified in the README file that comes with the CVS source distribution.
+ *    as specified in the README file that comes with the CVS source
+ *    distribution.
  *
  * This is the main C driver for the CVS system.
  *
@@ -12,13 +13,12 @@
  *
  */
 
-#include <assert.h>
 #include "cvs.h"
 
 #ifdef HAVE_WINSOCK_H
-#include <winsock.h>
-#else
-extern int gethostname ();
+# include <winsock.h>
+#elif !HAVE_GETHOSTNAME
+extern int gethostname (char *, int);
 #endif
 
 const char *program_name;
@@ -41,12 +41,16 @@ int really_quiet = 0;
 int quiet = 0;
 int trace = 0;
 int noexec = 0;
+int readonlyfs = 0;
 int logoff = 0;
 
 /* Set if we should be writing CVSADM directories at top level.  At
    least for now we'll make the default be off (the CVS 1.9, not CVS
    1.9.2, behavior). */
 int top_level_admin = 0;
+#ifdef SUPPORT_OLD_INFO_FMT_STRINGS
+int UseNewInfoFmtStrings = 0;
+#endif /* SUPPORT_OLD_INFO_FMT_STRINGS */
 
 mode_t cvsumask = UMASK_DFLT;
 
@@ -124,6 +128,7 @@ static const struct cmd
     { "login",    "logon",    "lgn",       login,     0 },
     { "logout",   NULL,       NULL,        logout,    0 },
 #endif /* AUTH_CLIENT_SUPPORT */
+    { "ls",       "dir",      "list",      ls,        0 },
 #if (defined(AUTH_SERVER_SUPPORT) || defined (HAVE_GSSAPI)) && defined(SERVER_SUPPORT)
     { "pserver",  NULL,       NULL,        server,    CVS_CMD_MODIFIES_REPOSITORY | CVS_CMD_USES_WORK_DIR }, /* placeholder */
 #endif
@@ -132,6 +137,7 @@ static const struct cmd
     { "release",  "re",       "rel",       release,   0 },
     { "remove",   "rm",       "delete",    cvsremove, CVS_CMD_MODIFIES_REPOSITORY | CVS_CMD_USES_WORK_DIR },
     { "rlog",     "rl",       NULL,        cvslog,    0 },
+    { "rls",      "rdir",     "rlist",     ls,        0 },
     { "rtag",     "rt",       "rfreeze",   cvstag,    CVS_CMD_MODIFIES_REPOSITORY },
 #ifdef SERVER_SUPPORT
     { "server",   NULL,       NULL,        server,    CVS_CMD_MODIFIES_REPOSITORY | CVS_CMD_USES_WORK_DIR },
@@ -215,6 +221,7 @@ static const char *const cmd_usage[] =
     "        login        Prompt for password for authenticating server\n",
     "        logout       Removes entry in .cvspass for remote repository\n",
 #endif /* AUTH_CLIENT_SUPPORT */
+    "        ls           List files available from CVS\n",
 #if (defined(AUTH_SERVER_SUPPORT) || defined (HAVE_GSSAPI)) && defined(SERVER_SUPPORT)
     "        pserver      Password server mode\n",
 #endif
@@ -223,6 +230,7 @@ static const char *const cmd_usage[] =
     "        release      Indicate that a Module is no longer in use\n",
     "        remove       Remove an entry from the repository\n",
     "        rlog         Print out history information for a module\n",
+    "        rls          List files in a module\n",
     "        rtag         Add a symbolic tag to a module\n",
 #ifdef SERVER_SUPPORT
     "        server       Server mode\n",
@@ -248,7 +256,9 @@ static const char *const opt_usage[] =
     "    -r           Make checked-out files read-only.\n",
     "    -w           Make checked-out files read-write (default).\n",
     "    -n           Do not execute anything that will change the disk.\n",
-    "    -t           Show trace of program execution -- try with -n.\n",
+    "    -t           Show trace of program execution (repeat for more\n",
+    "                 verbosity) -- try with -n.\n",
+    "    -R           Assume repository is read-only, such as CDROM\n",
     "    -v           CVS version and copyright.\n",
     "    -T tmpdir    Use 'tmpdir' for temporary files.\n",
     "    -e editor    Use 'editor' for editing log information.\n",
@@ -268,9 +278,7 @@ static const char *const opt_usage[] =
 
 
 static int
-set_root_directory (p, ignored)
-    Node *p;
-    void *ignored;
+set_root_directory (Node *p, void *ignored)
 {
     if (current_root == NULL && p->data == NULL)
     {
@@ -282,7 +290,7 @@ set_root_directory (p, ignored)
 
 
 static const char * const*
-cmd_synonyms ()
+cmd_synonyms (void)
 {
     char ** synonyms;
     char ** line;
@@ -321,8 +329,7 @@ cmd_synonyms ()
 
 
 unsigned long int
-lookup_command_attribute (cmd_name)
-     char *cmd_name;
+lookup_command_attribute (char *cmd_name)
 {
     const struct cmd *cm;
 
@@ -337,9 +344,26 @@ lookup_command_attribute (cmd_name)
 }
 
 
+
+/*
+ * Exit with an error code and an informative message about the signal
+ * received.  This function, by virtue of causing an actual call to exit(),
+ * causes all the atexit() handlers to be called.
+ *
+ * INPUTS
+ *   sig	The signal recieved.
+ *
+ * ERRORS
+ *   The cleanup routines registered via atexit() and the error function
+ *   itself can potentially change the exit status.  They shouldn't do this
+ *   unless they encounter problems doing their own jobs.
+ *
+ * RETURNS
+ *   Nothing.  This function will always exit.  It should exit with an exit
+ *   status of 1, but might not, as noted in the ERRORS section above.
+ */
 static RETSIGTYPE
-main_cleanup (sig)
-    int sig;
+main_cleanup (int sig)
 {
 #ifndef DONT_USE_SIGNALS
     const char *name;
@@ -389,10 +413,10 @@ main_cleanup (sig)
 #endif /* !DONT_USE_SIGNALS */
 }
 
+
+
 int
-main (argc, argv)
-    int argc;
-    char **argv;
+main (int argc, char **argv)
 {
     char *CVSroot = CVSROOT_DFLT;
     char *cp, *end;
@@ -406,7 +430,7 @@ main (argc, argv)
     int help = 0;		/* Has the user asked for help?  This
 				   lets us support the `cvs -H cmd'
 				   convention to give help for cmd. */
-    static const char short_options[] = "+Qqrwtnvb:T:e:d:Hfz:s:xa";
+    static const char short_options[] = "+QqrwtnRvb:T:e:d:Hfz:s:xa";
     static struct option long_options[] =
     {
         {"help", 0, NULL, 'H'},
@@ -427,6 +451,12 @@ main (argc, argv)
     SYSTEM_INITIALIZE (&argc, &argv);
 #endif
 
+#ifdef SYSTEM_CLEANUP
+	/* Hook for OS-specific behavior, for example socket subsystems on
+	   NT and OS2 or dealing with windows and arguments on Mac.  */
+	cleanup_register (SYSTEM_CLEANUP);
+#endif
+
 #ifdef HAVE_TZSET
     /* On systems that have tzset (which is almost all the ones I know
        of), it's a good idea to call it.  */
@@ -469,6 +499,10 @@ main (argc, argv)
     }
     if (getenv (CVSREAD_ENV) != NULL)
 	cvswrite = 0;
+    if (getenv (CVSREADONLYFS_ENV) != NULL) {
+	readonlyfs = 1;
+	logoff = 1;
+    }
 
     /* Set this to 0 to force getopt initialization.  getopt() sets
        this to 1 internally.  */
@@ -532,25 +566,28 @@ main (argc, argv)
 		cvswrite = 1;
 		break;
 	    case 't':
-		trace = 1;
+		trace++;
+		break;
+	    case 'R':
+		readonlyfs = -1;
+		logoff = 1;
 		break;
 	    case 'n':
 		noexec = 1;
 		logoff = 1;
 		break;
 	    case 'v':
-		(void) fputs ("\n", stdout);
-		version (0, (char **) NULL);    
-		(void) fputs ("\n", stdout);
-		(void) fputs ("\
-Copyright (c) 1989-2004 Brian Berliner, david d `zoo' zuhn, \n\
-                        Jeff Polk, and other authors\n", stdout);
-		(void) fputs ("\n", stdout);
-		(void) fputs ("CVS may be copied only under the terms of the GNU General Public License,\n", stdout);
-		(void) fputs ("a copy of which can be found with the CVS distribution kit.\n", stdout);
-		(void) fputs ("\n", stdout);
-
-		(void) fputs ("Specify the --help option for further information about CVS\n", stdout);
+		fputs ("\n", stdout);
+		version (0, NULL);    
+		fputs (
+"\n"
+"Copyright (c) 1989-2004 Brian Berliner, david d `zoo' zuhn,\n"
+"                        Jeff Polk, and other authors\n"
+"\n"
+"CVS may be copied only under the terms of the GNU General Public License,\n"
+"a copy of which can be found with the CVS distribution kit.\n"
+"\n"
+"Specify the --help option for further information about CVS\n", stdout);
 
 		exit (0);
 		break;
@@ -562,11 +599,13 @@ Copyright (c) 1989-2004 Brian Berliner, david d `zoo' zuhn, \n\
 		   either new or old CVS.  */
 		break;
 	    case 'T':
+		if (free_Tmpdir) free (Tmpdir);
 		Tmpdir = xstrdup (optarg);
 		free_Tmpdir = 1;
 		tmpdir_update_env = 1;	/* need to update environment */
 		break;
 	    case 'e':
+		if (free_Editor) free (Editor);
 		Editor = xstrdup (optarg);
 		free_Editor = 1;
 		break;
@@ -575,9 +614,11 @@ Copyright (c) 1989-2004 Brian Berliner, david d `zoo' zuhn, \n\
 		    free (CVSroot_cmdline);
 		CVSroot_cmdline = xstrdup (optarg);
 		if (free_CVSroot)
+		{
 		    free (CVSroot);
-		CVSroot = xstrdup (optarg);
-		free_CVSroot = 1;
+		    free_CVSroot = 0;
+		}
+		CVSroot = CVSroot_cmdline;
 		cvs_update_env = 1;	/* need to update environment */
 		break;
 	    case 'H':
@@ -634,6 +675,12 @@ Copyright (c) 1989-2004 Brian Berliner, david d `zoo' zuhn, \n\
     if (argc < 1)
 	usage (usg);
 
+    if (readonlyfs && !really_quiet) {
+	error (0, 0,
+	       "WARNING: Read-only repository access mode selected via `cvs -R'.\n\
+Using this option to access a repository which some users write to may\n\
+cause intermittent sandbox corruption.");
+    }
 
     /* Look up the command name. */
 
@@ -666,6 +713,8 @@ Copyright (c) 1989-2004 Brian Berliner, david d `zoo' zuhn, \n\
 	/* The user didn't ask for help, so go ahead and authenticate,
            set up CVSROOT, and the rest of it. */
 
+	short int lock_cleanup_setup = 0;
+
 	/* The UMASK environment variable isn't handled with the
 	   others above, since we don't want to signal errors if the
 	   user has asked for help.  This won't work if somebody adds
@@ -721,14 +770,17 @@ Copyright (c) 1989-2004 Brian Berliner, david d `zoo' zuhn, \n\
 
 #endif /* SERVER_SUPPORT */
 
-	/* This is only used for writing into the history file.  For
-	   remote connections, it might be nice to have hostname
-	   and/or remote path, on the other hand I'm not sure whether
-	   it is worth the trouble.  */
 
 #ifdef SERVER_SUPPORT
 	if (server_active)
+	{
+	    /* This is only used for writing into the history file.  For
+	       remote connections, it might be nice to have hostname
+	       and/or remote path, on the other hand I'm not sure whether
+	       it is worth the trouble.  */
 	    CurDir = xstrdup ("<remote>");
+	    cleanup_register (server_cleanup);
+	}
 	else
 #endif
 	{
@@ -749,29 +801,18 @@ Copyright (c) 1989-2004 Brian Berliner, david d `zoo' zuhn, \n\
 	    (void) putenv (env);
 	    /* do not free env, as putenv has control of it */
 	}
+	{
+	    char *env;
+	    /* XXX pid < 10^32 */
+	    env = xmalloc (strlen (CVS_PID_ENV) + 1 + 32 + 1);
+	    (void) sprintf (env, "%s=%ld", CVS_PID_ENV, (long) getpid ());
+	    (void) putenv (env);
+	    /* do not free env, as putenv has control of it */
+	}
 #endif
 
-#ifndef DONT_USE_SIGNALS
 	/* make sure we clean up on error */
-#ifdef SIGABRT
-	(void) SIG_register (SIGABRT, main_cleanup);
-#endif
-#ifdef SIGHUP
-	(void) SIG_register (SIGHUP, main_cleanup);
-#endif
-#ifdef SIGINT
-	(void) SIG_register (SIGINT, main_cleanup);
-#endif
-#ifdef SIGQUIT
-	(void) SIG_register (SIGQUIT, main_cleanup);
-#endif
-#ifdef SIGPIPE
-	(void) SIG_register (SIGPIPE, main_cleanup);
-#endif
-#ifdef SIGTERM
-	(void) SIG_register (SIGTERM, main_cleanup);
-#endif
-#endif /* !DONT_USE_SIGNALS */
+	signals_register (main_cleanup);
 
 	gethostname(hostname, sizeof (hostname));
 
@@ -903,9 +944,9 @@ Copyright (c) 1989-2004 Brian Berliner, david d `zoo' zuhn, \n\
 		if ((current_parsed_root = parse_cvsroot (current_root)) == NULL)
 		    error (1, 0, "Bad CVSROOT: `%s'.", current_root);
 
-		if (trace)
-		    fprintf (stderr, "%s-> main loop with CVSROOT=%s\n",
-			   CLIENT_SERVER_STR, current_root);
+		TRACE ( TRACE_FUNCTION,
+			"main loop with CVSROOT=%s",
+			current_root ? current_root : "(null)");
 
 		/*
 		 * Check to see if the repository exists.
@@ -991,6 +1032,25 @@ Copyright (c) 1989-2004 Brian Berliner, david d `zoo' zuhn, \n\
 	    }
 #endif
 
+	    if (
+#ifdef SERVER_SUPPORT
+		/* Don't worry about lock_cleanup_setup when the server is
+		 * active since we can only go through this loop once in that
+		 * case anyhow.
+		 */
+		server_active ||
+#endif
+#ifdef CLIENT_SUPPORT
+		!current_parsed_root->isremote &&
+#endif
+		!lock_cleanup_setup)
+	    {
+		/* Set up to clean up any locks we might create on exit.  */
+		cleanup_register (Lock_Cleanup);
+		lock_cleanup_setup = 1;
+	    }
+
+	    /* Call our worker function.  */
 	    err = (*(cm->func)) (argc, argv);
 	
 	    /* Mark this root directory as done.  When the server is
@@ -1012,38 +1072,16 @@ Copyright (c) 1989-2004 Brian Berliner, david d `zoo' zuhn, \n\
 
 #ifdef SERVER_SUPPORT
 	    if (server_active)
-	    {
-		server_active = 0;
 		break;
-	    }
 #endif
 	} /* end of loop for cvsroot values */
 
     } /* end of stuff that gets done if the user DOESN'T ask for help */
 
-    Lock_Cleanup ();
-
-    /* It's okay to cast out the const below since we know we allocated this in
-     * this function.  The const was to keep other functions from messing with
-     * this.
-     */
-    free ((char *)program_path);
-    if (CVSroot_cmdline != NULL)
-	free (CVSroot_cmdline);
     if (free_CVSroot)
 	free (CVSroot);
-    if (free_Editor)
-	free (Editor);
-    if (free_Tmpdir)
-	free (Tmpdir);
     root_allow_free ();
 
-#ifdef SYSTEM_CLEANUP
-    /* Hook for OS-specific behavior, for example socket subsystems on
-       NT and OS2 or dealing with windows and arguments on Mac.  */
-    SYSTEM_CLEANUP ();
-#endif
-
     /* This is exit rather than return because apparently that keeps
        some tools which check for memory leaks happier.  */
     exit (err ? EXIT_FAILURE : 0);
@@ -1051,18 +1089,19 @@ Copyright (c) 1989-2004 Brian Berliner, david d `zoo' zuhn, \n\
 	return 0;
 }
 
-char *
-Make_Date (rawdate)
-    char *rawdate;
-{
-    time_t unixtime;
 
-    unixtime = get_date (rawdate, (struct timeb *) NULL);
-    if (unixtime == (time_t) - 1)
-	error (1, 0, "Can't parse date/time: %s", rawdate);
+
+char *
+Make_Date (char *rawdate)
+{
+    time_t unixtime = get_date (rawdate, NULL);
+    if (unixtime == (time_t)-1)
+	error (1, 0, "Can't parse date/time: `%s'", rawdate);
     return date_from_time_t (unixtime);
 }
 
+
+
 /* Convert a time_t to an RCS format date.  This is mainly for the
    use of "cvs history", because the CVSROOT/history file contains
    time_t format dates; most parts of CVS will want to avoid using
@@ -1073,8 +1112,7 @@ Make_Date (rawdate)
    Returns a newly malloc'd string.  */
 
 char *
-date_from_time_t (unixtime)
-    time_t unixtime;
+date_from_time_t (time_t unixtime)
 {
     struct tm *ftm;
     char date[MAXDATELEN];
@@ -1093,9 +1131,11 @@ date_from_time_t (unixtime)
 		    ftm->tm_mon + 1, ftm->tm_mday, ftm->tm_hour,
 		    ftm->tm_min, ftm->tm_sec);
     ret = xstrdup (date);
-    return (ret);
+    return ret;
 }
 
+
+
 /* Convert a date to RFC822/1123 format.  This is used in contexts like
    dates to send in the protocol; it should not vary based on locale or
    other such conventions for users.  We should have another routine which
@@ -1104,9 +1144,7 @@ date_from_time_t (unixtime)
    The SOURCE date is in our internal RCS format.  DEST should point to
    storage managed by the caller, at least MAXDATELEN characters.  */
 void
-date_to_internet (dest, source)
-    char *dest;
-    const char *source;
+date_to_internet (char *dest, const char *source)
 {
     struct tm date;
 
@@ -1114,10 +1152,10 @@ date_to_internet (dest, source)
     tm_to_internet (dest, &date);
 }
 
+
+
 void
-date_to_tm (dest, source)
-    struct tm *dest;
-    const char *source;
+date_to_tm (struct tm *dest, const char *source)
 {
     if (sscanf (source, SDATEFORM,
 		&dest->tm_year, &dest->tm_mon, &dest->tm_mday,
@@ -1134,6 +1172,8 @@ date_to_tm (dest, source)
     dest->tm_mon -= 1;
 }
 
+
+
 /* Convert a date to RFC822/1123 format.  This is used in contexts like
    dates to send in the protocol; it should not vary based on locale or
    other such conventions for users.  We should have another routine which
@@ -1142,9 +1182,7 @@ date_to_tm (dest, source)
    The SOURCE date is a pointer to a struct tm.  DEST should point to
    storage managed by the caller, at least MAXDATELEN characters.  */
 void
-tm_to_internet (dest, source)
-    char *dest;
-    const struct tm *source;
+tm_to_internet (char *dest, const struct tm *source)
 {
     /* Just to reiterate, these strings are from RFC822 and do not vary
        according to locale.  */
@@ -1153,16 +1191,117 @@ tm_to_internet (dest, source)
 	 "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
     
     sprintf (dest, "%d %s %d %02d:%02d:%02d -0000", source->tm_mday,
-	     source->tm_mon < 0 || source->tm_mon > 11 ? "???" : month_names[source->tm_mon],
-	     source->tm_year + 1900, source->tm_hour, source->tm_min, source->tm_sec);
+	     source->tm_mon < 0 || source->tm_mon > 11
+               ? "???" : month_names[source->tm_mon],
+	     source->tm_year + 1900, source->tm_hour, source->tm_min,
+             source->tm_sec);
 }
 
+
+
+/*
+ * Format a date for the current locale.
+ *
+ * INPUT
+ *   UNIXTIME	The UNIX seconds since the epoch.
+ *
+ * RETURNS
+ *   If my_strftime() encounters an error, this function can return NULL.
+ *
+ *   Otherwise, returns a date string in ISO8601 format, e.g.:
+ *
+ *	2004-04-29 13:24:22 -0700
+ *
+ *   It is the responsibility of the caller to return of this string.
+ */
+static char *
+format_time_t (time_t unixtime)
+{
+    static char buf[sizeof ("yyyy-mm-dd HH:MM:SS -HHMM")];
+    /* Convert to a time in the local time zone.  */
+    struct tm ltm = *(gmtime (&unixtime));
+
+    if (my_strftime (buf, sizeof (buf), "%Y-%m-%d %H:%M:%S %z", &ltm) == 0)
+	return NULL;
+
+    return xstrdup (buf);
+}
+
+
+
+/* Like format_time_t(), but return time in UTC.
+ */
+char *
+gmformat_time_t (time_t unixtime)
+{
+    static char buf[sizeof ("yyyy-mm-dd HH:MM:SS -HHMM")];
+    /* Convert to a time in the local time zone.  */
+    struct tm ltm = *(gmtime (&unixtime));
+
+    if (my_strftime (buf, sizeof (buf), "%Y-%m-%d %H:%M:%S %z", &ltm) == 0)
+	return NULL;
+
+    return xstrdup (buf);
+}
+
+
+
+/* Format a date in the local timezone using format_time_t() given a date from
+ * an arbitrary timezone in a string.
+ *
+ * INPUT
+ *   DATESTR	A string that looks like anything get_date() can parse, e.g.:
+ *
+ *                      2004-04-29 20:24:22
+ *
+ * ERRORS
+ *   As get_date() & format_time_t().  Prints a warning if either provide
+ *   error return values.  See RETURNS.
+ *
+ * RETURNS
+ *   A freshly allocated string that is a copy of the input string if either
+ *   get_date() or format_time_t() encounter an error and as format_time_t()
+ *   otherwise.
+ */
+char *
+format_date_alloc (char *datestr)
+{
+    time_t unixtime;
+    char *buf;
+
+    TRACE (TRACE_FUNCTION, "format_date (%s)", datestr);
+
+    /* Convert the date string to seconds since the epoch. */
+    unixtime = get_date (datestr, NULL);
+    if (unixtime == (time_t)-1)
+    {
+	error (0, 0, "Can't parse date/time: `%s'.", datestr);
+	goto as_is;
+    }
+
+    /* Get the time into a string.  */
+    if ((buf = format_time_t (unixtime)) == NULL)
+    {
+	error (0, 0, "Unable to reformat date `%s'.", datestr);
+	goto as_is;
+    }
+
+    return buf;
+
+ as_is:
+    return xstrdup (datestr);
+}
+
+
+
 void
-usage (cpp)
-    register const char *const *cpp;
+usage (register const char *const *cpp)
 {
     (void) fprintf (stderr, *cpp++, program_name, cvs_cmd_name);
     for (; *cpp; cpp++)
 	(void) fprintf (stderr, *cpp);
-    error_exit ();
+    exit (EXIT_FAILURE);
 }
+
+/* vim:tabstop=8:shiftwidth=4
+ */
