@@ -1,11 +1,6 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
- *
- * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
- *                                  and others.
- *
- * Portions Copyright (C) 1992, Brian Berliner and Jeff Polk
- * Portions Copyright (C) 1989-1992, Brian Berliner
+ * Copyright (c) 1992, Brian Berliner and Jeff Polk
+ * Copyright (c) 1989-1992, Brian Berliner
  * 
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with the CVS kit.  */
@@ -19,13 +14,13 @@
 #define	DBLKSIZ	4096			/* since GNU ndbm doesn't define it */
 #endif
 
-static int checkout_file PROTO((char *file, char *temp));
-static char *make_tempfile PROTO((void));
-static void rename_rcsfile PROTO((char *temp, char *real));
+static int checkout_file (char *file, char *temp);
+static char *make_tempfile (void);
+static void rename_rcsfile (char *temp, char *real);
 
 #ifndef MY_NDBM
-static void rename_dbmfile PROTO((char *temp));
-static void write_dbmfile PROTO((char *temp));
+static void rename_dbmfile (char *temp);
+static void write_dbmfile (char *temp);
 #endif				/* !MY_NDBM */
 
 /* Structure which describes an administrative file.  */
@@ -65,15 +60,20 @@ static const char *const loginfo_contents[] = {
     "# If the name ALL appears as a regular expression it is always used\n",
     "# in addition to the first matching regex or DEFAULT.\n",
     "#\n",
-    "# You may specify a format string as part of the\n",
-    "# filter.  The string is composed of a `%' followed\n",
-    "# by a single format character, or followed by a set of format\n",
-    "# characters surrounded by `{' and `}' as separators.  The format\n",
-    "# characters are:\n",
+    "# If any format strings are present in the filter, they will be replaced as follows:\n",
+    "#    %p = path relative to repository\n",
+    "#    %r = repository (path portion of $CVSROOT)\n",
+    "#    %{sVv} = attribute list = file name, old version number (pre-checkin),\n",
+    "#           new version number (post-checkin).  When either old or new revision is\n",
+    "#           unknown, doesn't exist, or isn't applicable, the string \"NONE\" will be\n",
+    "#           placed on the command line instead.\n",
     "#\n",
-    "#   s = file name\n",
-    "#   V = old version number (pre-checkin)\n",
-    "#   v = new version number (post-checkin)\n",
+    "# Note that %{sVv} is a list operator and not all elements are necessary.  Thus %{sv} is\n",
+    "# a legal format string, but will only be replaced with file name and new revision.\n",
+    "# it also generates multiple arguments for each file being operated upon.  i.e. if two\n",
+    "# files, file1 & file2, are being commited from 1.1 to version 1.1.2.1 and from 1.1.2.2\n",
+    "# to 1.1.2.3, respectively, %{sVv} will generate the following six arguments in this\n",
+    "# order: file1, 1.1, 1.1.2.1, file2, 1.1.2.2, 1.1.2.3.\n",
     "#\n",
     "# For example:\n",
     "#DEFAULT (echo \"\"; id; echo %s; date; cat) >> $CVSROOT/CVSROOT/commitlog\n",
@@ -99,30 +99,7 @@ static const char *const rcsinfo_contents[] = {
     NULL
 };
 
-static const char *const editinfo_contents[] = {
-    "# The \"editinfo\" file is used to allow verification of logging\n",
-    "# information.  It works best when a template (as specified in the\n",
-    "# rcsinfo file) is provided for the logging procedure.  Given a\n",
-    "# template with locations for, a bug-id number, a list of people who\n",
-    "# reviewed the code before it can be checked in, and an external\n",
-    "# process to catalog the differences that were code reviewed, the\n",
-    "# following test can be applied to the code:\n",
-    "#\n",
-    "#   Making sure that the entered bug-id number is correct.\n",
-    "#   Validating that the code that was reviewed is indeed the code being\n",
-    "#       checked in (using the bug-id number or a seperate review\n",
-    "#       number to identify this particular code set.).\n",
-    "#\n",
-    "# If any of the above test failed, then the commit would be aborted.\n",
-    "#\n",
-    "# Actions such as mailing a copy of the report to each reviewer are\n",
-    "# better handled by an entry in the loginfo file.\n",
-    "#\n",
-    "# One thing that should be noted is the the ALL keyword is not\n",
-    "# supported.  There can be only one entry that matches a given\n",
-    "# repository.\n",
-    NULL
-};
+
 
 static const char *const verifymsg_contents[] = {
     "# The \"verifymsg\" file is used to allow verification of logging\n",
@@ -140,6 +117,14 @@ static const char *const verifymsg_contents[] = {
     "#\n",
     "# If any of the above test failed, then the commit would be aborted.\n",
     "#\n",
+    "# Format strings present in the filter will be replaced as follows:\n",
+    "#    %p = path relative to repository\n",
+    "#    %r = repository (path portion of $CVSROOT)\n",
+    "#    %l = name of log file to be verified.\n",
+    "#\n",
+    "# If no format strings are present in the filter, a default \" %l\" will\n",
+    "# be appended to the filter, but this usage is deprecated.\n",
+    "#\n",
     "# Actions such as mailing a copy of the report to each reviewer are\n",
     "# better handled by an entry in the loginfo file.\n",
     "#\n",
@@ -160,6 +145,15 @@ static const char *const commitinfo_contents[] = {
     "# to the $CVSROOT.  For the first match that is found, then the remainder\n",
     "# of the line is the name of the filter to run.\n",
     "#\n",
+    "# Format strings present in the filter will be replaced as follows:\n",
+    "#    %p = path relative to repository\n",
+    "#    %r = repository (path portion of $CVSROOT)\n",
+    "#    %{s} = file name, file name, ...\n",
+    "#\n",
+    "# If no format strings are present in the filter string, a default of\n",
+    "# \" %r %s\" will be appended to the filter string, but this usage is\n",
+    "# deprecated.\n",
+    "#\n",
     "# If the repository name does not match any of the regular expressions in this\n",
     "# file, the \"DEFAULT\" line is used, if it is specified.\n",
     "#\n",
@@ -170,12 +164,32 @@ static const char *const commitinfo_contents[] = {
 
 static const char *const taginfo_contents[] = {
     "# The \"taginfo\" file is used to control pre-tag checks.\n",
-    "# The filter on the right is invoked with the following arguments:\n",
+    "# The filter on the right is invoked with the following arguments if no format strings are present:\n",
     "#\n",
     "# $1 -- tagname\n",
     "# $2 -- operation \"add\" for tag, \"mov\" for tag -F, and \"del\" for tag -d\n",
-    "# $3 -- repository\n",
-    "# $4->  file revision [file revision ...]\n",
+    "# $3 -- tagtype \"?\" on delete, \"T\" for branch, \"N\" for static\n",
+    "# $4 -- repository\n",
+    "# $5->  file revision [file revision ...]\n",
+    "#\n",
+    "# If any format strings are present in the filter, they will be replaced as follows:\n",
+    "#    %b = branch mode = \"?\" (delete ops - unknown) | \"T\" (branch) | \"N\" (not branch)\n",
+    "#    %o = operation = \"add\" | \"mov\" | \"del\"\n",
+    "#    %p = path relative to repository\n",
+    "#    %r = repository (path portion of $CVSROOT)\n",
+    "#    %t = tagname\n",
+    "#    %{sVv} = attribute list = file name, old version tag will be deleted from,\n",
+    "#           new version tag will be added to (or deleted from, but this feature is\n",
+    "#           deprecated.  When either old or new revision is unknown, doesn't exist,\n",
+    "#           or isn't applicable, the string \"NONE\" will be placed on the command\n",
+    "#           line.\n",
+    "#\n",
+    "# Note that %{sVv} is a list operator and not all elements are necessary.  Thus %{sV} is\n",
+    "# a legal format string, but will only be replaced with file name and old revision.\n",
+    "# it also generates multiple arguments for each file being operated upon.  i.e. if two\n",
+    "# files, file1 & file2, are having a tag moved from version 1.1 to versoin 1.1.2.9, %{sVv}\n",
+    "# will generate the following six arguments in this order: file1, 1.1, 1.1.2.9, file2, 1.1,\n",
+    "# 1.1.2.9.\n",
     "#\n",
     "# A non-zero exit of the filter program will cause the tag to be aborted.\n",
     "#\n",
@@ -249,8 +263,13 @@ static const char *const notify_contents[] = {
     "#\n",
     "# \"ALL\" or \"DEFAULT\" can be used in place of the regular expression.\n",
     "#\n",
+    "# format strings are replaceed as follows:\n",
+    "#    %p = path relative to repository\n",
+    "#    %r = repository (path portion of $CVSROOT)\n",
+    "#    %s = user to notify\n",
+    "#\n",
     "# For example:\n",
-    "#ALL mail -s \"CVS notification\" %s\n",
+    "#ALL (echo Committed to %r/%p; cat) |mail %s -s \"CVS notification\"\n",
     NULL
 };
 
@@ -261,6 +280,7 @@ static const char *const modules_contents[] = {
     "#	key [options] directory files...\n",
     "#\n",
     "# Where \"options\" are composed of:\n",
+    "#	-i prog		Run \"prog\" on \"cvs commit\" from top-level of module.\n",
     "#	-o prog		Run \"prog\" on \"cvs checkout\" of module.\n",
     "#	-e prog		Run \"prog\" on \"cvs export\" of module.\n",
     "#	-t prog		Run \"prog\" on \"cvs rtag\" of module.\n",
@@ -285,7 +305,7 @@ static const char *const modules_contents[] = {
 
 static const char *const config_contents[] = {
     "# Set this to \"no\" if pserver shouldn't check system users/passwords\n",
-    "#SystemAuth=yes\n",
+    "#SystemAuth=no\n",
     "\n",
     "# Put CVS lock files in this directory rather than directly in the repository.\n",
     "#LockDir=/var/lock/cvs\n",
@@ -312,6 +332,23 @@ static const char *const config_contents[] = {
     "# repositories.  Set it to `never' (the previous CVS behavior) to prevent\n",
     "# verifymsg scripts from changing the log message.\n",
     "#RereadLogAfterVerify=always\n",
+    "\n",
+    "# Set `UserAdminOptions' to the list of `cvs admin' commands (options)\n",
+    "# that users not in the `cvsadmin' group are allowed to run.  This\n",
+    "# defaults to `k', or only allowing the changing of the default\n",
+    "# keyword expansion mode for files for users not in the `cvsadmin' group.\n",
+    "# This value is ignored if the `cvsadmin' group does not exist.\n",
+    "#\n",
+    "# The following string would enable all `cvs admin' commands for all\n",
+    "# users:\n",
+    "#UserAdminOptions=aAbceIklLmnNostuU\n",
+#ifdef SUPPORT_OLD_INFO_FMT_STRINGS
+    "\n",
+    "# Set `UseNewInfoFmtStrings' to `no' if you must support a legacy system by\n",
+    "# enabling the deprecated old style info file command line format strings.\n",
+    "# Be warned that these strings could be disabled in any new version of CVS.\n",
+    "UseNewInfoFmtStrings=yes\n",
+#endif /* SUPPORT_OLD_INFO_FMT_STRINGS */
     NULL
 };
 
@@ -322,9 +359,6 @@ static const struct admin_file filelist[] = {
     {CVSROOTADM_RCSINFO,
 	"a %s file can be used to configure 'cvs commit' templates",
 	rcsinfo_contents},
-    {CVSROOTADM_EDITINFO,
-	"a %s file can be used to validate log messages",
-	editinfo_contents},
     {CVSROOTADM_VERIFYMSG,
 	"a %s file can be used to validate log messages",
 	verifymsg_contents},
@@ -382,8 +416,7 @@ static const struct admin_file filelist[] = {
 
 /* Rebuild the checked out administrative files in directory DIR.  */
 int
-mkmodules (dir)
-    char *dir;
+mkmodules (char *dir)
 {
     struct saved_cwd cwd;
     char *temp;
@@ -400,7 +433,7 @@ mkmodules (dir)
 	return 0;
 
     if (save_cwd (&cwd))
-	error_exit ();
+	exit (EXIT_FAILURE);
 
     if ( CVS_CHDIR (dir) < 0)
 	error (1, errno, "cannot chdir to %s", dir);
@@ -524,7 +557,7 @@ mkmodules (dir)
     }
 
     if (restore_cwd (&cwd, NULL))
-	error_exit ();
+	exit (EXIT_FAILURE);
     free_cwd (&cwd);
 
     return (0);
@@ -534,7 +567,7 @@ mkmodules (dir)
  * Yeah, I know, there are NFS race conditions here.
  */
 static char *
-make_tempfile ()
+make_tempfile (void)
 {
     static int seed = 0;
     int fd;
@@ -561,9 +594,7 @@ make_tempfile ()
    not a very clean convention).  On success, return 0.  */
 
 static int
-checkout_file (file, temp)
-    char *file;
-    char *temp;
+checkout_file (char *file, char *temp)
 {
     char *rcs;
     RCSNode *rcsnode;
@@ -580,17 +611,7 @@ checkout_file (file, temp)
 	free (rcs);
 	return (1);
     }
-
     rcsnode = RCS_parsercsfile (rcs);
-    if (!rcsnode)
-    {
-	/* Probably not necessary (?); RCS_parsercsfile already printed a
-	   message.  */
-	error (0, 0, "Failed to parse `%s'.", rcs);
-	free (rcs);
-	return 1;
-    }
-
     retcode = RCS_checkout (rcsnode, NULL, NULL, NULL, NULL, temp,
 			    (RCSCHECKOUTPROC) NULL, (void *) NULL);
     if (retcode != 0)
@@ -608,8 +629,7 @@ checkout_file (file, temp)
 #ifndef MY_NDBM
 
 static void
-write_dbmfile (temp)
-    char *temp;
+write_dbmfile( char *temp )
 {
     char line[DBLKSIZ], value[DBLKSIZ];
     FILE *fp;
@@ -716,8 +736,7 @@ write_dbmfile (temp)
 }
 
 static void
-rename_dbmfile (temp)
-    char *temp;
+rename_dbmfile( char *temp )
 {
     /* I think that the size of the buffer needed here is
        just determined by sizeof (CVSROOTADM_MODULES), the
@@ -806,9 +825,7 @@ rename_dbmfile (temp)
 #endif				/* !MY_NDBM */
 
 static void
-rename_rcsfile (temp, real)
-    char *temp;
-    char *real;
+rename_rcsfile (char *temp, char *real)
 {
     char *bak;
     struct stat statbuf;
@@ -853,9 +870,7 @@ const char *const init_usage[] = {
 };
 
 int
-init (argc, argv)
-    int argc;
-    char **argv;
+init (int argc, char **argv)
 {
     /* Name of CVSROOT directory.  */
     char *adm;
