@@ -1,11 +1,6 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
- *
- * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
- *                                  and others.
- *
- * Portions Copyright (C) 1992, Brian Berliner and Jeff Polk
- * Portions Copyright (C) 1989-1992, Brian Berliner
+ *    Copyright (c) 1992, Brian Berliner and Jeff Polk
+ *    Copyright (c) 1989-1992, Brian Berliner
  *
  *    You may distribute under the terms of the GNU General Public License
  *    as specified in the README file that comes with the CVS source
@@ -26,7 +21,6 @@
  *	command line.
  */
 
-#include <assert.h>
 #include "cvs.h"
 #include "savecwd.h"
 
@@ -56,8 +50,8 @@ struct sortrec
     char *comment;
 };
 
-static int sort_order PROTO((const PTR l, const PTR r));
-static void save_d PROTO((char *k, int ks, char *d, int ds));
+static int sort_order (const void *l, const void *r);
+static void save_d (char *k, int ks, char *d, int ds);
 
 
 /*
@@ -66,7 +60,7 @@ static void save_d PROTO((char *k, int ks, char *d, int ds));
  * a warning message is displayed and a NULL is returned.
  */
 DBM *
-open_module ()
+open_module (void)
 {
     char *mfile;
     DBM *retval;
@@ -90,8 +84,7 @@ open_module ()
  * Close the modules file, if the open succeeded, that is
  */
 void
-close_module (db)
-    DBM *db;
+close_module (DBM *db)
 {
     if (db != NULL)
 	dbm_close (db);
@@ -104,22 +97,11 @@ close_module (db)
  * It calls back the passed routine for each directory of a module
  * It runs the post checkout or post tag proc from the modules file
  */
-static int
-my_module (db, mname, m_type, msg, callback_proc, where, shorten,
-	   local_specified, run_module_prog, build_dirs, extra_arg,
-	   stack)
-    DBM *db;
-    char *mname;
-    enum mtype m_type;
-    char *msg;
-    CALLBACKPROC callback_proc;
-    char *where;
-    int shorten;
-    int local_specified;
-    int run_module_prog;
-    int build_dirs;
-    char *extra_arg;
-    List *stack;
+int
+my_module (DBM *db, char *mname, enum mtype m_type, char *msg,
+            CALLBACKPROC callback_proc, char *where, int shorten,
+            int local_specified, int run_module_prog, int build_dirs,
+            char *extra_arg, List *stack)
 {
     char *checkout_prog = NULL;
     char *export_prog = NULL;
@@ -129,7 +111,7 @@ my_module (db, mname, m_type, msg, callback_proc, where, shorten,
     char *line;
     int modargc;
     int xmodargc;
-    char **modargv;
+    char **modargv = NULL;
     char **xmodargv = NULL;
     /* Found entry from modules file, including options and such.  */
     char *value = NULL;
@@ -146,27 +128,12 @@ my_module (db, mname, m_type, msg, callback_proc, where, shorten,
 #ifdef SERVER_SUPPORT
     int restore_server_dir = 0;
     char *server_dir_to_restore = NULL;
-    if (trace)
-    {
-	char *buf;
-
-	/* We use cvs_outerr, rather than fprintf to stderr, because
-	   this may be called by server code with error_use_protocol
-	   set.  */
-	buf = xmalloc (100
-		       + strlen (mname)
-		       + strlen (msg)
-		       + (where ? strlen (where) : 0)
-		       + (extra_arg ? strlen (extra_arg) : 0));
-	sprintf (buf, "%s-> my_module (%s, %s, %s, %s)\n",
-		 CLIENT_SERVER_STR,
-		 mname, msg, where ? where : "",
-		 extra_arg ? extra_arg : "");
-	cvs_outerr (buf, 0);
-	free (buf);
-    }
 #endif
 
+    TRACE (TRACE_FUNCTION, "my_module (%s, %s, %s, %s)",
+           mname ? mname : "(null)", msg ? msg : "(null)",
+           where ? where : "NULL", extra_arg ? extra_arg : "NULL");
+
     /* Don't process absolute directories.  Anything else could be a security
      * problem.  Before this check was put in place:
      *
@@ -320,7 +287,7 @@ my_module (db, mname, m_type, msg, callback_proc, where, shorten,
 	       the case where we found a file/directory rather than
 	       finding an entry in the modules file.  */
 	    if (save_cwd (&cwd))
-		error_exit ();
+		exit (EXIT_FAILURE);
 	    cwd_saved = 1;
 
 	    err += callback_proc (modargc, modargv, where, mwhere, mfile,
@@ -331,7 +298,7 @@ my_module (db, mname, m_type, msg, callback_proc, where, shorten,
 
 	    /* cd back to where we started.  */
 	    if (restore_cwd (&cwd, NULL))
-		error_exit ();
+		exit (EXIT_FAILURE);
 	    free_cwd (&cwd);
 	    cwd_saved = 0;
 
@@ -376,7 +343,6 @@ my_module (db, mname, m_type, msg, callback_proc, where, shorten,
 	    /* mwhere gets just the module name */
 	    mwhere = xstrdup (mname);
 	    mfile = cp + 1;
-	    assert (strlen (mfile));
 
 	    /* put the / back in mname */
 	    *cp = '/';
@@ -402,7 +368,7 @@ my_module (db, mname, m_type, msg, callback_proc, where, shorten,
 
     /* remember where we start */
     if (save_cwd (&cwd))
-	error_exit ();
+	exit (EXIT_FAILURE);
     cwd_saved = 1;
 
     assert (value != NULL);
@@ -498,7 +464,7 @@ my_module (db, mname, m_type, msg, callback_proc, where, shorten,
 
     if (alias && nonalias_opt)
     {
-	/* The documentation has never said it is legal to specify
+	/* The documentation has never said it is valid to specify
 	   -a along with another option.  And I believe that in the past
 	   CVS has ignored the options other than -a, more or less, in this
 	   situation.  */
@@ -695,7 +661,7 @@ module `%s' is a request for a file in a module which is not a directory",
 
     /* cd back to where we started */
     if (restore_cwd (&cwd, NULL))
-	error_exit ();
+	exit (EXIT_FAILURE);
     free_cwd (&cwd);
     cwd_saved = 0;
 
@@ -728,7 +694,7 @@ module `%s' is a request for a file in a module which is not a directory",
 	    }
 
 	    /* XXX can we determine the line number for this entry??? */
-	    expanded_path = expand_path (prog, "modules", 0);
+	    expanded_path = expand_path (prog, "modules", 0, 0);
 	    if (expanded_path != NULL)
 	    {
 		run_setup (expanded_path);
@@ -750,7 +716,7 @@ module `%s' is a request for a file in a module which is not a directory",
 		err += run_exec (RUN_TTY, RUN_TTY, RUN_TTY, RUN_NORMAL);
 		free (expanded_path);
 	    }
-	    if (real_prog) free (real_prog);
+	    free (real_prog);
 	}
     }
 
@@ -782,19 +748,10 @@ module `%s' is a request for a file in a module which is not a directory",
  * accepts a stack argument to track alias recursion.
  */
 int
-do_module (db, mname, m_type, msg, callback_proc, where, shorten,
-	   local_specified, run_module_prog, build_dirs, extra_arg)
-    DBM *db;
-    char *mname;
-    enum mtype m_type;
-    char *msg;
-    CALLBACKPROC callback_proc;
-    char *where;
-    int shorten;
-    int local_specified;
-    int run_module_prog;
-    int build_dirs;
-    char *extra_arg;
+do_module (DBM *db, char *mname, enum mtype m_type, char *msg,
+           CALLBACKPROC callback_proc, char *where, int shorten,
+           int local_specified, int run_module_prog, int build_dirs,
+           char *extra_arg)
 {
     return my_module (db, mname, m_type, msg, callback_proc, where, shorten,
                        local_specified, run_module_prog, build_dirs, extra_arg,
@@ -844,9 +801,7 @@ static char def_status[] = "NONE";
      have to look further.
 */
 static int
-sort_order (l, r)
-    const PTR l;
-    const PTR r;
+sort_order (const void *l, const void *r)
 {
     int i;
     const struct sortrec *left = (const struct sortrec *) l;
@@ -862,11 +817,7 @@ sort_order (l, r)
 }
 
 static void
-save_d (k, ks, d, ds)
-    char *k;
-    int ks;
-    char *d;
-    int ds;
+save_d (char *k, int ks, char *d, int ds)
 {
     char *cp, *cp2;
     struct sortrec *s_rec;
@@ -949,8 +900,7 @@ save_d (k, ks, d, ds)
    non-zero, print out status information for each module. */
 
 void
-cat_module (status)
-    int status;
+cat_module (int status)
 {
     DBM *db;
     datum key, val;
@@ -976,7 +926,7 @@ cat_module (status)
     close_module (db);
 
     /* Sort the list as requested */
-    qsort ((PTR) s_head, s_count, sizeof (struct sortrec), sort_order);
+    qsort ((void *) s_head, s_count, sizeof (struct sortrec), sort_order);
 
     /*
      * Run through the sorted array and format the entries
