@@ -13,7 +13,6 @@
  * size, and this code works fine.
  */
 
-#include <assert.h>
 #include "cvs.h"
 #include "getline.h"
 
@@ -22,16 +21,13 @@
 #   define O_ACCMODE (O_RDONLY | O_WRONLY | O_RDWR)
 # endif /* defined O_ACCMODE */
 
-static void mydbm_load_file PROTO ((FILE *, List *, char *));
+static void mydbm_load_file (FILE *, List *, char *);
 
 /* Returns NULL on error in which case errno has been set to indicate
    the error.  Can also call error() itself.  */
 /* ARGSUSED */
 DBM *
-mydbm_open (file, flags, mode)
-    char *file;
-    int flags;
-    int mode;
+mydbm_open (char *file, int flags, int mode)
 {
     FILE *fp;
     DBM *db;
@@ -55,12 +51,10 @@ mydbm_open (file, flags, mode)
     return (db);
 }
 
-static int write_item PROTO ((Node *, void *));
+static int write_item (Node *, void *);
 
 static int
-write_item (node, data)
-    Node *node;
-    void *data;
+write_item (Node *node, void *data)
 {
     FILE *fp = (FILE *)data;
     fputs (node->key, fp);
@@ -71,8 +65,7 @@ write_item (node, data)
 }
 
 void
-mydbm_close (db)
-    DBM *db;
+mydbm_close (DBM *db)
 {
     if (db->modified)
     {
@@ -90,9 +83,7 @@ mydbm_close (db)
 }
 
 datum
-mydbm_fetch (db, key)
-    DBM *db;
-    datum key;
+mydbm_fetch (DBM *db, datum key)
 {
     Node *p;
     char *s;
@@ -119,8 +110,7 @@ mydbm_fetch (db, key)
 }
 
 datum
-mydbm_firstkey (db)
-    DBM *db;
+mydbm_firstkey (DBM *db)
 {
     Node *head, *p;
     datum key;
@@ -142,8 +132,7 @@ mydbm_firstkey (db)
 }
 
 datum
-mydbm_nextkey (db)
-    DBM *db;
+mydbm_nextkey (DBM *db)
 {
     Node *head, *p;
     datum key;
@@ -169,11 +158,7 @@ mydbm_nextkey (db)
    it gives a warning, rather than either DBM_INSERT or DBM_REPLACE
    behavior.  */
 int
-mydbm_store (db, key, value, flags)
-    DBM *db;
-    datum key;
-    datum value;
-    int flags;
+mydbm_store (DBM *db, datum key, datum value, int flags)
 {
     Node *node;
 
@@ -199,10 +184,10 @@ mydbm_store (db, key, value, flags)
 }
 
 static void
-mydbm_load_file (fp, list, filename)
-    FILE *fp;
-    List *list;
-    char *filename;	/* Used in error messages. */
+mydbm_load_file (FILE *fp, List *list, char *filename)
+             
+               
+                   	/* Used in error messages. */
 {
     char *line = NULL;
     size_t line_size;
@@ -218,8 +203,8 @@ mydbm_load_file (fp, list, filename)
 
     cont = 0;
     line_num=0;
-    while ((line_length = 
-            getstr (&line, &line_size, fp, '\012', 0, GETLINE_NO_LIMIT)) >= 0)
+    while( ( line_length = 
+             getdelim( &line, &line_size, '\012', fp ) ) >= 0 )
     {
 	line_num++;
 	if (line_length > 0 && line[line_length - 1] == '\012')
