@@ -8,23 +8,22 @@
 
 #include "cvs.h"
 #include "getline.h"
-#include <assert.h>
 
 extern char *logHistory;
 
+
+
 /*
  * Parse the INFOFILE file for the specified REPOSITORY.  Invoke CALLPROC for
- * the first line in the file that matches the REPOSITORY, or if ALL != 0, any lines
- * matching "ALL", or if no lines match, the last line matching "DEFAULT".
+ * the first line in the file that matches the REPOSITORY, or if ALL != 0, any
+ * lines matching "ALL", or if no lines match, the last line matching
+ * "DEFAULT".
  *
  * Return 0 for success, -1 if there was not an INFOFILE, and >0 for failure.
  */
 int
-Parse_Info (infofile, repository, callproc, all)
-    char *infofile;
-    char *repository;
-    CALLPROC callproc;
-    int all;
+Parse_Info (const char *infofile, const char *repository, CALLPROC callproc,
+            int opt, void *closure)
 {
     int err = 0;
     FILE *fp_info;
@@ -32,9 +31,11 @@ Parse_Info (infofile, repository, callproc, all)
     char *line = NULL;
     size_t line_allocated = 0;
     char *default_value = NULL;
-    char *expanded_value= NULL;
+    int default_line = 0;
+    char *expanded_value;
     int callback_done, line_number;
-    char *cp, *exp, *value, *srepos, bad;
+    char *cp, *exp, *value;
+    const char *srepos;
     const char *regex_err;
 
     if (current_parsed_root == NULL)
@@ -64,14 +65,8 @@ Parse_Info (infofile, repository, callproc, all)
     /* strip off the CVSROOT if repository was absolute */
     srepos = Short_Repository (repository);
 
-    if (trace)
-	(void) fprintf (stderr, "%s-> Parse_Info (%s, %s, %s)\n",
-#ifdef SERVER_SUPPORT
-			server_active ? "S" : " ",
-#else
-			"",
-#endif
-			infopath, srepos, all ? "ALL" : "not ALL");
+    TRACE ( 1, "Parse_Info (%s, %s, %s)",
+	    infopath, srepos,  (opt & PIOPT_ALL) ? "ALL" : "not ALL");
 
     /* search the info file for lines that match */
     callback_done = line_number = 0;
@@ -114,10 +109,6 @@ Parse_Info (infofile, repository, callproc, all)
 	if ((cp = strrchr (value, '\n')) != NULL)
 	    *cp = '\0';
 
-	if (expanded_value != NULL)
-	    free (expanded_value);
-	expanded_value = expand_path (value, infofile, line_number);
-
 	/*
 	 * At this point, exp points to the regular expression, and value
 	 * points to the value to call the callback routine with.  Evaluate
@@ -128,12 +119,14 @@ Parse_Info (infofile, repository, callproc, all)
 	/* save the default value so we have it later if we need it */
 	if (strcmp (exp, "DEFAULT") == 0)
 	{
-	    /* Is it OK to silently ignore all but the last DEFAULT
-               expression?  */
-	    if (default_value != NULL && default_value != &bad)
+	    if (default_value != NULL)
+	    {
+		error (0, 0, "Multiple `DEFAULT' lines (%d and %d) in %s file",
+		       default_line, line_number, infofile);
 		free (default_value);
-	    default_value = (expanded_value != NULL ?
-			     xstrdup (expanded_value) : &bad);
+	    }
+	    default_value = xstrdup(value);
+	    default_line = line_number;
 	    continue;
 	}
 
@@ -144,11 +137,16 @@ Parse_Info (infofile, repository, callproc, all)
 	 */
 	if (strcmp (exp, "ALL") == 0)
 	{
-	    if (!all)
-		error(0, 0, "Keyword `ALL' is ignored at line %d in %s file",
-		      line_number, infofile);
-	    else if (expanded_value != NULL)
-		err += callproc (repository, expanded_value);
+	    if (!(opt & PIOPT_ALL))
+		error (0, 0, "Keyword `ALL' is ignored at line %d in %s file",
+		       line_number, infofile);
+	    else if ((expanded_value = expand_path (value, infofile,
+	                                            line_number, 1))
+	             != NULL )
+	    {
+		err += callproc (repository, expanded_value, closure);
+		free (expanded_value);
+	    }
 	    else
 		err++;
 	    continue;
@@ -169,8 +167,12 @@ Parse_Info (infofile, repository, callproc, all)
 	    continue;				/* no match */
 
 	/* it did, so do the callback and note that we did one */
-	if (expanded_value != NULL)
-	    err += callproc (repository, expanded_value);
+	if( ( expanded_value = expand_path( value, infofile, line_number, 1 )
+	    ) != NULL )
+	{
+	    err += callproc( repository, expanded_value, closure );
+	    free (expanded_value);
+	}
 	else
 	    err++;
 	callback_done = 1;
@@ -183,17 +185,20 @@ Parse_Info (infofile, repository, callproc, all)
     /* if we fell through and didn't callback at all, do the default */
     if (callback_done == 0 && default_value != NULL)
     {
-	if (default_value != &bad)
-	    err += callproc (repository, default_value);
+	if( ( expanded_value = expand_path( default_value, infofile,
+	                                    line_number, 1 )
+	    ) != NULL )
+	{
+	    err += callproc( repository, expanded_value, closure );
+	    free (expanded_value);
+	}
 	else
 	    err++;
     }
 
     /* free up space if necessary */
-    if (default_value != NULL && default_value != &bad)
+    if (default_value != NULL)
 	free (default_value);
-    if (expanded_value != NULL)
-	free (expanded_value);
     free (infopath);
     if (line != NULL)
 	free (line);
@@ -209,14 +214,14 @@ Parse_Info (infofile, repository, callproc, all)
    KEYWORD=VALUE.  There is currently no way to have a multi-line
    VALUE (would be nice if there was, probably).
 
-   CVSROOT is the $CVSROOT directory (current_parsed_root->directory might not be
-   set yet).
+   CVSROOT is the $CVSROOT directory
+   (current_parsed_root->directory might not be set yet, so this
+   function takes the cvsroot as a function argument).
 
    Returns 0 for success, negative value for failure.  Call
    error(0, ...) on errors in addition to the return value.  */
 int
-parse_config (cvsroot)
-    char *cvsroot;
+parse_config (char *cvsroot)
 {
     char *infopath;
     FILE *fp_info;
@@ -234,7 +239,7 @@ parse_config (cvsroot)
 	return 0;
     parsed = 1;
 
-    infopath = malloc (strlen (cvsroot)
+    infopath = xmalloc (strlen (cvsroot)
 			+ sizeof (CVSROOTADM_CONFIG)
 			+ sizeof (CVSROOTADM)
 			+ 10);
@@ -285,7 +290,7 @@ parse_config (cvsroot)
 	   for making sure the syntax is consistent.  Any good examples
 	   to follow there (Apache?)?  */
 
-	/* Strip the training newline.  There will be one unless we
+	/* Strip the trailing newline.  There will be one unless we
 	   read a partial line without a newline, and then got end of
 	   file (or error?).  */
 
@@ -340,6 +345,16 @@ parse_config (cvsroot)
 		goto error_return;
 	    }
 	}
+	else if (strcmp (line, "LocalKeyword") == 0)
+	{
+	    RCS_setlocalid(p);
+		
+	}
+	else if (strcmp (line, "KeywordExpand") == 0)
+	{
+	    RCS_setincexc(p);
+		
+	}
 	else if (strcmp (line, "PreservePermissions") == 0)
 	{
 	    if (strcmp (p, "no") == 0)
@@ -385,7 +400,7 @@ warning: this CVS does not support PreservePermissions");
 	{
 	    if (strcmp (p, "all") != 0)
 	    {
-		logHistory=malloc(strlen (p) + 1);
+		logHistory=xmalloc(strlen (p) + 1);
 		strcpy (logHistory, p);
 	    }
 	}
@@ -398,6 +413,25 @@ warning: this CVS does not support PreservePermissions");
 	    else if (strcmp (p, "stat") == 0)
 	      RereadLogAfterVerify = LOGMSG_REREAD_STAT;
 	}
+	else if (strcmp (line, "UserAdminOptions") == 0)
+	{
+	    UserAdminOptions = xmalloc(strlen(p) + 1);
+	    strcpy(UserAdminOptions, p);
+	}
+#ifdef SUPPORT_OLD_INFO_FMT_STRINGS
+	else if (strcmp (line, "UseNewInfoFmtStrings") == 0)
+	{
+	    if (strcmp (p, "no") == 0)
+		UseNewInfoFmtStrings = 0;
+	    else if (strcmp (p, "yes") == 0)
+		UseNewInfoFmtStrings = 1;
+	    else
+	    {
+		error (0, 0, "unrecognized value '%s' for UseNewInfoFmtStrings", p);
+		goto error_return;
+	    }
+	}
+#endif /* SUPPORT_OLD_INFO_FMT_STRINGS */
 	else
 	{
 	    /* We may be dealing with a keyword which was added in a
