@@ -1,11 +1,6 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
- *
- * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
- *                                  and others.
- *
- * Portions Copyright (C) 1992, Brian Berliner and Jeff Polk
- * Portions Copyright (C) 1989-1992, Brian Berliner
+ * Copyright (c) 1992, Brian Berliner and Jeff Polk
+ * Copyright (c) 1989-1992, Brian Berliner
  * 
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with the CVS source distribution.
@@ -17,18 +12,17 @@
  * release as either a date or a revision number.
  */
 
-#include <assert.h>
 #include "cvs.h"
 #include "getline.h"
 
-static RETSIGTYPE patch_cleanup PROTO((void));
-static Dtype patch_dirproc PROTO ((void *callerdat, const char *dir,
-				   const char *repos, const char *update_dir,
-				   List *entries));
-static int patch_fileproc PROTO ((void *callerdat, struct file_info *finfo));
-static int patch_proc PROTO((int argc, char **argv, char *xwhere,
+static RETSIGTYPE patch_cleanup (void);
+static Dtype patch_dirproc (void *callerdat, const char *dir,
+                            const char *repos, const char *update_dir,
+                            List *entries);
+static int patch_fileproc (void *callerdat, struct file_info *finfo);
+static int patch_proc (int argc, char **argv, char *xwhere,
 		       char *mwhere, char *mfile, int shorten,
-		       int local_specified, char *mname, char *msg));
+		       int local_specified, char *mname, char *msg);
 
 static int force_tag_match = 1;
 static int patch_short = 0;
@@ -47,7 +41,7 @@ static int unidiff = 0;
 
 static const char *const patch_usage[] =
 {
-    "Usage: %s %s [-flR] [-c|-u] [-s|-t] [-V %%d] [-k kopt]\n",
+    "Usage: %s %s [-flR] [-c|-u] [-s|-t] [-V %%d]\n",
     "    -r rev|-D date [-r rev2 | -D date2] modules...\n",
     "\t-f\tForce a head revision match if tag/date not found.\n",
     "\t-l\tLocal directory only, not recursive\n",
@@ -56,10 +50,9 @@ static const char *const patch_usage[] =
     "\t-u\tUnidiff format.\n",
     "\t-s\tShort patch - one liner per file.\n",
     "\t-t\tTop two diffs - last change made to the file.\n",
-    "\t-V vers\tUse RCS Version \"vers\" for keyword expansion.\n",
-    "\t-k kopt\tSpecify keyword expansion mode.\n",
     "\t-D date\tDate.\n",
     "\t-r rev\tRevision - symbolic or numeric.\n",
+    "\t-V vers\tUse RCS Version \"vers\" for keyword expansion.\n",
     "(Specify the --help global option for a list of other help options)\n",
     NULL
 };
@@ -67,9 +60,7 @@ static const char *const patch_usage[] =
 
 
 int
-patch (argc, argv)
-    int argc;
-    char **argv;
+patch (int argc, char **argv)
 {
     register int i;
     int local = 0;
@@ -87,9 +78,11 @@ patch (argc, argv)
 	{
 	    case 'Q':
 	    case 'q':
+#ifdef SERVER_SUPPORT
 		/* The CVS 1.5 client sends these options (in addition to
 		   Global_option requests), so we must ignore them.  */
 		if (!server_active)
+#endif
 		    error (1, 0,
 			   "-q or -Q must be specified before \"%s\"",
 			   cvs_cmd_name);
@@ -273,17 +266,8 @@ patch (argc, argv)
  */
 /* ARGSUSED */
 static int
-patch_proc (argc, argv, xwhere, mwhere, mfile, shorten, local_specified,
-	    mname, msg)
-    int argc;
-    char **argv;
-    char *xwhere;
-    char *mwhere;
-    char *mfile;
-    int shorten;
-    int local_specified;
-    char *mname;
-    char *msg;
+patch_proc (int argc, char **argv, char *xwhere, char *mwhere, char *mfile,
+            int shorten, int local_specified, char *mname, char *msg)
 {
     char *myargv[2];
     int err = 0;
@@ -291,12 +275,21 @@ patch_proc (argc, argv, xwhere, mwhere, mfile, shorten, local_specified,
     char *repository;
     char *where;
 
+    TRACE ( TRACE_FUNCTION, "patch_proc ( %s, %s, %s, %d, %d, %s, %s )",
+	    xwhere ? xwhere : "(null)",
+	    mwhere ? mwhere : "(null)",
+	    mfile ? mfile : "(null)",
+	    shorten, local_specified,
+	    mname ? mname : "(null)",
+	    msg ? msg : "(null)" );
+
     repository = xmalloc (strlen (current_parsed_root->directory)
                           + strlen (argv[0])
-			  + (mfile == NULL ? 0 : strlen (mfile) + 1) + 2);
+                          + (mfile == NULL ? 0 : strlen (mfile) + 1) + 2);
     (void)sprintf (repository, "%s/%s",
                    current_parsed_root->directory, argv[0]);
-    where = xmalloc (strlen (argv[0]) + (mfile == NULL ? 0 : strlen (mfile) + 1)
+    where = xmalloc (strlen (argv[0])
+                     + (mfile == NULL ? 0 : strlen (mfile) + 1)
 		     + 1);
     (void)strcpy (where, argv[0]);
 
@@ -338,11 +331,10 @@ patch_proc (argc, argv, xwhere, mwhere, mfile, shorten, local_specified,
     }
 
     /* cd to the starting repository */
-    if ( CVS_CHDIR (repository) < 0)
+    if (CVS_CHDIR (repository) < 0)
     {
 	error (0, errno, "cannot chdir to %s", repository);
 	free (repository);
-	free (where);
 	return 1;
     }
 
@@ -365,10 +357,9 @@ patch_proc (argc, argv, xwhere, mwhere, mfile, shorten, local_specified,
     }
 
     /* start the recursion processor */
-    err = start_recursion (patch_fileproc, (FILESDONEPROC)NULL, patch_dirproc,
-			   (DIRLEAVEPROC)NULL, NULL,
+    err = start_recursion (patch_fileproc, NULL, patch_dirproc, NULL, NULL,
 			   argc - 1, argv + 1, local_specified,
-			   which, 0, CVS_LOCK_READ, where, 1, repository);
+			   which, 0, CVS_LOCK_READ, where, 1, repository );
     free (repository);
     free (where);
 
@@ -383,14 +374,11 @@ patch_proc (argc, argv, xwhere, mwhere, mfile, shorten, local_specified,
  */
 /* ARGSUSED */
 static int
-patch_fileproc (callerdat, finfo)
-    void *callerdat;
-    struct file_info *finfo;
+patch_fileproc (void *callerdat, struct file_info *finfo)
 {
     struct utimbuf t;
     char *vers_tag, *vers_head;
     char *rcs = NULL;
-    char *rcs_orig = NULL;
     RCSNode *rcsfile;
     FILE *fp1, *fp2, *fp3;
     int ret = 0;
@@ -405,9 +393,6 @@ patch_fileproc (callerdat, finfo)
     char *cp1, *cp2;
     FILE *fp;
     int line_length;
-    int dargc = 0;
-    size_t darg_allocated = 0;
-    char **dargv = NULL;
 
     line1 = NULL;
     line1_chars_allocated = 0;
@@ -424,8 +409,8 @@ patch_fileproc (callerdat, finfo)
     if ((rcsfile->flags & VALID) && (rcsfile->flags & INATTIC))
 	isattic = 1;
 
-    rcs_orig = rcs = xmalloc (strlen (finfo->file) + sizeof (RCSEXT) + 5);
-    (void) sprintf (rcs, "%s%s", finfo->file, RCSEXT);
+    rcs = xmalloc (strlen (finfo->file) + sizeof (RCSEXT) + 5);
+    (void)sprintf (rcs, "%s%s", finfo->file, RCSEXT);
 
     /* if vers_head is NULL, may have been removed from the release */
     if (isattic && rev2 == NULL && date2 == NULL)
@@ -433,7 +418,7 @@ patch_fileproc (callerdat, finfo)
     else
     {
 	vers_head = RCS_getversion (rcsfile, rev2, date2, force_tag_match,
-				    (int *) NULL);
+				    NULL);
 	if (vers_head != NULL && RCS_isdead (rcsfile, vers_head))
 	{
 	    free (vers_head);
@@ -461,8 +446,7 @@ patch_fileproc (callerdat, finfo)
 	    goto out2;
 	}
     }
-    vers_tag = RCS_getversion (rcsfile, rev1, date1, force_tag_match,
-			       (int *) NULL);
+    vers_tag = RCS_getversion (rcsfile, rev1, date1, force_tag_match, NULL);
     if (vers_tag != NULL && RCS_isdead (rcsfile, vers_tag))
     {
         free (vers_tag);
@@ -480,7 +464,7 @@ patch_fileproc (callerdat, finfo)
 	goto out2;
     }
 
-    if( patch_short && ( vers_tag == NULL || vers_head == NULL ) )
+    if (patch_short && (vers_tag == NULL || vers_head == NULL))
     {
 	/* For adds & removes with a short patch requested, we can print our
 	 * error message now and get out.
@@ -489,18 +473,18 @@ patch_fileproc (callerdat, finfo)
 	cvs_output (finfo->fullname, 0);
 	if (vers_tag == NULL)
 	{
-	    cvs_output( " is new; ", 0 );
-	    cvs_output( rev2 ? rev2 : date2 ? date2 : "current", 0 );
-	    cvs_output( " revision ", 0 );
+	    cvs_output (" is new; ", 0);
+	    cvs_output (rev2 ? rev2 : date2 ? date2 : "current", 0);
+	    cvs_output (" revision ", 0);
 	    cvs_output (vers_head, 0);
 	    cvs_output ("\n", 1);
 	}
 	else
 	{
-	    cvs_output( " is removed; ", 0 );
-	    cvs_output( rev1 ? rev1 : date1, 0 );
-	    cvs_output( " revision ", 0 );
-	    cvs_output( vers_tag, 0 );
+	    cvs_output (" is removed; ", 0);
+	    cvs_output (rev1 ? rev1 : date1, 0);
+	    cvs_output (" revision ", 0);
+	    cvs_output (vers_tag, 0);
 	    cvs_output ("\n", 1);
 	}
 	ret = 0;
@@ -518,8 +502,7 @@ patch_fileproc (callerdat, finfo)
      */
     if ((fp1 = cvs_temp_file (&tmpfile1)) == NULL)
     {
-	error (0, errno, "cannot create temporary file %s",
-	       tmpfile1 ? tmpfile1 : "(null)");
+	error (0, errno, "cannot create temporary file %s", tmpfile1);
 	ret = 1;
 	goto out;
     }
@@ -528,8 +511,7 @@ patch_fileproc (callerdat, finfo)
 	    error (0, errno, "warning: cannot close %s", tmpfile1);
     if ((fp2 = cvs_temp_file (&tmpfile2)) == NULL)
     {
-	error (0, errno, "cannot create temporary file %s",
-	       tmpfile2 ? tmpfile2 : "(null)");
+	error (0, errno, "cannot create temporary file %s", tmpfile2);
 	ret = 1;
 	goto out;
     }
@@ -538,8 +520,7 @@ patch_fileproc (callerdat, finfo)
 	    error (0, errno, "warning: cannot close %s", tmpfile2);
     if ((fp3 = cvs_temp_file (&tmpfile3)) == NULL)
     {
-	error (0, errno, "cannot create temporary file %s",
-	       tmpfile3 ? tmpfile3 : "(null)");
+	error (0, errno, "cannot create temporary file %s", tmpfile3);
 	ret = 1;
 	goto out;
     }
@@ -549,9 +530,8 @@ patch_fileproc (callerdat, finfo)
 
     if (vers_tag != NULL)
     {
-	retcode = RCS_checkout (rcsfile, (char *)NULL, vers_tag,
-				rev1, options, tmpfile1,
-				(RCSCHECKOUTPROC)NULL, (void *)NULL);
+	retcode = RCS_checkout (rcsfile, NULL, vers_tag, rev1, options,
+                                tmpfile1, NULL, NULL);
 	if (retcode != 0)
 	{
 	    error (0, 0,
@@ -564,7 +544,7 @@ patch_fileproc (callerdat, finfo)
 						    (char *) 0, 0)) != -1)
 	    /* I believe this timestamp only affects the dates in our diffs,
 	       and therefore should be on the server, not the client.  */
-	    (void) utime (tmpfile1, &t);
+	    (void)utime (tmpfile1, &t);
     }
     else if (toptwo_diffs)
     {
@@ -573,9 +553,8 @@ patch_fileproc (callerdat, finfo)
     }
     if (vers_head != NULL)
     {
-	retcode = RCS_checkout (rcsfile, (char *)NULL, vers_head,
-				rev2, options, tmpfile2,
-				(RCSCHECKOUTPROC)NULL, (void *)NULL);
+	retcode = RCS_checkout (rcsfile, NULL, vers_head, rev2, options,
+                                tmpfile2, NULL, NULL);
 	if (retcode != 0)
 	{
 	    error (0, 0,
@@ -590,10 +569,8 @@ patch_fileproc (callerdat, finfo)
 	    (void)utime (tmpfile2, &t);
     }
 
-    if (unidiff) run_add_arg_p (&dargc, &darg_allocated, &dargv, "-u");
-    else run_add_arg_p (&dargc, &darg_allocated, &dargv, "-c");
-    switch (diff_exec (tmpfile1, tmpfile2, NULL, NULL, dargc, dargv,
-		       tmpfile3))
+    switch (diff_exec (tmpfile1, tmpfile2, NULL, NULL, unidiff ? "-u" : "-c",
+                       tmpfile3))
     {
 	case -1:			/* fork/wait failure */
 	    error (1, errno, "fork for diff failed on %s", rcs);
@@ -608,7 +585,7 @@ patch_fileproc (callerdat, finfo)
 	     * user wanted a short patch.  In that case, just output the short
 	     * message.
 	     */
-	    if( patch_short )
+	    if (patch_short)
 	    {
 		cvs_output ("File ", 0);
 		cvs_output (finfo->fullname, 0);
@@ -755,41 +732,24 @@ failed to read diff file header %s for %s: end of file", tmpfile3, rcs);
         free (line1);
     if (line2)
         free (line2);
-    if (tmpfile1 != NULL)
-    {
-	if (CVS_UNLINK (tmpfile1) < 0)
-	    error (0, errno, "cannot unlink %s", tmpfile1);
-	free (tmpfile1);
-	tmpfile1 = NULL;
-    }
-    if (tmpfile2 != NULL)
-    {
-	if (CVS_UNLINK (tmpfile2) < 0)
-	    error (0, errno, "cannot unlink %s", tmpfile2);
-	free (tmpfile2);
-	tmpfile2 = NULL;
-    }
-    if (tmpfile3 != NULL)
-    {
-	if (CVS_UNLINK (tmpfile3) < 0)
-	    error (0, errno, "cannot unlink %s", tmpfile3);
-	free (tmpfile3);
-	tmpfile3 = NULL;
-    }
-
-    if (dargc)
-    {
-	run_arg_free_p (dargc, dargv);
-	free (dargv);
-    }
+    if (CVS_UNLINK (tmpfile1) < 0)
+	error (0, errno, "cannot unlink %s", tmpfile1);
+    if (CVS_UNLINK (tmpfile2) < 0)
+	error (0, errno, "cannot unlink %s", tmpfile2);
+    if (CVS_UNLINK (tmpfile3) < 0)
+	error (0, errno, "cannot unlink %s", tmpfile3);
+    free (tmpfile1);
+    free (tmpfile2);
+    free (tmpfile3);
+    tmpfile1 = tmpfile2 = tmpfile3 = NULL;
 
  out2:
     if (vers_tag != NULL)
 	free (vers_tag);
     if (vers_head != NULL)
 	free (vers_head);
-    if (rcs_orig)
-	free (rcs_orig);
+    if (rcs != NULL)
+	free (rcs);
     return ret;
 }
 
@@ -800,23 +760,21 @@ failed to read diff file header %s for %s: end of file", tmpfile3, rcs);
  */
 /* ARGSUSED */
 static Dtype
-patch_dirproc (callerdat, dir, repos, update_dir, entries)
-    void *callerdat;
-    const char *dir;
-    const char *repos;
-    const char *update_dir;
-    List *entries;
+patch_dirproc (void *callerdat, const char *dir, const char *repos,
+               const char *update_dir, List *entries)
 {
     if (!quiet)
 	error (0, 0, "Diffing %s", update_dir);
-    return (R_PROCESS);
+    return R_PROCESS;
 }
 
+
+
 /*
  * Clean up temporary files
  */
 static RETSIGTYPE
-patch_cleanup ()
+patch_cleanup (void)
 {
     /* Note that the checks for existence_error are because we are
        called from a signal handler, without SIG_begincrsect, so
