@@ -10,7 +10,6 @@
  */
 
 #include "cvs.h"
-#include <assert.h>
 #include <stdio.h>
 #include "diffrun.h"
 
@@ -51,8 +50,8 @@
    On a related note, see the comments at diff_exec, later in this file,
    for more on the diff library.  */
 
-static void RCS_output_diff_options PROTO ((const char *, const char *,
-                                            const char *, const char *));
+static void RCS_output_diff_options (const char *, const char *, const char *,
+                                     const char *);
 
 
 /* Stuff to deal with passing arguments the way libdiff.a wants to deal
@@ -73,20 +72,19 @@ static char **call_diff_argv;
 static int call_diff_argc;
 static int call_diff_argc_allocated;
 
-static void call_diff_add_arg PROTO ((const char *));
-static void call_diff_setup PROTO ((const char *prog));
-static int call_diff PROTO ((const char *out));
-static int call_diff3 PROTO ((char *out));
+static void call_diff_add_arg (const char *);
+static void call_diff_setup (const char *prog);
+static int call_diff (const char *out);
+static int call_diff3 (char *out);
 
-static void call_diff_write_output PROTO((const char *, size_t));
-static void call_diff_flush_output PROTO((void));
-static void call_diff_write_stdout PROTO((const char *));
-static void call_diff_error PROTO((const char *, const char *, const char *));
+static void call_diff_write_output (const char *, size_t);
+static void call_diff_flush_output (void);
+static void call_diff_write_stdout (const char *);
+static void call_diff_error (const char *, const char *, const char *);
 
 /* VARARGS */
 static void 
-call_diff_setup (prog)
-    const char *prog;
+call_diff_setup (const char *prog)
 {
     char *cp;
     int i;
@@ -114,15 +112,13 @@ call_diff_setup (prog)
 }
 
 static void
-call_diff_arg (s)
-    const char *s;
+call_diff_arg (const char *s)
 {
     call_diff_add_arg (s);
 }
 
 static void
-call_diff_add_arg (s)
-    const char *s;
+call_diff_add_arg (const char *s)
 {
     /* allocate more argv entries if we've run out */
     if (call_diff_argc >= call_diff_argc_allocated)
@@ -144,9 +140,7 @@ call_diff_add_arg (s)
    file.  This is used when we are producing output to stdout.  */
 
 static void
-call_diff_write_output (text, len)
-    const char *text;
-    size_t len;
+call_diff_write_output (const char *text, size_t len)
 {
     if (len > 0)
 	cvs_output (text, len);
@@ -156,7 +150,7 @@ call_diff_write_output (text, len)
    This is used when we are producing output to stdout.  */
 
 static void
-call_diff_flush_output ()
+call_diff_flush_output (void)
 {
     cvs_flushout ();
 }
@@ -164,8 +158,7 @@ call_diff_flush_output ()
 /* Call back function for the diff library to write to stdout.  */
 
 static void
-call_diff_write_stdout (text)
-    const char *text;
+call_diff_write_stdout (const char *text)
 {
     cvs_output (text, 0);
 }
@@ -173,10 +166,7 @@ call_diff_write_stdout (text)
 /* Call back function for the diff library to write to stderr.  */
 
 static void
-call_diff_error (format, a1, a2)
-    const char *format;
-    const char *a1;
-    const char *a2;
+call_diff_error (const char *format, const char *a1, const char *a2)
 {
     /* FIXME: Should we somehow indicate that this error is coming from
        the diff library?  */
@@ -199,8 +189,8 @@ static struct diff_callbacks call_diff_stdout_callbacks =
 
 static struct diff_callbacks call_diff_file_callbacks =
 {
-    (void (*) PROTO((const char *, size_t))) NULL,
-    (void (*) PROTO((void))) NULL,
+    (void (*) (const char *, size_t)) NULL,
+    (void (*) (void)) NULL,
     call_diff_write_stdout,
     call_diff_error
 };
@@ -208,22 +198,20 @@ static struct diff_callbacks call_diff_file_callbacks =
 
 
 static int
-call_diff (out)
-    const char *out;
+call_diff (const char *out)
 {
     if (out == RUN_TTY)
-	return diff_run (call_diff_argc, call_diff_argv, NULL,
-			 &call_diff_stdout_callbacks);
+	return diff_run( call_diff_argc, call_diff_argv, NULL,
+			 &call_diff_stdout_callbacks );
     else
-	return diff_run (call_diff_argc, call_diff_argv, out,
-			 &call_diff_file_callbacks);
+	return diff_run( call_diff_argc, call_diff_argv, out,
+			 &call_diff_file_callbacks );
 }
 
 
 
 static int
-call_diff3 (out)
-    char *out;
+call_diff3 (char *out)
 {
     if (out == RUN_TTY)
 	return diff3_run (call_diff_argc, call_diff_argv, NULL,
@@ -238,13 +226,8 @@ call_diff3 (out)
 /* Merge revisions REV1 and REV2. */
 
 int
-RCS_merge(rcs, path, workfile, options, rev1, rev2)
-    RCSNode *rcs;
-    const char *path;
-    const char *workfile;
-    const char *options;
-    const char *rev1;
-    const char *rev2;
+RCS_merge (RCSNode *rcs, const char *path, const char *workfile,
+           const char *options, const char *rev1, const char *rev2)
 {
     char *xrev1, *xrev2;
     char *tmp1, *tmp2;
@@ -259,7 +242,9 @@ RCS_merge(rcs, path, workfile, options, rev1, rev2)
     cvs_output ("\n", 1);
 
     /* Calculate numeric revision numbers from rev1 and rev2 (may be
-       symbolic). */
+       symbolic).
+       FIXME - No they can't.  Both calls to RCS_merge are passing in
+       numeric revisions.  */
     xrev1 = RCS_gettag (rcs, rev1, 0, NULL);
     xrev2 = RCS_gettag (rcs, rev2, 0, NULL);
 
@@ -275,7 +260,7 @@ RCS_merge(rcs, path, workfile, options, rev1, rev2)
 		      (RCSCHECKOUTPROC)0, NULL))
     {
 	cvs_outerr ("rcsmerge: co failed\n", 0);
-	error_exit();
+	exit (EXIT_FAILURE);
     }
 
     cvs_output ("retrieving revision ", 0);
@@ -287,7 +272,7 @@ RCS_merge(rcs, path, workfile, options, rev1, rev2)
 		      (RCSCHECKOUTPROC)0, NULL))
     {
 	cvs_outerr ("rcsmerge: co failed\n", 0);
-	error_exit();
+	exit (EXIT_FAILURE);
     }
 
     /* Merge changes. */
@@ -323,7 +308,7 @@ RCS_merge(rcs, path, workfile, options, rev1, rev2)
     if (retval == 1)
 	cvs_outerr ("rcsmerge: warning: conflicts during merge\n", 0);
     else if (retval == 2)
-	error_exit();
+	exit (EXIT_FAILURE);
 
     if (diffout)
 	copy_file (diffout, workfile);
@@ -382,17 +367,9 @@ RCS_merge(rcs, path, workfile, options, rev1, rev2)
    about this--any such features are undocumented in the context of
    CVS, and I'm not sure how important to users.  */
 int
-RCS_exec_rcsdiff(rcsfile, opts, options, rev1, rev1_cache, rev2,
-                 label1, label2, workfile )
-    RCSNode *rcsfile;
-    const char *opts;
-    const char *options;
-    const char *rev1;
-    const char *rev1_cache;
-    const char *rev2;
-    const char *label1;
-    const char *label2;
-    const char *workfile;
+RCS_exec_rcsdiff (RCSNode *rcsfile, const char *opts, const char *options,
+                  const char *rev1, const char *rev1_cache, const char *rev2,
+                  const char *label1, const char *label2, const char *workfile)
 {
     char *tmpfile1 = NULL;
     char *tmpfile2 = NULL;
@@ -545,13 +522,8 @@ RCS file: ", 0);
    message on stderr.  */
 
 int
-diff_exec (file1, file2, label1, label2, options, out)
-    const char *file1;
-    const char *file2;
-    const char *label1;
-    const char *label2;
-    const char *options;
-    const char *out;
+diff_exec (const char *file1, const char *file2, const char *label1,
+           const char *label2, const char *options, const char *out)
 {
     char *args;
 
@@ -611,11 +583,8 @@ diff_exec (file1, file2, label1, label2, options, out)
    that I have seen. */
 
 static void
-RCS_output_diff_options (opts, rev1, rev2, workfile)
-    const char *opts;
-    const char *rev1;
-    const char *rev2;
-    const char *workfile;
+RCS_output_diff_options (const char *opts, const char *rev1, const char *rev2,
+                         const char *workfile)
 {
     char *tmp;
 
