@@ -1,24 +1,23 @@
 /*
  * Copyright (c) 1992, Brian Berliner and Jeff Polk
- * 
+ *
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with the CVS source distribution.
- * 
+ *
  * General recursion handler
- * 
+ *
  */
 
 #include "cvs.h"
 #include "savecwd.h"
 #include "fileattr.h"
 #include "edit.h"
-#include <assert.h>
 
-static int do_dir_proc PROTO((Node * p, void *closure));
-static int do_file_proc PROTO((Node * p, void *closure));
-static void addlist PROTO((List ** listp, char *key));
-static int unroll_files_proc PROTO((Node *p, void *closure));
-static void addfile PROTO((List **listp, char *dir, char *file));
+static int do_dir_proc (Node * p, void *closure);
+static int do_file_proc (Node * p, void *closure);
+static void addlist (List ** listp, char *key);
+static int unroll_files_proc (Node *p, void *closure);
+static void addfile (List **listp, char *dir, char *file);
 
 static char *update_dir;
 static char *repository = NULL;
@@ -39,7 +38,7 @@ struct recursion_frame {
     char *repository;			/* Keep track of repository for rtag */
 };
 
-static int do_recursion PROTO ((struct recursion_frame *frame));
+static int do_recursion (struct recursion_frame *frame);
 
 /* I am half tempted to shove a struct file_info * into the struct
    recursion_frame (but then we would need to modify or create a
@@ -62,60 +61,97 @@ struct frame_and_entries {
 
 
 /* Start a recursive command.
-
-   Command line arguments (ARGC, ARGV) dictate the directories and
-   files on which we operate.  In the special case of no arguments, we
-   default to ".".  */
+ *
+ * INPUT
+ *
+ *   fileproc
+ *     Function called with each file as an argument.
+ *
+ *   filesdoneproc
+ *     Function called after all the files in a directory have been processed,
+ *     before subdirectories have been processed.
+ *
+ *   direntproc
+ *     Function called immediately upon entering a directory, before processing
+ *     any files or subdirectories.
+ *
+ *   dirleaveproc
+ *     Function called upon finishing a directory, immediately before leaving
+ *     it and returning control to the function processing the parent
+ *     directory.
+ *
+ *   callerdat
+ *   	This void * is passed to the functions above.
+ *
+ *   argc, argv
+ *     The files on which to operate, interpreted as command line arguments.
+ *     In the special case of no arguments, defaults to operating on the
+ *     current directory (`.').
+ *
+ *   local
+ *
+ *   which
+ *     specifies the kind of recursion.  There are several cases:
+ *
+ *       1.  W_LOCAL is not set but W_REPOS or W_ATTIC is.  The current
+ *       directory when we are called must be the repository and
+ *       recursion proceeds according to what exists in the repository.
+ *
+ *       2a.  W_LOCAL is set but W_REPOS and W_ATTIC are not.  The
+ *       current directory when we are called must be the working
+ *       directory.  Recursion proceeds according to what exists in the
+ *       working directory, never (I think) consulting any part of the
+ *       repository which does not correspond to the working directory
+ *       ("correspond" == Name_Repository).
+ *
+ *       2b.  W_LOCAL is set and so is W_REPOS or W_ATTIC.  This is the
+ *       weird one.  The current directory when we are called must be
+ *       the working directory.  We recurse through working directories,
+ *       but we recurse into a directory if it is exists in the working
+ *       directory *or* it exists in the repository.  If a directory
+ *       does not exist in the working directory, the direntproc must
+ *       either tell us to skip it (R_SKIP_ALL), or must create it (I
+ *       think those are the only two cases).
+ *
+ *   aflag
+ *   locktype
+ *   update_preload
+ *   dosrcs
+ *
+ *   repository_in
+ *     keeps track of the repository string.  This is only for the remote mode,
+ *     specifically, r* commands (rtag, rdiff, co, ...) where xgetwd() was used
+ *     to locate the repository.  Things would break when xgetwd() was used
+ *     with a symlinked repository because xgetwd() would return the true path
+ *     and in some cases this would cause the path to be printed as other than
+ *     the user specified in error messages and in other cases some of CVS's
+ *     security assertions would fail.
+ *
+ * GLOBALS
+ *   ???
+ *
+ * OUTPUT
+ *
+ *  callerdat can be modified by the FILEPROC, FILESDONEPROC, DIRENTPROC, and
+ *  DIRLEAVEPROC.
+ *
+ * RETURNS
+ *   A count of errors counted by walking the argument list with
+ *   unroll_files_proc() and do_recursion().
+ *
+ * ERRORS
+ *   Fatal errors occur:
+ *     1.  when there were no arguments and the current directory
+ *         does not contain CVS metadata.
+ *     2.  when all but the last path element from an argument from ARGV cannot
+ *         be found to be a local directory.
+ */
 int
-start_recursion (fileproc, filesdoneproc, direntproc, dirleaveproc, callerdat,
-		 argc, argv, local, which, aflag, locktype,
-		 update_preload, dosrcs, repository_in)
-    FILEPROC fileproc;
-    FILESDONEPROC filesdoneproc;
-    DIRENTPROC 	direntproc;
-    DIRLEAVEPROC dirleaveproc;
-    void *callerdat;
-
-    int argc;
-    char **argv;
-    int local;
-
-    /* This specifies the kind of recursion.  There are several cases:
-
-       1.  W_LOCAL is not set but W_REPOS or W_ATTIC is.  The current
-       directory when we are called must be the repository and
-       recursion proceeds according to what exists in the repository.
-
-       2a.  W_LOCAL is set but W_REPOS and W_ATTIC are not.  The
-       current directory when we are called must be the working
-       directory.  Recursion proceeds according to what exists in the
-       working directory, never (I think) consulting any part of the
-       repository which does not correspond to the working directory
-       ("correspond" == Name_Repository).
-
-       2b.  W_LOCAL is set and so is W_REPOS or W_ATTIC.  This is the
-       weird one.  The current directory when we are called must be
-       the working directory.  We recurse through working directories,
-       but we recurse into a directory if it is exists in the working
-       directory *or* it exists in the repository.  If a directory
-       does not exist in the working directory, the direntproc must
-       either tell us to skip it (R_SKIP_ALL), or must create it (I
-       think those are the only two cases).  */
-    int which;
-
-    int aflag;
-    int locktype;
-    char *update_preload;
-    int dosrcs;
-    /* Keep track of the repository string.  This is only for the remote mode,
-     * specifically, r* commands (rtag, rdiff, co, ...) where xgetwd() was
-     * used to locate the repository.  Things would break when xgetwd() was
-     * used with a symlinked repository because xgetwd() would return the true
-     * path and in some cases this would cause the path to be printed as other
-     * than the user specified in error messages and in other cases some of
-     * CVS's security assertions would fail.
-     */
-    char *repository_in;
+start_recursion (FILEPROC fileproc, FILESDONEPROC filesdoneproc,
+                 DIRENTPROC direntproc, DIRLEAVEPROC dirleaveproc,
+                 void *callerdat, int argc, char **argv, int local,
+                 int which, int aflag, int locktype,
+                 char *update_preload, int dosrcs, char *repository_in)
 {
     int i, err = 0;
 #ifdef CLIENT_SUPPORT
@@ -124,6 +160,36 @@ start_recursion (fileproc, filesdoneproc, direntproc, dirleaveproc, callerdat,
     List *files_by_dir = NULL;
     struct recursion_frame frame;
 
+#ifdef HAVE_PRINTF_PTR
+    TRACE ( TRACE_FLOW,
+	    "start_recursion ( fileproc=%p, filesdoneproc=%p,\n"
+       "                       direntproc=%p, dirleavproc=%p,\n"
+       "                       callerdat=%p, argc=%d, argv=%p,\n"
+       "                       local=%d, which=%d, aflag=%d,\n"
+       "                       locktype=%d, update_preload=%s\n"
+       "                       dosrcs=%d, repository_in=%s )",
+	       (void *) fileproc, (void *) filesdoneproc,
+	       (void *) direntproc, (void *) dirleaveproc,
+	       (void *) callerdat, argc, (void *) argv,
+	       local, which, aflag, locktype,
+	       update_preload ? update_preload : "(null)", dosrcs,
+	       repository_in ? repository_in : "(null)");
+#else
+    TRACE ( TRACE_FLOW,
+	    "start_recursion ( fileproc=%lx, filesdoneproc=%lx,\n"
+       "                       direntproc=%lx, dirleavproc=%lx,\n"
+       "                       callerdat=%lx, argc=%d, argv=%lx,\n"
+       "                       local=%d, which=%d, aflag=%d,\n"
+       "                       locktype=%d, update_preload=%s\n"
+       "                       dosrcs=%d, repository_in=%s )",
+	       (unsigned long) fileproc, (unsigned long) filesdoneproc,
+	       (unsigned long) direntproc, (unsigned long) dirleaveproc,
+	       (unsigned long) callerdat, argc, (unsigned long) argv,
+	       local, which, aflag, locktype,
+	       update_preload ? update_preload : "(null)", dosrcs,
+	       repository_in ? repository_in : "(null)");
+#endif
+
     frame.fileproc = fileproc;
     frame.filesdoneproc = filesdoneproc;
     frame.direntproc = direntproc;
@@ -134,25 +200,6 @@ start_recursion (fileproc, filesdoneproc, direntproc, dirleaveproc, callerdat,
     frame.aflag = aflag;
     frame.locktype = locktype;
     frame.dosrcs = dosrcs;
-
-    /* If our repository_in has a trailing "/.", remove it before storing it
-     * for do_recursion().
-     *
-     * FIXME: This is somewhat of a hack in the sense that many of our callers
-     * painstakingly compute and add the trailing '.' we now remove.
-     */
-    while (repository_in && strlen (repository_in) >= 2
-           && repository_in[strlen (repository_in) - 2] == '/'
-           && repository_in[strlen (repository_in) - 1] == '.')
-    {
-	/* Beware the case where the string is exactly "/." or "//.".
-	 * Paths with a leading "//" are special on some early UNIXes.
-	 */
-	if (strlen (repository_in) == 2 || strlen (repository_in) == 3)
-	    repository_in[strlen (repository_in) - 1] = '\0';
-	else
-	    repository_in[strlen (repository_in) - 2] = '\0';
-    }
     frame.repository = repository_in;
 
     expand_wild (argc, argv, &argc, &argv);
@@ -166,7 +213,7 @@ start_recursion (fileproc, filesdoneproc, direntproc, dirleaveproc, callerdat,
     if (repository)
     {
 	free (repository);
-	repository = (char *) NULL;
+	repository = NULL;
     }
     if (filelist)
 	dellist (&filelist); /* FIXME-krp: no longer correct. */
@@ -190,24 +237,21 @@ start_recursion (fileproc, filesdoneproc, direntproc, dirleaveproc, callerdat,
 	    && CVSroot_cmdline == NULL
 	    && current_parsed_root->isremote)
 	{
-	    cvsroot_t *root = Name_Root (NULL, update_dir);
-	    if (root)
-	    {
-		if (strcmp (root->original, current_parsed_root->original))
-		    /* We're skipping this directory because it is for
-		     * a different root.  Therefore, we just want to
-		     * do the subdirectories only.  Processing files would
-		     * cause a working directory from one repository to be
-		     * processed against a different repository, which could
-		     * cause all kinds of spurious conflicts and such.
-		     *
-		     * Question: what about the case of "cvs update foo"
-		     * where we process foo/bar and not foo itself?  That
-		     * seems to be handled somewhere (else) but why should
-		     * it be a separate case?  Needs investigation...  */
-		    just_subdirs = 1;
-		free_cvsroot_t (root);
-	    }
+	    char *root = Name_Root (NULL, update_dir);
+	    if (root && strcmp (root, current_parsed_root->original) != 0)
+		/* We're skipping this directory because it is for
+		   a different root.  Therefore, we just want to
+		   do the subdirectories only.  Processing files would
+		   cause a working directory from one repository to be
+		   processed against a different repository, which could
+		   cause all kinds of spurious conflicts and such.
+
+		   Question: what about the case of "cvs update foo"
+		   where we process foo/bar and not foo itself?  That
+		   seems to be handled somewhere (else) but why should
+		   it be a separate case?  Needs investigation...  */
+		just_subdirs = 1;
+	    free (root);
 	}
 #endif
 
@@ -220,7 +264,7 @@ start_recursion (fileproc, filesdoneproc, direntproc, dirleaveproc, callerdat,
 	 */
 	if (just_subdirs)
 	{
-	    dirlist = Find_Directories ((char *) NULL, W_LOCAL, (List *) NULL);
+	    dirlist = Find_Directories (NULL, W_LOCAL, NULL);
 	    /* If there are no sub-directories, there is a certain logic in
 	       favor of doing nothing, but in fact probably the user is just
 	       confused about what directory they are in, or whether they
@@ -250,9 +294,9 @@ start_recursion (fileproc, filesdoneproc, direntproc, dirleaveproc, callerdat,
 		/* This is the same call to Find_Directories as above.
                    FIXME: perhaps it would be better to write a
                    function that duplicates a list. */
-		args_to_send_when_finished = Find_Directories ((char *) NULL,
+		args_to_send_when_finished = Find_Directories (NULL,
 							       W_LOCAL,
-							       (List *) NULL);
+							       NULL);
 	    }
 #endif
 	}
@@ -274,7 +318,7 @@ start_recursion (fileproc, filesdoneproc, direntproc, dirleaveproc, callerdat,
      * other possibility is named entities whicha are not currently in
      * the working directory.
      */
-    
+
     for (i = 0; i < argc; i++)
     {
 	/* if this argument is a directory, then add it to the list of
@@ -292,11 +336,13 @@ start_recursion (fileproc, filesdoneproc, direntproc, dirleaveproc, callerdat,
 	    char *comp;
 	    char *file_to_try;
 
-	    /* Now break out argv[i] into directory part (DIR) and file part (COMP).
-		   DIR and COMP will each point to a newly malloc'd string.  */
+	    /* Now break out argv[i] into directory part (DIR) and file part
+	     * (COMP).  DIR and COMP will each point to a newly malloc'd
+	     * string.
+	     */
 	    dir = xstrdup (argv[i]);
-	    /* Its okay to discard the const below - we know we just allocated
-	     * dir ourselves.
+	    /* It's okay to cast out last_component's const below since we know
+	     * we just allocated dir here and have control of it.
 	     */
 	    comp = (char *)last_component (dir);
 	    if (comp == dir)
@@ -306,10 +352,8 @@ start_recursion (fileproc, filesdoneproc, direntproc, dirleaveproc, callerdat,
 	    }
 	    else
 	    {
-		char *p = comp;
-
-		p[-1] = '\0';
-		comp = xstrdup (p);
+		comp[-1] = '\0';
+		comp = xstrdup (comp);
 	    }
 
 	    /* if this argument exists as a file in the current
@@ -318,7 +362,8 @@ start_recursion (fileproc, filesdoneproc, direntproc, dirleaveproc, callerdat,
 	    if (!(which & W_LOCAL))
 	    {
 		/* If doing rtag, we've done a chdir to the repository. */
-		file_to_try = xmalloc (strlen (argv[i]) + sizeof (RCSEXT) + 5);
+		file_to_try = xmalloc( strlen( argv[i] )
+		                       + sizeof( RCSEXT ) + 1 );
 		sprintf (file_to_try, "%s%s", argv[i], RCSEXT);
 	    }
 	    else
@@ -345,16 +390,16 @@ start_recursion (fileproc, filesdoneproc, direntproc, dirleaveproc, callerdat,
 		    strcpy (tmp_update_dir, update_dir);
 
 		    if (*tmp_update_dir != '\0')
-			(void) strcat (tmp_update_dir, "/");
+			strcat (tmp_update_dir, "/");
 
-		    (void) strcat (tmp_update_dir, dir);
+		    strcat (tmp_update_dir, dir);
 
 		    /* look for it in the repository. */
 		    repos = Name_Repository (dir, tmp_update_dir);
 		    reposfile = xmalloc (strlen (repos)
 					 + strlen (comp)
 					 + 5);
-		    (void) sprintf (reposfile, "%s/%s", repos, comp);
+		    sprintf (reposfile, "%s/%s", repos, comp);
 		    free (repos);
 
 		    if (!wrap_name_has (comp, WRAP_TOCVS) && isdir (reposfile))
@@ -436,7 +481,7 @@ start_recursion (fileproc, filesdoneproc, direntproc, dirleaveproc, callerdat,
 	   current_cvsroot.
 
 	   The algorithm should be:
-		   
+
 	   1) construct a tree of all directory names where each
 	   element contains a directory name and a flag which notes if
 	   that directory is checked out from current_cvsroot
@@ -469,7 +514,7 @@ start_recursion (fileproc, filesdoneproc, direntproc, dirleaveproc, callerdat,
 
 	   3) Walk the tree and spit out "Argument" commands to tell
 	   the server which directories to munge.
-		   
+
 	   Yuck.  It's not clear this is worth spending time on, since
 	   we might want to disable cvs commands entirely from
 	   directories that do not have CVSADM files...
@@ -525,17 +570,18 @@ start_recursion (fileproc, filesdoneproc, direntproc, dirleaveproc, callerdat,
 	dellist (&args_to_send_when_finished);
     }
 #endif
-    
+
     return (err);
 }
 
+
+
 /*
  * Implement the recursive policies on the local directory.  This may be
- * called directly, or may be called by start_recursion
+ * called directly, or may be called by start_recursion.
  */
 static int
-do_recursion (frame)
-    struct recursion_frame *frame;
+do_recursion (struct recursion_frame *frame)
 {
     int err = 0;
     int dodoneproc = 1;
@@ -544,9 +590,15 @@ do_recursion (frame)
     int locktype;
     int process_this_directory = 1;
 
+#ifdef HAVE_PRINT_PTR
+    TRACE (TRACE_FLOW, "do_recursion ( frame=%p )", (void *) frame);
+#else
+    TRACE (TRACE_FLOW, "do_recursion ( frame=%lx )", (unsigned long) frame);
+#endif
+
     /* do nothing if told */
     if (frame->flags == R_SKIP_ALL)
-	return (0);
+	return 0;
 
     locktype = noexec ? CVS_LOCK_NONE : frame->locktype;
 
@@ -588,7 +640,7 @@ do_recursion (frame)
      * generating data, to give the buffers a chance to drain to the
      * remote client.  We should not have locks active at this point,
      * but if there are writelocks around, we cannot pause here.  */
-    if (server_active && locktype != CVS_LOCK_WRITE)
+    if (server_active && locktype != CVS_LOCK_NONE)
 	server_pause_check();
 #endif
 
@@ -598,7 +650,7 @@ do_recursion (frame)
        usual.  THIS_ROOT might be NULL if we're doing an initial
        checkout -- check before using it.  The default should be that
        we process a directory's contents and only skip those contents
-       if a CVS/Root file exists. 
+       if a CVS/Root file exists.
 
        If we're running the server, we want to process all
        directories, since we're guaranteed to have only one CVSROOT --
@@ -621,30 +673,26 @@ do_recursion (frame)
 #endif
 	)
     {
-	cvsroot_t *this_root = Name_Root ((char *) NULL, update_dir);
+	char *this_root = Name_Root (NULL, update_dir);
 	if (this_root != NULL)
 	{
-	    if (findnode (root_directories, this_root->original))
-	    {
-		process_this_directory = !strcmp (current_parsed_root->original,
-						  this_root->original);
-		free_cvsroot_t (this_root);
-	    }
-	    else
+	    if (findnode (root_directories, this_root) == NULL)
 	    {
 		/* Add it to our list. */
 
 		Node *n = getnode ();
 		n->type = NT_UNKNOWN;
-		n->key = xstrdup (this_root->original);
-		n->data = this_root;
+		n->key = xstrdup (this_root);
 
 		if (addnode (root_directories, n))
-		    error (1, 0, "cannot add new CVSROOT %s",
-			   this_root->original);
-
-		process_this_directory = 0;
+		    error (1, 0, "cannot add new CVSROOT %s", this_root);
+	
 	    }
+	
+	    process_this_directory =
+		    (strcmp (current_parsed_root->original, this_root) == 0);
+
+	    free (this_root);
 	}
     }
 
@@ -655,7 +703,7 @@ do_recursion (frame)
     {
 	if (isdir (CVSADM))
 	{
-	    repository = Name_Repository ((char *) NULL, update_dir);
+	    repository = Name_Repository (NULL, update_dir);
 	    srepository = repository;		/* remember what to free */
 	}
 	else
@@ -706,10 +754,9 @@ do_recursion (frame)
 	       reasonable error message.  */
 	    if (repository == NULL)
 	    {
-		Name_Repository ((char *) NULL, update_dir);
-		assert (!"Not reached.  Please report this problem to <bug-cvs@gnu.org>");
+		Name_Repository (NULL, update_dir);
+	        assert (!"Not reached.  Please report this problem to <bug-cvs@gnu.org>");
 	    }
-
 	    /* find the files and fill in entries if appropriate */
 	    if (process_this_directory)
 	    {
@@ -749,7 +796,7 @@ do_recursion (frame)
 	struct file_info finfo_struct;
 	struct frame_and_file frfile;
 
-	/* read lock it if necessary */
+	/* Lock the repository, if necessary. */
 	if (repository)
 	{
 	    if (locktype == CVS_LOCK_READ)
@@ -786,7 +833,7 @@ do_recursion (frame)
 	    repository
 	    /* and when asked for a read or write lock. */
 	    && locktype != CVS_LOCK_NONE)
-	    Lock_Cleanup ();
+	    Simple_Lock_Cleanup ();
 
 	/* clean up */
 	dellist (&filelist);
@@ -809,7 +856,7 @@ do_recursion (frame)
 
 	frent.frame = frame;
 	frent.entries = entries;
-	err += walklist (dirlist, do_dir_proc, (void *) &frent);
+	err += walklist (dirlist, do_dir_proc, &frent);
     }
 #if 0
     else if (frame->dirleaveproc != NULL)
@@ -817,7 +864,7 @@ do_recursion (frame)
 #endif
     dellist (&dirlist);
 
-    if (entries) 
+    if (entries)
     {
 	Entries_Close (entries);
 	entries = NULL;
@@ -828,7 +875,13 @@ do_recursion (frame)
     {
 	free (srepository);
     }
-    repository = (char *) NULL;
+    repository = NULL;
+
+#ifdef HAVE_PRINT_PTR
+    TRACE (TRACE_FLOW, "Leaving do_recursion ( frame=%p )", (void *) frame);
+#else
+    TRACE (TRACE_FLOW, "Leaving do_recursion ( frame=%lx )", (unsigned long) frame);
+#endif
 
     return err;
 }
@@ -839,9 +892,7 @@ do_recursion (frame)
  * Process each of the files in the list with the callback proc
  */
 static int
-do_file_proc (p, closure)
-    Node *p;
-    void *closure;
+do_file_proc (Node *p, void *closure)
 {
     struct frame_and_file *frfile = (struct frame_and_file *)closure;
     struct file_info *finfo = frfile->finfo;
@@ -879,7 +930,7 @@ do_file_proc (p, closure)
 	    return 0;
 	}
     }
-    else 
+    else
         finfo->rcs = (RCSNode *) NULL;
     finfo->fullname = tmp;
     ret = frfile->frame->fileproc (frfile->frame->callerdat, finfo);
@@ -901,9 +952,7 @@ do_file_proc (p, closure)
  * Process each of the directories in the list (recursing as we go)
  */
 static int
-do_dir_proc (p, closure)
-    Node *p;
-    void *closure;
+do_dir_proc (Node *p, void *closure)
 {
     struct frame_and_entries *frent = (struct frame_and_entries *) closure;
     struct recursion_frame *frame = frent->frame;
@@ -1073,30 +1122,25 @@ but CVS uses %s for its own purposes; skipping %s directory",
 #endif
 	)
     {
-	cvsroot_t *this_root = Name_Root (dir, update_dir);
+	char *this_root = Name_Root (dir, update_dir);
 	if (this_root != NULL)
 	{
-	    if (findnode (root_directories, this_root->original))
-	    {
-		process_this_directory = !strcmp (current_parsed_root->original,
-						  this_root->original);
-		free_cvsroot_t (this_root);
-	    }
-	    else
+	    if (findnode (root_directories, this_root) == NULL)
 	    {
 		/* Add it to our list. */
 
 		Node *n = getnode ();
 		n->type = NT_UNKNOWN;
-		n->key = xstrdup (this_root->original);
-		n->data = this_root;
+		n->key = xstrdup (this_root);
 
 		if (addnode (root_directories, n))
-		    error (1, 0, "cannot add new CVSROOT %s",
-			   this_root->original);
+		    error (1, 0, "cannot add new CVSROOT %s", this_root);
 
-		process_this_directory = 0;
 	    }
+
+	    process_this_directory = (strcmp (current_parsed_root->original, this_root) == 0);
+
+	    free (this_root);
 	}
     }
 
@@ -1130,7 +1174,7 @@ but CVS uses %s for its own purposes; skipping %s directory",
     {
 	/* save our current directory and static vars */
         if (save_cwd (&cwd))
-	    error_exit ();
+	    exit (EXIT_FAILURE);
 	sdirlist = dirlist;
 	srepository = repository;
 	dirlist = NULL;
@@ -1189,7 +1233,7 @@ but CVS uses %s for its own purposes; skipping %s directory",
 
 	/* get back to where we started and restore state vars */
 	if (restore_cwd (&cwd, NULL))
-	    error_exit ();
+	    exit (EXIT_FAILURE);
 	free_cwd (&cwd);
 	dirlist = sdirlist;
 	repository = srepository;
@@ -1205,9 +1249,7 @@ but CVS uses %s for its own purposes; skipping %s directory",
  * Add a node to a list allocating the list if necessary.
  */
 static void
-addlist (listp, key)
-    List **listp;
-    char *key;
+addlist (List **listp, char *key)
 {
     Node *p;
 
@@ -1221,10 +1263,7 @@ addlist (listp, key)
 }
 
 static void
-addfile (listp, dir, file)
-    List **listp;
-    char *dir;
-    char *file;
+addfile (List **listp, char *dir, char *file)
 {
     Node *n;
     List *fl;
@@ -1247,9 +1286,7 @@ addfile (listp, dir, file)
 }
 
 static int
-unroll_files_proc (p, closure)
-    Node *p;
-    void *closure;
+unroll_files_proc (Node *p, void *closure)
 {
     Node *n;
     struct recursion_frame *frame = (struct recursion_frame *) closure;
@@ -1273,7 +1310,7 @@ unroll_files_proc (p, closure)
     if (strcmp(p->key, ".") != 0)
     {
         if (save_cwd (&cwd))
-	    error_exit ();
+	    exit (EXIT_FAILURE);
 	if ( CVS_CHDIR (p->key) < 0)
 	    error (1, errno, "could not chdir to %s", p->key);
 
@@ -1297,7 +1334,7 @@ unroll_files_proc (p, closure)
 	update_dir = save_update_dir;
 
 	if (restore_cwd (&cwd, NULL))
-	    error_exit ();
+	    exit (EXIT_FAILURE);
 	free_cwd (&cwd);
     }
 
@@ -1306,3 +1343,5 @@ unroll_files_proc (p, closure)
 	dellist (&filelist);
     return(err);
 }
+/* vim:tabstop=8:shiftwidth=4
+ */
