@@ -1,20 +1,16 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
- *
- * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
- *                                  and others.
- *
- * Portions Copyright (C) 1992, Brian Berliner and Jeff Polk
- * Portions Copyright (C) 1989-1992, Brian Berliner
+ * Copyright (c) 1992, Brian Berliner and Jeff Polk
+ * Copyright (c) 1989-1992, Brian Berliner
  * 
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with the CVS source distribution.
  */
 
-#include <assert.h>
 #include "cvs.h"
 #include "getline.h"
 
+
+
 /* Determine the name of the RCS repository for directory DIR in the
    current working directory, or for the current working directory
    itself if DIR is NULL.  Returns the name in a newly-malloc'd
@@ -25,9 +21,7 @@
    invoked.  */
 
 char *
-Name_Repository (dir, update_dir)
-    const char *dir;
-    const char *update_dir;
+Name_Repository (const char *dir, const char *update_dir)
 {
     FILE *fpin;
     const char *xupdate_dir;
@@ -70,37 +64,36 @@ Name_Repository (dir, update_dir)
 
 	if (!isdir (cvsadm))
 	{
-	    error (0, 0, "in directory %s:", xupdate_dir);
-	    error (1, 0, "there is no version here; do '%s checkout' first",
+	    error (0, 0, "in directory `%s':", xupdate_dir);
+	    error (1, 0, "there is no version here; do `%s checkout' first",
 		   program_name);
 	}
 	free (cvsadm);
 
 	if (existence_error (save_errno))
 	{
-	    /* FIXME: This is a very poorly worded error message.  It
-	       occurs at least in the case where the user manually
-	       creates a directory named CVS, so the error message
-	       should be more along the lines of "CVS directory found
-	       without administrative files; use CVS to create the CVS
-	       directory, or rename it to something else if the
-	       intention is to store something besides CVS
-	       administrative files".  */
-	    error (0, 0, "in directory %s:", xupdate_dir);
-	    error (1, 0, "*PANIC* administration files missing");
+	    /* This occurs at least in the case where the user manually
+	     * creates a directory named CVS.
+	     */
+	    error (0, 0, "in directory `%s':", xupdate_dir);
+	    error (0, 0, "CVS directory found without administrative files.");
+	    error (0, 0, "Use CVS to create the CVS directory, or rename the");
+	    error (0, 0, "directory if it is intended to store something");
+	    error (0, 0, "besides CVS administrative files.");
+	    error (1, 0, "*PANIC* administration files missing!");
 	}
 
-	error (1, save_errno, "cannot open %s", tmp);
+	error (1, save_errno, "cannot open `%s'", tmp);
     }
 
     if (getline (&repos, &repos_allocated, fpin) < 0)
     {
 	/* FIXME: should be checking for end of file separately.  */
-	error (0, 0, "in directory %s:", xupdate_dir);
-	error (1, errno, "cannot read %s", CVSADM_REP);
+	error (0, 0, "in directory `%s':", xupdate_dir);
+	error (1, errno, "cannot read `%s'", CVSADM_REP);
     }
     if (fclose (fpin) < 0)
-	error (0, errno, "cannot close %s", tmp);
+	error (0, errno, "cannot close `%s'", tmp);
     free (tmp);
 
     if ((cp = strrchr (repos, '\n')) != NULL)
@@ -117,16 +110,16 @@ Name_Repository (dir, update_dir)
 
 	if (current_parsed_root == NULL)
 	{
-	    error (0, 0, "in directory %s:", xupdate_dir);
+	    error (0, 0, "in directory `%s:", xupdate_dir);
 	    error (0, 0, "must set the CVSROOT environment variable\n");
-	    error (0, 0, "or specify the '-d' option to %s.", program_name);
-	    error (1, 0, "illegal repository setting");
+	    error (0, 0, "or specify the '-d' option to `%s'.", program_name);
+	    error (1, 0, "invalid repository setting");
 	}
 	if (pathname_levels (repos) > 0)
 	{
-	    error (0, 0, "in directory %s:", xupdate_dir);
+	    error (0, 0, "in directory `%s':", xupdate_dir);
 	    error (0, 0, "`..'-relative repositories are not supported.");
-	    error (1, 0, "illegal source repository");
+	    error (1, 0, "invalid source repository");
 	}
 	newrepos = xmalloc (strlen (current_parsed_root->directory)
 	                    + strlen (repos) + 2);
@@ -147,8 +140,7 @@ Name_Repository (dir, update_dir)
  * possibly fully qualified repository
  */
 const char *
-Short_Repository (repository)
-    const char *repository;
+Short_Repository (const char *repository)
 {
     if (repository == NULL)
 	return NULL;
@@ -192,8 +184,7 @@ Short_Repository (repository)
  */
 
 void
-Sanitize_Repository_Name (repository)
-    char *repository;
+Sanitize_Repository_Name (char *repository)
 {
     size_t len;
 
@@ -204,7 +195,7 @@ Sanitize_Repository_Name (repository)
     len = strlen (repository);
     if (len >= 2
 	&& repository[len - 1] == '.'
-	&& ISDIRSEP (repository[len - 2]))
+	&& ISSLASH (repository[len - 2]))
     {
 	repository[len - 2] = '\0';
     }
