@@ -23,14 +23,11 @@ const char method_names[][16] = {
 
 #ifndef DEBUG
 
-cvsroot_t *
-Name_Root (dir, update_dir)
-    const char *dir;
-    const char *update_dir;
+char *
+Name_Root (char *dir, char *update_dir)
 {
     FILE *fpin;
-    cvsroot_t *ret;
-    const char *xupdate_dir;
+    char *ret, *xupdate_dir;
     char *root = NULL;
     size_t root_allocated = 0;
     char *tmp;
@@ -88,7 +85,7 @@ Name_Root (dir, update_dir)
 	goto out;
     }
     fclose (fpin);
-    cp = root + len - 1;
+    cp = root + (len - 1);
     if (*cp == '\n')
 	*cp = '\0';			/* strip the newline */
 
@@ -97,38 +94,43 @@ Name_Root (dir, update_dir)
      * absolute pathname or specify a remote server.
      */
 
-    ret = parse_cvsroot (root);
-    if (ret == NULL)
+    if (
+#ifdef CLIENT_SUPPORT
+	(strchr (root, ':') == NULL) &&
+#endif
+    	! isabsolute (root))
     {
 	error (0, 0, "in directory %s:", xupdate_dir);
 	error (0, 0,
-	       "ignoring %s because it does not contain a valid root.",
+	       "ignoring %s because it does not contain an absolute pathname.",
 	       CVSADM_ROOT);
+	ret = NULL;
 	goto out;
     }
 
-    if (
 #ifdef CLIENT_SUPPORT
-        !ret->isremote &&
-#endif
-        !isdir (ret->directory))
+    if ((strchr (root, ':') == NULL) && !isdir (root))
+#else /* ! CLIENT_SUPPORT */
+    if (!isdir (root))
+#endif /* CLIENT_SUPPORT */
     {
 	error (0, 0, "in directory %s:", xupdate_dir);
 	error (0, 0,
 	       "ignoring %s because it specifies a non-existent repository %s",
 	       CVSADM_ROOT, root);
-	free_cvsroot_t (ret);
 	ret = NULL;
 	goto out;
     }
 
-
+    /* allocate space to return and fill it in */
+    strip_trailing_slashes (root);
+    ret = xstrdup (root);
  out:
     free (cvsadm);
     free (tmp);
     if (root != NULL)
 	free (root);
-    return ret;
+    return (ret);
 }
 
 
@@ -139,9 +141,7 @@ Name_Root (dir, update_dir)
  * future work.
  */
 void
-Create_Root (dir, rootdir)
-    const char *dir;
-    const char *rootdir;
+Create_Root (const char *dir, const char *rootdir)
 {
     FILE *fout;
     char *tmp;
@@ -173,7 +173,7 @@ Create_Root (dir, rootdir)
 #endif /* ! DEBUG */
 
 
-/* The root_allow_* stuff maintains a list of legal CVSROOT
+/* The root_allow_* stuff maintains a list of valid CVSROOT
    directories.  Then we can check against them when a remote user
    hands us a CVSROOT directory.  */
 
@@ -182,8 +182,7 @@ static char **root_allow_vector;
 static int root_allow_size;
 
 void
-root_allow_add (arg)
-    char *arg;
+root_allow_add (char *arg)
 {
     char *p;
 
@@ -211,7 +210,7 @@ root_allow_add (arg)
 	    printf ("E Fatal server error, aborting.\n\
 error ENOMEM Virtual memory exhausted.\n");
 
-	    error_exit ();
+	    exit (EXIT_FAILURE);
 	}
     }
     p = xmalloc (strlen (arg) + 1);
@@ -222,7 +221,7 @@ error ENOMEM Virtual memory exhausted.\n");
 }
 
 void
-root_allow_free ()
+root_allow_free (void)
 {
     if (root_allow_vector != NULL)
 	free_names (&root_allow_count, root_allow_vector);
@@ -230,8 +229,7 @@ root_allow_free ()
 }
 
 int
-root_allow_ok (arg)
-    char *arg;
+root_allow_ok (char *arg)
 {
     int i;
 
@@ -248,7 +246,7 @@ root_allow_ok (arg)
 	   expects responses.  */
 	printf ("\
 error 0 Server configuration missing --allow-root in inetd.conf\n");
-	error_exit ();
+	exit (EXIT_FAILURE);
     }
 
     for (i = 0; i < root_allow_count; ++i)
@@ -279,8 +277,11 @@ cvsroot_t *current_parsed_root = NULL;
  * Some of the other zeroes remain meaningful as, "never set, use default",
  * or the like
  */
+/* Functions which allocate memory are not pure.  */
+static cvsroot_t *new_cvsroot_t(void)
+    __attribute__( (__malloc__) );
 static cvsroot_t *
-new_cvsroot_t ()
+new_cvsroot_t (void)
 {
     cvsroot_t *newroot;
 
@@ -307,8 +308,7 @@ new_cvsroot_t ()
 
 /* Dispose of a cvsroot_t and its component parts */
 void
-free_cvsroot_t (root)
-    cvsroot_t *root;
+free_cvsroot_t (cvsroot_t *root)
 {
     if (root->original != NULL)
 	free (root->original);
@@ -357,8 +357,7 @@ free_cvsroot_t (root)
  * 	free_cvsroot_t()
  */
 cvsroot_t *
-parse_cvsroot (root_in)
-    const char *root_in;
+parse_cvsroot (const char *root_in)
 {
     cvsroot_t *newroot;			/* the new root to be returned */
     char *cvsroot_save;			/* what we allocated so we can dispose
@@ -369,9 +368,13 @@ parse_cvsroot (root_in)
 					 */
     char *cvsroot_copy, *p, *q;		/* temporary pointers for parsing */
 #ifdef CLIENT_SUPPORT
-    int check_hostname, no_port, no_password;
+    int check_hostname, no_port, no_password, no_proxy;
 #endif /* CLIENT_SUPPORT */
 
+    assert (root_in != NULL);
+
+    TRACE (TRACE_FUNCTION, "parse_cvsroot ( %s )", root_in);
+
     /* allocate some space */
     newroot = new_cvsroot_t();
 
@@ -404,21 +407,9 @@ parse_cvsroot (root_in)
 
 #ifdef CLIENT_SUPPORT
 	/* Look for method options, for instance, proxy, proxyport.
-	 * We don't handle these, but we like to try and warn the user that
-	 * they are being ignored.
+	 * Calling strtok again is saved until after parsing the method.
 	 */
-	if (p = strchr (method, ';'))	
-	{
-	    *p++ = '\0';
-	    if (!really_quiet)
-	    {
-		error (0, 0,
-"WARNING: Ignoring method options found in CVSROOT: `%s'.",
-		       p);
-		error (0, 0,
-"Use CVS version 1.12.7 or later to handle method options.");
-	    }
-	}
+	method = strtok (method, ";");
 #endif /* CLIENT_SUPPORT */
 
 	/* Now we have an access method -- see if it's valid. */
@@ -442,6 +433,50 @@ parse_cvsroot (root_in)
 	    error (0, 0, "Unknown method (`%s') in CVSROOT.", method);
 	    goto error_exit;
 	}
+
+#ifdef CLIENT_SUPPORT
+	/* Parse the method options, for instance, proxy, proxyport */
+	while (p = strtok (NULL, ";"))
+	{
+	    char *q = strchr (p, '=');
+	    if (q == NULL)
+	    {
+	        error (0, 0, "Option (`%s') has no argument in CVSROOT.",
+                       p);
+	        goto error_exit;
+	    }
+
+	    *q++ = '\0';
+	    if (strcmp (p, "proxy") == 0)
+	    {
+		newroot->proxy_hostname = xstrdup (q);
+	    }
+	    else if (strcmp (p, "proxyport") == 0)
+	    {
+		char *r = q;
+		if (*r == '-') r++;
+		while (*r)
+		{
+		    if (!isdigit(*r++))
+		    {
+			error (0, 0,
+"CVSROOT may only specify a positive, non-zero, integer proxy port (not `%s').",
+			       q);
+			goto error_exit;
+		    }
+		}
+		if ((newroot->proxy_port = atoi (q)) <= 0)
+		    error (0, 0,
+"CVSROOT may only specify a positive, non-zero, integer proxy port (not `%s').",
+			   q);
+	    }
+	    else
+	    {
+	        error (0, 0, "Unknown option (`%s') in CVSROOT.", p);
+	        goto error_exit;
+	    }
+	}
+#endif /* CLIENT_SUPPORT */
     }
     else
     {
@@ -453,13 +488,35 @@ parse_cvsroot (root_in)
 			  : local_method);
     }
 
+    /*
+     * There are a few sanity checks we can do now, only knowing the
+     * method of this root.
+     */
+#ifndef DEBUG
+    /* Why do we avoid these checks when DEBUG is set?  How is this used?  */
+# ifndef CLIENT_SUPPORT
+    if (newroot->method != local_method)
+    {
+	error (0, 0, "CVSROOT is set for a remote access method but your");
+	error (0, 0, "CVS executable doesn't support it.");
+	goto error_exit;
+    }
+# endif
+#endif /* ! DEBUG */
+
 #ifdef CLIENT_SUPPORT
     newroot->isremote = (newroot->method != local_method);
-#endif /* CLIENT_SUPPORT */
 
+    if (readonlyfs && newroot->isremote)
+	error (1, 0,
+"Read-only repository feature unavailable with remote roots (cvsroot = %s)",
+	       cvsroot_copy);
 
     if ((newroot->method != local_method)
-	&& (newroot->method != fork_method))
+#ifdef CLIENT_SUPPORT
+	&& (newroot->method != fork_method)
+#endif /* SERVER_SUPPORT */
+       )
     {
 	/* split the string into [[user][:password]@]host[:[port]] & /path
 	 *
@@ -477,10 +534,6 @@ parse_cvsroot (root_in)
 	firstslash = p;		/* == NULL if '/' not in string */
 	*p = '\0';
 
-        /* Don't parse username, password, hostname, or port without client
-         * support.
-         */
-#ifdef CLIENT_SUPPORT
 	/* Check to see if there is a username[:password] in the string. */
 	if ((p = strchr (cvsroot_copy, '@')) != NULL)
 	{
@@ -551,8 +604,8 @@ parse_cvsroot (root_in)
 	/* restore the '/' */
 	cvsroot_copy = firstslash;
 	*cvsroot_copy = '/';
-#endif /* CLIENT_SUPPORT */
     }
+#endif /* CLIENT_SUPPORT */
 
     /*
      * Parse the path for all methods.
@@ -561,31 +614,13 @@ parse_cvsroot (root_in)
      * called on a CVSROOT now.  cvsroot->original is saved for error messages
      * and, otherwise, we want no trailing slashes.
      */
-    Sanitize_Repository_Name( cvsroot_copy );
-    newroot->directory = xstrdup(cvsroot_copy);
+    Sanitize_Repository_Name (cvsroot_copy);
+    newroot->directory = xstrdup (cvsroot_copy);
 
     /*
      * Do various sanity checks.
      */
 
-#if ! defined (CLIENT_SUPPORT) && ! defined (DEBUG)
-    if (newroot->method != local_method)
-    {
-	error (0, 0, "CVSROOT is set for a remote access method but your");
-	error (0, 0, "CVS executable doesn't support it.");
-	goto error_exit;
-    }
-#endif
-
-#if ! defined (SERVER_SUPPORT) && ! defined (DEBUG)
-    if (newroot->method == fork_method)
-    {
-	error (0, 0, "CVSROOT is set to use the :fork: access method but your");
-	error (0, 0, "CVS executable doesn't support it.");
-	goto error_exit;
-     }
-#endif
-
 #ifdef CLIENT_SUPPORT
     if (newroot->username && ! newroot->hostname)
     {
@@ -593,8 +628,10 @@ parse_cvsroot (root_in)
 	goto error_exit;
     }
 
+    /* We won't have attempted to parse these without CLIENT_SUPPORT */
     check_hostname = 0;
     no_password = 1;
+    no_proxy = 1;
     no_port = 0;
 #endif /* CLIENT_SUPPORT */
     switch (newroot->method)
@@ -607,8 +644,6 @@ parse_cvsroot (root_in)
 	    error (0, 0, "when using local access method.");
 	    goto error_exit;
 	}
-	no_port = 1;
-	/* no_password already set */
 #endif /* CLIENT_SUPPORT */
 	/* cvs.texinfo has always told people that CVSROOT must be an
 	   absolute pathname.  Furthermore, attempts to use a relative
@@ -622,6 +657,14 @@ parse_cvsroot (root_in)
 	    error (0, 0, "when using local access method.");
 	    goto error_exit;
 	}
+#ifdef CLIENT_SUPPORT
+	/* We don't need to check for these in :local: mode, really, since
+	 * we shouldn't be able to hit the code above which parses them, but
+	 * I'm leaving them here in lieu of assertions.
+	 */
+	no_port = 1;
+	/* no_password already set */
+#endif /* CLIENT_SUPPORT */
 	break;
 #ifdef CLIENT_SUPPORT
     case fork_method:
@@ -662,6 +705,7 @@ parse_cvsroot (root_in)
 	goto error_exit;
 # else
 	check_hostname = 1;
+	no_proxy = 0;
 	/* no_password already set */
 	break;
 # endif
@@ -673,6 +717,7 @@ parse_cvsroot (root_in)
 	break;
     case pserver_method:
 	no_password = 0;
+	no_proxy = 0;
 	check_hostname = 1;
 	break;
 #endif /* CLIENT_SUPPORT */
@@ -687,6 +732,19 @@ parse_cvsroot (root_in)
 	error (0, 0, "pserver connection method.");
 	goto error_exit;
     }
+    if (no_proxy && (newroot->proxy_hostname || newroot->proxy_port))
+    {
+	error (0, 0,
+"CVSROOT proxy specification is only valid for gserver and");
+	error (0, 0, "pserver connection methods.");
+	goto error_exit;
+    }
+
+    if (!newroot->proxy_hostname && newroot->proxy_port)
+    {
+	error (0, 0, "Proxy port specified in CVSROOT without proxy host.");
+	goto error_exit;
+    }
 
     if (check_hostname && !newroot->hostname)
     {
@@ -695,12 +753,13 @@ parse_cvsroot (root_in)
     }
 
     if (no_port && newroot->port)
-	{
-	    error (0, 0, "CVSROOT port specification is only valid for gserver, kserver,");
-	    error (0, 0, "and pserver connection methods.");
-	    goto error_exit;
-	}
-#endif /* CLIENT_SUPPORT */
+    {
+        error (0, 0,
+"CVSROOT port specification is only valid for gserver, kserver,");
+        error (0, 0, "and pserver connection methods.");
+        goto error_exit;
+    }
+#endif /*CLIENT_SUPPORT */
 
     if (*newroot->directory == '\0')
     {
@@ -732,15 +791,11 @@ error_exit:
  * caller to be responsible for our return value
  */
 char *
-normalize_cvsroot (root)
-    const cvsroot_t *root;
+normalize_cvsroot (const cvsroot_t *root)
 {
     char *cvsroot_canonical;
-    char *p, *hostname, *username;
-    char port_s[64];
-
-    /* get the appropriate port string */
-    sprintf (port_s, "%d", get_cvs_port_number (root));
+    char *p, *hostname;
+    size_t length;
 
     /* use a lower case hostname since we know hostnames are case insensitive */
     /* Some logic says we should be tacking our domain name on too if it isn't
@@ -749,20 +804,18 @@ normalize_cvsroot (root)
      * the DNS trickery that makes life easier for sysadmins when they want to
      * move a repository or the like
      */
-    p = hostname = xstrdup(root->hostname);
+    p = hostname = xstrdup (root->hostname);
     while (*p)
     {
-	*p = tolower(*p);
+	*p = tolower (*p);
 	p++;
     }
 
-    /* get the username string */
-    username = root->username ? root->username : getcaller();
-    cvsroot_canonical = xmalloc ( strlen(username)
-				+ strlen(hostname) + strlen(port_s)
-				+ strlen(root->directory) + 12);
-    sprintf (cvsroot_canonical, ":pserver:%s@%s:%s%s",
-	    username, hostname, port_s, root->directory);
+    cvsroot_canonical = asnprintf (NULL, &length, ":pserver:%s@%s:%d%s",
+                                   root->username ? root->username
+                                                  : getcaller(),
+                                   hostname, get_cvs_port_number (root),
+                                   root->directory);
 
     free (hostname);
     return cvsroot_canonical;
@@ -774,8 +827,7 @@ normalize_cvsroot (root)
 /* allocate and return a cvsroot_t structure set up as if we're using the local
  * repository DIR.  */
 cvsroot_t *
-local_cvsroot (dir)
-    const char *dir;
+local_cvsroot (const char *dir)
 {
     cvsroot_t *newroot = new_cvsroot_t();
 
@@ -808,23 +860,14 @@ char *cvs_cmd_name = "parse_cvsroot";		/* XXX is this used??? */
    these make various bad assumptions, but they're pretty easy to
    debug when something goes wrong.  */
 
-void
-error_exit PROTO ((void))
-{
-    exit (1);
-}
-
 int
-isabsolute (dir)
-    const char *dir;
+isabsolute( const char *dir )
 {
     return (dir && (*dir == '/'));
 }
 
 void
-main (argc, argv)
-    int argc;
-    char *argv[];
+main( int argc, char *argv[] )
 {
     program_name = argv[0];
 
