@@ -15,12 +15,12 @@
 #include "cvs.h"
 
 #ifndef HAVE_UNISTD_H
-extern int execvp PROTO((char *file, char **argv));
+extern int execvp (char *file, char **argv);
 #endif
 
-static void run_add_arg PROTO((const char *s));
+static void run_add_arg (const char *s);
+
 
-extern char *strtok ();
 
 /*
  * To exec a program under CVS, first call run_setup() to setup initial
@@ -40,12 +40,15 @@ static int run_argc_allocated;
 
 /* VARARGS */
 void 
-run_setup (prog)
-    const char *prog;
+run_setup( const char *prog )
 {
-    char *cp;
     int i;
     char *run_prog;
+    char *buf, *d, *s;
+    size_t length;
+    size_t doff;
+    char inquotes;
+    int dolastarg;
 
     /* clean out any malloc'ed values from run_argv */
     for (i = 0; i < run_argc; i++)
@@ -60,22 +63,61 @@ run_setup (prog)
 
     run_prog = xstrdup (prog);
 
+    s = run_prog;
+    d = buf = NULL;
+    length = 0;
+    dolastarg = 1;
+    inquotes = '\0';
+    doff = d - buf;
+    expand_string(&buf, &length, doff + 1);
+    d = buf + doff;
+    while ((*d = *s++) != '\0')
+    {
+	switch (*d)
+	{
+	    case '\\':
+		if (*s) *d = *s++;
+		d++;
+		break;
+	    case '"':
+	    case '\'':
+		if (inquotes == *d) inquotes = '\0';
+		else inquotes = *d;
+		break;
+	    case ' ':
+	    case '\t':
+		if (inquotes) d++;
+		else
+		{
+		    *d = '\0';
+		    run_add_arg (buf);
+		    d = buf;
+		    while (isspace(*s)) s++;
+		    if (!*s) dolastarg = 0;
+		}
+		break;
+	    default:
+		d++;
+		break;
+	}
+	doff = d - buf;
+	expand_string(&buf, &length, doff + 1);
+	d = buf + doff;
+    }
+    if (dolastarg) run_add_arg (buf);
     /* put each word into run_argv, allocating it as we go */
-    for (cp = strtok (run_prog, " \t"); cp; cp = strtok ((char *) NULL, " \t"))
-	run_add_arg (cp);
+    if (buf) free (buf);
     free (run_prog);
 }
 
 void
-run_arg (s)
-    const char *s;
+run_arg (const char *s)
 {
     run_add_arg (s);
 }
 
 static void
-run_add_arg (s)
-    const char *s;
+run_add_arg (const char *s)
 {
     /* allocate more argv entries if we've run out */
     if (run_argc >= run_argc_allocated)
@@ -94,11 +136,7 @@ run_add_arg (s)
 
 
 int
-run_exec (stin, stout, sterr, flags)
-    const char *stin;
-    const char *stout;
-    const char *sterr;
-    int flags;
+run_exec (const char *stin, const char *stout, const char *sterr, int flags)
 {
     int shin, shout, sherr;
     int mode_out, mode_err;
@@ -352,11 +390,10 @@ run_exec (stin, stout, sterr, flags)
 
 
 void
-run_print (fp)
-    FILE *fp;
+run_print (FILE *fp)
 {
     int i;
-    void (*outfn) PROTO ((const char *, size_t));
+    void (*outfn) (const char *, size_t);
 
     if (fp == stderr)
 	outfn = cvs_outerr;
@@ -381,30 +418,27 @@ run_print (fp)
     }
 }
 
+
+
 /* Return value is NULL for error, or if noexec was set.  If there was an
    error, return NULL and I'm not sure whether errno was set (the Red Hat
    Linux 4.1 popen manpage was kind of vague but discouraging; and the noexec
    case complicates this even aside from popen behavior).  */
 
 FILE *
-run_popen (cmd, mode)
-    const char *cmd;
-    const char *mode;
+run_popen (const char *cmd, const char *mode)
 {
-    if (trace)
-	(void) fprintf (stderr, "%s-> run_popen(%s,%s)\n",
-			CLIENT_SERVER_STR, cmd, mode);
+    TRACE ( 1, "run_popen(%s,%s)", cmd, mode );
     if (noexec)
 	return (NULL);
 
     return (popen (cmd, mode));
 }
 
+
+
 int
-piped_child (command, tofdp, fromfdp)
-     const char **command;
-     int *tofdp;
-     int *fromfdp;
+piped_child (char *const *command, int *tofdp, int *fromfdp)
 {
     int pid;
     int to_child_pipe[2];
@@ -456,8 +490,7 @@ piped_child (command, tofdp, fromfdp)
 
 
 void
-close_on_exec (fd)
-     int fd;
+close_on_exec (int fd)
 {
 #ifdef F_SETFD
     if (fcntl (fd, F_SETFD, 1) == -1)
