@@ -1,17 +1,12 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
+ * Copyright (c) 1992, Brian Berliner and Jeff Polk
+ * Copyright (c) 1989-1992, Brian Berliner
  *
- * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
- *                                  and others.
- *
- * Portions Copyright (C) 1992, Brian Berliner and Jeff Polk
- * Portions Copyright (C) 1989-1992, Brian Berliner
- * 
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with the CVS source distribution.
- * 
+ *
  * Tag and Rtag
- * 
+ *
  * Add or delete a symbolic name to an RCS file, or a collection of RCS files.
  * Tag uses the checked out revision in the current directory, rtag uses
  * the modules database, if necessary.
@@ -20,42 +15,43 @@
 #include "cvs.h"
 #include "savecwd.h"
 
-static int rtag_proc PROTO((int argc, char **argv, char *xwhere,
+static int rtag_proc (int argc, char **argv, char *xwhere,
 		      char *mwhere, char *mfile, int shorten,
-		      int local_specified, char *mname, char *msg));
-static int check_fileproc PROTO ((void *callerdat, struct file_info *finfo));
-static int check_filesdoneproc PROTO ((void *callerdat, int err,
-                                       const char *repos,
-                                       const char *update_dir,
-                                       List *entries));
-static int pretag_proc PROTO((const char *repository, const char *filter));
-static void masterlist_delproc PROTO((Node *p));
-static void tag_delproc PROTO((Node *p));
-static int pretag_list_proc PROTO((Node *p, void *closure));
+		      int local_specified, char *mname, char *msg);
+static int check_fileproc (void *callerdat, struct file_info *finfo);
+static int check_filesdoneproc (void *callerdat, int err,
+				const char *repos, const char *update_dir,
+				List *entries);
+static int pretag_proc (const char *_repository, const char *_filter,
+                        void *_closure);
+static void masterlist_delproc (Node *_p);
+static void tag_delproc (Node *_p);
+static int pretag_list_to_args_proc (Node *_p, void *_closure);
 
-static Dtype tag_dirproc PROTO ((void *callerdat, const char *dir,
-                                 const char *repos, const char *update_dir,
-                                 List *entries));
-static int rtag_fileproc PROTO ((void *callerdat, struct file_info *finfo));
-static int rtag_delete PROTO((RCSNode *rcsfile));
-static int tag_fileproc PROTO ((void *callerdat, struct file_info *finfo));
+static Dtype tag_dirproc (void *callerdat, const char *dir,
+                          const char *repos, const char *update_dir,
+                          List *entries);
+static int rtag_fileproc (void *callerdat, struct file_info *finfo);
+static int rtag_delete (RCSNode *rcsfile);
+static int tag_fileproc (void *callerdat, struct file_info *finfo);
 
 static char *numtag;			/* specific revision to tag */
-static int numtag_validated = 0;
+static bool numtag_validated = false;
 static char *date = NULL;
 static char *symtag;			/* tag to add or delete */
-static int delete_flag;			/* adding a tag by default */
-static int branch_mode;			/* make an automagic "branch" tag */
-static int disturb_branch_tags = 0;	/* allow -F,-d to disturb branch tags */
-static int force_tag_match = 1;		/* force tag to match by default */
-static int force_tag_move;		/* don't force tag to move by default */
-static int check_uptodate;		/* no uptodate-check by default */
-static int attic_too;			/* remove tag from Attic files */
-static int is_rtag;
+static bool delete_flag;		/* adding a tag by default */
+static bool branch_mode;		/* make an automagic "branch" tag */
+static bool disturb_branch_tags = false;/* allow -F,-d to disturb branch tags */
+static bool force_tag_match = true;	/* force tag to match by default */
+static bool force_tag_move;		/* don't force tag to move by default */
+static bool check_uptodate;		/* no uptodate-check by default */
+static bool attic_too;			/* remove tag from Attic files */
+static bool is_rtag;
 
 struct tag_info
 {
     Ctype status;
+    char *oldrev;
     char *rev;
     char *tag;
     char *options;
@@ -67,7 +63,6 @@ struct master_lists
 };
 
 static List *mtlist;
-static List *tlist;
 
 static const char rtag_opts[] = "+aBbdFflnQqRr:D:";
 static const char *const rtag_usage[] =
@@ -106,18 +101,18 @@ static const char *const tag_usage[] =
     NULL
 };
 
+
+
 int
-cvstag (argc, argv)
-    int argc;
-    char **argv;
+cvstag (int argc, char **argv)
 {
-    int local = 0;			/* recursive by default */
+    bool local = false;			/* recursive by default */
     int c;
     int err = 0;
-    int run_module_prog = 1;
+    bool run_module_prog = true;
 
     is_rtag = (strcmp (cvs_cmd_name, "rtag") == 0);
-    
+
     if (argc == -1)
 	usage (is_rtag ? rtag_usage : tag_usage);
 
@@ -127,31 +122,31 @@ cvstag (argc, argv)
 	switch (c)
 	{
 	    case 'a':
-		attic_too = 1;
+		attic_too = true;
 		break;
 	    case 'b':
-		branch_mode = 1;
+		branch_mode = true;
 		break;
 	    case 'B':
-		disturb_branch_tags = 1;
+		disturb_branch_tags = true;
 		break;
 	    case 'c':
-		check_uptodate = 1;
+		check_uptodate = true;
 		break;
 	    case 'd':
-		delete_flag = 1;
+		delete_flag = true;
 		break;
             case 'F':
-		force_tag_move = 1;
+		force_tag_move = true;
 		break;
 	    case 'f':
-		force_tag_match = 0;
+		force_tag_match = false;
 		break;
 	    case 'l':
-		local = 1;
+		local = true;
 		break;
 	    case 'n':
-		run_module_prog = 0;
+		run_module_prog = false;
 		break;
 	    case 'Q':
 	    case 'q':
@@ -165,7 +160,7 @@ cvstag (argc, argv)
 			   cvs_cmd_name);
 		break;
 	    case 'R':
-		local = 0;
+		local = false;
 		break;
             case 'r':
                 numtag = optarg;
@@ -205,23 +200,23 @@ cvstag (argc, argv)
 	ign_setup ();
 
 	if (attic_too)
-	    send_arg("-a");
+	    send_arg ("-a");
 	if (branch_mode)
-	    send_arg("-b");
+	    send_arg ("-b");
 	if (disturb_branch_tags)
-	    send_arg("-B");
+	    send_arg ("-B");
 	if (check_uptodate)
-	    send_arg("-c");
+	    send_arg ("-c");
 	if (delete_flag)
-	    send_arg("-d");
+	    send_arg ("-d");
 	if (force_tag_move)
-	    send_arg("-F");
+	    send_arg ("-F");
 	if (!force_tag_match)
 	    send_arg ("-f");
 	if (local)
-	    send_arg("-l");
+	    send_arg ("-l");
 	if (!run_module_prog)
-	    send_arg("-n");
+	    send_arg ("-n");
 
 	if (numtag)
 	    option_with_arg ("-r", numtag);
@@ -263,11 +258,11 @@ cvstag (argc, argv)
 	for (i = 0; i < argc; i++)
 	{
 	    /* XXX last arg should be repository, but doesn't make sense here */
-	    history_write ('T', (delete_flag ? "D" : (numtag ? numtag : 
+	    history_write ('T', (delete_flag ? "D" : (numtag ? numtag :
 			   (date ? date : "A"))), symtag, argv[i], "");
 	    err += do_module (db, argv[i], TAG,
 			      delete_flag ? "Untagging" : "Tagging",
-			      rtag_proc, (char *) NULL, 0, local, run_module_prog,
+			      rtag_proc, NULL, 0, local, run_module_prog,
 			      0, symtag);
 	}
 	close_module (db);
@@ -278,25 +273,18 @@ cvstag (argc, argv)
 			 NULL);
     }
 
-    return (err);
+    return err;
 }
 
+
+
 /*
  * callback proc for doing the real work of tagging
  */
 /* ARGSUSED */
 static int
-rtag_proc (argc, argv, xwhere, mwhere, mfile, shorten, local_specified,
-	   mname, msg)
-    int argc;
-    char **argv;
-    char *xwhere;
-    char *mwhere;
-    char *mfile;
-    int shorten;
-    int local_specified;
-    char *mname;
-    char *msg;
+rtag_proc (int argc, char **argv, char *xwhere, char *mwhere, char *mfile,
+           int shorten, int local_specified, char *mname, char *msg)
 {
     /* Begin section which is identical to patch_proc--should this
        be abstracted out somehow?  */
@@ -306,22 +294,50 @@ rtag_proc (argc, argv, xwhere, mwhere, mfile, shorten, local_specified,
     char *repository;
     char *where;
 
+#ifdef HAVE_PRINTF_PTR
+    TRACE (TRACE_FUNCTION,
+	   "rtag_proc (argc=%d, argv=%p, xwhere=%s,\n"
+      "                mwhere=%s, mfile=%s, shorten=%d,\n"
+      "                local_specified=%d, mname=%s, msg=%s)",
+	    argc, (void *)argv, xwhere ? xwhere : "(null)",
+	    mwhere ? mwhere : "(null)", mfile ? mfile : "(null)",
+	    shorten, local_specified,
+	    mname ? mname : "(null)", msg ? msg : "(null)" );
+#else
+    TRACE (TRACE_FUNCTION,
+	   "rtag_proc (argc=%d, argv=%lx, xwhere=%s,\n"
+      "                mwhere=%s, mfile=%s, shorten=%d,\n"
+      "                local_specified=%d, mname=%s, msg=%s )",
+	    argc, (unsigned long)argv, xwhere ? xwhere : "(null)",
+	    mwhere ? mwhere : "(null)", mfile ? mfile : "(null)",
+	    shorten, local_specified,
+	    mname ? mname : "(null)", msg ? msg : "(null)" );
+#endif
+
     if (is_rtag)
     {
-	repository = xmalloc (strlen (current_parsed_root->directory) + strlen (argv[0])
-			      + (mfile == NULL ? 0 : strlen (mfile) + 1) + 2);
-	(void) sprintf (repository, "%s/%s", current_parsed_root->directory, argv[0]);
-	where = xmalloc (strlen (argv[0]) + (mfile == NULL ? 0 : strlen (mfile) + 1)
+	repository = xmalloc (strlen (current_parsed_root->directory)
+                              + strlen (argv[0])
+			      + (mfile == NULL ? 0 : strlen (mfile) + 1)
+                              + 2);
+	(void) sprintf (repository, "%s/%s", current_parsed_root->directory,
+                        argv[0]);
+	where = xmalloc (strlen (argv[0])
+                         + (mfile == NULL ? 0 : strlen (mfile) + 1)
 			 + 1);
 	(void) strcpy (where, argv[0]);
 
-	/* if mfile isn't null, we need to set up to do only part of the module */
+	/* If MFILE isn't null, we need to set up to do only part of the
+         * module.
+         */
 	if (mfile != NULL)
 	{
 	    char *cp;
 	    char *path;
 
-	    /* if the portion of the module is a path, put the dir part on repos */
+	    /* If the portion of the module is a path, put the dir part on
+             * REPOS.
+             */
 	    if ((cp = strrchr (mfile, '/')) != NULL)
 	    {
 		*cp = '\0';
@@ -353,11 +369,11 @@ rtag_proc (argc, argv, xwhere, mwhere, mfile, shorten, local_specified,
 	}
 
 	/* cd to the starting repository */
-	if ( CVS_CHDIR (repository) < 0)
+	if (CVS_CHDIR (repository) < 0)
 	{
 	    error (0, errno, "cannot chdir to %s", repository);
 	    free (repository);
-	    return (1);
+	    return 1;
 	}
 	/* End section which is identical to patch_proc.  */
 
@@ -375,57 +391,65 @@ rtag_proc (argc, argv, xwhere, mwhere, mfile, shorten, local_specified,
 
     if (numtag != NULL && !numtag_validated)
     {
-	tag_check_valid (numtag, argc - 1, argv + 1, local_specified, 0, repository);
-	numtag_validated = 1;
+	tag_check_valid (numtag, argc - 1, argv + 1, local_specified, 0,
+			 repository );
+	numtag_validated = true;
     }
 
-    /* check to make sure they are authorized to tag all the 
+    /* check to make sure they are authorized to tag all the
        specified files in the repository */
 
     mtlist = getlist();
     err = start_recursion (check_fileproc, check_filesdoneproc,
-                           (DIRENTPROC) NULL, (DIRLEAVEPROC) NULL, NULL,
+                           NULL, NULL, NULL,
 			   argc - 1, argv + 1, local_specified, which, 0,
 			   CVS_LOCK_READ, where, 1, repository);
-    
+
     if (err)
     {
        error (1, 0, "correct the above errors first!");
     }
-     
+
     /* It would be nice to provide consistency with respect to
        commits; however CVS lacks the infrastructure to do that (see
        Concurrency in cvs.texinfo and comment in do_recursion).  */
 
     /* start the recursion processor */
-    err = start_recursion (is_rtag ? rtag_fileproc : tag_fileproc,
-			   (FILESDONEPROC) NULL, tag_dirproc,
-			   (DIRLEAVEPROC) NULL, NULL, argc - 1, argv + 1,
-			   local_specified, which, 0, CVS_LOCK_WRITE, where, 1,
-			   repository);
-    if ( which & W_REPOS ) free ( repository );
+    err = start_recursion
+	(is_rtag ? rtag_fileproc : tag_fileproc,
+	 NULL, tag_dirproc, NULL, NULL, argc - 1, argv + 1,
+	 local_specified, which, 0, CVS_LOCK_WRITE, where, 1,
+	 repository);
     dellist (&mtlist);
+    if (which & W_REPOS) free (repository);
     if (where != NULL)
 	free (where);
-    return (err);
+    return err;
 }
 
+
+
 /* check file that is to be tagged */
 /* All we do here is add it to our list */
-
 static int
-check_fileproc (callerdat, finfo)
-    void *callerdat;
-    struct file_info *finfo;
+check_fileproc (void *callerdat, struct file_info *finfo)
 {
     const char *xdir;
     Node *p;
     Vers_TS *vers;
-    
-    if (check_uptodate) 
+    List *tlist;
+    struct tag_info *ti;
+    int addit = 1;
+
+    TRACE (TRACE_FUNCTION, "check_fileproc (%s, %s, %s)",
+	   finfo->repository ? finfo->repository : "(null)",
+	   finfo->fullname ? finfo->fullname : "(null)",
+	   finfo->rcs ? (finfo->rcs->path ? finfo->rcs->path : "(null)")
+	   : "NULL");
+
+    if (check_uptodate)
     {
-	switch (Classify_File (finfo, (char *) NULL, (char *) NULL,
-				      (char *) NULL, 1, 0, &vers, 0))
+	switch (Classify_File (finfo, NULL, NULL, NULL, 1, 0, &vers, 0))
 	{
 	case T_UPTODATE:
 	case T_CHECKOUT:
@@ -441,12 +465,12 @@ check_fileproc (callerdat, finfo)
 	default:
 	    error (0, 0, "%s is locally modified", finfo->fullname);
 	    freevers_ts (&vers);
-	    return (1);
+	    return 1;
 	}
     }
     else
 	vers = Version_TS (finfo, NULL, NULL, NULL, 0, 0);
-    
+
     if (finfo->update_dir[0] == '\0')
 	xdir = ".";
     else
@@ -458,13 +482,12 @@ check_fileproc (callerdat, finfo)
     else
     {
 	struct master_lists *ml;
-        
+
 	tlist = getlist ();
 	p = getnode ();
 	p->key = xstrdup (xdir);
 	p->type = UPDATE;
-	ml = (struct master_lists *)
-	    xmalloc (sizeof (struct master_lists));
+	ml = xmalloc (sizeof (struct master_lists));
 	ml->tlist = tlist;
 	p->data = ml;
 	p->delproc = masterlist_delproc;
@@ -481,27 +504,25 @@ check_fileproc (callerdat, finfo)
 	    error (0, 0, "nothing known about %s", finfo->file);
 	freevers_ts (&vers);
 	freenode (p);
-	return (1);
+	return 1;
     }
 
     /* Here we duplicate the calculation in tag_fileproc about which
        version we are going to tag.  There probably are some subtle races
        (e.g. numtag is "foo" which gets moved between here and
        tag_fileproc).  */
+    p->data = ti = xmalloc (sizeof (struct tag_info));
     if (!is_rtag && numtag == NULL && date == NULL)
-	p->data = xstrdup (vers->vn_user);
+	ti->rev = xstrdup (vers->vn_user);
     else
-	p->data = RCS_getversion (vers->srcfile, numtag, date,
+	ti->rev = RCS_getversion (vers->srcfile, numtag, date,
 				  force_tag_match, NULL);
 
-    if (p->data != NULL)
+    if (ti->rev != NULL)
     {
-        int addit = 1;
-        char *oversion;
-        
-        oversion = RCS_getversion (vers->srcfile, symtag, (char *) NULL, 1,
-				   (int *) NULL);
-        if (oversion == NULL) 
+        ti->oldrev = RCS_getversion (vers->srcfile, symtag, NULL, 1, NULL);
+
+	if (ti->oldrev == NULL)
         {
             if (delete_flag)
             {
@@ -512,42 +533,48 @@ check_fileproc (callerdat, finfo)
         }
 	else if (delete_flag)
 	{
-	    free (p->data);
-	    p->data = xstrdup (oversion);
+	    free (ti->rev);
+#ifdef SUPPORT_OLD_INFO_FMT_STRINGS
+	    /* a hack since %v used to mean old or new rev */
+	    ti->rev = xstrdup (ti->oldrev);
+#else /* SUPPORT_OLD_INFO_FMT_STRINGS */
+	    ti->rev = NULL;
+#endif /* SUPPORT_OLD_INFO_FMT_STRINGS */
 	}
-        else if (strcmp(oversion, p->data) == 0)
-        {
+        else if (strcmp(ti->oldrev, p->data) == 0)
             addit = 0;
-        }
         else if (!force_tag_move)
-        {
             addit = 0;
-        }
-        if (oversion != NULL)
-        {
-            free(oversion);
-        }
-        if (!addit)
-        {
-            free(p->data);
-            p->data = NULL;
-        }
+    }
+    else
+	addit = 0;
+    if (!addit)
+    {
+	free(p->data);
+	p->data = NULL;
     }
     freevers_ts (&vers);
-    (void) addnode (tlist, p);
-    return (0);
+    (void)addnode (tlist, p);
+    return 0;
 }
-                         
+
+
+
+struct pretag_proc_data {
+     List *tlist;
+     bool delete_flag;
+     bool force_tag_move;
+     char *symtag;
+};
+
 static int
-check_filesdoneproc (callerdat, err, repos, update_dir, entries)
-    void *callerdat;
-    int err;
-    const char *repos;
-    const char *update_dir;
-    List *entries;
+check_filesdoneproc (void *callerdat, int err, const char *repos,
+                     const char *update_dir, List *entries)
 {
     int n;
     Node *p;
+    List *tlist;
+    struct pretag_proc_data ppd;
 
     p = findnode(mtlist, update_dir);
     if (p != NULL)
@@ -562,51 +589,111 @@ check_filesdoneproc (callerdat, err, repos, update_dir, entries)
     {
         return (err);
     }
-    if ((n = Parse_Info(CVSROOTADM_TAGINFO, repos, pretag_proc, 1)) > 0)
+
+    ppd.tlist = tlist;
+    ppd.delete_flag = delete_flag;
+    ppd.force_tag_move = force_tag_move;
+    ppd.symtag = symtag;
+    if ((n = Parse_Info(CVSROOTADM_TAGINFO, repos, pretag_proc, PIOPT_ALL, &ppd)) > 0)
     {
         error (0, 0, "Pre-tag check failed");
         err += n;
     }
-    return (err);
+    return err;
 }
 
-static int
-pretag_proc (repository, filter)
-    const char *repository;
-    const char *filter;
-{
-    if (filter[0] == '/')
-    {
-        char *s, *cp;
 
-        s = xstrdup(filter);
-        for (cp=s; *cp; cp++)
-        {
-            if (isspace ((unsigned char) *cp))
-            {
-                *cp = '\0';
-                break;
-            }
-        }
-        if (!isfile(s))
-        {
-            error (0, errno, "cannot find pre-tag filter '%s'", s);
-            free(s);
-            return (1);
-        }
-        free(s);
+
+/*
+ * called from Parse_Info, this routine processes a line that came out
+ * of a taginfo file and turns it into a command and executes it.
+ *
+ * RETURNS
+ *    the absolute value of the return value of run_exec, which may or
+ *    may not be the return value of the child process.  this is
+ *    contrained to return positive values because Parse_Info is adding up
+ *    return values and testing for non-zeroness to signify one or more
+ *    of its callbacks having returned an error.
+ */
+static int
+pretag_proc (const char *repository, const char *filter, void *closure)
+{
+    char *newfilter = NULL;
+    char *cmdline;
+    const char *srepos = Short_Repository (repository);
+    struct pretag_proc_data *ppd = (struct pretag_proc_data *)closure;
+
+#ifdef SUPPORT_OLD_INFO_FMT_STRINGS
+    if (!strchr(filter, '%'))
+    {
+	error(0,0,
+              "warning: taginfo line contains no format strings:\n"
+              "    \"%s\"\n"
+              "Filling in old defaults ('%%t %%o %%p %%{sv}'), but please be aware that this\n"
+              "usage is deprecated.", filter);
+	newfilter = xmalloc (strlen(filter) + 16);
+	strcpy (newfilter, filter);
+	strcat (newfilter, " %t %o %p %{sv}");
+	filter = newfilter;
     }
-    run_setup (filter);
-    run_arg (symtag);
-    run_arg (delete_flag ? "del" : force_tag_move ? "mov" : "add");
-    run_arg (repository);
-    walklist(tlist, pretag_list_proc, NULL);
-    return (run_exec (RUN_TTY, RUN_TTY, RUN_TTY, RUN_NORMAL));
+#endif /* SUPPORT_OLD_INFO_FMT_STRINGS */
+
+    /* %t = tag being added/moved/removed
+     * %o = operation = "add" | "mov" | "del"
+     * %b = branch mode = "?" (delete ops - unknown) | "T" (branch) | "N" (not branch)
+     * %p = path from $CVSROOT
+     * %r = path from root
+     * %{sVv} = attribute list = file name, old version tag will be deleted from,
+     *             new version tag will be added to (or deleted from until
+     *             SUPPORT_OLD_INFO_FMT_STRINGS is undefined)
+     */
+    cmdline = format_cmdline(
+#ifdef SUPPORT_OLD_INFO_FMT_STRINGS
+	0, srepos,
+#endif /* SUPPORT_OLD_INFO_FMT_STRINGS */
+	filter,
+	"t", "s", ppd->symtag,
+	"o", "s", ppd->delete_flag ? "del" :
+	          ppd->force_tag_move ? "mov" : "add",
+    	"b", "c", delete_flag ? '?' : branch_mode ? 'T' : 'N',
+    	"p", "s", srepos,
+	"r", "s", current_parsed_root->directory,
+	"sVv", ",", ppd->tlist, pretag_list_to_args_proc, (void *) NULL,
+	(char *)NULL
+	);
+
+    if (newfilter) free (newfilter);
+
+    if (!cmdline || !strlen (cmdline))
+    {
+	if (cmdline) free (cmdline);
+	error(0, 0, "pretag proc resolved to the empty string!");
+	return 1;
+    }
+
+    run_setup (cmdline);
+
+    /* FIXME - the old code used to run the following here:
+     *
+     * if (!isfile(s))
+     * {
+     *     error (0, errno, "cannot find pre-tag filter '%s'", s);
+     *     free(s);
+     *     return (1);
+     * }
+     *
+     * not sure this is really necessary.  it might give a little finer grained
+     * error than letting the execution attempt fail but i'm not sure.  in any
+     * case it should be easy enough to add a function in run.c to test its
+     * first arg for fileness & executability.
+     */
+
+    free (cmdline);
+    return abs (run_exec (RUN_TTY, RUN_TTY, RUN_TTY, RUN_NORMAL));
 }
 
 static void
-masterlist_delproc(p)
-    Node *p;
+masterlist_delproc(Node *p)
 {
     struct master_lists *ml = p->data;
 
@@ -616,28 +703,93 @@ masterlist_delproc(p)
 }
 
 static void
-tag_delproc(p)
-    Node *p;
+tag_delproc(Node *p)
 {
+    struct tag_info *ti;
     if (p->data != NULL)
     {
+	ti = (struct tag_info *) p->data;
+	if (ti->oldrev) free (ti->oldrev);
+	if (ti->rev) free (ti->rev);
         free(p->data);
         p->data = NULL;
     }
     return;
 }
 
+/* to be passed into walklist with a list of tags
+ * p->key = tagname
+ * p->data = struct tag_info *
+ * p->data->oldrev = rev tag will be deleted from
+ * p->data->rev = rev tag will be added to
+ *
+ * closure will be a struct format_cmdline_walklist_closure
+ * where closure is undefined
+ */
 static int
-pretag_list_proc(p, closure)
-    Node *p;
-    void *closure;
+pretag_list_to_args_proc(Node *p, void *closure)
 {
-    if (p->data != NULL)
+    struct tag_info *taginfo = (struct tag_info *)p->data;
+    struct format_cmdline_walklist_closure *c =
+            (struct format_cmdline_walklist_closure *)closure;
+    char *arg = NULL;
+    const char *f;
+    char *d;
+    size_t doff;
+
+    if (p->data == NULL) return 1;
+
+    f = c->format;
+    d = *c->d;
+    /* foreach requested attribute */
+    while (*f)
     {
-        run_arg(p->key);
-        run_arg(p->data);
+   	switch (*f++)
+	{
+	    case 's':
+		arg = p->key;
+		break;
+	    case 'v':
+		arg = taginfo->rev ? taginfo->rev : "NONE";
+		break;
+	    case 'V':
+		arg = taginfo->oldrev ? taginfo->oldrev : "NONE";
+		break;
+	    default:
+		error(1,0,
+                      "Unknown format character or not a list attribute: %c",
+		      f[-1]);
+		break;
+	}
+	/* copy the attribute into an argument */
+	if (c->quotes)
+	{
+	    arg = cmdlineescape (c->quotes, arg);
+	}
+	else
+	{
+	    arg = cmdlinequote ('"', arg);
+	}
+
+	doff = d - *c->buf;
+	expand_string (c->buf, c->length, doff + strlen (arg));
+	d = *c->buf + doff;
+	strncpy (d, arg, strlen (arg));
+	d += strlen (arg);
+
+	free (arg);
+
+	/* and always put the extra space on.  we'll have to back up a char when we're
+	 * done, but that seems most efficient
+	 */
+	doff = d - *c->buf;
+	expand_string (c->buf, c->length, doff + 1);
+	d = *c->buf + doff;
+	*d++ = ' ';
     }
-    return (0);
+    /* correct our original pointer into the buff */
+    *c->d = d;
+    return 0;
 }
 
 
@@ -647,9 +799,7 @@ pretag_list_proc(p, closure)
  */
 /* ARGSUSED */
 static int
-rtag_fileproc (callerdat, finfo)
-    void *callerdat;
-    struct file_info *finfo;
+rtag_fileproc (void *callerdat, struct file_info *finfo)
 {
     RCSNode *rcsfile;
     char *version, *rev;
@@ -657,7 +807,7 @@ rtag_fileproc (callerdat, finfo)
 
     /* find the parsed RCS data */
     if ((rcsfile = finfo->rcs) == NULL)
-	return (1);
+	return 1;
 
     /*
      * For tagging an RCS file which is a symbolic link, you'd best be
@@ -666,7 +816,7 @@ rtag_fileproc (callerdat, finfo)
      */
 
     if (delete_flag)
-	return (rtag_delete (rcsfile));
+	return rtag_delete (rcsfile);
 
     /*
      * If we get here, we are adding a tag.  But, if -a was specified, we
@@ -676,27 +826,26 @@ rtag_fileproc (callerdat, finfo)
     if (attic_too && (!numtag && !date))
     {
 	if ((rcsfile->flags & VALID) && (rcsfile->flags & INATTIC))
-	    return (rtag_delete (rcsfile));
+	    return rtag_delete (rcsfile);
     }
 
-    version = RCS_getversion (rcsfile, numtag, date, force_tag_match,
-			      (int *) NULL);
+    version = RCS_getversion (rcsfile, numtag, date, force_tag_match, NULL);
     if (version == NULL)
     {
 	/* If -a specified, clean up any old tags */
 	if (attic_too)
-	    (void) rtag_delete (rcsfile);
+	    (void)rtag_delete (rcsfile);
 
 	if (!quiet && !force_tag_match)
 	{
 	    error (0, 0, "cannot find tag `%s' in `%s'",
 		   numtag ? numtag : "head", rcsfile->path);
-	    return (1);
+	    return 1;
 	}
-	return (0);
+	return 0;
     }
     if (numtag
-	&& isdigit ((unsigned char) *numtag)
+	&& isdigit ((unsigned char)*numtag)
 	&& strcmp (numtag, version) != 0)
     {
 
@@ -716,7 +865,7 @@ rtag_fileproc (callerdat, finfo)
     else
     {
 	char *oversion;
-       
+
 	/*
 	 * As an enhancement for the case where a tag is being re-applied to
 	 * a large body of a module, make one extra call to RCS_getversion to
@@ -743,35 +892,35 @@ rtag_fileproc (callerdat, finfo)
 		free (version);
 		return (0);
 	    }
-	  
+
 	    if (!force_tag_move)
 	    {
 		/* we're NOT going to move the tag */
-		(void) printf ("W %s", finfo->fullname);
+		(void)printf ("W %s", finfo->fullname);
 
-		(void) printf (" : %s already exists on %s %s", 
-			       symtag, isbranch ? "branch" : "version",
-			       oversion);
-		(void) printf (" : NOT MOVING tag to %s %s\n", 
-			       branch_mode ? "branch" : "version", rev);
+		(void)printf (" : %s already exists on %s %s",
+			      symtag, isbranch ? "branch" : "version",
+			      oversion);
+		(void)printf (" : NOT MOVING tag to %s %s\n",
+			      branch_mode ? "branch" : "version", rev);
 		free (oversion);
 		free (version);
-		if (branch_mode) free(rev);
-		return (0);
+		if (branch_mode) free (rev);
+		return 0;
 	    }
 	    else /* force_tag_move is set and... */
 		if ((isbranch && !disturb_branch_tags) ||
 		    (!isbranch && disturb_branch_tags))
 	    {
-	        error(0,0, "%s: Not moving %s tag `%s' from %s to %s%s.", 
+	        error(0,0, "%s: Not moving %s tag `%s' from %s to %s%s.",
 			finfo->fullname,
 			isbranch ? "branch" : "non-branch",
 			symtag, oversion, rev,
-			isbranch ? "" : " due to `-B' option"); 
+			isbranch ? "" : " due to `-B' option");
 		if (branch_mode) free(rev);
 		free (oversion);
 		free (version);
-		return (0);
+		return 0;
 	    }
 	    free (oversion);
 	}
@@ -788,28 +937,29 @@ rtag_fileproc (callerdat, finfo)
         if (branch_mode)
 	    free (rev);
         free (version);
-        return (1);
+        return 1;
     }
     if (branch_mode)
 	free (rev);
     free (version);
-    return (0);
+    return 0;
 }
 
+
+
 /*
  * If -d is specified, "force_tag_match" is set, so that this call to
  * RCS_getversion() will return a NULL version string if the symbolic
  * tag does not exist in the RCS file.
- * 
+ *
  * If the -r flag was used, numtag is set, and we only delete the
  * symtag from files that have numtag.
- * 
+ *
  * This is done here because it's MUCH faster than just blindly calling
  * "rcs" to remove the tag... trust me.
  */
 static int
-rtag_delete (rcsfile)
-    RCSNode *rcsfile;
+rtag_delete (RCSNode *rcsfile)
 {
     char *version;
     int retcode, isbranch;
@@ -823,24 +973,23 @@ rtag_delete (rcsfile)
 	free (version);
     }
 
-    version = RCS_getversion (rcsfile, symtag, (char *) NULL, 1,
-			      (int *) NULL);
+    version = RCS_getversion (rcsfile, symtag, NULL, 1, NULL);
     if (version == NULL)
-	return (0);
+	return 0;
     free (version);
 
 
     isbranch = RCS_nodeisbranch (rcsfile, symtag);
-    if ((isbranch && !disturb_branch_tags) || 
+    if ((isbranch && !disturb_branch_tags) ||
 	(!isbranch && disturb_branch_tags))
     {
 	if (!quiet)
-	    error(0, 0,
-		"Not removing %s tag `%s' from `%s'%s.", 
-		isbranch ? "branch" : "non-branch",
-		symtag, rcsfile->path,
-		isbranch ? "" : " due to `-B' option"); 
-	return (1);
+	    error (0, 0,
+                   "Not removing %s tag `%s' from `%s'%s.",
+                   isbranch ? "branch" : "non-branch",
+                   symtag, rcsfile->path,
+                   isbranch ? "" : " due to `-B' option");
+	return 1;
     }
 
     if ((retcode = RCS_deltag(rcsfile, symtag)) != 0)
@@ -849,22 +998,21 @@ rtag_delete (rcsfile)
 	    error (0, retcode == -1 ? errno : 0,
 		   "failed to remove tag `%s' from `%s'", symtag,
 		   rcsfile->path);
-	return (1);
+	return 1;
     }
     RCS_rewrite (rcsfile, NULL, NULL);
-    return (0);
+    return 0;
 }
 
 
+
 /*
  * Called to tag a particular file (the currently checked out version is
  * tagged with the specified tag - or the specified tag is deleted).
  */
 /* ARGSUSED */
 static int
-tag_fileproc (callerdat, finfo)
-    void *callerdat;
-    struct file_info *finfo;
+tag_fileproc (void *callerdat, struct file_info *finfo)
 {
     char *version, *oversion;
     char *nversion = NULL;
@@ -881,7 +1029,7 @@ tag_fileproc (callerdat, finfo)
                                   numtag,
                                   date,
                                   force_tag_match,
-				  (int *) NULL);
+				  NULL);
         if (nversion == NULL)
 	    goto free_vars_and_return;
     }
@@ -893,7 +1041,7 @@ tag_fileproc (callerdat, finfo)
 	 * If -d is specified, "force_tag_match" is set, so that this call to
 	 * RCS_getversion() will return a NULL version string if the symbolic
 	 * tag does not exist in the RCS file.
-	 * 
+	 *
 	 * This is done here because it's MUCH faster than just blindly calling
 	 * "rcs" to remove the tag... trust me.
 	 */
@@ -906,20 +1054,20 @@ tag_fileproc (callerdat, finfo)
 	free (version);
 
 	isbranch = RCS_nodeisbranch (finfo->rcs, symtag);
-	if ((isbranch && !disturb_branch_tags) || 
+	if ((isbranch && !disturb_branch_tags) ||
 	    (!isbranch && disturb_branch_tags))
 	{
 	    if (!quiet)
 		error(0, 0,
-		       "Not removing %s tag `%s' from `%s'%s.", 
+		       "Not removing %s tag `%s' from `%s'%s.",
 			isbranch ? "branch" : "non-branch",
 			symtag, vers->srcfile->path,
-			isbranch ? "" : " due to `-B' option"); 
+			isbranch ? "" : " due to `-B' option");
 	    retval = 1;
 	    goto free_vars_and_return;
 	}
 
-	if ((retcode = RCS_deltag(vers->srcfile, symtag)) != 0) 
+	if ((retcode = RCS_deltag(vers->srcfile, symtag)) != 0)
 	{
 	    if (!quiet)
 		error (0, retcode == -1 ? errno : 0,
@@ -992,7 +1140,7 @@ tag_fileproc (callerdat, finfo)
 	int isbranch = RCS_nodeisbranch (finfo->rcs, symtag);
 
 	/*
-	 * if versions the same and neither old or new are branches don't have 
+	 * if versions the same and neither old or new are branches don't have
 	 * to do anything
 	 */
 	if (strcmp (version, oversion) == 0 && !branch_mode && !isbranch)
@@ -1028,11 +1176,11 @@ tag_fileproc (callerdat, finfo)
 		if ((isbranch && !disturb_branch_tags) ||
 		    (!isbranch && disturb_branch_tags))
 	{
-	    error(0,0, "%s: Not moving %s tag `%s' from %s to %s%s.", 
+	    error(0,0, "%s: Not moving %s tag `%s' from %s to %s%s.",
 		    finfo->fullname,
 		    isbranch ? "branch" : "non-branch",
 		    symtag, oversion, rev,
-		    isbranch ? "" : " due to `-B' option"); 
+		    isbranch ? "" : " due to `-B' option");
 	    free (oversion);
 	    if (branch_mode)
 		free (rev);
@@ -1070,17 +1218,15 @@ tag_fileproc (callerdat, finfo)
     return (retval);
 }
 
+
+
 /*
  * Print a warm fuzzy message
  */
 /* ARGSUSED */
 static Dtype
-tag_dirproc (callerdat, dir, repos, update_dir, entries)
-    void *callerdat;
-    const char *dir;
-    const char *repos;
-    const char *update_dir;
-    List *entries;
+tag_dirproc (void *callerdat, const char *dir, const char *repos,
+             const char *update_dir, List *entries)
 {
 
     if (ignore_directory (update_dir))
@@ -1092,10 +1238,13 @@ tag_dirproc (callerdat, dir, repos, update_dir, entries)
     }
 
     if (!quiet)
-	error (0, 0, "%s %s", delete_flag ? "Untagging" : "Tagging", update_dir);
-    return (R_PROCESS);
+	error (0, 0, "%s %s", delete_flag ? "Untagging" : "Tagging",
+               update_dir);
+    return R_PROCESS;
 }
-
+
+
+
 /* Code relating to the val-tags file.  Note that this file has no way
    of knowing when a tag has been deleted.  The problem is that there
    is no way of knowing whether a tag still exists somewhere, when we
@@ -1109,12 +1258,10 @@ struct val_args {
     int found;
 };
 
-static int val_fileproc PROTO ((void *callerdat, struct file_info *finfo));
+static int val_fileproc (void *callerdat, struct file_info *finfo);
 
 static int
-val_fileproc (callerdat, finfo)
-    void *callerdat;
-    struct file_info *finfo;
+val_fileproc (void *callerdat, struct file_info *finfo)
 {
     RCSNode *rcsdata;
     struct val_args *args = (struct val_args *)callerdat;
@@ -1137,16 +1284,9 @@ val_fileproc (callerdat, finfo)
 
 
 
-static Dtype val_direntproc PROTO ((void *, const char *, const char *,
-                                    const char *, List *));
-
 static Dtype
-val_direntproc (callerdat, dir, repository, update_dir, entries)
-    void *callerdat;
-    const char *dir;
-    const char *repository;
-    const char *update_dir;
-    List *entries;
+val_direntproc (void *callerdat, const char *dir, const char *repository,
+                const char *update_dir, List *entries)
 {
     /* This is not quite right--it doesn't get right the case of "cvs
        update -d -r foobar" where foobar is a tag which exists only in
@@ -1170,13 +1310,8 @@ val_direntproc (callerdat, dir, repository, update_dir, entries)
    tag is found in CVSROOTADM_VALTAGS, but there is not (yet) any
    local directory.  */
 void
-tag_check_valid (name, argc, argv, local, aflag, repository)
-    char *name;
-    int argc;
-    char **argv;
-    int local;
-    int aflag;
-    char *repository;
+tag_check_valid (char *name, int argc, char **argv, int local, int aflag,
+                 char *repository)
 {
     DBM *db;
     char *valtags_filename;
@@ -1186,6 +1321,20 @@ tag_check_valid (name, argc, argv, local, aflag, repository)
     struct saved_cwd cwd;
     int which;
 
+#ifdef HAVE_PRINTF_PTR
+    TRACE ( TRACE_FUNCTION,
+	    "tag_check_valid ( name=%s, argc=%d, argv=%p, local=%d,\n"
+       "                       aflag=%d, repository=%s )",
+	    name ? name : "(name)", argc, (void *)argv, local, aflag,
+	    repository ? repository : "(null)" );
+#else
+    TRACE ( TRACE_FUNCTION,
+	    "tag_check_valid ( name=%s, argc=%d, argv=%lx, local=%d,\n"
+       "                       aflag=%d, repository=%s )",
+	    name ? name : "(name)", argc, (unsigned long)argv, local, aflag,
+	    repository ? repository : "(null)" );
+#endif
+
     /* Numeric tags require only a syntactic check.  */
     if (isdigit ((unsigned char) name[0]))
     {
@@ -1204,11 +1353,6 @@ Numeric tag %s contains characters other than digits and '.'", name);
 	|| strcmp (name, TAG_HEAD) == 0)
 	return;
 
-    /* Verify that the tag is valid syntactically.  Some later code once made
-     * assumptions about this.
-     */
-    RCS_check_tag (name);
-
     /* FIXME: This routine doesn't seem to do any locking whatsoever
        (and it is called from places which don't have locks in place).
        If two processes try to write val-tags at the same time, it would
@@ -1268,28 +1412,24 @@ Numeric tag %s contains characters other than digits and '.'", name);
 
     which = W_REPOS | W_ATTIC;
 
-    if (repository != NULL)
+    if (repository == NULL || repository[0] == '\0')
+	which |= W_LOCAL;
+    else
     {
-	if (repository[0] == '\0')
-	    which |= W_LOCAL;
-	else
-	{
-	    if (save_cwd (&cwd))
-		error_exit ();
-	    if (CVS_CHDIR (repository) < 0)
-		error (1, errno, "cannot change to %s directory", repository);
-	}
+	if (save_cwd (&cwd))
+	    exit (EXIT_FAILURE);
+	if (CVS_CHDIR (repository) < 0)
+	    error (1, errno, "cannot change to %s directory", repository);
     }
 
-    start_recursion (val_fileproc, (FILESDONEPROC) NULL,
-		     val_direntproc, (DIRLEAVEPROC) NULL,
-		     (void *)&the_val_args,
-		     argc, argv, local, which, aflag,
-		     CVS_LOCK_READ, NULL, 1, repository);
+    start_recursion
+	(val_fileproc, NULL, val_direntproc, NULL,
+	 &the_val_args, argc, argv, local, which, aflag,
+	 CVS_LOCK_READ, NULL, 1, repository);
     if (repository != NULL && repository[0] != '\0')
     {
 	if (restore_cwd (&cwd, NULL))
-	    error_exit ();
+	    exit (EXIT_FAILURE);
 	free_cwd (&cwd);
     }
 
@@ -1338,13 +1478,7 @@ Numeric tag %s contains characters other than digits and '.'", name);
  */
 
 void
-tag_check_valid_join (join_tag, argc, argv, local, aflag, repository)
-     char *join_tag;
-     int argc;
-     char **argv;
-     int local;
-     int aflag;
-     char *repository;
+tag_check_valid_join (char *join_tag, int argc, char **argv, int local, int aflag, char *repository)
 {
     char *c, *s;
 
