@@ -1,11 +1,6 @@
 /*
- * Copyright (C) 1986-2005 The Free Software Foundation, Inc.
- *
- * Portions Copyright (C) 1998-2005 Derek Price, Ximbiot <http://ximbiot.com>,
- *                                  and others.
- *
- * Portions Copyright (C) 1992, Brian Berliner and Jeff Polk
- * Portions Copyright (C) 1989-1992, Brian Berliner
+ * Copyright (c) 1992, Brian Berliner and Jeff Polk
+ * Copyright (c) 1989-1992, Brian Berliner
  *
  * You may distribute under the terms of the GNU General Public License as
  * specified in the README file that comes with the CVS source distribution.
@@ -39,7 +34,6 @@
  */
 
 #include "cvs.h"
-#include <assert.h>
 #include "savecwd.h"
 #ifdef SERVER_SUPPORT
 # include "md5.h"
@@ -51,34 +45,33 @@
 #include "buffer.h"
 #include "hardlink.h"
 
-static int checkout_file PROTO ((struct file_info *finfo, Vers_TS *vers_ts,
-				 int adding, int merging, int update_server));
+static int checkout_file (struct file_info *finfo, Vers_TS *vers_ts,
+				 int adding, int merging, int update_server);
 #ifdef SERVER_SUPPORT
-static void checkout_to_buffer PROTO ((void *, const char *, size_t));
-static int patch_file PROTO ((struct file_info *finfo,
-			      Vers_TS *vers_ts, 
-			      int *docheckout, struct stat *file_info,
-			      unsigned char *checksum));
-static void patch_file_write PROTO ((void *, const char *, size_t));
-#endif /* SERVER_SUPPORT */
-static int merge_file PROTO ((struct file_info *finfo, Vers_TS *vers));
-static int scratch_file PROTO((struct file_info *finfo, Vers_TS *vers));
-static Dtype update_dirent_proc PROTO ((void *callerdat, const char *dir,
-                                        const char *repository,
-                                        const char *update_dir,
-                                        List *entries));
-static int update_dirleave_proc PROTO ((void *callerdat, const char *dir,
-					int err, const char *update_dir,
-					List *entries));
-static int update_fileproc PROTO ((void *callerdat, struct file_info *));
-static int update_filesdone_proc PROTO ((void *callerdat, int err,
-                                         const char *repository,
-                                         const char *update_dir,
-                                         List *entries));
-#ifdef PRESERVE_PERMISSIONS_SUPPORT
-static int get_linkinfo_proc PROTO ((void *callerdat, struct file_info *));
+static void checkout_to_buffer (void *, const char *, size_t);
+static int patch_file (struct file_info *finfo,
+                       Vers_TS *vers_ts, 
+                       int *docheckout, struct stat *file_info,
+                       unsigned char *checksum);
+static void patch_file_write (void *, const char *, size_t);
 #endif
-static void join_file PROTO ((struct file_info *finfo, Vers_TS *vers_ts));
+static int merge_file (struct file_info *finfo, Vers_TS *vers);
+static int scratch_file (struct file_info *finfo, Vers_TS *vers);
+static Dtype update_dirent_proc (void *callerdat, const char *dir,
+                                 const char *repository,
+                                 const char *update_dir,
+                                 List *entries);
+static int update_dirleave_proc (void *callerdat, const char *dir,
+                                 int err, const char *update_dir,
+                                 List *entries);
+static int update_fileproc (void *callerdat, struct file_info *);
+static int update_filesdone_proc (void *callerdat, int err,
+                                  const char *repository,
+                                  const char *update_dir, List *entries);
+#ifdef PRESERVE_PERMISSIONS_SUPPORT
+static int get_linkinfo_proc( void *_callerdat, struct _finfo * );
+#endif
+static void join_file (struct file_info *finfo, Vers_TS *vers_ts);
 
 static char *options = NULL;
 static char *tag = NULL;
@@ -88,9 +81,11 @@ static char *date = NULL;
    end, once we have the right value for nonbranch, we call WriteTag
    again.  I don't know whether the first call is necessary or not.
    rewrite_tag is nonzero if we are going to have to make that second
-   call.  */
+   call.  warned is nonzero if we've already warned the user that the
+   tag occurs as both a revision tag and a branch tag.  */
 static int rewrite_tag;
 static int nonbranch;
+static int warned;
 
 /* If we set the tag or date for a subdirectory, we use this to undo
    the setting.  See update_dirent_proc.  */
@@ -133,13 +128,13 @@ static const char *const update_usage[] =
     NULL
 };
 
+
+
 /*
  * update is the argv,argc based front end for arg parsing
  */
 int
-update (argc, argv)
-    int argc;
-    char **argv;
+update (int argc, char **argv)
 {
     int c, err;
     int local = 0;			/* recursive by default */
@@ -182,9 +177,11 @@ update (argc, argv)
 		break;
 	    case 'Q':
 	    case 'q':
+#ifdef SERVER_SUPPORT
 		/* The CVS 1.5 client sends these options (in addition to
 		   Global_option requests), so we must ignore them.  */
 		if (!server_active)
+#endif
 		    error (1, 0,
 			   "-q or -Q must be specified before \"%s\"",
 			   cvs_cmd_name);
@@ -199,7 +196,6 @@ update (argc, argv)
 		tag = optarg;
 		break;
 	    case 'D':
-		if (date) free (date);
 		date = Make_Date (optarg);
 		break;
 	    case 'P':
@@ -401,10 +397,11 @@ update (argc, argv)
 	if (aflag || tag || date)
 	{
 	    char *repos = Name_Repository (NULL, NULL);
-	    WriteTag ((char *) NULL, tag, date, 0, ".", repos);
+	    WriteTag (NULL, tag, date, 0, ".", repos);
 	    free (repos);
 	    rewrite_tag = 1;
-	    nonbranch = 0;
+	    nonbranch = -1;
+	    warned = 0;
 	}
     }
 
@@ -418,8 +415,7 @@ update (argc, argv)
     /* call the command line interface */
     err = do_update (argc, argv, options, tag, date, force_tag_match,
 		     local, update_build_dirs, aflag, update_prune_dirs,
-		     pipeout, which, join_rev1, join_rev2, (char *) NULL, 1,
-		     (char *) NULL);
+		     pipeout, which, join_rev1, join_rev2, NULL, 1, NULL);
 
     /* free the space Make_Date allocated if necessary */
     if (date != NULL)
@@ -432,28 +428,17 @@ update (argc, argv)
 
 /*
  * Command line interface to update (used by checkout)
+ *
+ * repository = cvsroot->repository + update_dir.  This is necessary for
+ * checkout so that start_recursion can determine our repository.  In the
+ * update case, start_recursion can use the CVS/Root & CVS/Repository file
+ * to determine this value.
  */
 int
-do_update (argc, argv, xoptions, xtag, xdate, xforce, local, xbuild, xaflag,
-	   xprune, xpipeout, which, xjoin_rev1, xjoin_rev2, preload_update_dir,
-	   xdotemplate, repository)
-    int argc;
-    char **argv;
-    char *xoptions;
-    char *xtag;
-    char *xdate;
-    int xforce;
-    int local;
-    int xbuild;
-    int xaflag;
-    int xprune;
-    int xpipeout;
-    int which;
-    char *xjoin_rev1;
-    char *xjoin_rev2;
-    char *preload_update_dir;
-    int xdotemplate;
-    char *repository;
+do_update (int argc, char **argv, char *xoptions, char *xtag, char *xdate,
+           int xforce, int local, int xbuild, int xaflag, int xprune,
+           int xpipeout, int which, char *xjoin_rev1, char *xjoin_rev2,
+           char *preload_update_dir, int xdotemplate, char *repository)
 {
     int err = 0;
     char *cp;
@@ -478,14 +463,14 @@ do_update (argc, argv, xoptions, xtag, xdate, xforce, local, xbuild, xaflag,
 	date_rev1 = Make_Date (cp);
     }
     else
-	date_rev1 = (char *) NULL;
+	date_rev1 = NULL;
     if (join_rev2 && (cp = strchr (join_rev2, ':')) != NULL)
     {
 	*cp++ = '\0';
 	date_rev2 = Make_Date (cp);
     }
     else
-	date_rev2 = (char *) NULL;
+	date_rev2 = NULL;
 
 #ifdef PRESERVE_PERMISSIONS_SUPPORT
     if (preserve_perms)
@@ -498,10 +483,10 @@ do_update (argc, argv, xoptions, xtag, xdate, xforce, local, xbuild, xaflag,
 	/* FIXME-twp: the arguments to start_recursion make me dizzy.  This
 	   function call was copied from the update_fileproc call that
 	   follows it; someone should make sure that I did it right. */
-	err = start_recursion (get_linkinfo_proc, (FILESDONEPROC) NULL,
-			       (DIRENTPROC) NULL, (DIRLEAVEPROC) NULL, NULL,
-			       argc, argv, local, which, aflag, CVS_LOCK_READ,
-			       preload_update_dir, 1, (char *) NULL);
+	err = start_recursion
+	    (get_linkinfo_proc, NULL, NULL, NULL, NULL,
+	     argc, argv, local, which, aflag, CVS_LOCK_READ,
+	     preload_update_dir, 1, NULL);
 	if (err)
 	    return err;
 
@@ -519,8 +504,13 @@ do_update (argc, argv, xoptions, xtag, xdate, xforce, local, xbuild, xaflag,
 			   argc, argv, local, which, aflag, CVS_LOCK_READ,
 			   preload_update_dir, 1, repository);
 
+#ifdef SERVER_SUPPORT
+    if (server_active)
+	return err;
+#endif
+
     /* see if we need to sleep before returning to avoid time-stamp races */
-    if (!server_active && last_register_time)
+    if (last_register_time)
     {
 	sleep_past (last_register_time);
     }
@@ -528,6 +518,8 @@ do_update (argc, argv, xoptions, xtag, xdate, xforce, local, xbuild, xaflag,
     return err;
 }
 
+
+
 #ifdef PRESERVE_PERMISSIONS_SUPPORT
 /*
  * The get_linkinfo_proc callback adds each file to the hardlist
@@ -535,9 +527,7 @@ do_update (argc, argv, xoptions, xtag, xdate, xforce, local, xbuild, xaflag,
  */
 
 static int
-get_linkinfo_proc (callerdat, finfo)
-    void *callerdat;
-    struct file_info *finfo;
+get_linkinfo_proc( void *callerdat, struct file_info *finfo )
 {
     char *fullpath;
     Node *linkp;
@@ -585,11 +575,9 @@ get_linkinfo_proc (callerdat, finfo)
  * appropriate magic for checkout
  */
 static int
-update_fileproc (callerdat, finfo)
-    void *callerdat;
-    struct file_info *finfo;
+update_fileproc (void *callerdat, struct file_info *finfo)
 {
-    int retval;
+    int retval, nb;
     Ctype status;
     Vers_TS *vers;
 
@@ -598,16 +586,24 @@ update_fileproc (callerdat, finfo)
 
     /* Keep track of whether TAG is a branch tag.
        Note that if it is a branch tag in some files and a nonbranch tag
-       in others, treat it as a nonbranch tag.  It is possible that case
-       should elicit a warning or an error.  */
+       in others, treat it as a nonbranch tag.  */
     if (rewrite_tag
 	&& tag != NULL
 	&& finfo->rcs != NULL)
     {
 	char *rev = RCS_getversion (finfo->rcs, tag, NULL, 1, NULL);
 	if (rev != NULL
-	    && !RCS_nodeisbranch (finfo->rcs, tag))
-	    nonbranch = 1;
+	    && nonbranch != (nb = !RCS_nodeisbranch (finfo->rcs, tag)))
+	{
+	    if (nonbranch >= 0 && !warned && !quiet)
+	    {
+		error (0, 0,
+"warning: %s is a branch tag in some files and a revision tag in others.",
+			tag);
+		warned = 1;
+	    }
+	    if (nonbranch < nb) nonbranch = nb;
+	}
 	if (rev != NULL)
 	    free (rev);
     }
@@ -676,7 +672,11 @@ update_fileproc (callerdat, finfo)
                     bakname = backup_file (finfo->file, vers->vn_user);
                     /* This behavior is sufficiently unexpected to
                        justify overinformativeness, I think. */
-                    if (!really_quiet && !server_active)
+#ifdef SERVER_SUPPORT
+                    if ((! really_quiet) && (! server_active))
+#else /* ! SERVER_SUPPORT */
+                    if (! really_quiet)
+#endif /* SERVER_SUPPORT */
                         (void) printf ("(Locally modified %s moved to %s)\n",
                                        finfo->file, bakname);
                     free (bakname);
@@ -691,7 +691,8 @@ update_fileproc (callerdat, finfo)
                 {
                     if (vers->ts_conflict)
                     {
-			if (file_has_markers (finfo))
+			if (file_has_conflict (finfo, vers->ts_conflict)
+			    || file_has_markers (finfo))
                         {
                             write_letter (finfo, 'C');
                             retval = 1;
@@ -781,12 +782,8 @@ update_fileproc (callerdat, finfo)
 
 
 
-static void update_ignproc PROTO ((const char *, const char *));
-
 static void
-update_ignproc (file, dir)
-    const char *file;
-    const char *dir;
+update_ignproc (const char *file, const char *dir)
 {
     struct file_info finfo;
     char *tmp;
@@ -813,13 +810,10 @@ update_ignproc (file, dir)
 
 /* ARGSUSED */
 static int
-update_filesdone_proc (callerdat, err, repository, update_dir, entries)
-    void *callerdat;
-    int err;
-    const char *repository;
-    const char *update_dir;
-    List *entries;
+update_filesdone_proc (void *callerdat, int err, const char *repository,
+                       const char *update_dir, List *entries)
 {
+    if (nonbranch < 0) nonbranch = 0;
     if (rewrite_tag)
     {
 	WriteTag (NULL, tag, date, nonbranch, update_dir, repository);
@@ -842,11 +836,15 @@ update_filesdone_proc (callerdat, err, repository, update_dir, entries)
 	if (unlink_file_dir (CVSADM) < 0 && !existence_error (errno))
 	    error (0, errno, "cannot remove %s directory", CVSADM);
     }
+#ifdef SERVER_SUPPORT
     else if (!server_active && !pipeout)
+#else
+    else if (!pipeout)
+#endif /* SERVER_SUPPORT */
     {
         /* If there is no CVS/Root file, add one */
         if (!isfile (CVSADM_ROOT))
-	    Create_Root ((char *) NULL, current_parsed_root->original);
+	    Create_Root (NULL, current_parsed_root->original);
     }
 
     return err;
@@ -863,12 +861,8 @@ update_filesdone_proc (callerdat, err, repository, update_dir, entries)
  * recursion code should skip this directory.
  */
 static Dtype
-update_dirent_proc (callerdat, dir, repository, update_dir, entries)
-    void *callerdat;
-    const char *dir;
-    const char *repository;
-    const char *update_dir;
-    List *entries;
+update_dirent_proc (void *callerdat, const char *dir, const char *repository,
+                    const char *update_dir, List *entries)
 {
     if (ignore_directory (update_dir))
     {
@@ -895,11 +889,15 @@ update_dirent_proc (callerdat, dir, repository, update_dir, entries)
 	   is when update -d is specified, and the working directory
 	   is gone but the subdirectory is still mentioned in
 	   CVS/Entries).  */
-	/* In the remote case, the client should refrain from
-	   sending us the directory in the first place.  So we
-	   want to continue to give an error, so clients make
-	   sure to do this.  */
-	if (!server_active && !isdir (repository))
+	if (1
+#ifdef SERVER_SUPPORT
+	    /* In the remote case, the client should refrain from
+	       sending us the directory in the first place.  So we
+	       want to continue to give an error, so clients make
+	       sure to do this.  */
+	    && !server_active
+#endif
+	    && !isdir (repository))
 	    return R_SKIP_ALL;
 
 	if (noexec)
@@ -939,8 +937,9 @@ update_dirent_proc (callerdat, dir, repository, update_dir, entries)
 			  0,
 			  dotemplate);
 	    rewrite_tag = 1;
-	    nonbranch = 0;
-	    Subdir_Register (entries, (char *) NULL, dir);
+	    nonbranch = -1;
+	    warned = 0;
+	    Subdir_Register (entries, NULL, dir);
 	}
     }
     /* Do we need to check noexec here? */
@@ -993,9 +992,12 @@ update_dirent_proc (callerdat, dir, repository, update_dir, entries)
 	{
 	    WriteTag (dir, tag, date, 0, update_dir, repository);
 	    rewrite_tag = 1;
-	    nonbranch = 0;
+	    nonbranch = -1;
+	    warned = 0;
 	}
 
+	WriteTemplate (update_dir, dotemplate, repository);
+
 	/* initialize the ignore list for this directory */
 	ignlist = getlist ();
     }
@@ -1016,12 +1018,8 @@ update_dirent_proc (callerdat, dir, repository, update_dir, entries)
  */
 /* ARGSUSED */
 static int
-update_dirleave_proc (callerdat, dir, err, update_dir, entries)
-    void *callerdat;
-    const char *dir;
-    int err;
-    const char *update_dir;
-    List *entries;
+update_dirleave_proc (void *callerdat, const char *dir, int err,
+                      const char *update_dir, List *entries)
 {
     /* Delete the ignore list if it hasn't already been done.  */
     if (ignlist)
@@ -1043,7 +1041,8 @@ update_dirleave_proc (callerdat, dir, err, update_dir, entries)
 	    free (date);
 	    date = NULL;
 	}
-	nonbranch = 0;
+	nonbranch = -1;
+	warned = 0;
 	free (tag_update_dir);
 	tag_update_dir = NULL;
     }
@@ -1060,7 +1059,7 @@ update_dirleave_proc (callerdat, dir, err, update_dir, entries)
 	       this code used to ignore all errors, I'll play it safe.	*/
 	    if (unlink_file_dir (dir) < 0 && !existence_error (errno))
 		error (0, errno, "cannot remove %s directory", dir);
-	    Subdir_Deregister (entries, (char *) NULL, dir);
+	    Subdir_Deregister (entries, NULL, dir);
 	}
     }
 
@@ -1069,13 +1068,9 @@ update_dirleave_proc (callerdat, dir, err, update_dir, entries)
 
 
 
-static int isremoved PROTO ((Node *, void *));
-
 /* Returns 1 if the file indicated by node has been removed.  */
 static int
-isremoved (node, closure)
-    Node *node;
-    void *closure;
+isremoved (Node *node, void *closure)
 {
     Entnode *entdata = node->data;
 
@@ -1090,9 +1085,7 @@ isremoved (node, closure)
    existence of the CVS directory entry.  Zero otherwise.  If MIGHT_NOT_EXIST
    and the directory doesn't exist, then just return 0.  */
 int
-isemptydir (dir, might_not_exist)
-    const char *dir;
-    int might_not_exist;
+isemptydir (const char *dir, int might_not_exist)
 {
     DIR *dirp;
     struct dirent *dp;
@@ -1129,7 +1122,7 @@ isemptydir (dir, might_not_exist)
 		struct saved_cwd cwd;
 
 		if (save_cwd (&cwd))
-		    error_exit ();
+		    exit (EXIT_FAILURE);
 
 		if (CVS_CHDIR (dir) < 0)
 		    error (1, errno, "cannot change directory to %s", dir);
@@ -1138,7 +1131,7 @@ isemptydir (dir, might_not_exist)
 		Entries_Close (l);
 
 		if (restore_cwd (&cwd, NULL))
-		    error_exit ();
+		    exit (EXIT_FAILURE);
 		free_cwd (&cwd);
 
 		if (files_removed != 0)
@@ -1168,9 +1161,7 @@ isemptydir (dir, might_not_exist)
  * scratch the Entries file entry associated with a file
  */
 static int
-scratch_file (finfo, vers)
-    struct file_info *finfo;
-    Vers_TS *vers;
+scratch_file (struct file_info *finfo, Vers_TS *vers)
 {
     history_write ('W', finfo->update_dir, "", finfo->file, finfo->repository);
     Scratch_Entry (finfo->entries, finfo->file);
@@ -1179,18 +1170,18 @@ scratch_file (finfo, vers)
     {
 	if (vers->ts_user == NULL)
 	    server_scratch_entry_only ();
-	server_updated (finfo, vers,
-		SERVER_UPDATED, (mode_t) -1,
-		(unsigned char *) NULL,
-		(struct buffer *) NULL);
+	server_updated (finfo, vers, SERVER_UPDATED, (mode_t) -1, NULL, NULL);
     }
 #endif
     if (unlink_file (finfo->file) < 0 && ! existence_error (errno))
 	error (0, errno, "unable to remove %s", finfo->fullname);
-    else if (!server_active)
-    {
+    else
+#ifdef SERVER_SUPPORT
 	/* skip this step when the server is running since
 	 * server_updated should have handled it */
+	if (!server_active)
+#endif
+    {
 	/* keep the vers structure up to date in case we do a join
 	 * - if there isn't a file, it can't very well have a version number, can it?
 	 */
@@ -1214,12 +1205,8 @@ scratch_file (finfo, vers)
  * Check out a file.
  */
 static int
-checkout_file (finfo, vers_ts, adding, merging, update_server)
-    struct file_info *finfo;
-    Vers_TS *vers_ts;
-    int adding;
-    int merging;
-    int update_server;
+checkout_file (struct file_info *finfo, Vers_TS *vers_ts, int adding,
+               int merging, int update_server)
 {
     char *backup;
     int set_time, retval = 0;
@@ -1232,7 +1219,11 @@ checkout_file (finfo, vers_ts, adding, merging, update_server)
 
     /* Don't screw with backup files if we're going to stdout, or if
        we are the server.  */
-    if (!pipeout && !server_active)
+    if (!pipeout
+#ifdef SERVER_SUPPORT
+	&& ! server_active
+#endif
+	)
     {
 	backup = xmalloc (strlen (finfo->file)
 			  + sizeof (CVSADM)
@@ -1290,8 +1281,8 @@ VERS: ", 0);
 	    && ! joining ()
 	    && ! wrap_name_has (finfo->file, WRAP_FROMCVS))
 	{
-	    revbuf = buf_nonio_initialize ((BUFMEMERRPROC) NULL);
-	    status = RCS_checkout (vers_ts->srcfile, (char *) NULL,
+	    revbuf = buf_nonio_initialize (NULL);
+	    status = RCS_checkout (vers_ts->srcfile, NULL,
 				   vers_ts->vn_rcs, vers_ts->tag,
 				   vers_ts->options, RUN_TTY,
 				   checkout_to_buffer, revbuf);
@@ -1301,8 +1292,7 @@ VERS: ", 0);
 	    status = RCS_checkout (vers_ts->srcfile,
 				   pipeout ? NULL : finfo->file,
 				   vers_ts->vn_rcs, vers_ts->tag,
-				   vers_ts->options, RUN_TTY,
-				   (RCSCHECKOUTPROC) NULL, (void *) NULL);
+				   vers_ts->options, RUN_TTY, NULL, NULL);
     }
     if (file_is_dead || status == 0)
     {
@@ -1322,11 +1312,9 @@ VERS: ", 0);
 		   That would also fix the kludge with noexec, above, which
 		   is here only because noexec doesn't write srcfile->path
 		   for us to stat.  */
-		if (stat (vers_ts->srcfile->path, &sb) < 0)
+		if( CVS_STAT( vers_ts->srcfile->path, &sb ) < 0 )
 		{
-#if defined (SERVER_SUPPORT) || defined (CLIENT_SUPPORT)
 		    buf_free (revbuf);
-#endif /* defined (SERVER_SUPPORT) || defined (CLIENT_SUPPORT) */
 		    error (1, errno, "cannot stat %s",
 			   vers_ts->srcfile->path);
 		}
@@ -1430,7 +1418,7 @@ VERS: ", 0);
 			  adding ? "0" : xvers_ts->vn_rcs,
 			  xvers_ts->ts_user, xvers_ts->options,
 			  xvers_ts->tag, xvers_ts->date,
-			  (char *)0); /* Clear conflict flag on fresh checkout */
+			  NULL); /* Clear conflict flag on fresh checkout */
 
 	    /* fix up the vers structure, in case it is used by join */
 	    if (join_rev1)
@@ -1493,10 +1481,8 @@ VERS: ", 0);
 	free (backup);
     }
 
-#if defined (SERVER_SUPPORT) || defined (CLIENT_SUPPORT)
     if (revbuf != NULL)
 	buf_free (revbuf);
-#endif /* defined (SERVER_SUPPORT) || defined (CLIENT_SUPPORT) */
     return retval;
 }
 
@@ -1508,10 +1494,7 @@ VERS: ", 0);
    into a buffer.  */
 
 static void
-checkout_to_buffer (callerdat, data, len)
-     void *callerdat;
-     const char *data;
-     size_t len;
+checkout_to_buffer (void *callerdat, const char *data, size_t len)
 {
     struct buffer *buf = (struct buffer *) callerdat;
 
@@ -1544,12 +1527,7 @@ struct patch_file_data
  * itself.
  */
 static int
-patch_file (finfo, vers_ts, docheckout, file_info, checksum)
-    struct file_info *finfo;
-    Vers_TS *vers_ts;
-    int *docheckout;
-    struct stat *file_info;
-    unsigned char *checksum;
+patch_file (struct file_info *finfo, Vers_TS *vers_ts, int *docheckout, struct stat *file_info, unsigned char *checksum)
 {
     char *backup;
     char *file1;
@@ -1697,9 +1675,7 @@ patch_file (finfo, vers_ts, docheckout, file_info, checksum)
     retcode = 0;
     if (! fail)
     {
-	int dargc = 0;
-	size_t darg_allocated = 0;
-	char **dargv = NULL;
+	char *diff_options;
 
 	/* If the client does not support the Rcs-diff command, we
            send a context diff, and the client must invoke patch.
@@ -1707,13 +1683,16 @@ patch_file (finfo, vers_ts, docheckout, file_info, checksum)
            new approach only requires running diff in the server; the
            client can handle everything without invoking an external
            program.  */
-	if (!rcs_diff_patches)
+	if (! rcs_diff_patches)
+	{
 	    /* We use -c, not -u, because that is what CVS has
 	       traditionally used.  Kind of a moot point, now that
 	       Rcs-diff is preferred, so there is no point in making
 	       the compatibility issues worse.  */
-	    run_add_arg_p (&dargc, &darg_allocated, &dargv, "-c");
+	    diff_options = "-c";
+	}
 	else
+	{
 	    /* Now that diff is librarified, we could be passing -a if
 	       we wanted to.  However, it is unclear to me whether we
 	       would want to.  Does diff -a, in any significant
@@ -1723,11 +1702,10 @@ patch_file (finfo, vers_ts, docheckout, file_info, checksum)
 	       'binary'.  Conversely, do they tend to be much larger
 	       in the bad cases?  This needs some more
 	       thought/investigation, I suspect.  */
-	    run_add_arg_p (&dargc, &darg_allocated, &dargv, "-n");
-	retcode = diff_exec (file1, file2, NULL, NULL, dargc, dargv,
-			     finfo->file);
-	run_arg_free_p (dargc, dargv);
-	free (dargv);
+
+	    diff_options = "-n";
+	}
+	retcode = diff_exec (file1, file2, NULL, NULL, diff_options, finfo->file);
 
 	/* A retcode of 0 means no differences.  1 means some differences.  */
 	if (retcode != 0
@@ -1853,10 +1831,7 @@ patch_file (finfo, vers_ts, docheckout, file_info, checksum)
    patch_file via RCS_checkout.  */
 
 static void
-patch_file_write (callerdat, buffer, len)
-     void *callerdat;
-     const char *buffer;
-     size_t len;
+patch_file_write (void *callerdat, const char *buffer, size_t len)
 {
     struct patch_file_data *data = (struct patch_file_data *) callerdat;
 
@@ -1876,9 +1851,7 @@ patch_file_write (callerdat, buffer, len)
  * of a single letter and the name.
  */
 void
-write_letter (finfo, letter)
-    struct file_info *finfo;
-    int letter;
+write_letter (struct file_info *finfo, int letter)
 {
     if (!really_quiet)
     {
@@ -1918,62 +1891,16 @@ write_letter (finfo, letter)
 
 
 
-/* Reregister a file after a merge.  */
-static void
-RegisterMerge PROTO((struct file_info *finfo, Vers_TS *vers,
-		     const char *backup, int has_conflicts));
-static void
-RegisterMerge (finfo, vers, backup, has_conflicts)
-    struct file_info *finfo;
-    Vers_TS *vers;
-    const char *backup;
-    int has_conflicts;
-{
-    /* This file is the result of a merge, which means that it has
-       been modified.  We use a special timestamp string which will
-       not compare equal to any actual timestamp.  */
-    char *cp = NULL;
-
-    if (has_conflicts)
-    {
-	time (&last_register_time);
-	cp = time_stamp (finfo->file);
-    }
-    Register (finfo->entries, finfo->file, vers->vn_rcs ? vers->vn_rcs : "0",
-	      "Result of merge", vers->options, vers->tag, vers->date, cp);
-    if (cp)
-	free (cp);
-
-#ifdef SERVER_SUPPORT
-    /* Send the new contents of the file before the message.  If we
-       wanted to be totally correct, we would have the client write
-       the message only after the file has safely been written.  */
-    if (server_active)
-    {
-        server_copy_file (finfo->file, finfo->update_dir, finfo->repository,
-			  backup);
-	server_updated (finfo, vers, SERVER_MERGED, (mode_t) -1, NULL, NULL);
-    }
-#endif
-}
-
-
-
 /*
  * Do all the magic associated with a file which needs to be merged
  */
 static int
-merge_file (finfo, vers)
-    struct file_info *finfo;
-    Vers_TS *vers;
+merge_file (struct file_info *finfo, Vers_TS *vers)
 {
     char *backup;
     int status;
-    int retcode = 0;
     int retval;
 
-    assert (vers->vn_user);
-
     /*
      * The users currently modified file is moved to a backup file name
      * ".#filename.version", so that it will stay around for a few days
@@ -2011,21 +1938,13 @@ merge_file (finfo, vers)
 	   thought needs to go into this, and in the meantime it is safe
 	   to treat any such mismatch as an automatic conflict. -twp */
 
-	retcode = RCS_checkout (finfo->rcs, finfo->file,
-				vers->vn_rcs, vers->tag,
-				vers->options, NULL, NULL, NULL);
-	if (retcode)
-	{
-	    error (0, 0, "failed to check out `%s' file", finfo->fullname);
-	    error (0, 0, "restoring `%s' from backup file `%s'",
-		   finfo->fullname, backup);
-	    rename_file (backup, finfo->file);
-	    retval = 1;
-	    goto out;
-	}
-	xchmod (finfo->file, 1);
+#ifdef SERVER_SUPPORT
+	if (server_active)
+	    server_copy_file (finfo->file, finfo->update_dir,
+			      finfo->repository, backup);
+#endif
 
-	RegisterMerge (finfo, vers, backup, 1);
+	status = checkout_file (finfo, vers, 0, 1, 1);
 
 	/* Is there a better term than "nonmergeable file"?  What we
 	   really mean is, not something that CVS cannot or does not
@@ -2059,6 +1978,24 @@ merge_file (finfo, vers)
     if (strcmp (vers->options, "-V4") == 0)
 	vers->options[0] = '\0';
 
+    /* This file is the result of a merge, which means that it has
+       been modified.  We use a special timestamp string which will
+       not compare equal to any actual timestamp.  */
+    {
+	char *cp = 0;
+
+	if (status)
+	{
+	    (void) time (&last_register_time);
+	    cp = time_stamp (finfo->file);
+	}
+	Register (finfo->entries, finfo->file, vers->vn_rcs,
+		  "Result of merge", vers->options, vers->tag,
+		  vers->date, cp);
+	if (cp)
+	    free (cp);
+    }
+
     /* fix up the vers structure, in case it is used by join */
     if (join_rev1)
     {
@@ -2069,24 +2006,19 @@ merge_file (finfo, vers)
 	vers->vn_user = xstrdup (vers->vn_rcs);
     }
 
-    RegisterMerge (finfo, vers, backup, status);
-
-    /* FIXME: the noexec case is broken.  RCS_merge could be doing the
-       xcmp on the temporary files without much hassle, I think.  */
-    if (!noexec && !xcmp (backup, finfo->file))
+#ifdef SERVER_SUPPORT
+    /* Send the new contents of the file before the message.  If we
+       wanted to be totally correct, we would have the client write
+       the message only after the file has safely been written.  */
+    if (server_active)
     {
-	cvs_output (finfo->fullname, 0);
-	cvs_output (" already contains the differences between ", 0);
-	cvs_output (vers->vn_user, 0);
-	cvs_output (" and ", 0);
-	cvs_output (vers->vn_rcs, 0);
-	cvs_output ("\n", 1);
-
-	history_write ('G', finfo->update_dir, vers->vn_rcs, finfo->file,
-		       finfo->repository);
-	retval = 0;
-	goto out;
+        server_copy_file (finfo->file, finfo->update_dir, finfo->repository,
+			  backup);
+	server_updated (finfo, vers, SERVER_MERGED,
+			(mode_t) -1, (unsigned char *) NULL,
+			(struct buffer *) NULL);
     }
+#endif
 
     if (status == 1)
     {
@@ -2098,16 +2030,27 @@ merge_file (finfo, vers)
 	               finfo->repository);
 
     }
-    else if (retcode == -1)
+    else /* status == 0 */
     {
-	error (1, errno, "fork failed while examining update of %s",
-	       finfo->fullname);
-    }
-    else
-    {
-	write_letter (finfo, 'M');
 	history_write ('G', finfo->update_dir, vers->vn_rcs, finfo->file,
 		       finfo->repository);
+
+	/* FIXME: the noexec case is broken.  RCS_merge could be doing the
+	   xcmp on the temporary files without much hassle, I think.  */
+	if (!noexec && !xcmp (backup, finfo->file))
+	{
+	    cvs_output (finfo->fullname, 0);
+	    cvs_output (" already contains the differences between ", 0);
+	    cvs_output (vers->vn_user, 0);
+	    cvs_output (" and ", 0);
+	    cvs_output (vers->vn_rcs, 0);
+	    cvs_output ("\n", 1);
+
+	    retval = 0;
+	    goto out;
+	}
+
+	write_letter (finfo, 'M');
     }
     retval = 0;
  out:
@@ -2134,9 +2077,7 @@ merge_file (finfo, vers)
  *   1.  Is not called in client mode.
  */
 static void
-join_file (finfo, vers)
-    struct file_info *finfo;
-    Vers_TS *vers;
+join_file (struct file_info *finfo, Vers_TS *vers)
 {
     char *backup;
     char *t_options;
@@ -2149,16 +2090,14 @@ join_file (finfo, vers)
     char *jdate1;
     char *jdate2;
 
-    if (trace)
-	fprintf (stderr, "%s-> join_file(%s, %s%s%s%s, %s, %s)\n",
-		CLIENT_SERVER_STR,
-		finfo->file,
-		vers->tag ? vers->tag : "",
-		vers->tag ? " (" : "",
-		vers->vn_rcs ? vers->vn_rcs : "",
-		vers->tag ? ")" : "",
-		join_rev1 ? join_rev1 : "",
-		join_rev2 ? join_rev2 : "");
+    TRACE ( 1, "join_file(%s, %s%s%s%s, %s, %s)",
+	    finfo->file,
+	    vers->tag ? vers->tag : "",
+	    vers->tag ? " (" : "",
+	    vers->vn_rcs ? vers->vn_rcs : "",
+	    vers->tag ? ")" : "",
+	    join_rev1 ? join_rev1 : "",
+	    join_rev2 ? join_rev2 : "" );
 
     jrev1 = join_rev1;
     jrev2 = join_rev2;
@@ -2361,7 +2300,7 @@ join_file (finfo, vers)
 			       finfo->repository);
 #endif
 	if (! really_quiet)
-	    error (0, 0, "scheduling %s for removal", finfo->fullname);
+	    error (0, 0, "scheduling `%s' for removal", finfo->fullname);
 
 	return;
     }
@@ -2663,7 +2602,31 @@ join_file (finfo, vers)
        RCS_checkout above, and we aren't running as the server.
        However, that is not the normal case, and calling Register
        again won't cost much in that case.  */
-    RegisterMerge (finfo, vers, backup, status);
+    {
+	char *cp = 0;
+
+	if (status)
+	{
+	    (void) time (&last_register_time);
+	    cp = time_stamp (finfo->file);
+	}
+	Register (finfo->entries, finfo->file,
+		  vers->vn_rcs ? vers->vn_rcs : "0", "Result of merge",
+		  vers->options, vers->tag, vers->date, cp);
+	if (cp)
+	    free(cp);
+    }
+
+#ifdef SERVER_SUPPORT
+    if (server_active)
+    {
+	server_copy_file (finfo->file, finfo->update_dir, finfo->repository,
+			  backup);
+	server_updated (finfo, vers, SERVER_MERGED,
+			(mode_t) -1, (unsigned char *) NULL,
+			(struct buffer *) NULL);
+    }
+#endif
 
 out:
     free (rev1);
@@ -2687,10 +2650,7 @@ out:
  */
 
 int
-special_file_mismatch (finfo, rev1, rev2)
-    struct file_info *finfo;
-    char *rev1;
-    char *rev2;
+special_file_mismatch (struct file_info *finfo, char *rev1, char *rev2)
 {
 #ifdef PRESERVE_PERMISSIONS_SUPPORT
     struct stat sb;
@@ -2987,7 +2947,7 @@ special_file_mismatch (finfo, rev1, rev2)
 
 
 int
-joining ()
+joining (void)
 {
     return join_rev1 != NULL;
 }
