@@ -17,13 +17,14 @@
 
 const char *const watch_usage[] =
 {
-    "Usage: %s %s [on|off|add|remove] [-lR] [-a action] [files...]\n",
+    "Usage: %s %s {on|off|add|remove} [-lR] [-a <action>]... [<path>]...\n",
     "on/off: turn on/off read-only checkouts of files\n",
     "add/remove: add or remove notification on actions\n",
     "-l (on/off/add/remove): Local directory only, not recursive\n",
-    "-R (on/off/add/remove): Process directories recursively\n",
+    "-R (on/off/add/remove): Process directories recursively (default)\n",
     "-a (add/remove): Specify what actions, one of\n",
-    "    edit,unedit,commit,all,none\n",
+    "    edit,unedit,commit,all,none (defaults to all, multiple -a\n",
+    "    options are permitted)\n",
     "(Specify the --help global option for a list of other help options)\n",
     NULL
 };
@@ -31,9 +32,7 @@ const char *const watch_usage[] =
 static struct addremove_args the_args;
 
 void
-watch_modify_watchers (file, what)
-    char *file;
-    struct addremove_args *what;
+watch_modify_watchers (const char *file, struct addremove_args *what)
 {
     char *curattr = fileattr_get0 (file, "_watchers");
     char *p;
@@ -213,40 +212,33 @@ watch_modify_watchers (file, what)
 	free (mynewattr);
 }
 
-static int addremove_fileproc PROTO ((void *callerdat,
-				      struct file_info *finfo));
+static int addremove_fileproc (void *callerdat,
+				      struct file_info *finfo);
 
 static int
-addremove_fileproc (callerdat, finfo)
-    void *callerdat;
-    struct file_info *finfo;
+addremove_fileproc (void *callerdat, struct file_info *finfo)
 {
     watch_modify_watchers (finfo->file, &the_args);
     return 0;
 }
 
-static int addremove_filesdoneproc PROTO ((void *, int, char *, char *,
-					   List *));
+
 
 static int
-addremove_filesdoneproc (callerdat, err, repository, update_dir, entries)
-    void *callerdat;
-    int err;
-    char *repository;
-    char *update_dir;
-    List *entries;
+addremove_filesdoneproc (void *callerdat, int err, const char *repository,
+                         const char *update_dir, List *entries)
 {
     if (the_args.setting_default)
 	watch_modify_watchers (NULL, &the_args);
     return err;
 }
 
-static int watch_addremove PROTO ((int argc, char **argv));
+
+
+static int watch_addremove (int argc, char **argv);
 
 static int
-watch_addremove (argc, argv)
-    int argc;
-    char **argv;
+watch_addremove (int argc, char **argv)
 {
     int c;
     int local = 0;
@@ -308,7 +300,7 @@ watch_addremove (argc, argv)
     }
 
 #ifdef CLIENT_SUPPORT
-    if (client_active)
+    if (current_parsed_root->isremote)
     {
 	start_server ();
 	ign_setup ();
@@ -318,25 +310,14 @@ watch_addremove (argc, argv)
 	/* FIXME: copes poorly with "all" if server is extended to have
 	   new watch types and client is still running an old version.  */
 	if (the_args.edit)
-	{
-	    send_arg ("-a");
-	    send_arg ("edit");
-	}
+	    option_with_arg ("-a", "edit");
 	if (the_args.unedit)
-	{
-	    send_arg ("-a");
-	    send_arg ("unedit");
-	}
+	    option_with_arg ("-a", "unedit");
 	if (the_args.commit)
-	{
-	    send_arg ("-a");
-	    send_arg ("commit");
-	}
+	    option_with_arg ("-a", "commit");
 	if (!the_args.edit && !the_args.unedit && !the_args.commit)
-	{
-	    send_arg ("-a");
-	    send_arg ("none");
-	}
+	    option_with_arg ("-a", "none");
+	send_arg ("--");
 	send_files (argc, argv, local, 0, SEND_NO_CONTENTS);
 	send_file_names (argc, argv, SEND_EXPAND_WILD);
 	send_to_server (the_args.adding ?
@@ -348,39 +329,34 @@ watch_addremove (argc, argv)
 
     the_args.setting_default = (argc <= 0);
 
-    lock_tree_for_write (argc, argv, local, 0);
+    lock_tree_promotably (argc, argv, local, W_LOCAL, 0);
 
-    err = start_recursion (addremove_fileproc, addremove_filesdoneproc,
-			   (DIRENTPROC) NULL, (DIRLEAVEPROC) NULL, NULL,
-			   argc, argv, local, W_LOCAL, 0, 0, (char *)NULL,
-			   1);
+    err = start_recursion
+	(addremove_fileproc, addremove_filesdoneproc,
+	 NULL, NULL, NULL,
+	 argc, argv, local, W_LOCAL, 0, CVS_LOCK_WRITE,
+	 NULL, 1, NULL);
 
     Lock_Cleanup ();
     return err;
 }
 
 int
-watch_add (argc, argv)
-    int argc;
-    char **argv;
+watch_add (int argc, char **argv)
 {
     the_args.adding = 1;
     return watch_addremove (argc, argv);
 }
 
 int
-watch_remove (argc, argv)
-    int argc;
-    char **argv;
+watch_remove (int argc, char **argv)
 {
     the_args.adding = 0;
     return watch_addremove (argc, argv);
 }
 
 int
-watch (argc, argv)
-    int argc;
-    char **argv;
+watch (int argc, char **argv)
 {
     if (argc <= 1)
 	usage (watch_usage);
@@ -422,13 +398,11 @@ static const char *const watchers_usage[] =
     NULL
 };
 
-static int watchers_fileproc PROTO ((void *callerdat,
-				     struct file_info *finfo));
+static int watchers_fileproc (void *callerdat,
+				     struct file_info *finfo);
 
 static int
-watchers_fileproc (callerdat, finfo)
-    void *callerdat;
-    struct file_info *finfo;
+watchers_fileproc (void *callerdat, struct file_info *finfo)
 {
     char *them;
     char *p;
@@ -473,13 +447,12 @@ watchers_fileproc (callerdat, finfo)
 	cvs_output ("\n", 1);
     }
   out:;
+    free (them);
     return 0;
 }
 
 int
-watchers (argc, argv)
-    int argc;
-    char **argv;
+watchers (int argc, char **argv)
 {
     int local = 0;
     int c;
@@ -508,13 +481,14 @@ watchers (argc, argv)
     argv += optind;
 
 #ifdef CLIENT_SUPPORT
-    if (client_active)
+    if (current_parsed_root->isremote)
     {
 	start_server ();
 	ign_setup ();
 
 	if (local)
 	    send_arg ("-l");
+	send_arg ("--");
 	send_files (argc, argv, local, 0, SEND_NO_CONTENTS);
 	send_file_names (argc, argv, SEND_EXPAND_WILD);
 	send_to_server ("watchers\012", 0);
@@ -522,8 +496,9 @@ watchers (argc, argv)
     }
 #endif /* CLIENT_SUPPORT */
 
-    return start_recursion (watchers_fileproc, (FILESDONEPROC) NULL,
-			    (DIRENTPROC) NULL, (DIRLEAVEPROC) NULL, NULL,
-			    argc, argv, local, W_LOCAL, 0, 1, (char *)NULL,
-			    1);
+    return start_recursion
+	( watchers_fileproc, (FILESDONEPROC) NULL,
+	  (DIRENTPROC) NULL, (DIRLEAVEPROC) NULL, NULL,
+	  argc, argv, local, W_LOCAL, 0, CVS_LOCK_READ,
+	  (char *) NULL, 1, (char *) NULL );
 }
